#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include "videosystem.h"
#include "algorithms.h"
#include "videofileload.h"
#include "tine.h"
#include "listener.h"

typedef struct VideoHdrStruct
{ 
  int kennung;
  int compression;
  int len_total;
  int len_data;
  int len_fmthdr;
  int frameno;
  struct _timeb timestamp;
  int cameraport;
  int widthframe;
  int heightframe;
  int compressed;
  unsigned char Reserved[12];
  double scr_scale;
  int bpp;
  int bpp_effective;
  unsigned char reserved1[8];
} VIDHDR;

#ifndef _TINEDEFS_H_
#define UINT32 unsigned int
#define SINT32 int

#define IF_GRAY   0
#define IF_RGB    1
#define IF_RGBA   2
#define IF_RGB15  3
#define IF_RGB16  4
#define IF_JPEG   5
#define IF_TIFF   6
#define IF_YUV411 7
#define IF_YUV422 8
#define IF_YUV444 9
#define IF_HUFFYUV 10
#define IF_BITMAP 11

/* image flags */
#define IFLG_LITTLE_ENDIAN   0x1
#define IFLG_LOSSLESS		 0x2 // SW! Aug 31, 2010 changed naming -- this flag is only to tag that 
                             // SW! Aug 31, 2010 the frame has not been altered in such a way that 
								             // SW! Aug 31, 2010 original data can not be retrieved 100% (lossless)
#define IFLG_HORIZONTAL_FLIP 0x4
#define IFLG_VERTICAL_FLIP   0x8
#define IFLG_XYSTART_ONE_BASED 0x10 // SW! Aug 31, 2010 added this here it was introduced ~1 year ago for doocs
                                    // SW! Aug 31, 2010 image server compatibility

// SW! there are more flags, but as they don't have anything to do with image itself (but rather with 
// SW! event number synchronisation, they are omitted here

/**
 * \struct TImageSourceHeader
 * \brief Defines a sourceHeader header (used in conjuction with an image header) 
 *
 */
typedef struct
{
  UINT32 baseTag; /**< base id */
  UINT32 cameraPortId; /**< specific id */
  UINT32 versionTag;    /**< version */
  UINT32 totalLength; /**< total length in bytes (all headers + frame) */
  SINT32 timestampSeconds; /**< image timestamp (UTC seconds) */
  SINT32 timestampMicroseconds; /**< image timestamp (micro seconds) */
  char cameraPortName[64];  /**< camera port name */
} TImageSourceHeader;

/**
 * \struct TImageFrameHeader
 * \brief Defines a canonical image frame header 
 *
 * Standard image frame header, 100 bytes long (not divisible by 8!)
 */
typedef struct
{
  SINT32 sourceWidth;
  SINT32 sourceHeight;
  SINT32 aoiWidth;
  SINT32 aoiHeight;
  SINT32 xStart;
  SINT32 yStart;
  SINT32 bytesPerPixel;
  SINT32 effectiveBitsPerPixel;
  SINT32 horizontalBinning;
  SINT32 verticalBinning;
  SINT32 sourceFormat;
  SINT32 imageFormat;
  UINT32 frameNumber;
  UINT32 eventNumber;
  float xScale;
  float yScale;
  float imageRotation;
  float fspare1;
  float fspare2;
  float fspare3;
  SINT32 imageFlags;
  SINT32 ispare1;
  SINT32 ispare2;
  SINT32 ispare3;
  SINT32 appendedFrameSize;
} TImageFrameHeader;

typedef struct
{
  TImageSourceHeader sourceHeader;
  TImageFrameHeader frameHeader;
} hdrIMAGE;

/**
 * \struct DIMAGE
 * \brief Defines a TINE image data object containing an image source header, an image
 * frame header and the image frame itself.
 *
 * \note format CF_AIMAGE is used to send/receive arrays of images, in which case
 *  the dArrayLength field of DTYPE gives the number of images.  If CF_IMAGE is
 *  used, the dArrayLength field gives the size (in bytes) of the image frame to be sent or
 *  received.
 *  Either CF_AIMAGE or CF_IMAGE can be used in structures.
 *
 */
typedef struct
{
  TImageSourceHeader sourceHeader; /**< is the TImageSourceHeader defining the image source and camera parameters */
  TImageFrameHeader frameHeader;   /**< frameHeader is the TImageFrameHeader giving all of the image frame characteristics */
  UINT32 frameBufferSize;          /**< frameBufferSize if the size in bytes of the frame buffer
                                        With CF_IMAGE (non-adjustable) this value is an assigned fixed value giving
                                        the maximum frame size. With CF_AIMAGE (adjustable) this value will be updated
                                        with each frame, gving the size of the current image. */
  BYTE *frameBuffer;               /**< frameBuffer is a reference to the frame buffer holding the current image.
                                        With CF_IMAGE (non-adjustable) this is a fixed reference to a frame buffer.
                                        With CF_AIMAGE (adjustable) this is a volatile reference to the current image 
                                        frame.  */
} DIMAGE;
#endif

VS_EXPORT INT32 WINAPI convert_formatA( struct VIDINFHDR *vinf, UINT32 newtype, UINT32 downscale_function)
{
 return convert_format(vinf,newtype, downscale_function);
}
VS_EXPORT INT32 WINAPI xrayfilterA( struct VIDINFHDR *vinout)
{
  return xrayfilter(vinout);
}
VS_EXPORT INT32 WINAPI normalizeframes_rectA( struct VIDINFHDR *vinout, UINT32 x1, UINT32 y1, UINT32 x2, UINT32 y2, BOOL upperonly )
{
  return normalizeframes_rect(vinout,x1,y1,x2,y2,upperonly);
}
VS_EXPORT INT32 WINAPI normalizeframesA( struct VIDINFHDR *vinout, BOOL upperonly)
{
  return normalizeframes(vinout,upperonly);
}
VS_EXPORT void WINAPI initialize_vidinfhdrA	( struct VIDINFHDR *vinf )
{
  initialize_vidinfhdr(vinf);
}
VS_EXPORT void WINAPI free_vidinfhdrA	( struct VIDINFHDR *vinf )
{
  free_vidinfhdr(vinf );
}

VS_EXPORT INT32 WINAPI load_imm_fileA	( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_imm_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_bkg_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_bkg_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_imc_bkc_fileA		( CHAR *filename, struct VIDINFHDR *vinf, INT32 type)
{
  return load_imc_bkc_file( filename, vinf,type);
}

VS_EXPORT INT32 WINAPI load_imc_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_imc_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_bkc_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_bkc_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI  load_fileA			( INT32 type, char *filename, struct VIDINFHDR *vinf )
{
  return load_file( type, filename, vinf );
}

VS_EXPORT INT32 WINAPI  save_fileA			( INT32 type, char *filename, struct VIDINFHDR *vinf )
{
  return save_file(type, filename, vinf );
}

VS_EXPORT INT32 WINAPI  save_imm_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_imm_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI  save_bkg_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_bkg_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI  save_bkc_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_bkc_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI save_imc_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_imc_file( outfilename, vinf );
}

VS_EXPORT VOID WINAPI decompress_huffyuvA( VOIDP uncompressed, const VOIDP compressed, UINT32 size_uncompressed)
{
  decompress_huffyuv(uncompressed, compressed, size_uncompressed);
}

BYTE *imgRawBuffer = NULL;
int imgRawBufferSize = 0;

/* SW! Aug 31, 2010 added documentation of function 

	- limitations of function (at the moment Aug 31, 2010)
	   - only grayscale images of 8 bytes per pixel are supported (no gray >8, no colour, no jpeg)
	   - IF_HUFFYUV decompression is supported here although currently this mode is not existing in reality somewhere

     The function is meant to decode video system 3 header and bits into a simple buffer containing 8 bit grayscale
	 data plus three data fields: frameSize, frameWidth and frameHeight

	parameters:
		frame [out] - decoded frame contents (top-down, left to right, pixel by pixel, no padding, 
		              no offset 8 bits per pixel grayscale data)

		imageHdr [in] - pointer to header of VSv3 DIMAGE type (frameHeader and sourceHeader as TINE hdrIMAGE struct)
		imageBytes [in] - pointer to buffer containing raw bits described by imageHdr
		
 	    frameSize [in/out] - in: size of buffer at 'frame'
		                     out: in case of return dimension_error it will show required size
							      in case of return 0 (success) it will show size that has been used

		frameWidth [out] - width (number of pixels in x) of 'frame'
		frameHeight [out] - height (number of pixels in y) of 'frame' 
		   


    return value:
		tine completion code
			20
			0
			dimension_error
			not_implemented (added by SW! Aug 31, 2010)
  
	Limitations:
		validity/integrity of header fields (behind imageHdr) is not checked
		aoi handling is limited so that only the contents inside the AOI are returned in frame
		(there is no possibility of black rectangle surrounding aoi (and depicting whole sourceWidthxsourceHeight original
		 frame)

  */


VS_EXPORT INT32 getFrameFromImageBytes(BYTE *frame,const BYTE *imageHdr,const BYTE *imageBytes,int *frameSize, int *frameWidth, int *frameHeight)
{
	int siz, rawSiz, fHeight, fWidth;
	hdrIMAGE *hdr = (hdrIMAGE *)imageHdr;
	static int sizvh = sizeof(hdrIMAGE);

	if (frameSize == NULL || imageHdr == NULL || imageBytes == NULL || frame == NULL) return 20; // argument_list_error;

	if (hdr->frameHeader.imageFormat != IF_GRAY && hdr->frameHeader.imageFormat != IF_HUFFYUV)
	return not_implemented;

	if (hdr->frameHeader.bytesPerPixel != 1) return not_implemented;

	fHeight = hdr->frameHeader.aoiHeight != -1 ? hdr->frameHeader.aoiHeight : hdr->frameHeader.sourceHeight;
	fWidth = hdr->frameHeader.aoiWidth != -1 ? hdr->frameHeader.aoiWidth : hdr->frameHeader.sourceWidth;

	siz = fHeight * fWidth * hdr->frameHeader.bytesPerPixel;
	rawSiz = hdr->frameHeader.appendedFrameSize;
	
	if (siz > *frameSize)
	{
		*frameSize = siz;
		return dimension_error;
	}
      
	if (hdr->frameHeader.imageFormat == IF_HUFFYUV)
	{ 
		// is compressed
		if (frame == imageBytes)
		{
			if (imgRawBufferSize < rawSiz)
			{
				imgRawBufferSize = rawSiz + (rawSiz + 4096)%4096;
		    if (imgRawBuffer != NULL) free(imgRawBuffer);
				if ((imgRawBuffer=(BYTE *)malloc(imgRawBufferSize)) == NULL) return out_of_local_memory;
			}
			memcpy(imgRawBuffer,imageBytes,rawSiz);
			imageBytes = imgRawBuffer;
		}
		
		decompress_huffyuv(frame,(VOIDP)imageBytes,siz);
	}
	else
	{
		// is not compressed    
		if (frame != imageBytes) memcpy(frame,imageBytes,siz);
	}
  
	*frameSize = siz;

	if (frameWidth != NULL) *frameWidth = fWidth; //SW! Aug 31, 2010 fixed, was: sourceWidth
	if (frameHeight != NULL) *frameHeight = fHeight; //SW! Aug 31, 2010 fixed, was: sourceHeight

	return 0;
}

VS_EXPORT INT32 WINAPI getFrameFromImageBytesA(BYTE *frame,const BYTE *imageHdr,const BYTE *imageBytes,int *frameSize, int *frameWidth, int *frameHeight)
{
  return getFrameFromImageBytes(frame,imageHdr,imageBytes,frameSize,frameWidth,frameHeight);
}

VS_EXPORT INT32 GetImageFromVideoServer(char *fullDeviceNameAndProperty,BYTE *frame,int *frameSize, int *frameWidth, int *frameHeight,double *dTimeStamp)
{
  int cc = 0;
  DTYPE dout;
  DIMAGE img;

  memset(&dout,0,sizeof(DTYPE));
  img.frameBuffer = frame;
  img.frameBufferSize = *frameSize;
  dout.dArrayLength = *frameSize;
  dout.dFormat = CF_IMAGE;
  dout.data.vptr = &img;

  cc = alsnCall(fullDeviceNameAndProperty,&dout,NULL,CA_READ,dTimeStamp);
  if (cc == 0)
  {
    cc = getFrameFromImageBytes(frame,(BYTE *)&img,img.frameBuffer,frameSize,frameWidth,frameHeight);
  }
  return cc;
}
VS_EXPORT INT32 WINAPI GetImageFromVideoServerA(char *fullDeviceNameAndProperty,BYTE *frame,int *frameSize, int *frameWidth, int *frameHeight,double *dTimeStamp)
{
  return GetImageFromVideoServer(fullDeviceNameAndProperty,frame,frameSize,frameWidth,frameHeight,dTimeStamp);
}

VS_EXPORT INT32 getFrameFromImageBlob(unsigned char *frame,const unsigned char *imageBlob,int *frameSize, int *frameWidth, int *frameHeight)
{
  int siz;
  VIDHDR *videoHdr = (VIDHDR *)imageBlob;
  static int sizvh = sizeof(VIDHDR);
  
  if (frameSize == NULL) return 20; // argument_list_error;
  
  //memcpy(videoHdr,imageBlob,sizvh); // SW! disabled Aug 31, 2010 - dangerous and imo (sorry) stupid

  if (videoHdr->kennung != 0x11223344)
  {
    *frameSize = 0;
    return 103; // invalid_data;
  }

  siz = videoHdr->heightframe * videoHdr->widthframe * videoHdr->bpp / 8;
  if (siz > *frameSize)
  {
    *frameSize = siz;
    return 23; //dimension_error;
  }
  if (videoHdr->compressed == 1)
  { // is compressed
    decompress_huffyuv(frame,(VOIDP)&imageBlob[sizvh + videoHdr->len_fmthdr],siz);
  }
  else
  { // is not compressed    
    memcpy(frame,&imageBlob[sizvh],siz);
  }
  *frameSize = siz;
  if (frameWidth != NULL) *frameWidth = videoHdr->widthframe;
  if (frameHeight != NULL) *frameHeight = videoHdr->heightframe;
  return 0;
}

VS_EXPORT INT32 WINAPI getFrameFromImageBlobA(unsigned char *frame,const unsigned char *imageBlob,int *frameSize, int *frameWidth, int *frameHeight)
{
  return getFrameFromImageBlob(frame,imageBlob,frameSize,frameWidth,frameHeight);
}

BOOL WINAPI DllMain(HINSTANCE hInstDll,DWORD fdwReason,LPVOID lpvReserved)
{
  switch (fdwReason)
  {
    case DLL_PROCESS_ATTACH:  // DLL being loaded
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
    case DLL_PROCESS_DETACH:  // DLL being unloaded
      break;
  }
  return (TRUE);
}
