#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include "videosystem.h"
#include "videofileload.h"

typedef struct VideoHdrStruct
{ 
  int kennung;
  int compression;
  int len_total;
  int len_data;
  int len_fmthdr;
  int frameno;
  struct _timeb timestamp;
  int cameraport;
  int widthframe;
  int heightframe;
  int compressed;
  unsigned char Reserved[12];
  double scr_scale;
  int bpp;
  int bpp_effective;
  unsigned char reserved1[8];
} VIDHDR;

VS_EXPORT void WINAPI initialize_vidinfhdrA		( struct VIDINFHDR *vinf )
{
  initialize_vidinfhdr(vinf);
}
VS_EXPORT void WINAPI free_vidinfhdrA	( struct VIDINFHDR *vinf )
{
  free_vidinfhdr(vinf );
}

VS_EXPORT INT32 WINAPI load_imm_fileA	( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_imm_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_bkg_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_bkg_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_imc_bkc_fileA		( CHAR *filename, struct VIDINFHDR *vinf, INT32 type)
{
  return load_imc_bkc_file( filename, vinf,type);
}

VS_EXPORT INT32 WINAPI load_imc_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_imc_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI load_bkc_fileA			( CHAR *filename, struct VIDINFHDR *vinf )
{
  return load_bkc_file( filename, vinf );
}

VS_EXPORT INT32 WINAPI  load_fileA			( INT32 type, char *filename, struct VIDINFHDR *vinf )
{
  return load_file( type, filename, vinf );
}

VS_EXPORT INT32 WINAPI  save_fileA			( INT32 type, char *filename, struct VIDINFHDR *vinf )
{
  return save_file(type, filename, vinf );
}

VS_EXPORT INT32 WINAPI  save_imm_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_imm_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI  save_bkg_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_bkg_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI  save_bkc_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_bkc_file( outfilename, vinf );
}

VS_EXPORT INT32 WINAPI save_imc_fileA			( char *outfilename, struct VIDINFHDR *vinf )
{
  return save_imc_file( outfilename, vinf );
}

VS_EXPORT VOID WINAPI decompress_huffyuvA( VOIDP uncompressed, const VOIDP compressed, UINT32 size_uncompressed)
{
  decompress_huffyuv(uncompressed, compressed, size_uncompressed);
}

VS_EXPORT INT32 getFrameFromImageBlob(unsigned char *frame,const unsigned char *imageBlob,int *frameSize, int *frameWidth, int *frameHeight)
{
  int siz;
  VIDHDR *videoHdr = (VIDHDR *)imageBlob;
  static int sizvh = sizeof(VIDHDR);

  if (frameSize == NULL) return 20; // argument_list_error;
  memcpy(videoHdr,imageBlob,sizvh);

  if (videoHdr->kennung != 0x11223344)
  {
    *frameSize = 0;
    return 103; // invalid_data;
  }

  siz = videoHdr->heightframe * videoHdr->widthframe * videoHdr->bpp / 8;
  if (siz > *frameSize)
  {
    *frameSize = siz;
    return 23; //dimension_error;
  }
  if (videoHdr->compressed == 1)
  { // is compressed
    decompress_huffyuv(frame,(VOIDP)&imageBlob[sizvh + videoHdr->len_fmthdr],siz);
  }
  else
  { // is not compressed    
    memcpy(frame,&imageBlob[sizvh],siz);
  }
  *frameSize = siz;
  if (frameWidth != NULL) *frameWidth = videoHdr->widthframe;
  if (frameHeight != NULL) *frameHeight = videoHdr->heightframe;
  return 0;
}

VS_EXPORT INT32 WINAPI getFrameFromImageBlobA(unsigned char *frame,const unsigned char *imageBlob,int *frameSize, int *frameWidth, int *frameHeight)
{
  return getFrameFromImageBlob(frame,imageBlob,frameSize,frameWidth,frameHeight);
}

BOOL WINAPI DllMain(HINSTANCE hInstDll,DWORD fdwReason,LPVOID lpvReserved)
{
  switch (fdwReason)
  {
    case DLL_PROCESS_ATTACH:  // DLL being loaded
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
    case DLL_PROCESS_DETACH:  // DLL being unloaded
      break;
  }
  return (TRUE);
}
