Attribute VB_Name = "VideoSystem"
Option Explicit

Global Const FCN_NONE = 0
Global Const FCN_AVERAGE = 1
Global Const FCN_ENVELOPE = 2
Global Const FCN_SIGNIFICANCE = 3


Global Const TYPE_NONE_EMPTY = -1
Global Const TYPE_CHOOSE = 0
Global Const TYPE_IMM = 1
Global Const TYPE_IMC = 2
Global Const TYPE_BKG = 3
Global Const TYPE_BKC = 4
Global Const TYPE_SIGNIFICANCE = 5

Global Const SRCCOPY = &HCC0020               '/* dest = source                   */
Global Const SRCPAINT = &HEE0086              '/* dest = source OR dest           */
Global Const SRCAND = &H8800C6                '/* dest = source AND dest          */
Global Const SRCINVERT = &H660046             '/* dest = source XOR dest          */
Global Const SRCERASE = &H440328              '/* dest = source AND (NOT dest )   */
Global Const NOTSRCCOPY = &H330008            '/* dest = (NOT source)             */
Global Const NOTSRCERASE = &H1100A6           '/* dest = (NOT src) AND (NOT dest) */
Global Const MERGECOPY = &HC000CA             '/* dest = (source AND pattern)     */
Global Const MERGEPAINT = &HBB0226            '/* dest = (NOT source) OR dest     */
Global Const PATCOPY = &HF00021               '/* dest = pattern                  */
Global Const PATPAINT = &HFB0A09              '/* dest = DPSnoo                   */
Global Const PATINVERT = &H5A0049             '/* dest = pattern XOR dest         */
Global Const DSTINVERT = &H550009             '/* dest = (NOT dest)               */
Global Const BLACKNESS = &H42                 '/* dest = BLACK                    */
Global Const WHITENESS = &HFF0062             '/* dest = WHITE                    */

' required structure



Type VIDINFHDR
  scr_width As Long
  scr_height As Long
  scr_bpp As Long
  scr_bpp_effective As Long
  scr_scale As Double
  scr_numframes As Long
  scr_type As Long
  scr_lplp_framebits As Long
  scr_lp_framescales As Long
End Type

Type TIMEB
  time As Long
  millitm As Integer
  timezone As Integer
  dstflag As Integer
  Reserved(1) As Byte
End Type

Type VIDHDR
  kennung As Long
  compression As Long
  len_total As Long
  len_data As Long
  len_fmthdr As Long
  frameno As Long
  timestamp As TIMEB
  cameraport As Long
  widthframe As Long
  heightframe As Long
  compressed As Long
  Reserved(11) As Byte

  scr_scale As Double
  bpp As Long
  bpp_effective As Long
  
  reserved1(7) As Byte
End Type

Type BITMAPINFOHEADER '40 bytes
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type
Type RGBQUAD '4 bytes
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type
Type BITMAPINFO '44 bytes
    bmiHeader As BITMAPINFOHEADER
    bmiColors(255) As RGBQUAD
End Type

Global rgbColors(255) As RGBQUAD
Global greyColors(255) As RGBQUAD
Global matlabColors(255) As RGBQUAD
Global pitzColors(255) As RGBQUAD

Global Const DIB_RGB_COLORS = 0&
Global Const BI_RGB = 0&

Declare Sub initialize_vidinfhdr Lib "videosystem.dll" Alias "initialize_vidinfhdrA" (vinf As VIDINFHDR)
Declare Sub free_vidinfhdr Lib "videosystem.dll" Alias "free_vidinfhdrA" (vinf As VIDINFHDR)

Declare Function load_imm_file Lib "videosystem.dll" Alias "load_imm_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function load_bkg_file Lib "videosystem.dll" Alias "load_bkg_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long

Declare Function load_imc_bkc_file Lib "videosystem.dll" Alias "load_imc_bkc_fileA" (ByVal filename As String, vinf As VIDINFHDR, ByVal scrType As Long) As Long
Declare Function load_imc_file Lib "videosystem.dll" Alias "load_imc_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function load_bkc_file Lib "videosystem.dll" Alias "load_bkc_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function load_file Lib "videosystem.dll" Alias "load_fileA" (ByVal scrType As Long, ByVal filename As String, vinf As VIDINFHDR) As Long

Declare Function save_file Lib "videosystem.dll" Alias "save_fileA" (ByVal scrType As Long, ByVal filename As String, vinf As VIDINFHDR) As Long

Declare Function save_imm_file Lib "videosystem.dll" Alias "save_imm_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function save_bkg_file Lib "videosystem.dll" Alias "save_bkg_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function save_bkc_file Lib "videosystem.dll" Alias "save_bkc_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long
Declare Function save_imc_file Lib "videosystem.dll" Alias "save_imc_fileA" (ByVal filename As String, vinf As VIDINFHDR) As Long

Declare Sub decompress_huffyuv Lib "videosystem.dll" Alias "decompress_huffyuvA" (uncompressed As Any, compressed As Any, ByVal size_compressed As Long)
Declare Function getFrameFromImageBlob Lib "videosystem.dll" Alias "getFrameFromImageBlobA" (frame As Byte, imageBlob As Byte, frameSize As Long, frameWidth As Long, frameHeight As Long) As Long

Declare Function SetDIBits Lib "gdi32" (ByVal hdc As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As BITMAPINFO, ByVal wUsage As Long) As Long


Declare Function StretchDIBits Lib "gdi32" ( _
  ByVal hdc As Long, _
  ByVal XDest As Long, _
  ByVal YDest As Long, _
  ByVal nDestWidth As Long, _
  ByVal nDestHeight As Long, _
  ByVal XSrc As Long, _
  ByVal YSrc As Long, _
  ByVal nSrcWidth As Long, _
  ByVal nSrcHeight As Long, _
  lpBits As Any, _
  lpBitsInfo As BITMAPINFO, _
  ByVal iUsage As Long, _
  ByVal dwRop As Long _
) As Long

Sub FillInColorTables()
Dim i As Integer
Dim different_colorvalues As Long
Dim different_effective_colorvalues As Long
Dim step1min As Long
Dim step2min As Long
Dim step3min As Long
Dim step4min As Long
Dim step5min As Long
Dim step1max As Long
Dim step2max As Long
Dim step3max As Long
Dim step4max As Long
Dim step5max As Long


Static done As Boolean
If done Then Exit Sub

For i = 0 To 255
 rgbColors(i).rgbRed = (IIf(i > 170, 1, 0) * ((i - 170) * 190 / 85 + 65)) Mod 255
 rgbColors(i).rgbGreen = (IIf(i > 85, 1, 0) * IIf(i < 171, 1, 0) * (190 * (i - 85) / 85 + 65) + 255 * IIf(i = 255, 1, 0)) Mod 255
 rgbColors(i).rgbBlue = (IIf(i > 0, 1, 0) * IIf(i < 150, 1, 0) * 255 * (150 - i + 1) / 150 + 255 * IIf(i = 255, 1, 0)) Mod 255
 rgbColors(i).rgbReserved = 0
 greyColors(i).rgbRed = i
 greyColors(i).rgbBlue = i
 greyColors(i).rgbGreen = i
 greyColors(i).rgbReserved = 0
Next

different_colorvalues = 256   '1 << m_iActualBpp
different_effective_colorvalues = 256 '1 << m_iActualBppEffective
step1min = 0
step1max = (different_effective_colorvalues / 8) - 1
step2min = step1max + 1
step2max = ((different_effective_colorvalues * 3) / 8) - 1
step3min = step2max + 1
step3max = ((different_effective_colorvalues * 5) / 8) - 1
step4min = step3max + 1
step4max = ((different_effective_colorvalues * 7) / 8) - 1
step5min = step4max + 1
step5max = different_effective_colorvalues - 1

matlabColors(different_effective_colorvalues - 1).rgbRed = 255
matlabColors(different_effective_colorvalues - 1).rgbGreen = 255
matlabColors(different_effective_colorvalues - 1).rgbBlue = 255

For i = step1min To step2min - 1
  matlabColors(i).rgbRed = 0
  matlabColors(i).rgbGreen = 0
  matlabColors(i).rgbBlue = CInt((((i / 2#) / step2min) + 0.5) * 255#)
Next

For i = step2min To step3min - 1
  matlabColors(i).rgbRed = 0
  matlabColors(i).rgbGreen = CInt(((i / (step2min * 2#)) - 0.5) * 255#)
  matlabColors(i).rgbBlue = 255
Next

For i = step3min To step4min - 1
  matlabColors(i).rgbRed = CInt(((i / (step2min * 2#)) - 1.5) * 255#)
  matlabColors(i).rgbGreen = 255
  matlabColors(i).rgbBlue = CInt((1# - ((i / (step2min * 2#)) - 1.5)) * 255#)
Next

For i = step4min To step5min - 1
  matlabColors(i).rgbRed = 255
  matlabColors(i).rgbGreen = CInt(((i / (-(step2min * 2#))) + 3.5) * 255#)
  matlabColors(i).rgbBlue = 0
Next

For i = step5min To step5max - 1
  matlabColors(i).rgbRed = CInt(((i / (-(step2min * 2#))) + 4.5) * 255#)
  matlabColors(i).rgbGreen = 0
  matlabColors(i).rgbBlue = 0
Next

different_colorvalues = 256   '1 << m_iActualBpp
different_effective_colorvalues = 256 '1 << m_iActualBppEffective
step1min = 1
step1max = different_effective_colorvalues / 3
step2min = step1max + 1
step2max = ((different_effective_colorvalues * 293) / 500)
step3min = step2max + 1
step3max = ((different_effective_colorvalues * 2) / 3)
step4min = step3max + 1
step4max = different_effective_colorvalues - 1
step5min = step4max + 1
step5max = different_effective_colorvalues - 1
      
pitzColors(different_effective_colorvalues - 1).rgbRed = 255
pitzColors(different_effective_colorvalues - 1).rgbGreen = 255
pitzColors(different_effective_colorvalues - 1).rgbBlue = 255

Dim m1, n1, m2, n2
  
m1 = -0.5608 / (different_effective_colorvalues / 3# - 1#)
n1 = 1# - m1 * 1#
For i = step1min To step2min - 1
  pitzColors(i).rgbRed = 0
  pitzColors(i).rgbGreen = 0
  pitzColors(i).rgbBlue = CInt((m1 * i + n1) * 255#)
Next
m1 = 0.561 / ((different_effective_colorvalues * 293# / 500#) - (different_effective_colorvalues / 3#))
n1 = 0.263 - (m1 * (different_effective_colorvalues / 3))
m2 = -0.4314 / ((different_effective_colorvalues * 293 / 500) - (different_effective_colorvalues / 3))
n2 = 0 - (m2 * (different_effective_colorvalues * 293 / 500))
For i = step2min To step3min - 1
  pitzColors(i).rgbRed = 0
  pitzColors(i).rgbGreen = CInt((m1 * i + n1) * 255#)
  pitzColors(i).rgbBlue = CInt((m2 * i + n2) * 255#)
Next
m1 = 0.176 / ((different_effective_colorvalues * 2 / 3) - (different_effective_colorvalues * 293 / 500))
n1 = 1 - (m1 * (different_effective_colorvalues * 2 / 3))
For i = step3min To step4min - 1
  pitzColors(i).rgbRed = 0
  pitzColors(i).rgbGreen = CInt((m1 * i + n1) * 255#) Mod 255
  pitzColors(i).rgbBlue = 0
Next
m1 = 0.737 / (different_effective_colorvalues - (different_effective_colorvalues * 2 / 3))
n1 = 1 - (m1 * different_effective_colorvalues)
For i = step4min To step4max - 1
  pitzColors(i).rgbRed = CInt((m1 * i + n1) * 255#)
  pitzColors(i).rgbGreen = 0
  pitzColors(i).rgbBlue = 0
Next

done = True
End Sub
Sub ImageToPictureBox(vh As VIDHDR, bp() As Byte, pic As PictureBox, colors() As RGBQUAD)
  Dim linesscanned As Long
  Dim bm As BITMAPINFO

  ' PICTURE-BOX pic MUST HAVE AUTOREDRAW = TRUE!
  ' and SCALEMODE = PIXEL
  
  ' copy color table
  MEMCPY bm.bmiColors(0), colors(0), 256 * 4

  ' fill in proper bitmpapinfoheader values
  bm.bmiHeader.biSize = Len(bm.bmiHeader)
  bm.bmiHeader.biWidth = vh.widthframe
  bm.bmiHeader.biHeight = -vh.heightframe
  bm.bmiHeader.biPlanes = 1
  bm.bmiHeader.biBitCount = 8
  bm.bmiHeader.biCompression = BI_RGB
  bm.bmiHeader.biSizeImage = 0 'UBound(bp)
  bm.bmiHeader.biXPelsPerMeter = 0
  bm.bmiHeader.biYPelsPerMeter = 0
  bm.bmiHeader.biClrUsed = 0
  bm.bmiHeader.biClrImportant = 0
  ' display it in the picture box
  'linesscanned = SetDIBits(pic.hdc, pic.Image, 0, vh.heightframe, bp(0), bm, DIB_RGB_COLORS) ' no stretching
  linesscanned = StretchDIBits(pic.hdc, 0, 0, pic.ScaleWidth, pic.ScaleHeight, 0, 0, vh.widthframe, vh.heightframe, bp(0), bm, DIB_RGB_COLORS, SRCCOPY)
  Debug.Print linesscanned
    
  pic.Visible = True
  pic.Refresh
  
End Sub

