//{{AFX_INCLUDES()
#include "srv.h"
//}}AFX_INCLUDES
#if !defined(AFX_BOGUSTINESERVERDLG_H__168E31B3_23C7_11D7_AFDB_0002B34B0F4F__INCLUDED_)
#define AFX_BOGUSTINESERVERDLG_H__168E31B3_23C7_11D7_AFDB_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// BogusTineServerDlg.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CBogusTineServerDlg dialog
//
// This class implements the TINE Server for delivering the video data using 
// multicast transmission to all via TINE protocol connected clients.
// After instantiating the class you have to call the InitializeTine() method 
// to initialize the underlying ActiveX-control and all data structures needed
// for the transfer. If the function returns TRUE everything is fine and you can
// call PutFrame() to input a single video frame into the class. This frame gets 
// send out on the next client request that asks for frame data.
//
// The TINE property where the frame data resides is:
// Device Context  : 'PITZ'
// Device Server   : (hostname of the server computer in upper case)+'F' e.g. 'ZNPPITZFG1F'
// Device Name     : 'device_0'
// Device Property : 'FRAME.GET'
//
// A valid video frame memory block (for PutFrame() ) consists of a video header (VIDHDR) 
// plus sometimes a frame header plus the frame data (bits), either compressed or 
// uncompressed.
//
// After sending out all frames (mostly this is the case when shutting down the server), one
// has to call the DeinitializeTine() method to stop the TINE server and to clean up the 
// memory structures and variables. After calling DeinitializeTine(), the class can be safely 
// destructed.
//
class CBogusTineServerDlg : public CDialog
{
// Construction
public:
    // constructs the class
	CBogusTineServerDlg(CWnd* pParent = NULL);   // standard constructor

    // initializes TINE server, returns TRUE on success, FALSE on error
    BOOL InitializeTine( CString servername, int size_in_bytes );

    // deinitializes TINE server, returns always TRUE
	BOOL DeinitializeTine();

	// puts a single video frame (VIDHDR+(fmthdr)+framebits) into the TINE server
	// the total size in bytes of the memory block must be passed via the framelength 
	// parameter.
	BOOL PutFrame( unsigned char *framebits, int framelength);

// Dialog Data
	//{{AFX_DATA(CBogusTineServerDlg)
	enum { IDD = IDD_DIALOG_BOGUSTINE };
	CSrv	m_CTineFrameServer;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBogusTineServerDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CBogusTineServerDlg)
	afx_msg void OnEqpFcnSrvctrl1Bogus(LPCTSTR devName, LPCTSTR devProperty, long outArrayLen, long inArrayLen, short devAccess);
	DECLARE_EVENTSINK_MAP()
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	// TRUE: TINE is successfully initialized
	BOOL m_bTineInitialized;
	
	// length of the current video frame
	int m_iAktFrameLength;
	
	// frame bits of the current video frame
	char *m_pAktFrameData;
	
    // TRUE: frame was not send out 
	BOOL m_iFirstTimeFrame;
	
	// variant for sending out the frame bits using TINE ActiveX control
	VARIANT m_vData;
	
	// points to the memory location occupied by m_vData
	char *m_pVariantData;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BOGUSTINESERVERDLG_H__168E31B3_23C7_11D7_AFDB_0002B34B0F4F__INCLUDED_)
