// Compression.h: interface for the CCompression class.
//
// This class compress the images that were acquired from the frame grabber
// using any VCM Codec you specify through the fourcc_codec parameter in the
// Init() function. Please note that temporal compression codecs do not work,
// just keyframe codecs like e.g. HuffYUV and MJPG are supported.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMPRESSION_H__180F6491_55C2_11D6_9B36_0004769003D4__INCLUDED_)
#define AFX_COMPRESSION_H__180F6491_55C2_11D6_9B36_0004769003D4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CCompression  
{
public:
	// construction/destruction
	CCompression();
	virtual ~CCompression();

    // initialize codec using the inputFmt image format specification
    BOOL Init( DWORD fourcc_codec, LPBITMAPINFOHEADER inputFmt);
	
	// compress a single frame ( frame -> compressedframe )
	BOOL CompressFrame( char *frame, char *compressedframe );

	// get length of the output format, which is generated by the codec 
	// for each frame
	BOOL GetOutputFormatLength( int *length );

	// get the output format of the last compressed frame
	BOOL GetOutputFormat( LPBITMAPINFOHEADER outputFmt );
	
	// deinitialize codec
	BOOL Done();

	// returns the maximum size of a compressed frame in bytes
	BOOL GetMaxCompressedSize( int *size );

    // returns the fourcc associated with the codec 
	DWORD ReturnFourccCodec();

protected:
    // own error handler, useful in debug version of the program
	int myErrorHandler( int errornum, const char *format, ... );

    // handle to the codec instance
	HIC m_pCompressor;

    // pointers to input and output format specifications
	LPBITMAPINFOHEADER m_pOutputFmt;
	LPBITMAPINFOHEADER m_pInputFmt;
    
	// are we initialized
	BOOL m_bInitialized;
private:
	// variable to hold the four character code
	DWORD m_iFourccCodec;
	// saves the last error code
	int m_iLastError;
};

#endif // !defined(AFX_COMPRESSION_H__180F6491_55C2_11D6_9B36_0004769003D4__INCLUDED_)
