// ConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Grabsrv.h"
#include "headers.h"
#include "IniEx.h"
#include "ConfigDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConfigDlg dialog
//
// The CConfigDlg class represents a configuration dialog for the grabber server.
// The class is based on the configuration dialog resource (IDD_CONFIGDLG_DIALOG).
// You can adjust certain settings. These are System number and board number 
// (PcVision grabber card settings), network settings (listen port and maximum 
// number of clients) and the four camera port description strings.
//

/////////////////////////////////////////////////////////////////////////////
//
// default construction created by class wizard, only the passing of nref and 
// pref (negative and positive reference of the ADC on the frame grabber card)
// was added
//
CConfigDlg::CConfigDlg(int nref0, int pref0, CWnd* pParent /*=NULL*/)
	: CDialog(CConfigDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConfigDlg)
	//}}AFX_DATA_INIT
	m_iNRef = nref0;
	m_iPRef = pref0;
}

/////////////////////////////////////////////////////////////////////////////
//
// data exchange function created by class wizard
//
void CConfigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigDlg)
	DDX_Control(pDX, IDC_EDIT6_IMAQINTERFACE, m_CEditImaqInterface);
	DDX_Control(pDX, IDC_EDIT9_TINEFRAMESERVER, m_CEditTineFrameServer);
	DDX_Control(pDX, IDC_EDIT7_TINEPORT, m_CEditTinePort);
	DDX_Control(pDX, IDC_EDIT8_TINECONTROLSERVER, m_CEditTineControlServer);
	DDX_Control(pDX, IDC_EDIT6_TINECONTEXT, m_CEditTineContext);
	DDX_Control(pDX, IDC_SLIDER1, m_CSliderJPEGQuality);
	DDX_Control(pDX, IDC_EDIT5, m_CEditJPEGQuality);
	DDX_Control(pDX, IDC_EDIT4_PREF0, m_CEditPRef0);
	DDX_Control(pDX, IDC_EDIT3_NREF0, m_CEditNRef0);
	DDX_Control(pDX, IDC_CHECK3_TRIGMOD5, m_CCheckTrigMod5);
	DDX_Control(pDX, IDC_CHECK2_EXTTRIG, m_CCheckExtTrig);
	DDX_Control(pDX, IDC_CHECK1, m_CCheckLossy);
	DDX_Control(pDX, IDC_EDIT4, m_CEditNumberClientsLossy);
	DDX_Control(pDX, IDC_EDIT3, m_CEditPortnumberLossy);
	DDX_Control(pDX, IDC_EDIT2, m_CEditNumberClients);
	DDX_Control(pDX, IDC_EDIT1, m_CEditPortnumber);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
//
// message map generated by class wizard
//
BEGIN_MESSAGE_MAP(CConfigDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigDlg)
	ON_BN_CLICKED(IDC_CHECK1, OnCheck1)
	ON_BN_CLICKED(IDC_CHECK2_EXTTRIG, OnCheck2Exttrig)
	ON_BN_CLICKED(IDC_CHECK3_TRIGMOD5, OnCheck3Trigmod5)
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigDlg message handlers


/////////////////////////////////////////////////////////////////////////////
//
// The OnOK function is called when the user presses the 'OK' button. The 
// settings are written to the CConfiguration instance and the dialog is 
// closed.
//
void CConfigDlg::OnOK() 
{
	CString msg;

    if (CheckParameters() == FALSE) return; // some parameters are not valid -> do not process ok

    // save PcVision settings into the configuration file
    m_CEditImaqInterface.GetWindowText(msg);
	m_pIniFile->SetValue( "DEFAULT", "ImaqInterface", msg );

   	m_CEditTinePort.GetWindowText(msg);
	m_pIniFile->SetValue("DEFAULT", "TINEPORTNUMBER", msg);

	m_CEditTineContext.GetWindowText(msg);
	m_pIniFile->SetValue("DEFAULT", "TINECONTEXT", msg);

	m_CEditTineControlServer.GetWindowText(msg);
    m_pIniFile->SetValue("DEFAULT", "TINECONTROLEQPNAME", msg);
	
	m_CEditTineFrameServer.GetWindowText(msg);
    m_pIniFile->SetValue("DEFAULT", "TINEFRAMEEQPNAME", msg);
	
	// obtain and save network settings

	m_CEditPortnumber.GetWindowText(msg);
	m_pIniFile->SetValue("DEFAULT", "LISTENPORTNUMBER", msg);

	m_CEditNumberClients.GetWindowText(msg);
    m_pIniFile->SetValue("DEFAULT", "MAXCLIENTS", msg);

	m_CEditPortnumberLossy.GetWindowText(msg);
	m_pIniFile->SetValue("DEFAULT", "LISTENPORTNUMBERLOSSY", msg);

	m_CEditNumberClientsLossy.GetWindowText(msg);
    m_pIniFile->SetValue("DEFAULT", "MAXCLIENTSLOSSY", msg);

    CString tmp;

	int check = m_CCheckLossy.GetCheck();
	m_pIniFile->SetValueInt("DEFAULT", "LOSSYENABLED", check);

    int ret = m_CCheckExtTrig.GetCheck();
	if (ret == 1) tmp = "TRUE"; else tmp = "FALSE";
	m_pIniFile->SetValue("DEFAULT", "ExternalTrigger", tmp);

    ret = m_CCheckTrigMod5.GetCheck();
	if (ret == 1) tmp = "TRUE"; else tmp = "FALSE";
	m_pIniFile->SetValue("DEFAULT", "TriggerMod5", tmp);

	int jpegqual = m_CSliderJPEGQuality.GetPos();
    m_pIniFile->SetValueInt("DEFAULT", "JPEGQuality", jpegqual);

	CDialog::OnOK();
}


/////////////////////////////////////////////////////////////////////////////
//
// The OnCancel function is called when the user presses the 'Cancel' button.
// No settings are stored in the CIniEx object and the dialog is 
// closed.
//
void CConfigDlg::OnCancel() 
{
	CDialog::OnCancel();
}

/////////////////////////////////////////////////////////////////////////////
//
// The setConfiguration function should be called just after the dialog is 
// constructed. You have to pass a reference to the instantiated CIniEx
// class which manages permanent settings of the application. If you have not 
// passed this reference before showing the dialog (DoModal), the dialog will
// cease to function.
// 
void CConfigDlg::setConfiguration( CIniEx *the_config )
{
 m_pIniFile = the_config;
}

/////////////////////////////////////////////////////////////////////////////
//
// This function is called when the dialog is about to be displayed on 
// screen. It initializes some controls on the dialog and reads the current
// settings from the CConfiguration instance.
//
BOOL CConfigDlg::OnInitDialog() 
{
	// call base class
	CDialog::OnInitDialog();

    // obtain network and PCVision settings from the
	// configuration class
	CString imaqInterface = m_pIniFile->GetValue("DEFAULT", "ImaqInterface", "img0");
;
	int portnum = m_pIniFile->GetValueInt( "DEFAULT", "LISTENPORTNUMBER", 31777);
	int maxclients = m_pIniFile->GetValueInt( "DEFAULT", "MAXCLIENTS", 255);
	int portnum2 = m_pIniFile->GetValueInt( "DEFAULT", "LISTENPORTNUMBERLOSSY", 31778);
	int maxclients2 = m_pIniFile->GetValueInt( "DEFAULT", "MAXCLIENTSLOSSY", 255);
    int enabled2 = m_pIniFile->GetValueInt( "DEFAULT", "LOSSYENABLED", 1);
    CString exttrigenabled = m_pIniFile->GetValue( "DEFAULT", "ExternalTrigger", "TRUE");
	CString exttrigmod5 = m_pIniFile->GetValue( "DEFAULT", "TriggerMod5", "FALSE");

	CString m_sTineContext = m_pIniFile->GetValue( "DEFAULT", "TINECONTEXT", "PITZ");
	CString m_sTineControlServer = m_pIniFile->GetValue( "DEFAULT", "TINECONTROLEQPNAME", "");
	CString m_sTineFrameServer = m_pIniFile->GetValue( "DEFAULT", "TINEFRAMEEQPNAME", "");
	int m_iTinePort = m_pIniFile->GetValueInt("DEFAULT", "TINEPORTNUMBER", 11);

	int jpegqual = m_pIniFile->GetValueInt("DEFAULT", "JPEGQuality", 60);

    // initialize the controls using the four settings
	// obtained above

	CString msg;

    m_CSliderJPEGQuality.SetRange(10, 90);
    m_CSliderJPEGQuality.SetTicFreq(1);
    m_CSliderJPEGQuality.SetPos(jpegqual);

	msg.Format("%d", m_iTinePort);
	m_CEditTinePort.SetWindowText((LPCTSTR) msg);
	
	m_CEditTineContext.SetWindowText((LPCTSTR) m_sTineContext);
	m_CEditTineControlServer.SetWindowText((LPCTSTR) m_sTineControlServer);
	m_CEditTineFrameServer.SetWindowText((LPCTSTR) m_sTineFrameServer);

	m_CEditImaqInterface.SetWindowText((LPCTSTR) imaqInterface );

	msg.Format("%d %%", m_CSliderJPEGQuality.GetPos());
	m_CEditJPEGQuality.SetWindowText((LPCTSTR) msg);
	
	msg.Format("%d", portnum);
	m_CEditPortnumber.SetWindowText((LPCTSTR) msg);

	msg.Format("%d", maxclients );
	m_CEditNumberClients.SetWindowText((LPCTSTR) msg);

	msg.Format("%d", portnum2);
	m_CEditPortnumberLossy.SetWindowText( (LPCTSTR) msg);

    msg.Format("%d", maxclients2);
	m_CEditNumberClientsLossy.SetWindowText( (LPCTSTR) msg);

    if (exttrigenabled == "TRUE") m_CCheckExtTrig.SetCheck(1); else m_CCheckExtTrig.SetCheck(0);
	if (exttrigmod5 == "TRUE") m_CCheckTrigMod5.SetCheck(1); else m_CCheckTrigMod5.SetCheck(0);

	if (enabled2 == 0)
	{
		m_CEditPortnumberLossy.EnableWindow(FALSE);
		m_CEditNumberClientsLossy.EnableWindow(FALSE);
		m_CCheckLossy.SetCheck(0);
	}
	else m_CCheckLossy.SetCheck(1);

    // put negative and positive reference onto dialog

	CString tmp;

    tmp.Format("%f", m_iPRef / 1000.0);
	m_CEditPRef0.SetWindowText( (LPCTSTR) tmp );

    tmp.Format("%f", m_iNRef / 1000.0);
	m_CEditNRef0.SetWindowText( (LPCTSTR) tmp );

    // build orientation values and set the controls

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


///////////////////////////////////////////////////////////////////////////////////////
// CheckParameters() - checks for validity of the values inside the controls on the 
//                     configuration dialog
//
// Return-Value:
// TRUE  : every parameter is valid
// FALSE : at least one of the parameters is not valid
//
// This function outputs a MessageBox containing the error message of the first control
// that is not valid.
//
BOOL CConfigDlg::CheckParameters()
{
	CString msg;

   	m_CEditTinePort.GetWindowText(msg);
	int tpnum = atoi( (const char *) (LPCTSTR) msg);
	if (tpnum < 0 || tpnum > 30)
	{
		AfxMessageBox("The tine port offset is not in the desired range (0-30) !", MB_ICONEXCLAMATION | MB_OK);
	    return( FALSE );
    }

	m_CEditTineContext.GetWindowText(msg);
	if (msg.GetLength() > 16)
	{
		AfxMessageBox("The tine context string is too long. It must be less than 17 characters.", MB_ICONEXCLAMATION | MB_OK);
		return( FALSE );
	}

	m_CEditTineControlServer.GetWindowText(msg);
	if (msg.GetLength() > 16)
	{
		AfxMessageBox("The tine control server string is too long.\nIt must be less than 17 characters.", MB_ICONEXCLAMATION | MB_OK);
		return( FALSE );
	}
	
	m_CEditTineFrameServer.GetWindowText(msg);
	if (msg.GetLength() > 16)
	{
		AfxMessageBox("The tine frame server string is too long.\nIt must be less than 17 characters.", MB_ICONEXCLAMATION | MB_OK);
		return( FALSE );
	}

	// check whether network listen port is not inside the desired range (0..65535)

	m_CEditPortnumber.GetWindowText(msg);
	int lpnum = atoi( (const char *) (LPCTSTR) msg);
	if (lpnum < 0 || lpnum > 65536) 
	{
	 AfxMessageBox("The listen port number is not in the desired range (0-65535) !", MB_ICONEXCLAMATION | MB_OK);
	 return( FALSE );
    }

    // check maximum number of socket clients (0..255)
	m_CEditNumberClients.GetWindowText(msg);
	int numc = atoi( (const char *) (LPCTSTR) msg);
	if (numc < 0 || numc > 255) 
	{
	 AfxMessageBox("The maximum number of Socket clients is not in the desired range (0-255) !", MB_ICONEXCLAMATION | MB_OK);
	 return( FALSE );
    }


	m_CEditPortnumberLossy.GetWindowText(msg);
	lpnum = atoi( (const char *) (LPCTSTR) msg);
	if (lpnum < 0 || lpnum > 65536) 
	{
	 AfxMessageBox("The listen port number for lossy operation is not in the desired range (0-65535) !", MB_ICONEXCLAMATION | MB_OK);
	 return( FALSE );
    }

    // check maximum number of socket clients (0..255)
	m_CEditNumberClientsLossy.GetWindowText(msg);
	numc = atoi( (const char *) (LPCTSTR) msg);
	if (numc < 0 || numc > 255) 
	{
	 AfxMessageBox("The maximum number of Socket clients for lossy operation is not in the desired range (0-255) !", MB_ICONEXCLAMATION | MB_OK);
	 return( FALSE );
    }

    return( TRUE );
}

//////////////////////////////////////////////////////////////////////////
// mkOrientation (makeOrientation) - calculates the orientation value for
// the ComboBox out of a more descriptive string
//
int CConfigDlg::mkOrientation(CString value)
{
 CString normal("NORM");
 CString rot90cw("ROT90CW");
 CString rot180cw("ROT180CW");
 CString rot270cw("ROT270CW");
 CString mirrorvert("MIRRORVERT");
 CString mirrorhoriz("MIRRORHORIZ");
 CString rot90mirrhoriz("ROT90CWMIRRHORIZ");

 if (value == normal) return( 0 );
 if (value == rot90cw) return( 1 );
 if (value == rot180cw) return( 2 );
 if (value == rot270cw) return( 3 );
 if (value == mirrorvert) return( 4 );
 if (value == mirrorhoriz) return( 5 );
 if (value == rot90mirrhoriz) return( 6 );
 
 return( 0 );
}

//////////////////////////////////////////////////////////////////////////
// mkreOrientation (makeOrientation the other way round) - calculates the 
// more descriptive orientation string out of the orientation integer value
// obtained from the combo box
//
void CConfigDlg::mkreOrientation(int value, CString &valuestr)
{
 switch( value )
 {
  case 0:  valuestr = "NORM";
           break;
  case 1:  valuestr = "ROT90CW";
           break;
  case 2:  valuestr = "ROT180CW";
           break;
  case 3:  valuestr = "ROT270CW";
           break;
  case 4:  valuestr = "MIRRORVERT";
           break;
  case 5:  valuestr = "MIRRORHORIZ";
           break;
  case 6:  valuestr = "ROT90CWMIRRHORIZ";
           break;

  default: valuestr = "NORM";
           break;
 }
}


void CConfigDlg::OnCheck1() 
{
	int check = m_CCheckLossy.GetCheck();

	if (check == 1)
	{
		m_CEditPortnumberLossy.EnableWindow(TRUE);
		m_CEditNumberClientsLossy.EnableWindow(TRUE);
		m_CEditJPEGQuality.EnableWindow(TRUE);
		m_CSliderJPEGQuality.EnableWindow(TRUE);
	}
	else
	{
		m_CEditPortnumberLossy.EnableWindow(FALSE);
		m_CEditNumberClientsLossy.EnableWindow(FALSE);
		m_CEditJPEGQuality.EnableWindow(FALSE);
		m_CSliderJPEGQuality.EnableWindow(FALSE);
	}
	
}

void CConfigDlg::OnCheck2Exttrig() 
{

}

void CConfigDlg::OnCheck3Trigmod5() 
{
	
}

void CConfigDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CString msg;
	
	msg.Format("%d %%", m_CSliderJPEGQuality.GetPos());

	m_CEditJPEGQuality.SetWindowText((LPCTSTR) msg);

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}
