// Grab.h: interface for the CGrab class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAB_H__F26604D1_51FD_11D6_9B36_0004769003D4__INCLUDED_)
#define AFX_GRAB_H__F26604D1_51FD_11D6_9B36_0004769003D4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CGrab  
{

 public:
	 BOOL getEffectiveBpp(unsigned int &bpp_eff);
	 int getFrameBpp();
	 int getFrameHeight();
	 int getFrameWidth();

  // construction / destruction	 
  CGrab();
  virtual ~CGrab();

  // initialize the framegrabber card and ITEX system
  BOOL Init( const char *imaq_interface_name, BOOL trigger, BOOL triggermod5, CQueue *grab_queue, short nref, short pref);
  
  // deinitialize framegrabber card and ITEX system
  BOOL Done();
  // snaps a single video frame
  BOOL StartGrab();
  // stops grabbing
  BOOL StopGrab();  
  
  // returns TRUE when the class is grabbing at the moment
  BOOL IsGrabbing();

  // get actually selected camera port
  int getCamera();
  //CICamera *getCurrentCamera();
  // set camera port to cam value (valid values 0..3)
  BOOL setCamera( int cam );

  int GetFrameLength();

  // function to set and to retrieve the positive and negative reference
  // attention: read the information just before the function body before using
  // them !

  BOOL getNRef( short &nref);
  BOOL setNRef( short nref );

  BOOL getPRef( short &pref );
  BOOL setPRef( short pref );

 private:
	 BOOL setNRefPRefHW(int camera);
	 
	// NI IMAQ
	SESSION_ID   Sid;
	INTERFACE_ID Iid;

	// PCICapMod m_iCap;
  
	 //CICamera *m_pCam0;
  	 //CICamera *m_pCam1;
  	 //CICamera *m_pCam2;
  	 //CICamera *m_pCam3;

  BOOL m_bTrigMod5;
  BOOL m_bExtTrig;

  // queue to put the grabbed frames into
  CQueue * m_GrabQueue;
 
  // variable to save the system number 
  // (for proper deinitialization) 
  int m_iSystem;
  // very first frame number (just after grabbing)
  BOOL m_iFirstFrameNo;
  
  /* error handling */

  // catch ITEX error messages 
  // catch own error messages
  int myErrorHandler( int errornum, const char *format, ... );
  // saves the last error code of the class
  int m_iLastError;

  // true just after grabbing has started, needed for 
  // framerate calculation
  BOOL m_bFirstTime;
  
  // true if the class is successfully initialized
  BOOL m_bSuccessfulInitialized;

  // is the class grabbing at the moment ? 
  BOOL m_bIsGrabbing;
  BOOL m_bShutdownThread;

  // width and height of the frames that are grabbed
  unsigned short m_iFrameWidth;
  unsigned short m_iFrameHeight;
  unsigned int m_iFrameBppEffective;
  unsigned int m_iFrameBpp;

  /* ITEX data structures */
  //pMODCNF m_itx; // pointer to ITEX module
  //pMODCNF m_amv; // pointer to ITEX AMV module
  //HIFCGRAB m_hgrab; // grab handle
  //ITXINTR_ID m_intr; // interrupt handle

  UINT threadGrab();
  static UINT threadGrab_static(LPVOID pParam);

  // grabbing thread function
  //static UINT threadGrab(LPVOID pParam);
  // CWinThread object for grabbing thread
  CWinThread *m_pGrabThread;

  // double buffer for just acquired images
  BYTE *m_pDoubleImageBuffer; // double buffer for 2 Images

  /* values needed for proper describing the video frame */
  
  // pointer to currently acquired image
  BYTE *m_pAktImage;
  // camera port
  unsigned int m_iCamera;
  // actual frame number
  unsigned int m_iFrameNumber;
  // timestamp of current frame
  _timeb m_tTimeStamp;
  // very first timestamp (just after StartGrab() )
  _timeb m_tVeryFirstTimestamp;

  // member to store the frames per second 
  double m_framespersec; 

  short m_nref;
  short m_pref;
};

#endif // !defined(AFX_GRAB_H__F26604D1_51FD_11D6_9B36_0004769003D4__INCLUDED_)
