// SendReceive.cpp: implementation of the CSendReceive class
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#define _NIWIN  
#include "niimaq.h"
#include <vfw.h>
#include "Grabsrv.h"
#include "Compression.h"
class CGrab;
class CGrabsrvApp;
class CMainDlg;
class CSocketEncapsulation;
class CSocketEncapsulation2;
class SSyncSocket;
class CIniEx;
class CBogusTineServerDlg;
#include "fourierstat.h"
#include "stat.h"
#include "Queue.h"
#include "SendReceive.h"
#include "SendReceive2.h"
#include "headers.h"
#include "JPEGCompression.h"
#include "SocketEncapsulation1.h"
#include "SocketEncapsulation2.h"
#include "Resource.h"
#include "ConfigDlg.h"
#include "SSyncSocket.h"
#include "Grabsrv.h"
#include "GrabsrvDlg.h"
#include "Grab.h"
#include "IniEx.h"
#include "BogusTineServerDlg.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
//
// This class is responsible for sending out the compressed video frames
// to one of the connected clients. The sending and receiving is done in
// threads, because the underlying socket functions are blocking.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
// The constructor just does some basic initialization of class members.
//
CSendReceive2::CSendReceive2( CSocketEncapsulation2 *parent, SSyncSocket *the_sock )
{
    // initialize socket encapsulation reference
	m_pSockEncap = parent;
	
	// initialize socket reference
	m_pSock = the_sock;

	// set all flags to false
	m_bSendThreadRunning = FALSE;
	m_bReceiveThreadRunning = FALSE;
	m_bDropInProgress = FALSE;
}

//////////////////////////////////////////////////////////////////////
//
// The destructor frees up the memory occupied by the frames waiting 
// in the video queue to be send out. It also frees up the video queue.
//
CSendReceive2::~CSendReceive2()
{
    // obtain size of video queue
    int size = m_video_queue.GetSize();

    // free up memory occupied by the frames
    for (int i=0;i<size;i++)
	{
		char *tmpptr = (char *) m_video_queue.RemoveHead();
		if (tmpptr != NULL) memman.sw_free( (void *) tmpptr );
	}

}

//////////////////////////////////////////////////////////////////////
//
// The Start() function starts up the send and receive threads. After 
// calling this function, waiting video frames from the queue will be 
// sent out. The function returns always TRUE.
//
BOOL CSendReceive2::Start()
{
   // initialize thread flags to proper value
   m_bSendThreadRunning = FALSE;
   m_bReceiveThreadRunning = FALSE;
   m_bForceSendThreadToStop = FALSE;
   m_bForceReceiveThreadToStop = FALSE;

   // start up the two threads
   m_pReceiveThread = AfxBeginThread((AFX_THREADPROC) thread_receive_static, this);
   m_pSendThread = AfxBeginThread((AFX_THREADPROC) thread_send_static, this);

   return TRUE;
}


/////////////////////////////////////////////////////////////////
//
// Using this function, you can put a video frame into the video 
// queue of the class. The frame will (when it is the first frame
// in the queue) then be sent out. This functions also checks 
// whether there are too much frames in the queue and drops all
// of them if there are more than 5 frames in the queue.
//
// The function returns false if the frame could not be added to
// the video frame queue.
// 
BOOL CSendReceive2::PutInQueue(char *videoframe, int length)
{
	// drop frames if there are too much of them
    if (m_bDropInProgress == FALSE)
	{
	 // indicate that dropping is in progress so that no other part
	 // of the class is allowed to drop frames
	 m_bDropInProgress = TRUE;
	 // get number of frames
	 int size = m_video_queue.GetSize();
	 // only drop if there are more than 5 frames in the queue
	 if (size > 5)
	 {
		for (int z=0;z<size;z++)
		{
		 char *tmpbuf = (char *) m_video_queue.RemoveHead();
		 if (tmpbuf != NULL) memman.sw_free( (void *) tmpbuf );
        }
	 }
	 // end of critical section
	 m_bDropInProgress = FALSE;
    }
	
	// allocate temporary buffer for frame
	char *tmpdata = (char *) memman.sw_malloc( length );
	// return FALSE if we could not allocate that memory
	if (tmpdata == NULL) return( FALSE );

    // copy the frame into the newly allocated temporary buffer 
	memcpysw(tmpdata, videoframe, length);
	
	// insert buffer as last element in the queue
	m_video_queue.AddTail( tmpdata );

    // success
	return( TRUE );
}

////////////////////////////////////////////////////////////////////////////////
//
// This is just a wrapper around the normal c++ memcpy().
//
void CSendReceive2::memcpysw( char *dst, char *src, int lengthsize) 
{
 memcpy( (void *) dst, (const void *) src, lengthsize ); 
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the static function used to initialize the thread function 
// thread_receive(). This function just does a dynamic cast to call 
// the proper thread_receive function.
//
UINT CSendReceive2::thread_receive_static(LPVOID *p)
{
	return reinterpret_cast<CSendReceive2 *>(p)->thread_receive();
}


/////////////////////////////////////////////////////////////////////////////
//
// This is the actual receive thread which "reads" data from the socket. As 
// we have no client->server communication on the socket, this function is
// just there because recv() returns zero if the socket was closed. Using
// the thread_receive() function we are able to detect a closed connection.
//
UINT CSendReceive2::thread_receive()
{
 char tmpbuf[1024];
 int ret=0;

 // receive thread is running from now on
 m_bReceiveThreadRunning = TRUE;
 
 for (;;) // infinite loop
 {
  // if someone forces us to stop, then just do it
  if (m_bForceReceiveThreadToStop == TRUE) break;

  ret = m_pSock->Receive((void *) &(tmpbuf[0]), 1000); // dummy try to read data from the socket
  
  // there was an error on the socket and no one forces us to stop immediately...
  if (ret == SOCKET_ERROR && m_bForceReceiveThreadToStop == FALSE) 
  {
      // force the other thread to stop
      m_bForceSendThreadToStop = TRUE;
	  // tell main application that the current client 'this' is no longer active
	  AfxGetMainWnd()->PostMessage(ID_REMOVE_CLIENT, ID_REMOVE_CLIENT_2, (LPARAM) this);	  

	  break; // shut down thread
  }
 }
 // receive thread is not running any more
 m_bReceiveThreadRunning = FALSE;
 return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the static function used to initialize the thread function 
// thread_send(). This function just does a dynamic cast to call 
// the proper thread_send function.
//
UINT CSendReceive2::thread_send_static(LPVOID *p)
{
	return reinterpret_cast<CSendReceive2 *>(p)->thread_send();
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the actual send thread which takes frame after frame out of the 
// queue and sends it out using the underlying socket. In addition, this 
// thread drops frames if there are too much frames in the queue waiting to 
// be sent out.
//
UINT CSendReceive2::thread_send()
{
 char *outbuffer;
 VIDHDR vidH;

 // from now on the send thread is running
 m_bSendThreadRunning = TRUE;
 for (;;) // infinite loop
 {
    // someone forces us to stop: break for-loop
    if (m_bForceSendThreadToStop == TRUE) break;

	// drop frames if there are too much of them
    if (m_bDropInProgress == FALSE)
	{
	 // indicate that dropping is in progress so that no other part
	 // of the class is allowed to drop frames
	 m_bDropInProgress = TRUE;
	 // get number of frames
	 int size = m_video_queue.GetSize();
	 // only drop if there are more than 5 frames in the queue
	 if (size > 5)
	 {
		for (int z=0;z<size;z++)
		{
		 char *tmpbuf = (char *) m_video_queue.RemoveHead();
		 if (tmpbuf != NULL) memman.sw_free( (void *) tmpbuf );
        }
	 }
	 // clear critical section flag
	 m_bDropInProgress = FALSE;
    }

    // get the first frame out of the queue
    outbuffer = (char *) m_video_queue.RemoveHead();
	if (outbuffer != NULL)
	{
		// copy into video header (to obtain the size of the frame)
		memcpysw( (char *) &vidH, outbuffer, sizeof(VIDHDR) );
	    
		// send out the buffer
		int ret = m_pSock->Send( (void *) outbuffer, vidH.len_total);
	    
		// delete the buffer, we do not need it any more
		memman.sw_free( (void *) outbuffer );
	    
		// some error while sending out the data ? 
		// and no one forces us to stop ? 
		if (ret == SOCKET_ERROR && m_bForceSendThreadToStop == FALSE)
		{
		    // force the other thread to stop
			m_bForceReceiveThreadToStop = TRUE;
			// post message that the client 'this' is no longer valid
            AfxGetMainWnd()->PostMessage(ID_REMOVE_CLIENT, ID_REMOVE_CLIENT_2, (LPARAM) this);	  

	        break; // break for-loop
		}
		
	}
	 else // no frame in the queue... sleep for a while
	{     // as we are running at maximum 10 Hz, we can sleep 100ms
		 ::Sleep(100);
		 
		 // someone forces us to stop: break for-loop
		 if (m_bForceSendThreadToStop == TRUE) break;
	}
 }
 
 // send thread is not running any more 
 m_bSendThreadRunning = FALSE;
 
 return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
// This function returns whether both threads are not running (and thus shut 
// down). It returns TRUE if both threads are not running, FALSE if at least
// one of the two threads is still running.
//
BOOL CSendReceive2::ThreadsShutDown()
{
 if (m_bSendThreadRunning == FALSE && m_bReceiveThreadRunning == FALSE) return( TRUE );

 return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
//
// This function just returns the socket reference member of the class.
//
SSyncSocket *CSendReceive2::GetSocket()
{
 return( m_pSock );
}

/////////////////////////////////////////////////////////////////////////////
//
// This function is needed to set private member variables to force both
// threads to stop execution.
//
void CSendReceive2::ForceThreadsToStop()
{
 m_bForceSendThreadToStop = TRUE;
 m_bForceReceiveThreadToStop = TRUE;
}

BEGIN_MESSAGE_MAP(CSendReceive2, CWnd)
	//{{AFX_MSG_MAP(CSendReceive2)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

