// SocketEncapsulation.cpp: implementation of the CSocketEncapsulation class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#define _NIWIN  
#include "niimaq.h"
#include <vfw.h>
#include "oleauto.h"
#include "Oaidl.h"
#include "rpcdefs.h"
class CSocketEncapsulation2;
class CSocketEncapsulation;
class CGrab;
class CMainDlg;
class CGrabsrvApp;
class CQueue;
class CIniEx;
class CBogusTineServerDlg;
class CSendReceive;
class CSendReceive2;
#include "Queue.h"
#include "compression.h"
#include "headers.h"
#include "JPEGCompression.h"
#include "resource.h"
#include "SSyncSocket.h"
#include "BogusTineServerDlg.h"
#include "SocketEncapsulation2.h"
#include "SocketEncapsulation1.h"
#include "SendReceive.h"
#include "SendReceive2.h"
#include "ConfigDlg.h"
#include "fourierstat.h"
#include "stat.h"
#include "Grabsrv.h"
#include "GrabsrvDlg.h"
#include "Grab.h"
#include "IniEx.h"
#include "resource.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////////////
// CSocketEncapsulation class
//
// This class is responsible for all socket-based happenings. It starts up a
// listen thread which listens for incoming connections and accepts them. It 
// also fires up send and receive threads on the accepted sockets so that the
// application is able to send out the video frames. From application point of
// view you only have to initialize this class and put data in it. The data
// will be send out to all connected clients.
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// 
// In the constructor we just initialize some member variables.
//
CSocketEncapsulation2::CSocketEncapsulation2()
{
 m_bInitialized = FALSE;
 m_bShowingPossible = FALSE;
 m_bDoNotClose = FALSE;
 m_bNoRemoveClient = FALSE;
}

//////////////////////////////////////////////////////////////////////////////
// 
// If the class is still initialized when destruction takes place, just 
// deinitialize it.
//
CSocketEncapsulation2::~CSocketEncapsulation2()
{
 if (m_bInitialized == TRUE)
 {
  Done();
 }
}

//////////////////////////////////////////////////////////////////////////////
// 
// In the Init function the class is properly initialized. The listen socket
// is instantiated, the listen thread is started, and two important variables
// are initialized.
//
// Paramters:
//  listenport     - port on which the server listens for incoming connections
//  maxnumclients  - maximum number of allowed clients
//  the_dlg        - reference to main application dialog  
//  size_in_bytes  - size in bytes of the video frame bits
//
BOOL CSocketEncapsulation2::Init( int listenport, int maxnumclients, CMainDlg *the_dlg )
{
 if (m_bInitialized == TRUE) return( TRUE );
 // initialize member variables
 m_iNumClients = 0;
 UpdateNumberClients();
 m_bListenThreadRunning = FALSE;
 m_pListenThread = NULL;
 m_iMaxNumClients = maxnumclients;
 m_pMainDlg = the_dlg;

 // instantiate list of clients
 m_pClientList = new CQueue();
 if (m_pClientList == NULL)
 {
  return( FALSE );
 }

 // initialize listen socket
 m_pSSock = new SSyncSocket();
 if (m_pSSock == NULL)
 {
  delete m_pClientList;
  m_pClientList = NULL;
  return( FALSE );
 }
 BOOL ret = m_pSSock->Create(listenport);
 if (ret != TRUE)
 {
  delete m_pSSock;
  m_pSSock = NULL;
  delete m_pClientList;
  m_pClientList = NULL;
  return( FALSE );
 }

 ret = m_pSSock->Listen();
 if (ret != TRUE)
 {
  m_pSSock->Close();
  delete m_pSSock;
  m_pSSock = NULL;
  delete m_pClientList;
  m_pClientList = NULL;
  return( FALSE );
 }

 // fire up listen thread
 m_pListenThread = AfxBeginThread((AFX_THREADPROC) thread_listen_static, this);

 // indicate that class is properly initialized 
 m_bInitialized = TRUE;

 return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// 
// This function puts the buffer into the CSendReceive class of each connected 
// client so that it gets send out by the socket's send thread.
//
BOOL CSocketEncapsulation2::Send( char *buffer, int length )
{
 // set flag that no connections should be closed
 m_bDoNotClose = TRUE;
 
 // are there some clients connected ? 
 if (m_pClientList->GetSize() > 0)
 {
  int len = m_pClientList->GetSize(); // how many ? 
  int pos = 0;

  while( pos < len ) // iterate over all clients
  {
   //POSITION listPos;

   CSendReceive2 *sendrecv = (CSendReceive2 *) m_pClientList->GetElem(pos);

   if (sendrecv != NULL)
   {
     sendrecv->PutInQueue( buffer, length );
   }
/*
   // obtain list position based on index 'pos'
   listPos = m_pClientList->FindIndex(pos);
   if (listPos != NULL) // we have found the position
   {
    // cast CSendReceive class out of the client list
	CSendReceive *sendrecv = (CSendReceive *) m_pClientList->GetAt(listPos);
    
	// put the buffer into the responsible CSendReceive class, so that
	// it gets send out by the send thread
	sendrecv->PutInQueue( buffer, length );
   }
*/
   // increment pos
   pos++;
  }//while

 }//if
 
 // clear 'do-not-close-connections'-flag
 m_bDoNotClose = FALSE;
 
 return TRUE; 
}

//////////////////////////////////////////////////////////////////////////////
// 
// This function deinitializes the whole class. All clients will be disconnected
// and removed from the client list. In addition, their send and receive threads
// will be shut down. After this, the listen-thread is shut down and the listen
// socket is closed. This function will be called if the server goes out of 
// service (...will be shut down).
//
BOOL CSocketEncapsulation2::Done()
{
 if (m_bInitialized == FALSE) return( TRUE );

 // tell other program parts that it is not allowed to remove clients from now on
 // because we will be doing this
 m_bNoRemoveClient = TRUE;

 // free all clients
 if (m_pClientList->GetSize() > 0) // are there some clients connected ? 
 {
  int size = m_pClientList->GetSize(); // how many are connected ? 

  BOOL donext = TRUE;

  while( donext == TRUE )
  {
   CSendReceive2 *sendrecv = (CSendReceive2 *) m_pClientList->RemoveHead();

   if (sendrecv != NULL)
   {
     // obtain underlying socket from CSendReceive
     SSyncSocket *sock = sendrecv->GetSocket();

     // close the socket
     sock->Close();

     // force send and receive thread to stop
     sendrecv->ForceThreadsToStop();
   
     // wait for the threads to come down
     while( sendrecv->ThreadsShutDown() == FALSE)
	 {
	   ::Sleep(50);
	 }

     // delete the socket
     delete sock;
   
     // delete the CSendReceive class
     delete sendrecv;

     // decrement number of clients
     m_iNumClients--;
     // update the number of clients on the main dialog
     UpdateNumberClients();
   }
   else
	   donext = FALSE;
  }

 }

 // close the listen socket
 m_pSSock->Close();

 // wait for listen thread to shut down
 while( m_bListenThreadRunning == TRUE)
 {
	 ::Sleep(100);
 }

 // delete listen socket
 delete m_pSSock;
 m_pSSock = NULL;

 // indicate that we are not initialized any more
 m_bInitialized = FALSE;

 // return false (failure) if not all clients could be removed
 if (m_iNumClients != 0) return FALSE;

 // from now on the other program parts are allowed to remove clients 
 // ..although there should not be a single one now
 m_bNoRemoveClient = FALSE;

 return TRUE; // success

}

/////////////////////////////////////////////////////////////////////////////
//
// This is the static function used to initialize the thread function 
// thread_listen(). This function just does a dynamic cast to call 
// the proper thread_listen function.
//

UINT CSocketEncapsulation2::thread_listen_static(LPVOID *p)
{
	return reinterpret_cast<CSocketEncapsulation2 *>(p)->thread_listen();
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the actual listen thread which listens for incoming connections and
// accepts them. The connection is immediately closed if there are too much 
// (maximum number of) clients connected at the moment.
// If the socket is accepted successfully, a CSendReceive class is instantiated
// and started, so that data can be sent and received. The CSendReceive 
// instance is added to the list of clients. After this the number of clients is 
// updated.
//
UINT CSocketEncapsulation2::thread_listen()
{
 SSyncSocket *sock = NULL;

 // from now on listen thread is running
 m_bListenThreadRunning = TRUE;
 for (;;) // infinite loop
 {
  // instantiate clean new socket
  sock = new SSyncSocket();
 
  // block until there is a incoming connection request
  // it it comes in, accept it
  BOOL ret = m_pSSock->Accept(*sock);
  
  // error (e.g. closed listen socket) -> break for-loop
  if (ret == FALSE) break;

  // increment number of clients
  m_iNumClients++;
  if (m_iNumClients > m_iMaxNumClients) // more clients than we allow ? 
  {
	// close socket and do cleanup
	sock->Close();
	delete sock;
	// decrement number of clients
	m_iNumClients--;
  }
   else // socket is accepted
  {
       // instantiate CSendReceive class
       CSendReceive2 *sendrecv = new CSendReceive2(this, sock);

	   if (sendrecv != NULL)
	   {
	     // fire up send and receive threads
	     sendrecv->Start();
	   
	     // add client to client list
	     m_pClientList->AddTail( (void *) sendrecv );
	   }
	   else
	   {
           sock->Close();
		   delete sock;
	   }
  }
  // update the number of clients (on main dialog window)
  UpdateNumberClients();
 }
 
 // if for-loop is broken
 // close socket for safety
 sock->Close();
 // clean up listen socket
 delete sock;

 // listen thread is not running any more
 m_bListenThreadRunning = FALSE;
 return( 0 );
}



/////////////////////////////////////////////////////////////////////////////
//
// This function removes a client from the list of connected clients. This 
// mostly happen when the client closes a connection. The CSendReceive class
// is also shut down and the 'number of clients'-control on the main dialog 
// is updated.
//
UINT CSocketEncapsulation2::RemoveClient( CSendReceive2 *sendrecv)
{
    // get to-be-closed socket from the send receive class 
	SSyncSocket *sock = sendrecv->GetSocket();

    // are we allowed to remove the client (because if TRUE
	// Done() is actually happening and closes the clients there)
	if (m_bNoRemoveClient == TRUE) return( 0 );
    
	// are we allowed to close the client now ? 
	// [ send-function is in progress, so we wait for it to 
	//   finish before we shut down the client ]
	while(m_bDoNotClose == TRUE)
	{
		::Sleep(20);
	}
    
    // find position of client in client list
	void *pos = m_pClientList->RemoveElem((void *) sendrecv);
    
	if (pos != NULL) // found ? 
	{
	  // close the socket
	  sock->Close();

      // wait for the send and receive threds to shut down
	  while( sendrecv->ThreadsShutDown() == FALSE)
	  {
		  ::Sleep(50);
	  }
      
      // decrement number of clients
	  m_iNumClients--;
	  // update the number of clients on the main dialog
      UpdateNumberClients();

      // free socket memory
	  delete sock;

	  // free CSendReceive class memory
	  delete sendrecv;
	}

    return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
// This private function updates the 'number of clients' control on the main 
// application dialog if it is allowed at the moment.
//
void CSocketEncapsulation2::UpdateNumberClients()
{
    if (m_bShowingPossible == TRUE) // allowed ? 
	{
		// format string and print it onto the control
		CString msg;
		msg.Format("%d", m_iNumClients);
		m_pMainDlg->m_CEditLossyClients.SetWindowText((LPCTSTR) msg);
	}

}

/////////////////////////////////////////////////////////////////////////////
//
// This public function updates the 'number of clients' control on the main 
// application dialog. To check whether it is allowed, the variable 
// 'isitokayto' is passed accordingly. The parameter is also used in consecutive
// calls to UpdateNumberClients().
//
void CSocketEncapsulation2::doUpdateNumClients(BOOL isitokayto)
{
	m_bShowingPossible = isitokayto;
	UpdateNumberClients();
}

/*
BEGIN_MESSAGE_MAP(CSocketEncapsulation2, CWnd)
	//{{AFX_MSG_MAP(CSocketEncapsulation2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
*/

/////////////////////////////////////////////////////////////////////////////
// CSocketEncapsulation message handlers

