/*: ==================================================
    ===                servdef.h                  ====
    ==================================================   */
#ifndef SERVDEF_H_
#define SERVDEF_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include "prolog.h"
#include "rpctypes.h"
#include "almsrv.h"

/*: ==================================================
//
//         structs and macros of general use:
//
//  ==================================================   */

#ifndef SystemCalloc
# define SystemCalloc(nr,sz) calloc(nr,sz)
#endif

#ifndef SystemFree
# define SystemFree(ptr) free(ptr)
#endif

#define ccerr(e) {cc = e; goto err; }
#define MAXALLOWED_POLLRATE 65535L
#define MSECS(a,b) ((a).tv_sec-(b).tv_sec > MAXALLOWED_POLLRATE ? MAXALLOWED_POLLRATE :\
		    ((a).tv_sec-(b).tv_sec)*1000+((a).tv_usec-(b).tv_usec)/1000)
#define FEC_NAME_SIZE     20
#define ELEM_NAME_SIZE    15
#define EQP_NAME_SIZE      6
#define EQP_PROP_SIZE     32
#define EQP_DEVICE_SIZE   16

#ifndef MAX_ENTRIES
# define MAX_ENTRIES     100 /* connection table entries  */
#endif
#ifndef FEC_TIMEOUT
#define FEC_TIMEOUT      1   /* seconds */
#endif
#ifndef SERVER_TIMEOUT
#define SERVER_TIMEOUT   1   /* seconds */
#endif

#define IPX             0x01
#define SPX             0x02
#define TCP             0x04
#define UDP             0x08

#define getProtocolAsString(i) (\
  (i) == IPX ? "IPX" :\
  (i) == SPX ? "SPX" :\
  (i) == TCP ? "TCP" :\
  (i) == UDP ? "UDP" :\
  "---"\
)

#define POLLING         0x01
#define SELECTING       0x02

#ifndef DEFAULT_POLLRATE
#  ifndef SYSPOLL
#    define DEFAULT_POLLRATE  30000
#  else
#    define DEFAULT_POLLRATE SYSPOLL
#  endif
#endif

#ifndef MIN_POLLRATE
#  ifndef SYSPOLL
#    define MIN_POLLRATE 1000
#  else
#    define MIN_POLLRATE SYSPOLL
#  endif
#endif

#define DEFAULT_CLIENT_POLLRATE 1000

#define DEFAULT_MAX_TIMEOUT 30000

#define MAX_TRANSPORTSIZE 0xffff

#ifndef BURST_LIMIT
# define BURST_LIMIT 20
#endif
extern int gBurstLimit;
extern int gCycleDelay;
#define BCAST_ID 255

typedef struct EQPTask
{
  void (*tskfcn)(void); /* background task function */
  int weight;           /* background task weight factor */
  struct EQPTask *next;
} EQPTASK;
extern EQPTASK *EQPTaskList;

#define EQPBKG void (*)(void)
#define EQPINI void (*)(void)
#define EQPEXI void (*)(void)

typedef struct       /* net */
{
  UINT16 TotalSize;
  char UserName[16];
  short Protocol;
  short EqpSize;
} HeaderStruct;
#define HEADERSTRUCT_SIZE  22
#define SUB_PROT_LOCATION 18

typedef struct       /* net */
{
  char ElemName[16];
  char EqpModName[8];
  short EqpModNo;
  char Property[12];
  char Cycle[22];
  short Format;
  short DataSize;
} EqpInfoStruct;
#define EQPINFOSTRUCT_SIZE 64

typedef struct          /* local */
{              
  char name[16];        /* eqp function name */
  char prop[32];        /* eqp property */
  char deviceName[32];  /* eqp device name */
  char context[16];     /* eqp context */
  short EqNo;           /* eqp number */
  short access;         /* data access */
  short formatIn;       /* eqp data format down */
  short formatOut;      /* eqp data format up */
  UINT32 sizeIn;        /* data array size down */
  UINT32 sizeOut;       /* data array size up */
  char strTagIn[8];     /* eqp structure tag down */
  char strTagOut[8];    /* eqp structure tag up */
  short dataFlg;        /* signal new arrival (negative completion code) */
  short tocounter;      /* link timeout counter */
  int  RPCindex;        /* index into RPC database */
  int  FECindex;        /* index into FEC database */
  UINT32 RpcID;         /* connection ID */
  UINT32 pollRate;      /* polling rate for connectioned clients */
  UINT32 heartbeat;     /* used by subscriptions */
  UINT16 mode;          /* used by subscriptions */
  int RPCprotocol;      /* RPC header type */
  int protocol;         /* transport protocol (IPX,SPX,UDP,TCP) */
  time_t time;          /* last polling request */
  time_t starttime;     /* entry into table (used by subscriptions) */
  time_t RPCtimestamp;  /* arriving RPC timestamp */
  UINT32 lastDataSize;  /* arriving RPC data size */
  short RPCversion[2];
  int  TCPsocket;
  int  counter;         /* current subscription counter */
  char *dataIn;         /* pointer to associated data down */
  char *dataOut;        /* pointer to associated data up */
  char *bufOut;         /* pointer to raw data up buffer if necessary */
  BYTE *bufOutBlks;     /* pointer to data-up buffer block progress if necessary */
  short allocated;      /* is dataOut allocated memory ? */
  short useErrValue;    /* flag  */
  UINT16 numblks;       /* progress counters for incoming data */
  UINT16 blksin;        /* progress counters for incoming data */
  UINT32 bytesin;       /* progress counters for incoming data  */
  UINT32 nextDataSize;  /* next outgoing data size in bytes */
  UINT32 nextDataPtr;   /* next outgoing data array pointer */
  UINT32 BytesRemaining;/* number of bytes data remaining */
  UINT32 BytesTotal;    /* total number of bytes data */
  UINT16 datamtu;       /* outgoing data minimum transport unit */
  UINT16 blkidIn;       /* block Id of data to send */
  UINT16 numblksIn;     /* num blks input data to send */
  UINT16 blknumIn;      /* this blk num of input data */
  UINT32 callbackID;    /* pivate callback ID */
  void (*fcn)(int id,int cc); /* call back function */
  UINT8 grouped;        /* flag to indicate if element is grouped */
  UINT8 pending;        /* flag to indicate if element is pending */
  UINT16 srvID;         /* server subscription ID (applicable in MCAST) */
  short linkStatus;     /* current link status */
  short needsNotification; /* flag to call NotifyClient() or not */
  char RPCLastErrorString[32]; /* temperary storage for return status */
  BYTE  errValue[64];    /* buffer for assigned error value */
} CONN_TABLE;

typedef struct grpStruct
{
  short numPending;     /* num of group events pending */
  short numInGrp;       /* number of elements in group */
  UINT32 GrpID;         /* group identification */
  short  GrpCc;         /* group completion */
  struct grpStruct *next;
} GROUP_TABLE;

typedef struct         /* net */
{
  char  name[16];      /* broadcast element name  (16 bytes) */
  short size;          /* number of elements broadcast under "name" (2 bytes) */
  short format;        /* data format (as per RPC definitions)  (2 bytes) */
  long  time;          /* timestamp of broadcast data (4 bytes) */
} BrdcstData;
#define BRDCSTDATA_HEADER_SIZE (16 + 2 + 2 + 4)

#define EMPTY_PROP    0
#define TARGET_PROP   1
#define WILDCARD_PROP 2
#define STOCK_PROP    3

#define PROPERTY_HASH_SIZE 211
#define PROPERTY_HASH_INDEX(i) ((i) & 0xff)
#define PROPERTY_NUM_INDEX(i) (((i) >> 8) & 0xffff)
#define MAKE_PROPERTY_ID(i,indx) (((i) << 8) | (indx))

typedef struct PropertyEGU
{
  char units[16];
  double min;
  double max;
  BYTE graph;
  BYTE reserved[3];
} PropertyEGUStruct;

typedef struct PrpRedirBlkStruct
{
  NAME16 prpRedirection[4];  
  struct PrpRedirBlkStruct *next;
} PrpRedirBlk;

typedef struct ExportPropertyList
{
  char  prpName[32];       /* property name */
  long  prpId;             /* property id */
  UINT32 prpSize;          /* maximum supported array size */
  UINT32 prpSizeIn;        /* maximum supported input array size */
  short prpFormat;         /* format output data */
  short prpFormatIn;       /* format input data */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  short reserved;          /* round out the boundaries */
  NAME8 prpFormatTag;      /* output data structure tag if non zero-length */
  NAME8 prpFormatTagIn;    /* input data structure tag if non zero-length */
  PrpRedirBlk *prd;        /* re-director if not NULL */
  char  prpDescription[64];/* short description of property */
  PropertyEGUStruct egu;   /* EGU information for display */
  struct ExportPropertyList *next;
} ExportPropertyListStruct;

#define MODULE_HASH_SIZE 211
#define MODULE_HASH_PAYOFF 25 /* don't hash if fewer modules than this */

typedef struct ModuleXRef
{
  int index;
  struct ModuleXRef *next;
} ModuleXRefStruct;

#define STOCKPROP_HASH_SIZE 103

typedef struct StockPropXRef
{
  int index;
  struct StockPropXRef *next;
} StockPropXRefStruct;

extern StockPropXRefStruct **StockPropXRefTable;

typedef struct ExportListTag /* local */
{
  char   Context[16];       /* exported equipment name context */
  char   tagName[16];       /* exported equipment name tag */
  char   EqpName[8];        /* local equipment name tag */
  char   EqpAlias[8];       /* local equipment alias name tag */
  int    exportId;          /* local ID */
  ExportPropertyListStruct  /* pointer to chained equipment property hash list */
         *EqpProperty[PROPERTY_HASH_SIZE+1]; /* Add room for Wild-card list */
  int    (*EqpPropertyQueryFcn)(char *devName,PropertyQueryStruct **pqs);
  int    (*EqpPropertyQueryExFcn)(char *devName,PropertyQueryStructEx **pqsx);
  int    (*EqpDeviceQueryFcn)(char *devProperty,NAME16 **dq);
  NAME32 *EqpModuleName;    /* pointer to device name list */
  ModuleXRefStruct **EqpModuleXRef; /* ModuleName cross-reference */
  short  EqpNumProperties;  /* total number of properties supported */
  short  EqpNumModules;     /* total number of devices supported */
  short  registered;        /* registered with name server */
  ADS    *adsTable;         /* alarm definition structure table */
  short  alarmSystem;       /* alarm system flag */
  ALARM  **almLst;          /* associated alarm list */
  short  nalarms;           /* number of active alarms */
  time_t almMRT;            /* alarm Most Recent Timestamp */
  time_t almMRCT;           /* alarm Most Recent Complete Timestamp */
  short  almHSV;            /* alarm highest Severity */
  short  nAlmMRT;           /* number of alarms at MRT */
  short  nAlmMRCT;          /* number of alarms at MRCT */
  short  nAlmHSV;           /* number of alarms at HSV */
  short  nclients;          /* number of active clients */
  short  ncontracts;        /* number of active contracts */
# ifdef NODAL
  int  (*NdlFcn)(double *,int,int,char *); /* associated Nodal equipment function */
# else
  int  (*NdlFcn)(double *,short,short,short,char *);
# endif
  int  (*EqpFcn)(char *,char *,DTYPE *,DTYPE *,short);/* associated equipment function */
  void (*eqpBkg)(void);     /* associated background task */
  long rateBkg;             /* background polling rate msec */
  short bkgbusy;            /* background busy semaphore */
  struct timeval lastBkg;   /* background task last polled */
  void (*eqpIni)(void);     /* associated initialization  */
  void (*eqpExi)(void);     /* associated cleanup */
  struct ExportListTag *next;  /* Linked list */
} ExportListStruct;

extern ExportPropertyListStruct *ExportPropertyList;
extern ExportListStruct *ExportList;
extern int NrOfEqpTypes;

typedef struct                /* net: queries */
{
  char  FecName[FEC_NAME_SIZE+1];
  char  ExportName[ELEM_NAME_SIZE+1];
  char  EqName[EQP_NAME_SIZE+1];
  char  Context[16];
  short alarmSystem;
} RPCDataStruct;
#define RPCDATASTRUCT_SIZE (FEC_NAME_SIZE+ELEM_NAME_SIZE+EQP_NAME_SIZE+3+2+16)

extern UINT32 IncomingNetworkProtocol;
typedef struct               /* net: queries */
{
  char Name[FEC_NAME_SIZE+1];
  BYTE Network[4];
  BYTE Node[6];
  BYTE ImmediateAddress[6];
  char IP[16];
  BYTE PortOffset;
  char IPh_addr[4];
  short  Prot;
}RPCFecStruct;
#define RPCFECSTRUCT_SIZE (FEC_NAME_SIZE+1+16+21+2)

typedef struct
{
  char os[12];
  char desc[64];
  char loc[32];
  char hdw[48];
  char resp[32];
} RPCFecExStruct;

typedef struct
{
  char FecName[FEC_NAME_SIZE];
  time_t localtime;
  time_t starttime;
  long   systemPollingRate;
  short  numBkgTsks;
  short  numTotalContracts;
  short  numTotalClients;
  short  numTargetContracts;
  short  numTargetClients;
  short  numConnections;
  UINT32 numConnectionTimeouts;
  UINT32 numConnectionArrivals;
  UINT32 numUDPpkts;
  UINT32 numTCPpkts;
  UINT32 numIPXpkts;
  UINT32 numSPXpkts;
} ActivityQueryStruct;
#define ACTIVITYQUERYSTRUCT_SIZE (FEC_NAME_SIZE+4*3+2*6+4*6)

typedef struct
{
  char desc[64];
  BYTE dataFmt;
  BYTE dataSiz;
  BYTE data[64];
  time_t timeEntered;
  UINT32 codeNow;
  UINT32 codePrv;
  UINT32 codeNxt;
} ServerStateStruct;
#define SERVERSTATESTRUCT_SIZE (64+2+64+4*4)

/* =============================================================
//
// Subscription oriented Producer-consumer definitions:
//
// ===========================================================*/

#ifndef NCONSUMER
# define NCONSUMER     50
#endif
#ifndef NCONTRACT
# define NCONTRACT     150
#endif
#define RENEWAL       60
#define REMINDER      5
#define ACK_REQUEST   (2*REMINDER)
#define HEARTBEAT     60
#define REPLY_PENDING 3
#define SUB_PENDING   0x3fff
#define ACK_PENDING   0x7fff
#define CANCEL        CM_CANCEL
#define SINGLE        CM_SINGLE
#define REFRESH       CM_REFRESH
#define POLL          CM_POLL
#define REGISTER      CM_REGISTER
#define GROUPED       CM_GROUPED

#ifndef WORK_BUFFER_SIZE
# define WORK_BUFFER_SIZE 0x2000
#endif

#define IPX_DATA_SIZE   512
#define IPX_BUFFER_SIZE 1466

#ifndef UDP_BUFFER_SIZE
# define UDP_BUFFER_SIZE 1472
#endif
#ifndef WORK_SIZE
# define WORK_SIZE  (WORK_BUFFER_SIZE*sizeof(double)) /* 64 KB on most machines */
#endif
#define BRDCST_BUFFER_SIZE IPX_BUFFER_SIZE

extern char *SubType[];
extern int nconsumer;
extern int ncontract;
extern int StaleData;

typedef struct             /* net: Protocol 4 (in) */
{
  char  EqpName[8];
  char  Property[12];
  short EqpSize;
  short EqpFormat;
  short EqpNumber;
#ifdef NODAL
  int  (*fcn)(double *,int,int,char *);
#else
  int  (*fcn)(double *,short,short,short,char *);
#endif
} CONTRACT4;
#define CONTRACT4_SIZE 30

typedef struct             /* net: Protocol 5 (in) */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
} CONTRACT;
#define CONTRACT_SIZE (32+16+8+4*2+4*1+2*8)

typedef struct
{
  CONTRACT contract;
  long pollrate;
  long nconsumers;
} ContractQueryStruct;
#define CONTRACT_QUERY_SIZE (CONTRACT_SIZE + 4 + 4)

typedef struct                    /* first part net: Protocol 5 (query) */
{
  char               userName[16];
  IPXAddress         IPXaddress;  /* if 0 then IP transfer is assumed */
  BYTE               node[6];     /* IPX routing node (Immediate Address) */
  struct sockaddr_in IPaddress;
  short              count;       /* number of registrations */
  short              protocol;    /* target client protocol level */
  short              transport;   /* transport protocol */
  int                sck;         /* communication socket */
} CLN;
#define CLIENT_SIZE (16 + 12 + 6 + 16 + 2 + 2)

typedef struct                    /* net: Protocol 5 (query) */
{
  char               userName[16];
  IPXAddress         IPXaddress;  /* if 0 then IP transfer is assumed */
  BYTE               node[6];     /* IPX routing node (Immediate Address) */
  struct sockaddr_in IPaddress;
  short              count;       /* number of registrations */
  short              protocol;    /* target client protocol level */
} CLNQS;

typedef struct
{
  CLNQS cln;
} ClientQueryStruct;
#define CLIENT_QUERY_SIZE (16 + 12 + 6 + 16 + 2 + 2)

#define LOCK_UNLOCKED 0
#define LOCK_PREEMPTIVE 1
#define LOCK_PERSISTENT 2
#define LOCK_ABORT 3
typedef struct
{
  CLN cln;
  int lockType;
  int lockDuration;
  UINT32 lockToken;
  time_t lockStart;
} AccessLockStruct;

typedef struct                /* local */
{
  short  id;                  /* contract ID as seen by consumer */
  short  mode;                /* SINGLE,REFRESH,POLL,CANCEL */
  long   PollingRate;         /* requested polling rate */
  struct timeval lasttime;    /* time of most recent update */
  time_t starttime;           /* start of initial subscription */
  BYTE   counter;             /* counts remaining in subscription(s) */
  BYTE   Stale;               /* 1 = send new data to consumer */
  UINT16 numblks;             /* total number of message blocks */
  UINT16 blknum;              /* block number of this message */
  short  statusCode;          /* interim completion */
  UINT16 mtu;                 /* maximal transport unit */
  UINT32 nextDataSize;        /* next outgoing data size in bytes */
  UINT32 nextDataPtr;         /* next outgoing data array pointer */
  UINT32 BytesRemaining;      /* number of bytes data remaining */
  UINT32 BytesTotal;          /* total number of bytes data */
} ClientSTATUS;

#define MAXLONGGRAMS 5
typedef struct
{
  long   haddr;               /* identify client by IP address */
  UINT16 SubscriptionID;      /* supplied by the consumer upon registration */
  UINT16 blksin;              /* # blocks arrived from long datagram */
  UINT16 blkid;               /* block id of long datagram */
  UINT8  busy,
	 reserved;
  UINT8  *data;
  time_t blktime;
  time_t SubsStarttime;
} ContractProgress;

typedef struct ConStruct
{
  CLN              *client;
  ClientSTATUS     status;
  struct ConStruct *next;
} CONSUMER;

typedef struct                  /* local */
{
  CONSUMER *consumer;           /* assciated consumer list */
  int      nconsumer;           /* number of consumers for this contract */
  CONTRACT4 contract;
  long     PollingRate;         /* in seconds, smallest number wins ... */
  struct   timeval lasttime;    /* most recent data polled */
  struct   timeval lastmarked;  /* last time the data were marked STALE */
  time_t   RPCtimestamp;        /* data timestamp */
  BYTE     *Data;               /* data to send */
} ContractListStruct4;

typedef struct                  /* local */
{
  CONSUMER *consumer;           /* assciated consumer list */
  short    nconsumer;           /* number of consumers for this contract */
  short    contractID;          /* local contract ID */
  CONTRACT contract;            /* see above */
  long     PollingRate;         /* in mseconds, smallest number wins ... */
  struct   timeval lasttime;    /* most recent data polled */
  struct   timeval lastmarked;  /* last time the data were marked STALE */
  time_t   RPCtimestamp;        /* data timestamp */
  BYTE     *DataOut;            /* data to send  */
  BYTE     *DataIn;             /* data received */
  UINT32   dataSizeOut;         /* output data size < contract.dataSize */
  int      pending;             /* pending flag for DeliverData() vs ProduceData() */
  ExportListStruct *exp;        /* associated export element */
} ContractListStruct;

typedef struct                  /* net: Protocol 4 (out) */
{
  short msgsize;
  short SubscriptionID;         /* supplied by the consumer upon registration */
  short CompletionCode;
  short counter;                /* tell consumer how much is left */
  time_t timestamp;             /* RPCtimestamp */
  time_t starttime;             /* supplied by the consumer upon registration */
} ProducerHeaderStruct4;
#define PRODUCER4_SIZE 16

typedef struct                  /* net: Protocol 5 (out) */
{
  UINT16 msgsize;               /* message size in bytes */
  UINT16 SubscriptionID;        /* supplied by the consumer upon registration */
  UINT16 CompletionCode;        /* completion or interim status */
  UINT16 numblks;               /* total number of message blocks */
  UINT16 blknum;                /* block number of this message */
  UINT16 mtu;                   /* maximum transport unit */
  UINT16 EqpSize;               /* Eqp Size of returned data */
  BYTE   EqpFormat;             /* Eqp Format of returned data */
  BYTE   counter;               /* tell consumer how much is left */
  time_t timestamp;             /* RPCtimestamp */
  time_t ClnStarttime;          /* supplied by the consumer upon registration */
} ProducerHeaderStruct;
#define PRODUCER_SIZE (7*2+2*1+2*4)
#define PMTU(s)  (((s) ? IPX_BUFFER_SIZE : UDP_BUFFER_SIZE) - PRODUCER_SIZE - sizeof(UINT16))

struct subscription4            /* net: Protocol 4 (in) */
{
  short    msgsize;
  UINT16   PollingRate;
  short    id;
  short    mode;
  time_t   starttime;
  CONTRACT4 contract;
};
typedef struct subscription4 SUBSCRIPTION4;
#define SUBSCRIPTION4_SIZE (12+CONTRACT4_SIZE)

struct subscription         /* net: Protocol 5 (in) */
{
  UINT16    msgsize;
  UINT16    mtu;            /* maximum transport unit for incoming data */
  UINT8     numblks;        /* total number of message blocks           */
  UINT8     blknum;         /* block number of this message             */
  UINT16    id;
  short     mode;
  short     blkid;          /* (incremented) block id */
  UINT32    PollingRate;
  time_t    starttime;
  CONTRACT contract;
};
typedef struct subscription SUBSCRIPTION;
#define SUBSCRIPTION_SIZE (6*2+2*4+CONTRACT_SIZE)
#define SUB_MODE_LOCATION (HEADERSTRUCT_SIZE + 8)

extern int MaxNumContracts;
extern int MaxNumClients;
extern ContractListStruct **ContractList;
extern CLN **Consumers;                /* registered consumer list */

extern int clnSocketNumber;     /* SPX Client Socket                 */
extern int ProducerSocket;      /* UDP Producer Socket               */
extern int ConsumerSocket;      /* UDP Consumer Socket               */
extern int MCastConsumerSocket; /* UDP MCast/BCast Consumer Socket   */
extern int BrdcstSocket;        /* UDP Broadcast Socket              */
extern int GlobalSocket;        /* UDP Global listening Socket       */
extern int SyncSocket;          /* UDP trivial RPC Socket            */
extern struct sockaddr_in prod_name;
extern AccessLockStruct currentAccessLock;

void doConsumer(void);

/* =============================================================
//
//     general externals and prototypes :
//
// ===========================================================*/

typedef struct
{
  char prpName[32];
  UINT16 prpSize;          /* maximum supported array size */
  short prpFormat;         /* native format */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  char  prpDescription[32];/* short description of property */
  int  (*fcn)(CONTRACT *,char *,char *);
} StockPropertyStruct;

typedef struct
{
  char prpName[32];
  UINT32 prpSize;          /* maximum supported array size (output) */
  UINT32 prpSizeIn;        /* maximum supported array size (input)  */
  short prpFormat;         /* format (output) */
  short prpFormatIn;       /* format (input) */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  char  prpDescription[64];/* short description of property */
  int  (*fcn)(CONTRACT *,char *,char *);
} StockPropertyStructEx;

#ifndef _StockPropertyStructEx 
# define _StockPropertyStructEx StockPropertyStructEx
#endif
extern int NrStockProperties;
extern _StockPropertyStructEx StockProperty[];

typedef struct
{
  char modName[16];
  int  (*fcn)(CONTRACT *,char *,char *);
} StockModifierStruct;
#ifndef _StockModifierStruct
# define _StockModifierStruct StockModifierStruct
#endif
extern int NrStockModifiers;
extern _StockModifierStruct StockModifier[];

#ifdef SERVER_POLLING_MODE
#define ServerCycleMode    POLLING
#else
#define ServerCycleMode    SELECTING
#endif
extern int RequireAcknowledgments;
extern unsigned short MinPollingRate;
extern long SysPoll;
extern int AveBusyTime;
extern struct timeval thisCycleTime;
extern int broadcast, nglobals,TCPdebug,TCPerrcnt;
extern BYTE *BrdcstBuffer;
extern unsigned long TCPcumcnt,TCPinitcnt;
extern time_t RPCtimestamp,startuptime;
extern short RPCverion[2];
extern int RPCprotocol;
extern int hConn, hControl;
extern WORD ISwap(WORD w);
extern int connectionTableID,TCPloaded,IPXloaded,numErr;
extern char *erlst[];
extern char username[16];      /* login name of PC user */
extern BYTE *clientbuf;
extern BYTE *WorkArea;
extern BYTE *tmpWorkArea;
extern unsigned long WorkAreaSize;
extern CONSUMER *EQPconsumerlist;   /* global reference  */
extern int EQPnconsumers;
extern CONTRACT *EQPcontract;

extern char fecaddr[30],clientaddr[30],fecprot[4],timebuf[9],datebuf[9];
extern char *SRVDataBuffer;    /* global Buffer pointers for data exchange */
extern int ListenForMCasts;
extern int foreground;

extern int MaxNumConnections;
extern CONN_TABLE **connTable;
extern GROUP_TABLE *grpTable;
extern int ReturnToClientCycle;
extern int ReturnToServerCycle;
extern int nConnectionTableEntries;
extern int doConsumerSemaphore;
extern int ExecLinkSemaphore;
extern int AttachLinkSemaphore;
extern int recvNetGlobalSemaphore;
extern struct sockaddr_in srv_name,brdcst_name;
extern int TCPParent,TCPProducerSocket,UDPSocket;
extern fd_set TCPrset, rwset, UDPset;
extern struct timeval TCPto, fec_to;
extern int NrOfUsers,WRITEaccess;
extern BYTE rpc_socket[2];
extern const BYTE brdcst_socket[2];
extern BYTE reg_socket[2];
extern const BYTE mcast_socket[2];
extern char PATH[];
extern RPCFecStruct *RPCFec, *NameServer;
extern RPCDataStruct *RPCData;
extern RPCFecExStruct RPCFecEx;
extern int numNameServers,curNameServer;
extern int RPCNumRecsRead,RPCNumFecsRead,RPCNumEqpsRead,FECallocNum;
extern unsigned long TCPcount,UDPcount,SPXcount,IPXcount,BRDcount;
extern unsigned long nConnectionTimeouts,nArrivals;
extern int NrOfEqpTypes;
extern short NrEQPBckTsks;
extern int NumNodalFunctions;
extern time_t *tnull;
extern char gDeviceContext[17];
void ExitEQPModules(void);
void RegisterEQPtasks(char *name,void (*ini)(void),void (*tsk)(void),int rate,void (*exi)(void));
void checkTasklist(void);
ExportListStruct *findTagID(char *tag);
int getRegisteredExports(void);
int RegisterProperty(char *eqm,char *prop,int siz,short fmt,short acc,char *dsc);
int PrepSubBuffer(BYTE *data);
int SendSubBuffer(int FECindex,int sck,int transport,BYTE *data);
unsigned short execRequest(CLN *cln);
int ProduceData(struct timeval *thisTime);
int DeliverData(struct timeval *thisTIme);
int doRequest(int num, int *i);
int getConsumerRequest(CLN *cln,BYTE *buf,int packetsize);
int getConnectionEntry(DTYPE *dout,DTYPE *din,
    char *devcontxt,char *devgrp,char *devname,char *property);
void displayData(int format, int fec, int dir);
int dumpConsumerList(void);
int dumpContractList(void);
int dumpConnectionList(void);
void dumpRegisteredUsers(void);
int doClient(int num, int *id);
void consumeData(BYTE *buf);
int feclog(char *text);
int addFecToList(char   *name);
int initFecTable(void);
int datacmp(BYTE *d1,BYTE *d2,int siz,int fmt,double t);
int initBroadcastList(void);
void getGlobalUpdate(BYTE *buf);
void quitServer(void);
char *_strdate(char *buf);
char *_strtime(char *buf);
void hardDelay(int msec);
int IsMemberLockSet(CLN *cln);
CLN *LocateConsumerInList(struct sockaddr_in *ip,IPXAddress *nwAddr,BYTE *immAddr);
unsigned long ElfHash(const unsigned char *key);
void makeEqpModuleHashTable(ExportListStruct *el);
int makeStockPropXRefTable(void);
short IsStockProperty(char *Property);
short GetStockPropertyId(char *Property);
int lookupPropertyName(ExportListStruct *el,char *prpName,char *devName);
char *lookupPropertyId(ExportListStruct *el,short id);
int sndRegisteredExports(struct timeval *tv);
void logServerStats(struct timeval *tv);
void ParsePropertyDescription(char *str,ExportPropertyListStruct *prp);
void ParseRedirector(char *str,ExportPropertyListStruct *prp);
ExportListStruct *findTagID(char *nam);
int SetBurstLimit(int npackets);

int IsInForeground(void);
void checkIPXNets(struct timeval *tv);
int GetSPXStream(WORD ID,CLN *cln);
int initIPX(void);
void CleanupIPX(struct timeval *tv);
void closeIPXServer(void);
void AcceptIPX(struct timeval *tv);
void AcceptSPXStream(void);
void CloseSPXStream(WORD ID);
int SendIPXData(CLN *cln, BYTE *data);
int SendIPXBroadcast(BYTE *buf,int size);
void fulfillSPXStreamRequest(WORD ID);
CLN *getConsumerIPXAddress(void);
char *ipxnet_ntoa(BYTE *node);
void dumpIPXNets(void);

void NotifyClient(int i,int cc);
int doTCPClient(int fec);
int listenUDP(int *s,unsigned short port);
int sendUDPBuffer(struct sockaddr_in *client, int msgsize);
int connectTCP(int port,int i);
int initTCP(void);
int GetTCPStream(int socket,CLN *cln);
void CloseTCPStream(int socket);
int SendTCPData(int socket,BYTE *buf,long buflen);
int SendIPData(CLN *cln,BYTE *data);
int SendIPBroadcast(BYTE *buf,int size);
void fulfillTCPStreamRequest(int socket);
CLN *getConsumerIPAddress(int sck,int prot);
void AcceptTCPStream(void);
void dumpIPNets(void);
void AcceptIP(struct timeval *tv);
void CleanupIP(struct timeval *tv);
void closeIPServer(void);
int IsMemberIPNets(int num,struct sockaddr_in *lst,struct sockaddr_in *ipaddr);
int tUDPCall(int fecindex,BYTE *buf,int timout);
int tIPXCall(int fecindex,BYTE *buf,int timout);
int tRPC(int fecindex,char *data,char *name,short EqNo,char *prop,
           short size,short format,short timout);
int tExecLink(int fecindex,char *data,char *name,char *dev,char *prop,
           short size,short format,short timout);
void checkClientNotification(void);
int readFecLog(char *textbuf,int sizeInBytes);

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif  /* SERVDEF_H_ */
