/* =============================================================
//
//     UDP transport structs and macros
//
// ===========================================================*/

#ifndef SRVIP_H_
#define SRVIP_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include "servdef.h"

#ifdef UNIX
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <stddef.h>
#include "unixcmd.h"

typedef struct ipclsn
{
  struct sockaddr_in ipfec;
  IPXAddress ipxfec;
  struct ipclsn *next;
} IPCLSN;
#define IPC_PING    0x0001
#define IPC_LSNUDP  0x0002
#define IPC_LSNIPX  0x0003

extern int BroadcastServer;

#endif /* UNIX */

#define NumTCPChan	  5
#define	UDP_PORT       9003
#define TCP_LEGACYPORT 8003
#define TCP_PORT       8503
#define GCAST_PORT     8004
#define MCAST_PORT     9503
#ifndef UDP_CLN_PORT
# define UDP_CLN_PORT  8050
#endif
#ifndef UDP_CLN_RANGE
# define UDP_CLN_RANGE (UDP_CLN_PORT+1000)
#endif
#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif
#ifndef MCAST_TTL
# define MCAST_TTL 5
#endif
#ifndef MSDOS
# define CYCLE_DELAY   20000
#else
# define CYCLE_DELAY   20
#endif
#ifndef CLN_PATH
# define CLN_PATH "/var/tmp/"
#endif
#define CLN_PERM S_IRWXU

#ifndef MAX_TCPCONNECTIONS
# define MAX_TCPCONNECTIONS 32
#endif

#define MIN_FDSETS 16

#ifndef DEFAULT_TCP_CONNECTION_TIMEOUT
# define DEFAULT_TCP_CONNECTION_TIMEOUT 200
#endif

typedef struct tcpbckt
{
  int idx;                  /* FEC index             */
  int sck;                  /* TCP connection socket */
  int ptr;                  /* bucket pointer        */
  int siz;                  /* bucket target size    */
  BYTE buf[UDP_BUFFER_SIZE];/* bucket buffer         */ 
  struct tcpbckt *nxt;      /* this is a linked list */
} TCPBCKT;
extern TCPBCKT *bcktList;
TCPBCKT *findBucket(int sck);
TCPBCKT *getBucket(int sck);
void freeBucket(int sck);

extern int *TCPsck;
extern int nTCPsck;
extern int maxFdSets;
extern int maxTCPConnections;

extern int IPXProducerSocket;
extern int IPXConsumerSocket;
extern int IPXMCastConsumerSocket;
#define soerr(s) {strcat(txt,s); goto err;}

extern struct in_addr tMCastAddr;
extern struct in_addr tGCastAddr;
extern struct sockaddr_in tMCastAddrSck;
extern struct sockaddr_in tGCastAddrSck;
extern fd_set ext_set;
extern int externalFdSets;
extern struct sockaddr_in *IPNetList;
extern int NrOfIPNets;
extern struct sockaddr_in *IPBcastList;
extern int NrOfIPBcastNets;
void FD_MERGE(fd_set fdsrc,fd_set *fddst);
extern void (*socketAcceptEntryFcn)(void);
void CloseConnectionSocket(int sck);
void closeIPConnectionToFec(int index);
int ListenForIPMulticasts(void);
int ListenForIPNetGlobals(void);
extern int ListenForGlobals;
#ifdef WINNT
int GetErrno(void);
int WinsockLoaded(void);
#endif

#define ProcessData(sck,bf) {\
  if (IncomingNetworkProtocol == TCP) {\
    if ((bckt=findBucket(sck)) != NULL) {\
      nleft = nread;\
      while (nleft > 0) {\
        if (bckt->ptr == 0) bckt->siz = PeekWord(*(short *)(bf));\
        n = bckt->siz - bckt->ptr;\
        if (nleft >= n) {\
          memcpy(&bckt->buf[bckt->ptr],bf,n);\
          consumeData(bckt->buf);\
          bf += n;\
          nleft -= n;\
          bckt->ptr = 0;\
        } else {\
          memcpy(&bckt->buf[bckt->ptr],bf,nleft);\
          bckt->ptr += nleft;\
          nleft = 0;\
        }\
      }\
    }\
  } else {\
    consumeData(bf);\
  }\
}

#define recvIncomingIPData(sck) {\
  if (FD_ISSET(sck,&conset)) {\
    if ((nread=recvfrom(sck,(char *)clnWorkArea,WORK_SIZE,0,(struct sockaddr *)&IncomingIPfec,&len)) > 0 &&\
         nread >= (n=PeekWord(*(short *)clnWorkArea)) )\
    {\
      if (IncomingNetworkProtocol != TCP && nread > n) {\
        memcpy(&IncomingNetworkProtocol,&clnWorkArea[n],sizeof(UINT32));\
        if (IncomingNetworkProtocol == UDP)\
          memcpy(&IncomingIPfec,&clnWorkArea[n+4],sizeof(struct sockaddr));\
        else\
          memcpy(&IncomingIPXfec,&clnWorkArea[n+4],sizeof(IPXAddress));\
      }\
      inet_ntoa_b(IncomingIPfec.sin_addr,fecaddr);\
      bufptr = clnWorkArea; ProcessData(sck,bufptr);\
    }\
    if (NGdebug > 3) {\
      if (nread < 0) sprintf(dbgbuf,"%s: read error",fecprot);\
      else sprintf(dbgbuf,"%s: %d bytes read from %d",fecprot,nread,n);\
      dbgoutput(dbgbuf,tnull);\
    }\
  }\
}

#define setupGlobalsSocket(sck,p) {\
  if (listenUDP(&(sck),p)) {\
    sck = 0;\
    sprintf(str,"cannot open %s socket",p == GCAST_PORT ? "GLOBALS" : "MCAST");\
    feclog(str);\
  }\
  if (sck) FD_SET(sck,&conset);\
}

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* SRVIP_H_ */
