// FourierStat.h: interface for the CFourierStat class.
//
// Initially written by Velizar Miltchev (DESY Zeuthen) miltchev@ifh.de
// some additional code and modifications for Grabber server 
// by Stefan Weisse sweisse@ifh.de
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FOURIERSTAT_H__B552A989_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
#define AFX_FOURIERSTAT_H__B552A989_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Stat.h"

// define constants needed for fourier transformation
#define defdegree 5
#define defrecdep 1
#define pi 3.1415926535897932384626433832795

// inherit class CStat
class CFourierStat : public CStat  
{
public:

	// construction of CFourierStat class, same like CStat, but two new parameters deg and recdep, which
	// are needed for fourier transformation
	CFourierStat(unsigned char* data,
		unsigned short deg=defdegree,unsigned short recdep=defrecdep,
		unsigned int ImWi=768,unsigned int ImHi=574);

    // destruction of the class
	virtual ~CFourierStat();

	// do the calculation
	void START();

private:	
	// calculate X and Y spot size based on the distribution array and the spot center position
	// we do not use the member variables but parameters because of recursion
	double CalcYRMS(double *dist, double ym);
	double CalcXRMS(double *dist,double xm);

    // calculate X and Y spot center positions based on the distribution array and the three additional
	// parameters
	double CalcXMEAN( double* dist,unsigned short recnum,  int locleft,  int locright);
	double CalcYMEAN( double* dist,unsigned short recnum,  int locleft,  int locright);

    // recalculate X and Y distribution
	double* ReYDist( int locup,  int locdown);
	double* ReXDist( int locleft,  int locright);

    // calculate A and B coefficent needed for fourier transformation
	double* CalcAcoef(double* DistArray, int len,unsigned short deg=defdegree);
	double* CalcBcoef(double* DistArray, int len,unsigned short deg=defdegree);

    // recursion depth
	unsigned short RecursDepth;

	// degree
	unsigned short degree;

    // distribution members (different from the ones in CStat)
	double* FTYdist;
	double* FTXdist;

};

#endif // !defined(AFX_FOURIERSTAT_H__B552A989_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)

