// Grab.cpp: implementation of the CGrab class.
//
// based on code from BESSY framegrabbing software "Pitzcam". 
// written by Stefan Weisse (sweisse@ifh.de)
//////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include <ifc.h>
#include "headers.h"
#include <vfw.h>
#include <pcvision.h>
#include <ifcapi.h>
#include <amvcam.h>
#include "Queue.h"
#include "Grabsrv.h"
#include "Grab.h"
#include "Compression.h"
class CGrab;
class CGrabsrvApp;
class CSocketEncapsulation;
class CSocketEncapsulation2;
class CMainDlg;
class CSendReceive;
class CSendReceive2;
class CFourierStat;
class CStat;
class CIniEx;
class CBogusTineServerDlg;
#include "Resource.h"
#include "ConfigDlg.h"
#include "SSyncSocket.h"
#include "SocketEncapsulation1.h"
#include "SocketEncapsulation2.h"
#include "JPEGCompression.h"
#include "Grabsrv.h"
#include "GrabsrvDlg.h"
#include "Grab.h"
#include "SendReceive.h"
#include "SendReceive2.h"
#include "fourierstat.h"
#include "IniEx.h"
#include "stat.h"
#include "BogusTineServerDlg.h"
#include "OrientationHelper.h"

/*
#include "stdafx.h"
#include <ifc.h>
#include <ifcapi.h>
#include <amvcam.h>
#include <vfw.h>
#include "oleauto.h"
#include "Oaidl.h"
#include "rpcdefs.h"
#include "headers.h"
#include "Compression.h"
class CGrab;
class CGrabsrvApp;
class CSocketEncapsulation;
class CSocketEncapsulation2;
class CMainDlg;
class CSendReceive;
class CSendReceive2;
class CFourierStat;
class CStat;
class CIniEx;
class CBogusTineServerDlg;
#include "Queue.h"
#include "Resource.h"
#include "ConfigDlg.h"
#include "SSyncSocket.h"
#include "SocketEncapsulation1.h"
#include "SocketEncapsulation2.h"
#include "JPEGCompression.h"
#include "Grabsrv.h"
#include "GrabsrvDlg.h"
#include "Grab.h"
#include "SendReceive.h"
#include "SendReceive2.h"
#include "fourierstat.h"
#include "IniEx.h"
#include "stat.h"
#include "BogusTineServerDlg.h"
#include "OrientationHelper.h"
*/


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
//
// This class is meant as an encapsulation of the raw ITEX grabber 
// function which are needed to access the PCVision frame grabber 
// card. Using this class one is able to snap pictures, grab frames
// continuously and switch the camera port on the frame grabber card.
// The class issues a PostMessage (WM_COMMAND, ID_NOTIFY_GRABBED) to
// the main application dialog so that the main class is notified 
// whether there is a new frame grabbed. Then the main class can call
// the getActImageData() function to get all important data from the
// grabbed frame.
//
// The lifecycle is as follows:
// - instantiate the class
// - call Init() to initialize the class and the ITEX system
// - call StartGrab/StopGrab to start/stop continuous grabbing or
// - call Snap() to acquire a single image
// - call setCamera / getCamera to set or retrieve the camera port
// - when everything is finished, call Done() to deinitialize the 
//   class
//
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// 
// In construction of the class, just some variables are initialized to
// their default values.
CGrab::CGrab()
{
 // we are not grabbing at the moment
 m_bIsGrabbing = FALSE;
 
 // no grab thread is up and running
 m_pGrabThread = NULL;
 
 m_iLastError = 0; // no error
 
 // initialize width and height values
 m_iFrameWidth = 0;
 m_iFrameHeight = 0;

 // set camera port to 0
 m_iCamera = 0;

 // class is not yet initialized
 m_bSuccessfulInitialized = FALSE;
 
 // up to now no double buffer is allocated
 m_pDoubleImageBuffer = NULL;
 
 // starting frame number is zero
 m_iFrameNumber = 0;
 m_iFrameBpp = 8;
 m_iFrameBppEffective = 8;
}

//////////////////////////////////////////////////////////////////////
// 
// During destruction, we check whether the class is initialized and
// grabbing. If any of the conditions are true, we stop grabbing and/or
// call Done() to deinitialize the class.
//
CGrab::~CGrab()
{
	if (m_bSuccessfulInitialized == TRUE) // are we initialized?
	{
		if (m_bIsGrabbing == TRUE) // are we grabbing ? 
		{
			StopGrab(); // stop grabbing
		}

		Done(); // deinitialize class and ITEX
	}

}

//////////////////////////////////////////////////////////////////////
// 
// The function setCamera() is needed to set the camera port on the 
// PCVision frame grabber card. The parameter cam can have integer
// values of 0 to 3. The function returns TRUE if everything went 
// fine and false otherwise.
//
BOOL CGrab::setCamera( int cam )
{
	if (m_bSuccessfulInitialized == FALSE) // are we not initialized?
	    return FALSE; // then return FALSE

	if (cam == 0)
	{
		m_iCap->SetActiveCamera(m_pCam0);
		setNRefPRefHW(0);
		
	}
	 else if (cam == 1)
	{ 
		m_iCap->SetActiveCamera(m_pCam1);
		setNRefPRefHW(1);
	}
	else if (cam == 2)
	{
		m_iCap->SetActiveCamera(m_pCam2);
		setNRefPRefHW(2);
	}
	else // cam==3
	{
		m_iCap->SetActiveCamera(m_pCam3);
		setNRefPRefHW(3);
	}

    m_iCamera = cam;

    return TRUE; //success
}

//////////////////////////////////////////////////////////////////////
// 
// The function getCamera() retrieves the actually selected camera 
// port. The camera port is returned. If the function returns '-1'
// then something went wrong. Either the ITEX system is not initialized
// or the class was not able to retrieve the camera port.
//
int CGrab::getCamera()
{
	if (m_bSuccessfulInitialized == FALSE) // are we not initialized?
	     return -1; // return failure
	
	CICamera *temp = m_iCap->GetCam(IFC_ACTIVE_CAMERA_FLAG);

	if (temp == m_pCam0) return 0;
	if (temp == m_pCam1) return 1;
	if (temp == m_pCam2) return 2;
	if (temp == m_pCam3) return 3;

     // print error message
    myErrorHandler(16, "Can not query camera port");	
	return -1; // return failure
}

CICamera *CGrab::getCurrentCamera()
{
	static CICamera *temp;
	if (m_bSuccessfulInitialized == FALSE) // are we not initialized?
	     return NULL; // return failure
	
	temp = m_iCap->GetCam(IFC_ACTIVE_CAMERA_FLAG);

	return temp; // return failure
}


//////////////////////////////////////////////////////////////////////
// 
// The function Init() initializes the ITEX system and the class so 
// that we are able to snap/grab frames and are able to get and set
// the camera port value. The two parameters are informations passed
// to the underlying ITEX system. In normal case, both system and 
// boardno are zero. The function returns TRUE if the class and the
// ITEX system is successfully initialized; FALSE otherwise.
//
BOOL CGrab::Init( short system, short boardno, BOOL trigger, BOOL triggermod5, CQueue *grab_queue, short nref, short pref )
{
	CPCVision *pcv;

    m_nref = nref;
	m_pref = pref;

    m_GrabQueue = grab_queue;
	m_bExtTrig = trigger;
	m_bTrigMod5 = triggermod5;

	FILE *fp = fopen("PCVision.txt", "rt");
	if (fp != NULL) fclose(fp); else myErrorHandler(33456, "PCVision.txt could not be opened for reading!");

	IfxSetConsoleHandler( this->ifcerrorhandler );

	m_iCap=IfxCreateCaptureModule("ICP",(DWORD) system,"PCVision.txt");

	if (m_iCap == NULL)
	{
      myErrorHandler(1, "Could not load configuration file PCVision.txt !");
      return FALSE;
    }

	// SW! June 28, 2006 - was added to be more compatible if fg card is screwed up, does not seem to make 
	// behaviour better, but no side effects observed so it was kept in
	pcv = (CPCVision *) m_iCap;
	pcv->InitializeRegs();
	
	m_pCam0 = m_iCap->GetCam(0);
	m_pCam1 = m_iCap->GetCam(1);
	m_pCam2 = m_iCap->GetCam(2);
	m_pCam3 = m_iCap->GetCam(3);

    m_iSystem = (int) system;

    CAM_ATTR attr;

    m_pCam0->GetAttr(&attr, FALSE);

	m_iFrameHeight = (short) attr.dwHeight;
	m_iFrameWidth = (short) attr.dwWidth;

	//m_iCap->SetAcqTimeout(2000); // in ms (10 seconds) // SW! Apr 24, 2006: removed!


    m_pCam0->SetAcqParam(AMV_LOW_PASS_FILTER_EN, IFC_ENABLE);
    m_pCam1->SetAcqParam(AMV_LOW_PASS_FILTER_EN, IFC_ENABLE);
    m_pCam2->SetAcqParam(AMV_LOW_PASS_FILTER_EN, IFC_ENABLE);
    m_pCam3->SetAcqParam(AMV_LOW_PASS_FILTER_EN, IFC_ENABLE);

	if (m_bExtTrig == TRUE)
	{
      // enable external trigger
      m_pCam0->SetAcqParam(P_FRAME_RESET_MODE, IFC_ENABLE);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_MODE, IFC_ENABLE);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_MODE, IFC_ENABLE);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_MODE, IFC_ENABLE);

      m_pCam0->SetAcqParam(P_FRAME_RESET_POLARITY, IFC_ACTIVE_HIGH);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_POLARITY, IFC_ACTIVE_HIGH);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_POLARITY, IFC_ACTIVE_HIGH);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_POLARITY, IFC_ACTIVE_HIGH);

      m_pCam0->SetAcqParam(P_FRAME_RESET_OFFSET, 1);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_OFFSET, 1);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_OFFSET, 1);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_OFFSET, 1);

      m_pCam0->SetAcqParam(P_VERT_OFF, 18);
	  m_pCam1->SetAcqParam(P_VERT_OFF, 18);
	  m_pCam2->SetAcqParam(P_VERT_OFF, 18);
	  m_pCam3->SetAcqParam(P_VERT_OFF, 18);

      m_pCam0->SetAcqParam(P_FRAME_RESET_SIZE, IFC_FRAME_RESET_ONE_LINE);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_SIZE, IFC_FRAME_RESET_ONE_LINE);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_SIZE, IFC_FRAME_RESET_ONE_LINE);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_SIZE, IFC_FRAME_RESET_ONE_LINE);

      m_pCam0->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam1->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam2->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam3->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);

	  m_pCam0->SetAcqParam(P_TRIGGER_ENABLE, IFC_ENABLE);
	  m_pCam0->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam0->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

  	  m_pCam1->SetAcqParam(P_TRIGGER_ENABLE, IFC_ENABLE);
	  m_pCam1->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam1->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam2->SetAcqParam(P_TRIGGER_ENABLE, IFC_ENABLE);
	  m_pCam2->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam2->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

  	  m_pCam3->SetAcqParam(P_TRIGGER_ENABLE, IFC_ENABLE);
	  m_pCam3->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam3->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);
	}
	else
	{
    
      m_pCam0->SetAcqParam(P_VERT_OFF, 44);
	  m_pCam1->SetAcqParam(P_VERT_OFF, 44);
	  m_pCam2->SetAcqParam(P_VERT_OFF, 44);
	  m_pCam3->SetAcqParam(P_VERT_OFF, 44);

      m_pCam0->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);

      m_pCam0->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam1->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam2->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam3->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);

      m_pCam0->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam0->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam0->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam1->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam1->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam1->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam2->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam2->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam2->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam3->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam3->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam3->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);
	}

	// allocate memory needed for storing up to two frames (double buffering)
    int dbuflen = m_iFrameWidth*m_iFrameHeight*2*m_iFrameBpp/8;
	m_pDoubleImageBuffer = (unsigned char *) memman.sw_malloc( dbuflen ); // 2 images grayscale (8bit)
	if (m_pDoubleImageBuffer == NULL)
	{
     // cleanup, call error handler and return failure
     delete m_iCap;

	 myErrorHandler(17, "Out of memory allocating %d bytes for double image buffer (CGrab::Init)", dbuflen);
	 return FALSE;
	}

	m_iFrameBpp = 8;
	m_iFrameBppEffective = 8;


    // when we are here everything went fine
    m_bSuccessfulInitialized = TRUE;
	
	// success
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
// This function starts the continuous grabbing of images. It checks 
// whether everything is properly initialized and starts up the grabber
// thread which acquires the images. The function returns TRUE if 
// grabbing is started, FALSE otherwise.
// 
BOOL CGrab::StartGrab()
{
 // check whether class is properly initialized
 if (m_bSuccessfulInitialized == FALSE) return FALSE;
 
 // if we are actually grabbing, do nothing
 if (m_bIsGrabbing == TRUE) return(TRUE);

 // start the grabbing

 CICamera *cam;

 cam = getCurrentCamera();
 
 m_hgrab = cam->Grab(0, m_pDoubleImageBuffer, 2);
 if(!m_hgrab) 
 {
  myErrorHandler(10, "Could not start continuous grabbing.");
  return(FALSE);
 } 

 // fire up grabber thread with time critical priority
 m_bFirstTime = TRUE;
 m_pGrabThread = AfxBeginThread(threadGrab, this, THREAD_PRIORITY_TIME_CRITICAL); // maybe thread priority is needed, too: see MSDN doc
 if (!m_pGrabThread) // error ? 
 {
  myErrorHandler(11, "Could not create grabber thread.");
  return( FALSE ); // failure
 }
 
 // do not auto-delete grabber thread
 m_pGrabThread->m_bAutoDelete = false;
 
 // set grabbing flag to true
 m_bIsGrabbing = TRUE;

 return( TRUE ); // success
}

//////////////////////////////////////////////////////////////////////
// 
// This function stops the continuous grabbing of images. It checks 
// whether we are grabbing and stops the grabbing if necessary. The 
// function returns always TRUE.
//
BOOL CGrab::StopGrab()
{
 if (m_bIsGrabbing == TRUE) // are we grabbing
 {
  
  //((CMainDlg *)AfxGetMainWnd())->PrintServerMessage("B4 freeze.");


	 // stop grabbing
  CICamera *cam;
  cam = getCurrentCamera();
  cam->Freeze(m_hgrab);

  //((CMainDlg* )AfxGetMainWnd())->PrintServerMessage("After freeze.");
  
  //itx_host_grab_stop(m_hgrab);
  // set grabbing handle to zero (uninitialized), so that
  // the thread knows that it should stop
  m_hgrab = 0;
  // wait for the thread to come down
  WaitForSingleObject(m_pGrabThread->m_hThread, INFINITE);
  // delete grab thread
  delete m_pGrabThread;
  m_pGrabThread = 0;


  //((CMainDlg* )AfxGetMainWnd())->PrintServerMessage("After thread destroy.");

  // we are not grabbing any more
  m_bIsGrabbing = FALSE;
 }

 return( TRUE ); // success
}

//////////////////////////////////////////////////////////////////////
// 
// This function deinitializes the class and the ITEX grabbing system.
// After a call to this function, the class is going into the unin-
// intialized state. You can not grab, snap, get/set camera port any
// more. The function always returns TRUE. The two sleeps are there 
// because without them, ITEX was not properly uninitialized sometimes.
// ITEX could be uninitialized but not initialized any more.
//
BOOL CGrab::Done()
{
  // not initialized? -> then there is no need to shut down anything
  if (m_bSuccessfulInitialized == FALSE) return TRUE;

  // wait half a second
  // if we work using triggered acquisition, shut down
  // everything that was needed to do triggered acquisition
 

  if (m_bExtTrig == TRUE)
  {
      m_pCam0->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam1->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam2->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);
	  m_pCam3->SetAcqParam(P_FRAME_RESET_MODE, IFC_DISABLE);

      m_pCam0->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam1->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam2->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);
	  m_pCam3->SetAcqParam(AMV_FRAME_RESET_ON_VSYNC_OUTPUT, IFC_DISABLE);

      m_pCam0->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam0->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam0->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam1->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam1->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam1->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam2->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam2->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam2->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);

	  m_pCam3->SetAcqParam(P_TRIGGER_ENABLE, IFC_DISABLE);
	  m_pCam3->SetAcqParam(P_TRIGGER_SRC, IFC_EXT0_TRIG);
	  m_pCam3->SetAcqParam(P_TRIGGER_POLARITY, IFC_ACTIVE_HIGH);
  }


  // remove ITEX system
  delete m_iCap;
  
  // free memory needed by the double buffer
  memman.sw_free( (void *) m_pDoubleImageBuffer );
  
  // class is not successfully initialized any more
  m_bSuccessfulInitialized = FALSE;
  
  return( TRUE ); //success
}

//////////////////////////////////////////////////////////////////////
// 
// This function returns whether the class is actually grabbing images.
// It returns TRUE if grabbing is up and running, FALSE otherwise.
//
BOOL CGrab::IsGrabbing()
{
 return( m_bIsGrabbing );
}

//////////////////////////////////////////////////////////////////////
// 
// This is the thread which is needed for coninuous acquisition of 
// frames. After having successful acquired one frame, a message is
// posted to the main window indicating that a new frame was grabbed.
// To stop the thread just set the member m_hgrab to zero.
//
UINT CGrab::threadGrab(LPVOID pParam)
{
    BYTE *ptr=NULL;
    double passedmilliseconds;
	CGrab *_this = (CGrab *)pParam;
    int tmp=0;
	int last_ret = -1;

	while (_this->m_hgrab)
	{
		// Wait for the next image to be ready and transfer to host
		//int ret = itx_grab_latest_seqnum(_this->m_hgrab, TRUE, &ptr, TRUE);
        CICamera *cam;
	    cam = _this->getCurrentCamera();

		int iSeqNum = cam->GrabWaitFrameEx(_this->m_hgrab,&ptr,IFC_WAIT_NEWER_FRAME,1200,TRUE);
        int ret = iSeqNum; 
		if (!_this->m_hgrab) return 0;

		if ((ret != -1) && (last_ret != ret) && (ptr != NULL)) // success ? 
		{
    	  // set pointer to image
		  _this->m_pAktImage = (BYTE *) ptr;
		  // set timestamp for frame
		  ::_ftime( &_this->m_tTimeStamp );
		  // increase temporary frame counter
		  tmp++;

          // if we do not use triggered acquisition, just grab every 5th frame
		  // to have 5 FPS and not 25 FPS
		  if (_this->m_bTrigMod5 == TRUE)
		  {
		    if (tmp%5 != 0) 
		    {
              cam->GrabRelease(_this->m_hgrab,iSeqNum);
              last_ret = ret;
		      continue;
            }
  		  }

	      // increase frame number
		  _this->m_iFrameNumber++;

		  // calculate framerate 
		  // if this is the first time we are executing this,
		  // store the actual timestamp as very first time stamp
	      
    	  if (_this->m_bFirstTime == TRUE)
		  {
            memcpy((void *) &_this->m_tVeryFirstTimestamp, (const void *) &_this->m_tTimeStamp, sizeof(_timeb));
	        _this->m_bFirstTime = FALSE;
			_this->m_iFirstFrameNo = _this->m_iFrameNumber;
		  }
		  else
		  {
			// calculate passed milliseconds between first and actual frame
			passedmilliseconds = difftime(_this->m_tTimeStamp.time, _this->m_tVeryFirstTimestamp.time) * 1000.0;

			passedmilliseconds -= ((double)_this->m_tVeryFirstTimestamp.millitm);
			passedmilliseconds += ((double)_this->m_tTimeStamp.millitm);
		  }

          // calculate the frame rate (average calculation) 
          _this->m_framespersec = 1000.0 / ( passedmilliseconds / ((double)(_this->m_iFrameNumber-_this->m_iFirstFrameNo+1)));
		  /* calculate framerate end */
 
          // create memory block for storing video frame 
          char *block = (char *) memman.sw_malloc( sizeof(VIDHDR)+(_this->m_iFrameWidth*_this->m_iFrameHeight*_this->m_iFrameBpp/8) );
		  if (block != NULL)
		  {

           // build proper video header
           VIDHDR hdr;
	       hdr.kennung = 0x11223344;
		   hdr.compression = MAKEFOURCC('H','F','Y','U');
		   hdr.cameraport = _this->m_iCamera;
		   hdr.frameno = _this->m_iFrameNumber;
		   hdr.len_data = _this->m_iFrameWidth*_this->m_iFrameHeight;
		   hdr.len_fmthdr = 0;
		   hdr.len_total = hdr.len_data + hdr.len_fmthdr;
		   hdr.widthframe = _this->m_iFrameWidth;
		   hdr.heightframe = _this->m_iFrameHeight;
		   hdr.framerate = _this->m_framespersec;
		   hdr.bpp = _this->m_iFrameBpp;
		   hdr.bpp_effective = _this->m_iFrameBppEffective;
		   hdr.compressed = 0;
		   
		   memman.memcpysw((char *) &hdr.timestamp, (char *) &_this->m_tTimeStamp, sizeof(_timeb) );  

           // put video header inside the memory block  
           memman.memcpysw( block, (char *) &hdr, sizeof(VIDHDR) );
		  
		   // put video frame bits inside the memory location.
		   memman.memcpysw( block+sizeof(VIDHDR), (char *) _this->m_pAktImage, _this->m_iFrameWidth*_this->m_iFrameHeight*_this->m_iFrameBpp/8 );
   
 
           // drop frames if there are too many
	       int num_in_queue = _this->m_GrabQueue->GetSize();
	       if (num_in_queue > 5)
	       {
		       for (int i=0;i<num_in_queue-1;i++)
		       {
		         void *tmpptr = (void *) _this->m_GrabQueue->RemoveHead();
		         if (tmpptr != NULL) memman.sw_free( tmpptr );

			   }
		   }

           // insert frame into grab queue
           _this->m_GrabQueue->AddTail( (void *) block );
          
   		   // Notify our main window that we grabbed a new image
		   // grabbed image can be obtained by reading from the grab queue
		   AfxGetMainWnd()->PostMessage(WM_COMMAND, ID_NOTIFY_GRABBED);
		  }
          cam->GrabRelease(_this->m_hgrab,iSeqNum);
        }
        
        last_ret = ret;

	}
	
	return 0;
}


int CGrab::ifcerrorhandler(const char *fmt, va_list marker)
{
	char msg[200];
	int l;
	l=_vsnprintf(msg, 198, fmt, marker);
	
	CString tmp(msg);
	CString tmp1("IFCerr: ");

	tmp1 = tmp1 + tmp;

	((CMainDlg *)AfxGetMainWnd())->PrintServerMessage(tmp1);

	return l;
}


//////////////////////////////////////////////////////////////////////
// 
// This function is the error handler for errors happening at class 
// level. The parameter errornum is not really used at the moment. The 
// other parameters ( format, ...) are just like printf().
//
int CGrab::myErrorHandler( int errornum, const char *format, ... )
{

 char m_msg[512];
	
 va_list args;
 va_start(args, format);
 _vsnprintf(m_msg, 510, format, args);
 va_end(args);
 
 m_iLastError = errornum;
 ((CMainDlg *)AfxGetMainWnd())->PrintServerMessage(m_msg);

 //AfxMessageBox( m_msg, MB_ICONEXCLAMATION | MB_OK );

 return 1;
}

//////////////////////////////////////////////////////////////////////
// 
// This function returns the length in bytes of an acquired frame. If 
// the class is not yet initialized, the function returns zero.
//
int CGrab::GetFrameLength()
{
 if (m_bSuccessfulInitialized == TRUE) // successfully initialized ? 
 {
  return( m_iFrameWidth * m_iFrameHeight * m_iFrameBpp / 8 ); // return frame length
 }
  else
	  return 0; // return zero
}


BOOL CGrab::setNRefPRefHW(int cameraport)
{
 short nref;
 short pref;

 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 CICamera *cam = getCurrentCamera();

 nref = m_nref;
 pref = m_pref;

 switch( cameraport )
 {
  case 0: cam = m_pCam0;
          break;

  case 1: cam = m_pCam1;
          break;

  case 2: cam = m_pCam2;
          break;

  case 3: cam = m_pCam3;
          break;

  default: return FALSE;
 }

 cam->SetAcqParam(AMV_NREF_CONTROL, nref/1000.0);
 cam->SetAcqParam(AMV_PREF_CONTROL, pref/1000.0);

 return TRUE;
}  


BOOL CGrab::getPRef(short &pref)
{
	if (m_bSuccessfulInitialized == FALSE) return( FALSE );

	pref = m_pref;

	return TRUE;
}


BOOL CGrab::getNRef(short &nref)
{
	if (m_bSuccessfulInitialized == FALSE) return( FALSE );
	
	nref = m_nref;

	return TRUE;
}


BOOL CGrab::setNRef( short nref )
{
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 m_nref = nref;

 return TRUE;
}

BOOL CGrab::setPRef( short pref )
{
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 m_pref = pref;

 return TRUE;
}

int CGrab::getFrameWidth()
{
 return( m_iFrameWidth );
}

int CGrab::getFrameHeight()
{
 return( m_iFrameHeight );
}

int CGrab::getFrameBpp()
{
 return( m_iFrameBpp );
}

BOOL CGrab::getEffectiveBpp(unsigned int &bpp_eff)
{
	bpp_eff = m_iFrameBppEffective;

	return TRUE;
}

void CGrab::StartStopGrabWorkaround()
{
  // SW! disabled, takes too much time (around 12 seconds with this workaround)
  // SW! June 28, 2006
  // SW! but works no matter how much the framegrabber card was screwed up

 /*
 // check whether class is properly initialized
 if (m_bSuccessfulInitialized == FALSE) return;
 
 // if we are actually grabbing, do nothing
 if (m_bIsGrabbing == TRUE) return;

 // start the grabbing

 CICamera *cam;

 cam = getCurrentCamera();

 DWORD old = m_iCap->SetAcqTimeout(90);

 m_hgrab = cam->Grab(0, m_pDoubleImageBuffer, 2);
 if(!m_hgrab) 
 {
  myErrorHandler(10, "Could not start grabworkaround.");
  return;
 } 

 cam->Freeze(m_hgrab);

 m_iCap->SetAcqTimeout(old);
 // success
 */
}
