// NotQuiteNullDacl.h: interface for the NotQuiteNullDacl class.
// 
// This class allows the user to simply create a Not-Quite-Null 
// Discretionary Access Control List (Dacl).  This Dacl has 
// many of the attributes of the Null Dacl that is often used
// to open up access to file and kernel objects but denies the
// user access to the objects security attributes.  Therefore
// it is less subject to abuse than the simple Null Dacl.
//
// This class is based heavily on the concepts and sample code 
// found in Chapter 10 of Programming Server-Side Applications
// for Microsoft Windows 2000 by Richter and Clark.  It also 
// used ideas and code from the MSDN article "Ask Dr. Gui #49"
//
//////////////////////////////////////////////////////////////////////


//
// SW! Apr 21, 2006 downloaded this class and example code from 
//
// http://www.codeguru.com/cpp/w-p/win32/tutorials/article.php/c4545/
//
//

#if !defined(AFX_NOTQUITENULLDACL_H__0C1A24F2_8DFB_4F3D_B280_BFE81D692A45__INCLUDED_)
#define AFX_NOTQUITENULLDACL_H__0C1A24F2_8DFB_4F3D_B280_BFE81D692A45__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <aclapi.h>

class NotQuiteNullDacl  
{
public:
   // constructor/destructor
	NotQuiteNullDacl();
	virtual ~NotQuiteNullDacl();

   // accessors
   PACL  GetPDACL() {return m_pDACL;};
   DWORD GetErr() {return m_dwErr;}

   // the real work gets done here
   bool Create();

private:
   bool CreateEveryoneSid();
   ULONG CalculateAclSize();
   bool AddNotQuiteNullAces();


private:
	PACL                    m_pDACL;
   PSID                    m_pEveryoneSid;
   DWORD                   m_dwErr;
};

#endif // !defined(AFX_NOTQUITENULLDACL_H__0C1A24F2_8DFB_4F3D_B280_BFE81D692A45__INCLUDED_)
