#if !defined(_ASYNC_SOCKET_HPP_INCLUDED_)
#define _ASYNC_SOCKET_HPP_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// define some standard constants to be independent from windows sockets declarations

#define SD_SEND1  1
#define SW_SOCK_STREAM SOCK_STREAM


class SSyncSocket
	{
	public:
		SSyncSocket(); // Construction
		virtual ~SSyncSocket(); // virtual Destruction

		// creates a socket
		BOOL Create( UINT nSocketPort = 0, int nSocketType = SW_SOCK_STREAM, LPCTSTR lpszSocketAddress = NULL);

		// accepts an incoming connection
		BOOL Accept( SSyncSocket& rConnectedSocket, SOCKADDR *lpSockAddr = NULL, int *lpSockAddrLen = NULL);
		
		// closes the socket
		void Close(); 

		// binds the socket to a port and a host name/dotted IP address
		BOOL Bind( UINT nSocketPort, LPCTSTR lpszSocketAddress = NULL );
        // binds the socket to a address defined by the SOCKADDR structure
		BOOL Bind ( const SOCKADDR* lpSockAddr, int nSockAddrLen );

		// connects to host/dotted IP address and port number
        BOOL Connect( LPCTSTR lpszHostAddress, UINT nHostPort );
		// connectes to remote computer defined by SOCKADDR structure
		BOOL Connect( const SOCKADDR *lpSockAddr, int nSockAddrLen );

		// puts the socket in listening mode to detect incoming connections
		BOOL Listen( int nConnectionBacklog = SOMAXCONN );

		// receives exact nBufLen bytes from the socket
		int Receive( void *lpBuf, int nBufLen, int nFlags = 0);

		// sends out nBufLen bytes from memory location lpBuf
		int Send( const void *lpBuf, int nBufLen, int nFlags = 0); 

		// returns the dotted ip address and the port of the remote computer
		BOOL GetPeerName( LPSTR rPeerAddress, UINT & rPeerPort);
        // returns the remote computer address in a SOCKADDR structure
        BOOL GetPeerName( SOCKADDR *lpSockAddr, int *lpSockAddrLen);

		// gets the last error from underlying socket implementation
		int GetLastError(); // returns last socket error

		// get human readable string of last error inside this socket class
		void GetLastErrorString (CString& msg); // returns last socket error as human readable string

	protected:
		// no protected members

	private:
		// attach a socket handle to class
		bool attach( SOCKET s);

		// saves the last error code inside the class 
		int m_iLastError;
		
		// socket handle 
		SOCKET the_socket;

		// shuts down a send/receive or both communcation directions
        BOOL ShutDown( int nHow = SD_SEND1);
	};

#endif //_ASYNC_SOCKET_HPP_INCLUDED_
