// Stat.h: interface for the CStat class.
//
// Initially written by Velizar Miltchev (DESY Zeuthen) miltchev@ifh.de
// some additional code and modifications for Grabber server 
// by Stefan Weisse sweisse@ifh.de
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STAT_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
#define AFX_STAT_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////////////////
//
// This class does straightforward statistical analysis. Using weight of the masses 
// in x projection and y projetion the class is able to detect the spot center coor-
// dinates and the spot size. Straightforward statistics fail in some cases, e.g. if 
// there is more than one spot on the video frame. This class, in comparison to the 
// CStat implementation in the Videoclnt, does no subtraction of the background image and 
// no normalization of the area of interest.
//

class CStat  
{
public:
	// construction of the class
	CStat(unsigned char* data,unsigned int ImWi=768,unsigned int lmHi=574);

	// destruction of the class
	virtual ~CStat();

    // returns x mean value (spot center on x-axis)
	double XM();
	// returns y mean value (spot center on y-axis)
	double YM();

	// returns x rms value (spot size on x-axis)
	double X_RMS();
	// returns y rms value (spot size on y-axis)
	double Y_RMS();

    // returns the x-distribution (dist over all columns)
	double* X_DIST();
	// returns the y-distribution (dist over all rows)
	double* Y_DIST();

	// do the calculation of spot center and size
	void START();

protected:
    // floating point distribution arrays
	double* XDIST;
	double* YDIST;

    // integer distribution array (needed for calculation in inner loop)
	unsigned int *YDISTi;
	unsigned int *XDISTi;

    // member variables used to store the calculated spot center and size 
	double XMEAN;
	double YMEAN;
	double XRMS;
	double YRMS;

    // member variables to store important variables
	unsigned char* array; // input array
	unsigned char* BackGr;
	unsigned int left; // leftmost pixel position of area of interest
	unsigned int right; // rightmost pixel position of area of interest
	unsigned int up; // topmost pixel position of area of interest
	unsigned int down; // bottom-most pixel position of area of interest
	unsigned int ImageWidth; // width of the to-be-analyzed image
	unsigned int ImageHeight; // height of the to-be-analyzed image

	// calculate x distribution: deprecated, replaced by CalcXYDistSW()
	void CalcXDIST(unsigned char* data);
	// calculates y distribution: deprecated, replaced by NormalizeImage() and CalcXYDistSW()
	void CalcYDIST(unsigned char* data);

    // calculation of distributions 
	void CalcXYDistSW(unsigned char *data);

	// calculates x mean value
	double CalcXMEAN(double* dist);
	// calculates y mean value
	double CalcYMEAN(double* dist);
	// calculates x rms value
	double CalcXRMS(double* dist,double xm);
	// calculates y rms value
	double CalcYRMS(double* dist,double ym);

    // temporary storage of some max value  
	unsigned char MaxVal;

};



#endif // !defined(AFX_STAT_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
