#ifndef _HISTORY_H_
#define _HISTORY_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include "prolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "servdef.h"

#ifndef MAX_HISTORY_RECORDS
# define MAX_HISTORY_RECORDS 20
#endif

typedef struct
{
  CONTRACT c;
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthInSeconds;         /* for short term storage */
  int  depthInMonths;           /* for long term storage */
  int  dataPtr;
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
  int  recordIndex;            /* local index identifier */
  int  recordLength;
  time_t lastArchiveTime;
  char ArchiveFileName[16]; /* current working file name */
  time_t *timestamp;                  /* timestamp array */
  BYTE *data;
} HistoryRecord;

typedef struct
{
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthInSeconds;         /* for short term storage */
  int  depthInMonths;           /* for long term storage */
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
} HistorySpecification;

#ifdef UNIX
# define SH_DENYNONE (S_IWUSR|S_IRUSR|S_IRGRP|S_IROTH)
#endif
#ifdef VXWORKS
# undef SH_DENYNONE
# define SH_DENYNONE (0644)
#endif
#define HISTORY_FILENAME "history.csv"

#define HISTORY_BUFFER_SIZE 1024
#define STS_DEADBAND 2

extern char ArchiveLocation[128];
extern HistoryRecord *historyRecords;
extern int nHistoryRecords;
extern int maxHistoryRecords;

int AppendHistoryInformation(char *eqp,char *prp,char *dev,int len,int fmt,int idx,HistorySpecification *hspec);
int LoadHistoryInformation(char *fname);
void historyInit(void);
void historyCycle(void);
long loadLTSData(int i,int index,time_t starttime,time_t endtime,
                 void *datbuf,int datfmt,int bufsize,int sample);
long getLTSData(char *eqp,char *prp, char *dev,int index,
                 time_t starttime,time_t endtime,
                 BYTE *data,int fmt,int arraysize,int sample);
long getSTSData(char *eqp,char *prp, char *dev,int index,
                 time_t starttime,time_t endtime,
                 BYTE *data,int fmt,int arraysize,int sample);
long getLastStoredData(char *eqp,char *prp, char *dev,int index,
                 BYTE *data,int fmt,int arraysize);
int HISTQueryFunction(CONTRACT *con,char *dataIn,char *data);

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* _HISTORY_H_ */
