
#ifndef RPCTYPES_H_
#define RPCTYPES_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include <time.h>
#include "rpcdefs.h"


typedef struct
{
  char name[8];
} NAME8;

typedef struct
{
  char name[16];
} NAME16;

typedef struct
{
  char name[32];
} NAME32;

typedef struct
{
  char name[48];
} NAME48;

typedef struct
{
  char name[64];
} NAME64;

typedef struct
{
  char name[8];
  UINT32 ival;
} NAME8I;

typedef struct
{
  char name[16];
  UINT32 ival;
} NAME16I;

typedef struct
{
  char name[32];
  UINT32 ival;
} NAME32I;

typedef struct
{
  char name[48];
  UINT32 ival;
} NAME48I;

typedef struct
{
  char name[64];
  UINT32 ival;
} NAME64I;

typedef struct
{
  char   name[16];
  float  fval;
  UINT32 ival;
} NAME16FI;

typedef struct
{
  UINT16 sval;
  UINT32 ival;
} SHTINT;

typedef struct
{
  UINT32 lval;
  UINT32 ival;
} LNGINT;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
} INTINT;

typedef struct
{
  float  fval;
  UINT32 ival;
} FLTINT;

typedef struct
{
  double dval;
  UINT32 ival;
} DBLINT;

typedef struct
{
  float  fval;
  UINT32 i1val;
  UINT32 i2val;
} FLTINTINT;

typedef struct
{
  UINT32 i1val;
  float  fval;
  UINT32 i2val;
} INTFLTINT;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  time_t tm;
  char   str[80];
} DUSTRING;

typedef struct
{
  float  xval;
  float  yval;
  UINT32 ival;
} XYS;

typedef struct
{
  float f1val;
  float f2val;
  UINT32 ival;
} FLTFLTINT;

typedef struct
{
  float x;
  float y;
} DPOINT;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
  UINT32 i3val;
  UINT32 i4val;
} DADDRESS;

typedef struct
{
  time_t t1;
  time_t t2;
  UINT32 i1val;
  UINT32 i2val;
} DWINDOW;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
  float  f1val;
  float  f2val;
} FWINDOW;

typedef struct
{
  float  f1val;
  UINT32 i1val;
  float  f2val;
  UINT32 i2val;
} FLTINTFLTINT;

typedef struct
{
  time_t t;
  float  d;
  UINT16 s;
  UINT16 i;
} TDSI;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  float  f3val;
} DFILTER;

typedef struct
{
  char   comment[80];
  time_t tm;
  float  s_start;
  float  s_inc;
  UINT32 status;
} hdrSPECTRUM;

typedef struct
{
  char   comment[80];
  time_t tm;
  float  s_start;
  float  s_inc;
  UINT32 status;
  float  d_spect_array[1024];
} DSPECTRUM;

typedef union
{
  void   *vptr;
  char   *cptr;
  UINT8  *bptr;
  UINT16 *sptr;
  UINT32 *lptr;
  float  *fptr;
  double *dptr;
} DUNION;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[16];
  DUNION data;
} DTYPE;

#define RPCERR_SIZE 32
extern char RPCLastErrorString[RPCERR_SIZE];

/* ======================== servers.h ==remove far's 26.01.95===========*/

int Subscribe(char **Values,char *eqpname,int EqNo,char *property,int size,
		   int format,int pollingRate,void (*function)(int,int),int mode);

int hotRpc(char **Values, char *eqpname, int EqNo, char *property, int rwFlg,
		 int format, int pollingRate, void (*function)(int,int));

void Cancel(int i); /* cancels subscription #i (called automatically when you 'quit') */
#define CloseLink(i) Cancel(i)
extern int connectionTableID;

int RPC(char *Values, char *eqpname, int EqNo, char *property, int rwFlg, int format, int datasize);
int aRPC(char *Values, char *eqpname, int EqNo, char *property, int rwFlg, int format, int datasize, void (*function)(int,int));

int PACK(char *Name,char *Values,char *eqpname, int EqNo,
         char *property,int rwFlg,int format,int datasize,char **info);

extern int SystemRunning;
int _SystemReset(int level);
#define SystemReset(i) _SystemReset(i)
int _SystemCycle(int chkcmd);
#define SystemCycle(i) _SystemCycle(i)
void _SystemDelay(int msec);
#define SystemDelay(i) _SystemDelay(i)
int _SystemInit(int netchk);
#define SystemInit(i)  _SystemInit(i)
int _SystemScheduleProperty(char *eqm, char *prp);
#define SystemScheduleProperty(a,b) _SystemScheduleProperty(a,b)
char *GetRPCLastError(short cc);
void SetRPCCompletion(char *errstr);
int ExecLink(char *devName,char *devProperty,
             DTYPE *dout,DTYPE *din,short access);
int ExecLinkEx(char *devName,char *devProperty,
	     DTYPE *dout,DTYPE *din,short access,UINT16 timeout);
int AttachLink(char *devName,char *devProperty,
               DTYPE *dout,DTYPE *din,short access,
               int pollingRate,void (*callback)(int,int),int mode);
int AttachLinkEx(char *devName,char *devProperty,
							 DTYPE *dout,DTYPE *din,short access,
               int pollingRate,void (*callback)(int,int),int mode,UINT32 callbackID);
int ModifyLinkAttributes(int i,short access,int pollingRate,
               void (*callback)(int,int),int mode,UINT32 callbackID);
int recvNetGlobal(char *keyword,DTYPE *dout,void (*callback)(int,int));
int recvNetGlobalEx(char *keyword,DTYPE *dout,void (*callback)(int,int),UINT32 callbackID);

int PACKRPC(char *Name);
int RegisterFecNameEx(char *name,char *desc,char *os,char *loc,char *hdw,char *resp,BYTE poff,char *context);
int RegisterFecName(char *name,char *desc,char *os,char *loc,char *hdw,char *resp,BYTE poff);
int RegisterPropertyEx(char *eqm,char *prop,int siz,short fmt,short acc,char *dsc,int prpId);
int RegisterProperty(char *eqm,char *prop,int siz,short fmt,short acc,char *dsc);
int RegisterPropertyInformation(char *eqm,char *prop,DTYPE *dout,DTYPE *din,short acc,char *dsc,int propId,char *rdr);
int RegisterModuleName(char *eqname,char *devname,int devnr);
#define RegisterDeviceName(eqm,dev,devnr) RegisterModuleName(eqm,dev,devnr)
char *GetDeviceName(char *eqm,int devnr);
#define GetModuleName(eqm,devnr) GetDeviceName(eqm,devnr)
int GetDeviceNumber(char *eqm,char *dev);
#define GetModuleNumber(eqm,dev) GetDeviceNumber(eqm,dev)
int GetPropertyId(char *eqm,char *prpName);
int RegisterExport(char *tagname,char *eqpname,int numModules);
int RegisterEquipmentModule(
    char *tagname,
    char *eqpname,
    int NumModules,
    int (*fcn)(char *,char *,DTYPE *,DTYPE *,short),
    void (*ini)(void),
    void (*tsk)(void),
    int rate,
    void (*exi)(void)
);
int RegisterNodalModule(
    char *tagname,
    char *eqpname,
    int NumModules,
#   ifdef NODAL
    int (*fcn)(double *,int,int,char *),
#   else
    int (*fcn)(double *,short,short,short,char *),
#   endif
    void (*ini)(void),
    void (*tsk)(void),
    int rate,
    void (*exi)(void)
);
int PutRPCErrorValue(int i, void *errValue);

typedef struct
{
  char  prpName[32];
  char  prpDescription[32];
  UINT16 prpSize;
  BYTE prpFormat;
  BYTE prpAccess;
} PropertyQueryStruct;
#define PROPERTYQUERYSTRUCT_SIZE (32+32+2+1+1)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT32 prpHistoryDepth;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
} PropertyQueryStructEx;
#define PROPERTYQUERYSTRUCTEX_SIZE (32+64+32+8*2+16+4*6+4)

short RegisterPropertyQueryFunction(char *eqm,int (*fcn)(char *devName,PropertyQueryStruct **pqs),int numprops);
short RegisterPropertyQueryExFunction(char *eqm,int (*fcn)(char *devName,PropertyQueryStructEx **pqsx),int numprops);
int RegisterLocalAlias(char *eqpname,char *alias);
int AssignNumModules(char *eqname,int num);

int GetNumContracts(char *eqname);
int GetNumConsumers(char *eqname);
int GetNumCallers(char *eqname);
int GetCurrentLinkStatus(int i);
int GetCurrentLinkStatusFromCallbackId(int id);
char *GetCaller(void);
int GetCallerInfo(NAME16 *un,BYTE *ipx,UINT32 *ip,short *prot,int *num);
int GetCompletionDataSize(int i);
int GetCompletionDataSizeFromCallbackId(int id);
int GetCompletionDataType(int i);
int GetCompletionDataTypeFromCallbackId(int id);
char *GetExportedName(char *eqpname);
BYTE *SystemVersion(void);
int feclog(char *text);
int ExecLocalLink(char *devName,char *devProperty,DTYPE *dout,DTYPE *din,short access);

extern short EqpFormats[];
extern time_t RPCtimestamp;
extern short RPCversion[2];
extern int logServerFile;
extern char logClientFile;
extern int RequireAcknowledgments;
extern int AllowRemoteExit;
extern short SychronizeFecClock;
extern WORD watchdogBaseAddress;
extern long SysPoll;
extern unsigned short MinPollingRate;
extern unsigned short MaxPollingRate;
extern int defaultClientTimeout;
#define defaultClientTimeoutMsec defaultClientTimeout
extern int MaxNumContracts;
extern int MaxNumClients;
extern int MaxNumConnections;
extern unsigned long MaxRPCTransportSize;
extern char logServerFileName[80];
extern char logServerFileMode[8];
extern char logClientFileName[80];
extern char logClientFileMode[8];
extern BYTE PortOffset;
extern int FeclogDepth;
extern int StartupDebug;
extern UINT32 LastCompletionDataSize;
extern int allowLegacyConnections;
extern int longDeviceNames;
extern int putCommandsInFeclog;
extern int nofeclog;

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* RPCTYPES_H_ */
