/////////////////////////////////////////////////////////////////////////////
// TINE Multiplexer service
//
// BASED ON example and reference implementation of a simple TINE server "sine" as
//   a Windows service 
//
//   - example taken from PJ Naughters CNTService freeware example
//
//   - modified by Stefan Weisse / DESY Zeuthen, September - December 2005
//
//		- changes: removed references to CString object and CStringBuffer, 
//		  replaced by default string handling (char *, string.h)
//		- removed MFC (completely? imho) out of the project (to have less dependencies)
//		- integrated TINE sine server into it
//		- added "set working directory" to the directory of the executable in
//		  order to find the configuration files (*.csv)
//
//   OPEN POINTS:
//	  - error messages on command line are not sufficent (ie. not describing what really happened)
//
//   NOTES:
//	  - you have to specify a command line parameter (like -install, -uninstall or 
//      -help) otherwise the service is attempted to be "touched" by StartServiceControl
//	    which does only work from NT service manager (see "RunAsService")
//
// TINE Video Multiplexer (and general camera controller)
//
//    S. Weisse / DESY Zeuthen, Apr 26, 2005
/////////////////////////////////////////////////////////////////////////////
#define APPV_DATE "Wed Apr 26 13:00:00 2006"
#define APPV_MAJOR 0
#define APPV_MINOR 9
#define APPV_REVISION 100

#define SERVICEUNIQUENAME		"TINEVIDMUXSVC"
#define SERVICECALLNAME			"TINE Video and Multiplexer Service (TINEVIDMUXSVC)"
#define SERVICEDESCRIPTION		"Video Multiplexer server (TINE, as Service)"
#define SERVICEACCEPTFUNCTIONS	SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_PAUSE_CONTINUE
#define COMMANDLINEHELP         "This service controls as a kind of master the Video multiplexer device\nas well as the camera switching across servers.\nUsage: VideoService [-install | -uninstall | -help]"


#include "stdafx.h"

#include "prolog.h"
#include <direct.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include "servdef.h"
#include "rpctypes.h" 
#include "toolkit.h"
#include "ntcmd.h"
#include "ntserv_msg.h"

#include "app.h"


// TINE Forward declarations

// usually found inside "application".cpp

void PostSystemInit(void);
void PreSystemInit(void);

BOOL CALLBACK EnumServices(DWORD /*dwData*/, ENUM_SERVICE_STATUS& Service)
{
	TRACE(_T("Service name is %s\n"), Service.lpServiceName);
    TRACE(_T("Friendly name is %s\n"), Service.lpDisplayName);

    return TRUE; //continue enumeration
}


int main( void )
{

	CNTServiceCommandLineInfo cmdInfo;
    CMyService Service;
	Service.ParseCommandLine(cmdInfo);
	Service.ProcessShellCommand(cmdInfo);

	return 1;
}

CMyService::CMyService() : CNTService(_T(SERVICEUNIQUENAME), _T(SERVICECALLNAME), SERVICEACCEPTFUNCTIONS, _T(SERVICEDESCRIPTION)) 
{
	//Simple boolean which is set to request the service to stop
	m_bWantStop = FALSE;
	m_bPaused = FALSE;
}

void CMyService::ServiceMain(DWORD /*dwArgc*/, LPTSTR* /*lpszArgv*/)
{
	int cc=0;
	TCHAR szPath[MAX_PATH]; 
	char *pos;
    
	// Move current dir ("working directory") to the directory of the executable service file
    if( !GetModuleFileName( NULL, szPath, MAX_PATH ) )
    {
        printf("GetModuleFileName failed (%d)\n", GetLastError()); 
        exit(1);
    }
    
	if ((pos = strrchr((const char *) szPath, '\\')) != NULL) *pos = 0;

	if (_chdir(szPath) != 0) exit(1);
    // Move current dir ("working directory") to the directory of the executable service file END

	//register our control handler
	RegisterCtrlHandler();

	//Pretend that starting up takes some time
	ReportStatusToSCM(SERVICE_START_PENDING, NO_ERROR, 0, 1, 0);

	/* TINE: init RPC server: */
    /* call any non-default resource initialization before SystemInit(): */

	AssignFecApplicationInfo(APPV_MAJOR,APPV_MINOR,APPV_REVISION);
	    
	PreSystemInit();

	if ((cc=SystemInit(TRUE)) != 0) 
	{ 
		
		printf("TINE: SystemInit error: %s\n",erlst[cc]); 
		exit(1); 
	}

	/* call FEC specific initialization: */
	PostSystemInit();
	
	ReportStatusToSCM(SERVICE_RUNNING, NO_ERROR, 0, 1, 0);

    //Report to the event log that the service has started successfully
    m_EventLogSource.Report(EVENTLOG_INFORMATION_TYPE, CNTS_MSG_SERVICE_STARTED, m_sDisplayName);

	//The tight loop which constitutes the service


    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  
    BOOL bOldPause = m_bPaused;
	
	while (!m_bWantStop)
	{
		// TINE
	    if (!m_bPaused) SystemCycle(TRUE); else Sleep(10); 
		// Sleep(10); added in order not to overload the machine on paused server


		//SCM has requested a Pause / Continue
		if (m_bPaused != bOldPause)
		{
			if (m_bPaused)
			{
    			ReportStatusToSCM(SERVICE_PAUSED, NO_ERROR, 0, 1, 0);
				//Report to the event log that the service has paused successfully
				m_EventLogSource.Report(EVENTLOG_INFORMATION_TYPE, CNTS_MSG_SERVICE_PAUSED, m_sDisplayName);
			}
			else
			{
				ReportStatusToSCM(SERVICE_RUNNING, NO_ERROR, 0, 1, 0);
				//Report to the event log that the service has stopped continued
				m_EventLogSource.Report(EVENTLOG_INFORMATION_TYPE, CNTS_MSG_SERVICE_CONTINUED, m_sDisplayName);
			}
		}

		bOldPause = m_bPaused;
	}

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	//Pretend that closing down takes some time
	ReportStatusToSCM(SERVICE_STOP_PENDING, NO_ERROR, 0, 1, 0);
    
	// TINE
    feclog("Stopping RPC Server ...\n>");
    ServerExitCondition = 4;
    closeServers(0);
    ExitEQPModules();
    freeServerMemory();

	ReportStatusToSCM(SERVICE_STOPPED, NO_ERROR, 0, 1, 0);

    //Report to the event log that the service has stopped successfully
    m_EventLogSource.Report(EVENTLOG_INFORMATION_TYPE, CNTS_MSG_SERVICE_STOPPED, m_sDisplayName);
}

void CMyService::OnStop()
{
    ::EnterCriticalSection(&lock);

    //Change the current state to STOP_PENDING
    m_dwCurrentState = SERVICE_STOP_PENDING;  

	//Signal the other thread to end
	m_bWantStop = TRUE;
	
	::LeaveCriticalSection(&lock);
}

void CMyService::OnPause()
{
    ::EnterCriticalSection(&lock);

    //Change the current state
    m_dwCurrentState = SERVICE_PAUSE_PENDING;  

	//Signal the other thread
    m_bPaused = TRUE;

	::LeaveCriticalSection(&lock);
}

void CMyService::OnContinue()
{
    ::EnterCriticalSection(&lock); //synchronise access to the variables

    //Change the current state
    m_dwCurrentState = SERVICE_CONTINUE_PENDING;  

	//Signal the other thread
    m_bPaused = FALSE;

	::LeaveCriticalSection(&lock); //synchronise access to the variables
}

void CMyService::OnUserDefinedRequest(DWORD dwControl)
{
	dwControl;
	// NO user defined request implemented
}

void CMyService::ShowHelp()
{
	printf("%s\n", COMMANDLINEHELP);
}