// IniEx.cpp: implementation of the CIniEx class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>

#include "stringfunc.h"
#include "IniEx.h"

#include <assert.h>

#define ASSERT assert


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//GrowSize for Dynmiz Section Allocation
CIniEx::CIniEx(int GrowSize)
{
	TCHAR szPath[MAX_PATH]; 
	char *pos;
	
	m_GrowSize=GrowSize;

	m_SectionNo=0;
	m_writeWhenChange=FALSE;
	m_makeBackup=FALSE;
	m_NoCaseSensitive=TRUE;
	m_Changed=FALSE;
	
	m_Keys=NULL;
	m_Values=NULL;
	m_Sections = NULL;

	m_allocatedObjectCount=0;

	m_ErrStr[0] = 0;
	m_ExePath[0] = 0;
	m_BackupFileName[0] =0;
	m_FileName[0] = 0;

	if( !GetModuleFileName( NULL, szPath, MAX_PATH ) ) memset(szPath, 0, MAX_PATH);	
    
	if ((pos = strrchr((const char *) szPath, '\\')) != NULL) *pos = 0;

	if (strlen(szPath) == 0) 
		copywholestring(m_ExePath, "");
	else
		strncpy(m_ExePath, szPath, MAX_PATH_LENGTH-1);

}

CIniEx::~CIniEx()
{
	if (m_writeWhenChange)
		WriteFile(m_makeBackup);
	ResetContent();
}



BOOL CIniEx::OpenAtExeDirectory(const char *pFileName,BOOL writeWhenChange,	BOOL createIfNotExist, BOOL noCaseSensitive,
							BOOL makeBackup)
{
	char filePath[MAX_PATH_LENGTH];
	
	copywholestring(filePath, m_ExePath);

	if (isRightmostChar( filePath, '\\') == FALSE) addstr(filePath,"\\");
	addstr(filePath, pFileName);

	return Open(filePath,writeWhenChange,createIfNotExist,noCaseSensitive,makeBackup);
}

BOOL CIniEx::Open(const char *pFileName, BOOL writeWhenChange, BOOL createIfNotExist, BOOL noCaseSensitive,
				  BOOL makeBackup )
{

	BOOL bRet;

	static char Line[MAX_STRING_LENGTH];
	static char sectionStr[MAX_STRING_LENGTH];
	int nPlace;

	//if it's second ini file for this instance
	//we have to save it and delete member variables contents
	
	
	if (!IsEmpty(m_FileName)) 
	{
		WriteFile();
		ResetContent();
	}

	m_NoCaseSensitive=noCaseSensitive;
	m_writeWhenChange=writeWhenChange;
	m_makeBackup=makeBackup;

	// try whether it is there and can be opened

	char tmpKey[MAX_STRING_LENGTH];
	char tmpValue[MAX_STRING_LENGTH];

	FILE *fp;
	fp = fopen( pFileName, "rt");
	if (fp != NULL)
	{
		copywholestring(m_FileName,pFileName);

		//Grow the arrays as GrowSize(which given constructor)
		m_allocatedObjectCount = m_GrowSize;
		m_Keys=(char **) malloc( m_allocatedObjectCount * sizeof(char *) );
		//m_Keys_Size = (int *) malloc( m_allocatedObjectCount * sizeof(int) );

		m_Values=(char **) malloc( m_allocatedObjectCount * sizeof(char *) );
		//m_Values_Size = (int *) malloc( m_allocatedObjectCount * sizeof(int) );

		m_Sections = (char *) malloc( 2*sizeof(char) );
		
		m_Sections[0] = 0;
		m_Sections[1] = 0;

		for (int i=0;i<m_GrowSize;i++)
		{
			m_Keys[m_SectionNo+i]=NULL;
			//m_Keys_Size[m_SectionNo+i] = 0;
			m_Values[m_SectionNo+i]=NULL;
			//m_Values_Size[m_SectionNo+i] = 0;
		}

		while (TRUE)
		{
			//Read one line from given ini file
			bRet=ReadString(fp, Line);
			if (!bRet) break;

			TrimRight(Line, " \n\t");

			if (IsEmpty(Line)) continue;
								
			//if line's first character = '[' 
			//and last character = ']' it's section 
			if ((isLeftmostChar(Line, '[') == TRUE) && (isRightmostChar(Line, ']') == TRUE))
			{
				m_Keys[m_SectionNo]=new char[2];
				memset(m_Keys[m_SectionNo],0,2);

				//m_Keys_Size[m_SectionNo] = m_CurKeySize;
				
				m_Values[m_SectionNo]=new char[2];
				memset(m_Values[m_SectionNo], 0, 2);

				//m_Values_Size[m_SectionNo] = m_CurValueSize;
				
				m_SectionNo++;		
				GrowIfNecessary();
				
				Mid(sectionStr, Line, 1, strlen(Line)-2 );
				continue;
			}
			
			nPlace=find(Line, "=");
			
			if (nPlace==-1)
			{
				copywholestring(tmpKey,Line);
				copywholestring(tmpValue, "");
			}
			else
			{
				Left(tmpKey, Line, nPlace);
				Mid(tmpValue, Line, nPlace+1);
			}
			
			//Add( char **stringarray, const char *string )
			Add(&(m_Keys[m_SectionNo-1]), tmpKey);
			Add(&(m_Values[m_SectionNo-1]), tmpValue);
			SetAtGrow(&m_Sections,m_SectionNo-1, sectionStr);
		}
		fclose(fp);
		return TRUE;

	}
	else
		return FALSE;
}


char *CIniEx::GetValue(const char *Key)
{
	return GetValue("",Key);
}

//if Section Name="" -> looking up key for witout section
char *CIniEx::GetValue(const char *Section,const char *Key,const char *DefaultValue/*=""*/)
{
	static char retStr[MAX_STRING_LENGTH];

	int nIndex=LookupSection(Section);

	if (nIndex==-1) return (char *) DefaultValue;

	int nRet;
	
	for (int i=GetUpperBound(m_Keys[nIndex]);i>=0;i--)
	{
		nRet=CompareStrings(GetAt(m_Keys[nIndex],i),Key);
		if (nRet==0)
		{
			copywholestring(retStr, GetAt(m_Values[nIndex],i));
			//int nPlace=retStr.ReverseFind(';');
			//if (nPlace!=-1) retStr.Delete(nPlace,retStr.GetLength()-nPlace);
			return retStr;
		}
	}

	return (char *) DefaultValue;
}


int CIniEx::GetValueInt(const char *Section,const char *Key,int defval /*DefaultValue=""*/)
{
	char tmp[MAX_STRING_LENGTH];
	sprintf(tmp, "%d", defval);
	
	int ret = atoi( (const char *) GetValue(Section, Key, tmp) );
	return( ret );
}

double CIniEx::GetValueDouble(const char *Section,const char *Key,double defval)
{
	char tmp[MAX_STRING_LENGTH];
	sprintf(tmp, "%f", defval);
	
	double ret = atof( (const char *) GetValue(Section, Key, tmp) );
	return( ret );
}

//returns index of key for given section
//if no result returns -1
int CIniEx::LookupKey(int nSectionIndex,const char *Key)
{
	ASSERT(nSectionIndex<=m_SectionNo);
	int nRet;
	for (int i=GetUpperBound(m_Keys[nSectionIndex]);i>=0;i--)
	{
		nRet=CompareStrings(GetAt(m_Keys[nSectionIndex], i), Key);
		
		if (nRet==0) return i;
	}
	return -1;
}

//return given sections index in array
int CIniEx::LookupSection(const char *Section)
{
	int nRet;
	for (int i=0;i<GetSize(m_Sections);i++)
	{
		nRet=CompareStrings(GetAt(m_Sections,i),Section);
		if (nRet==0) return i;
	}
	return -1;
}

//Sets for Key=Value for without section
void CIniEx::SetValue(const char *Key,const char *Value)
{
	SetValue("",Key,Value);
}

//returns backup file name
//if you didn't want backup (when openning file) it returns ""
char *CIniEx::WriteFile(BOOL makeBackup/*=FALSE*/)
{
	//return m_BackupFileName;

	static char retStr[MAX_PATH_LENGTH];
	
	if (!m_Changed) 
		return "";

	char tmpFileName[MAX_PATH_LENGTH];
	
	copywholestring(tmpFileName, m_FileName);
	copywholestring(retStr, "");
	
	if (makeBackup)
	{
		if ( IsEmpty(m_BackupFileName) )
		{
			FindBackupFile();
		}

		BOOL ret=CopyFile(m_FileName,m_BackupFileName,FALSE);
	}

	
	FILE *fp = fopen(m_FileName, "wt");
	if (fp == NULL) { printf("File could not be opened for writing!!\n"); return retStr; }

	
	for (int i=0;i<GetSize(m_Sections);i++)
	{
		if (!IsEmpty(GetAt(m_Sections,i)))
		{
			fprintf(fp, "[%s]\n", GetAt(m_Sections,i));
		}

		if (!m_Keys[i]) continue;

		int max = GetUpperBound(m_Keys[i]);

		for (int j=0;j<=max;j++)
		{
			//if key is empts we don't write "="

			
			fprintf(fp, "%s%s%s\n",GetAt(m_Keys[i],j),
							IsEmpty(GetAt(m_Keys[i],j))?"":"=", GetAt(m_Values[i],j));

			if (j==max) fprintf(fp, "\n");
		
		}
	}

	fclose(fp);
	// m_Changed = FALSE; // ??? shouldn't this be here ??? 

	return m_BackupFileName;
}

BOOL CIniEx::GetWriteWhenChange(void)
{
	return m_writeWhenChange;
}


void CIniEx::SetWriteWhenChange(BOOL WriteWhenChange)
{
	m_writeWhenChange=WriteWhenChange;
}


void CIniEx::SetBackupFileName(const char *backupFile)
{
	copywholestring(m_BackupFileName,backupFile);
}

void CIniEx::FindBackupFile(void)
{
	char filePath[MAX_PATH_LENGTH];
	//char ext[MAX_PATH_LENGTH];

	WIN32_FIND_DATA ffData;
	BOOL bContinue=TRUE;
	
	copywholestring(filePath, m_FileName);

	int nPlace=reversefind(filePath, '.');

	DeleteString(filePath,nPlace,strlen(filePath)-nPlace);
	addstr(filePath,"*.*");

	int extNo=0;
	char *p;

	HANDLE handle=FindFirstFile(filePath,&ffData);

	while (bContinue)
	{
		bContinue=FindNextFile(handle,&ffData);
		p=ffData.cFileName;
		p+=strlen(ffData.cFileName)-3;
		if (atoi(p)>extNo) extNo=atoi(p);
	}
	
	sprintf(m_BackupFileName, "%s.%.3d",m_FileName,extNo+1);
}


void CIniEx::ResetContent()
{
	if (!m_Keys) return;
	if (m_Keys)
	{
		for (int i=0;i<m_allocatedObjectCount;i++)
		{
			if (m_Keys[i])
				delete [] m_Keys[i];
			if (m_Values[i])
				delete [] m_Values[i];
		}
		free(m_Keys);
		//free(m_Keys_Size);
		free(m_Values);
		//free(m_Values_Size);
		free( m_Sections );
	}
	m_Keys=NULL;
	m_Values=NULL;
	//m_Keys_Size=NULL;
	//m_Values_Size=NULL;
	m_Sections = NULL;
	
	m_SectionNo=0;
	
	copywholestring(m_FileName,"");
	
	m_Changed=FALSE;
}


//Removes key and it's value from given section
BOOL CIniEx::RemoveKey(const char *Section,const char *Key)
{
	int nIndex=LookupSection(Section);
	if (nIndex==-1) return FALSE;

	int nKeyIndex=LookupKey(nIndex,Key);
	if (nKeyIndex==-1) return FALSE;

	RemoveAt(&(m_Keys[nIndex]),nKeyIndex);
	RemoveAt(&(m_Values[nIndex]),nKeyIndex);

	m_Changed=TRUE;

	return TRUE;
}

//Removes key and it's value 
BOOL CIniEx::RemoveKey(const char *Key)
{
	return RemoveKey("",Key);
}


//Removes given section(including all keys and values)
//return FALSE when given section not found
//It won't couse memory leak because when deleting object
//the msize (malloc size) checking
BOOL CIniEx::RemoveSection(const char *Section)
{
	int nIndex=LookupSection(Section);
	if (nIndex==-1) return FALSE;

	//m_Keys[nIndex]->RemoveAll();
	//m_Values[nIndex]->RemoveAll();

	delete [] m_Keys[nIndex];
	delete [] m_Values[nIndex];

	m_Keys[nIndex]=NULL;
	m_Values[nIndex]=NULL;

	RemoveAt(&m_Sections,nIndex);
	m_SectionNo--;
	m_Changed=TRUE;
	
	return TRUE;
}

int CIniEx::CompareStrings(const char *str1, const char *str2)
{
	if (m_NoCaseSensitive)
		return stricmp(str1, str2);
	else
		return strcmp(str1, str2);
}

void CIniEx::GrowIfNecessary(void)
{
	//for first gives GrowSize
	if (m_SectionNo>=m_allocatedObjectCount)
	{
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//realloc for GrowSize
		m_allocatedObjectCount += m_GrowSize;
		
		m_Keys=(char **)realloc(m_Keys,sizeof(char *) * (m_allocatedObjectCount) );
		m_Values=(char **)realloc(m_Values,sizeof(char *) * (m_allocatedObjectCount) );
		
		//m_Keys_Size = (int *) realloc(m_Keys_Size,sizeof(int) * (m_allocatedObjectCount) );
		//m_Values_Size = (int *) realloc(m_Values_Size,sizeof(int) * (m_allocatedObjectCount) );
		
		//allocated + GrowSize
		//zero memory for new allocation
		
		for (int i=0;i<m_GrowSize;i++)
		{
			m_Keys[m_SectionNo+i]=NULL;
			//m_Keys_Size[m_SectionNo+i]=0;

			m_Values[m_SectionNo+i]=NULL;
			//m_Values_Size[m_SectionNo+i]=0;
		}
	}
}

//copy each string (section name) because 
//if sections parametter be a pointer it may clear content of member
void CIniEx::GetSections(char **sections)
{
	for (int i=0;i<GetSize(m_Sections);i++)
		Add(sections,GetAt(m_Sections,i));
}

void CIniEx::GetKeysInSection(const char *section,char **keys)
{
	int nIndex=LookupSection(section);
	if (nIndex==-1) return;

	for (int i=0;i<GetSize(m_Keys[nIndex]);i++)
	{
		Add(keys, GetAt(m_Keys[nIndex],i));
	}
}

//writes Key=value given section
void CIniEx::SetValue(const char *Section,const char *Key,const char *Value)
{
	//file opened?
	ASSERT(!IsEmpty(m_FileName));

	//if given key already existing, overwrite it
	int nIndex=LookupSection(Section);
	int nKeyIndex;
	if (nIndex==-1)
	{
		//if key not exist grow arrays (if necessary)
		m_Changed=TRUE;
		m_SectionNo++;
		GrowIfNecessary();
		
		m_Keys[m_SectionNo-1]=new char[2];
		memset( m_Keys[m_SectionNo-1], 0, 2);

		m_Values[m_SectionNo-1]=new char[2];
		memset( m_Values[m_SectionNo-1], 0, 2);
		
		nIndex=m_SectionNo-1;
		Add(&m_Sections,Section);
	}

	//looking up keys for section
	nKeyIndex=LookupKey(nIndex,Key);
	
	//if key exist -> overwrite it
	//if not add to end of array
	if (nKeyIndex!=-1) 
	{
		if (CompareStrings(GetAt(m_Values[nIndex],nKeyIndex),Value)!=0) 
			m_Changed=TRUE;
		
		SetAt(&(m_Values[nIndex]),nKeyIndex,Value);
	}
	else	//if not exist
	{
		m_Changed=TRUE;
		Add(&m_Keys[nIndex], Key);
		Add(&m_Values[nIndex],Value);
	}
}

void CIniEx::SetValueInt(const char *Section, const char *Key, int value )
{
	char tmp[MAX_STRING_LENGTH];

	sprintf(tmp, "%d", value);
	SetValue(Section, Key, tmp);
}

void CIniEx::SetValueDouble(const char *Section, const char *Key, double value )
{
	char tmp[MAX_STRING_LENGTH];

	sprintf(tmp, "%f", value);
	SetValue(Section, Key, tmp);
}
