// IniEx.h: interface for the CIniEx class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_INIEX_H_INCLUDED_)
#define _INIEX_H_INCLUDED_


#define MAX_SECTION_COUNT	512


class CIniEx  
{
private:
	//Functions
	int		LookupKey		(int nSectionIndex,const char *Key);
	int		LookupSection	(const char *Section);
	int		CompareStrings	(const char *str1,const char *str2);
	void    GrowIfNecessary	(void);
	void	FindBackupFile	(void);

	void GetKeysInSection(const char *section, char **keys ); // keys = stringarray
	void GetSections(char **sections); // sections = stringarray

private:
	//Variables
	char				m_ErrStr[MAX_STRING_LENGTH];
	char				m_ExePath[MAX_PATH_LENGTH];
	
	// StringArray == str1\0str2\0str3\0str4\0\0
	char **m_Keys; // StringArray
	//int  *m_Keys_Size;
	char **m_Values; // StringArray
	//int  *m_Values_Size;

	char *m_Sections; // StringArray
	//int  m_Sections_Size;

	//int		m_CurKeySize;
	//int		m_CurValueSize;

	int		m_SectionNo;
	int		m_GrowSize;
	int		m_allocatedObjectCount;
	BOOL	m_NoCaseSensitive;
	BOOL	m_writeWhenChange;
	char	m_BackupFileName[MAX_STRING_LENGTH];
	char    m_FileName[MAX_STRING_LENGTH];
	BOOL	m_makeBackup;
	BOOL	m_Changed;	

public:

	CIniEx(int GrowSize=4);
	virtual ~CIniEx();

	BOOL Open(LPCSTR pFileName,
			  BOOL writeWhenChange=TRUE,
			  BOOL createIfNotExist=TRUE,
			  BOOL noCaseSensitive=TRUE,
			  BOOL makeBackup=FALSE);
	BOOL OpenAtExeDirectory(LPCSTR pFileName,
			  BOOL writeWhenChange=TRUE,
			  BOOL createIfNotExist=TRUE,
			  BOOL noCaseSensitive=TRUE,
			  BOOL makeBackup=FALSE);
	void ResetContent();

	char *WriteFile(BOOL makeBackup=FALSE);

	char *GetValue(const char *Section,const char *Key,const char *DefaultValue="");
	char *GetValue(const char *Key);
	int     GetValueInt(const char *Section,const char *Key,int defval=0);
	double  GetValueDouble(const char *Section,const char *Key,double defval=0.0);

	void SetValue(const char *Key,const char *Value);
	void SetValue(const char *Section,const char *Key,const char *Value);
	void SetValueInt(const char *Section, const char *Key, int value );
    void SetValueDouble(const char *Section, const char *Key, double value );

	BOOL RemoveKey(const char *Key);
	BOOL RemoveKey(const char *Section,const char *Key);

	BOOL RemoveSection(const char *Section);

	BOOL GetWriteWhenChange();
	void SetWriteWhenChange(BOOL WriteWhenChange);

	void SetBackupFileName(const char *backupFile);
};

#endif // !defined(_INIEX_H_INCLUDED_)