
// version "NT service" 0.92 beta, May 05, 2006
// snapshot from old multiplexer server taken (V0.89 Beta dev from Oct 24, 2005)
//
// adjusted  : if connection to mplex lost, try a reconnect
//			   increase timeout on "select on read back data from mplex device" via tcp to 2 seconds

//				- all strings NAME48 instead of text (on read, NOT on write!)
//				- connect with 60 seconds timeout on startup (esp. important if server is started by SCM
//				  on windows startup)
//				- if service is paused sleep(10) instead of SystemCycle() in order not to overload 
//				  server
//				- registerfecnameex changed (fecname == servername ("Server" from MPServer.cfg) + "FEC")
//				- timeouts enlarged and socket telnet interface to rs232 handling improved

// changelog:
//
// 11. mai 2006
//	- fixed endless loop when switching a camera and no multiplexer device is installed (i.e. without multiplexer 
//	  device architecture)
//
//

/*
   Module: mps_srv.c  
*/
#include "prolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <windows.h>
#include "errors.h"
#include "rpctypes.h"
#include "swaplib.h"  /* for CF_STRUCT example */
#include "almsrv.h"
#include "history.h"
#include "stringfunc.h"
#include "IniEx.h"

/* 
   toolkit.c is an optional module, which you might want to
   compile and link into your project.  It contains many
   useful routines for querying or manipulating data.  This
   example makes use of calls to getValuesAsFloat() and
   putValuesFromFloat(), as well as PropertyDataExchange(),
   all of which are modules from toolkit.c
   
   So here we include the header with the protypes:
*/
#include "toolkit.h" 


#ifdef MSDOS
# include "doscmd.h"
#elif defined(UNIX)
# include "unixcmd.h"
# define random(r) (rand()%(r))
#elif defined(WINNT) 
# include "ntcmd.h"
# define random(r) (rand()%(r))
#else
# define random(r) ((i*(69069))%(r))
#endif

static char versionStr[49] = "VidMux Server Service V0.93 beta, May 09, 2006";

//#define DEVNUM_MPS		0

//#define DEVNUM_MPU		1
//#define DEVNUM_CAM		2
//#define DEVNUM_SRV		3

static const char devmpu[]="Multiplexer";
static const char devmps[]="MPServer";
static const char devsrv[]="Server";
static const char devcam[]="Camera";

static WSADATA wsaData;

#define EQPMODNAME "MPSEQM"

static SOCKET tcp_sock=INVALID_SOCKET;

static char newcamid[11];
static char newserverid[11];
static long newlock;

static char newcamid_forbkg[11];
static char newserverid_forbkg[11];
static long newlock_forbkg;

static long selectOngoing;
static long m_lSelectFinishedSuccessfully;

// is the multiplexer device needed (==1) or are all cameras on dedicated servers ? (==0)
static long m_lMultiplexerDeviceNeeded;

static long m_iSomethingChanged;

#define PRP_MPS_ALIVESINCE			1
#define PRP_MPS_VERSION				2
#define PRP_MPS_STATUS				3
#define PRP_MPS_NEWCAMID			4
#define PRP_MPS_NEWSERVERID			5
#define PRP_MPS_NEWLOCK				6
#define PRP_MPS_SELECT				7
#define PRP_MPS_SELECTREADBACK		8		
#define PRP_MPS_PROBESELECT			9	
#define PRP_MPS_LASTERROR			10	
#define PRP_MPS_NUMCAMERAS			11	
#define PRP_MPS_NUMSERVERS			12	
#define PRP_MPS_SOMETHINGCHANGED	13	

#define PRP_CAM_NAME				14
#define PRP_CAM_CAMID				15
#define PRP_CAM_STS					16
#define PRP_CAM_ENABLED				17
#define PRP_CAM_EXTTRIG				18	
#define PRP_CAM_TRIGMOD5			19	
#define PRP_CAM_WIDTH				20
#define PRP_CAM_HEIGHT				21
#define PRP_CAM_BPP					22
#define PRP_CAM_ORIENTATION			23		
#define PRP_CAM_SCALE				24
#define PRP_CAM_NREF				25
#define PRP_CAM_PREF				26
#define PRP_CAM_ISONMULTIPLEXER		27		
#define PRP_CAM_LOCALPORT			28	
#define PRP_CAM_DEDICATEDSERVERID	29		
#define PRP_CAM_MPINPUTPORT			30	

#define PRP_SRV_NAME				31
#define PRP_SRV_SERVERID			32
#define PRP_SRV_TCPPORT				33
#define PRP_SRV_TCPPORTLOSSY		34
#define PRP_SRV_TINEFRAMESNAME		35
#define PRP_SRV_TINECONTROLSNAME	36
#define PRP_SRV_TINECONTEXT			37
#define PRP_SRV_STS					38
#define PRP_SRV_CURCAMID			39
#define PRP_SRV_CURCAMNAME			40
#define PRP_SRV_LOCK				41
#define PRP_SRV_ISONMULTIPLEXER		42
#define PRP_SRV_MPOUTPUTPORT		43

#define PRP_MPU_STATUS				44
#define PRP_MPU_CONNSTATUS			45
#define PRP_MPU_NUMINPUTPORTS		46
#define PRP_MPU_NUMOUTPUTPORTS		47
#define PRP_MPU_CAMIDINPUTPORT		48
#define PRP_MPU_SRVIDOUTPUTPORT		49
#define PRP_MPU_CAMIDONOUTPUTPORT	50


#define CHANGEDSCHEDPROP "SomethingChanged"

 
XPropertyQueryStruct ServerXQueryStruct[] = {
  { "Name",				"server name",					"","","","",		0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_NAME,0 },
  { "ServerID",			"server id",					"","","","",		0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_SERVERID,0 },
  { "TCPPort",			"tcp port (lossless)",			"","","","",		0,	65535,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_TCPPORT,0 },
  { "TCPPortLossy",		"tcp port for lossy network",	"","","","",		0,	65535,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_TCPPORTLOSSY,0 },
  { "TineFramesName",	"tine frames server name",		"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_TINEFRAMESNAME,0 },
  { "TineControlName",	"tine control server name",		"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_TINECONTROLSNAME,0 },
  { "TineContext",		"tine context",					"","","","",		0,	768,	1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_TINECONTEXT,0 },
  { "Status",			"server status (good/bad) NOT IMPLEMENTED",		"","","","",		0,	100,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_STS,0 },
  { "CurCamID",			"current camera id",			"","","","",		0,	32,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_CURCAMID,0 },
  { "CurCamName",		"current camera name",			"","","","",		0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_CURCAMNAME,0 },
  { "Lock",				"locked? NOT IMPLEMENTED",						"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_LOCK,0 },
  { "IsOnMultiplexer",	"is on mplexer?",				"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_ISONMULTIPLEXER,0 },
  { "MPOutputPort",		"output port of mplex",			"","","","",		0,	100,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_SRV_MPOUTPUTPORT,0 }
};

XPropertyQueryStruct CameraXQueryStruct[] = {
  { "Name",				"camera name",				"","","","datetime",0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_NAME,0 },
  { "CamID",			"camera id",				"","","","",		0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_CAMID,0 },
  { "Status",			"server status (good/bad) NOT IMPLEMENTED",	"","","","",		0,	100,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_STS,0 },
  { "Enabled",			"enabled?",					"","","","",		0,	1,		1,  0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_ENABLED,0 },
  { "ExtTrig",			"external trig?",			"","","","",		0,	1,		1,  0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_EXTTRIG,0 },
  { "TrigMod5",			"trig mod 5?",				"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_TRIGMOD5,0 },
  { "Width",			"src width of camera",		"","","","",		0,	768,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_WIDTH,0 },
  { "Height",			"src height of camera",		"","","","",		0,	576,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_HEIGHT,0 },
  { "BPP",				"bits per pixel of cam",	"","","","",		0,	32,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_BPP,0 },
  { "Orientation",		"orientation",				"","","","",		0,	0,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_ORIENTATION,0 },
  { "Scale",			"scale factor of cam",		"","","","",		0,	1000,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_SCALE,0 },
  { "NRef",				"negative reference",		"","","","mV",		1,	1000,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_NREF,0 },
  { "PRef",				"positive reference",		"","","","mV",		1,	2000,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_PREF,0 },
  { "IsOnMultiplexer",	"is on multiplexer ?",		"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_ISONMULTIPLEXER,0 },
  { "LocalPort",		"local port: [dedic]",		"","","","",		0,	100,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_LOCALPORT,0 },
  { "DedicatedServerID","is on dedicated server ?",	"","","","",		0,	1,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_DEDICATEDSERVERID,0 },
  { "MPInputPort",		"input port [isonmplex]",	"","","","",		0,	100,	1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,	GT_LIN,"",0,0,1,1,0,0,PRP_CAM_MPINPUTPORT,0 }
};


XPropertyQueryStruct MultiplexerServerXQueryStruct[] = {
  { "AliveSince",		"srv online since",			"","","","datetime",0,	1000,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_ALIVESINCE,0 },
  { "Version",			"server version build id",	"","","","",		0,	1000,		1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_VERSION,0 },
  { "Status",			"server status (good/bad)",	"","","","",		0,	100,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_LONG),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_STATUS,0 },
  { "NewCamID",			"new camera id",			"","","","",		0,	0,			1, 48,1,0,0,BFMT(CF_NAME48),	BFMT(CF_TEXT),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPS_NEWCAMID,0 },
  { "NewServerID",		"new server id",			"","","","",		0,	0,			1, 48,1,0,0,BFMT(CF_NAME48),	BFMT(CF_TEXT),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPS_NEWSERVERID,0 },
  { "NewLock",			"lock server",				"","","","",		0,	1,			1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPS_NEWLOCK,0 },
  { "Select",			"update configuration",		"","","","",		0,	1,			1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPS_SELECT,0 },
  { "SelectReadback",	"update succeeded?",		"","","","",		0,	1,			1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_SELECTREADBACK,0 },
  { "ProbeSelect",		"probe update",				"","","","",		0,	1,			1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPS_PROBESELECT,0 },
  { "LastError",		"last error as string",		"","","","",		0,	0,			1,	0,1,0,0,BFMT(CF_NAME48),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_LASTERROR,0 },
  { "NumCameras",		"number of cameras",		"","","","",		1,	1000,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_NUMCAMERAS,0 },
  { "NumServers",		"number of server",			"","","","",		1,	1000,		1,	0,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_NUMSERVERS,0 },
  { CHANGEDSCHEDPROP,	"increased any camchange",  "","","","",		0,	0xFFFF,		1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPS_SOMETHINGCHANGED,0 }
};

XPropertyQueryStruct MultiplexerUnitXQueryStruct[] = {
  { "Status",			"mplex unit sts (good/bad) NOT IMPLEMENTED","","","","",		0,	1,		1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPU_STATUS,0 },
  { "ConnStatus",		"mplex tcp sts (good/bad)",	"","","","",		0,	1,		1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPU_CONNSTATUS,0 },
  { "NumInputPorts",	"number of input ports",	"","","","",		0,	1,		1,  1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPU_NUMINPUTPORTS,0 },
  { "NumOutputPorts",	"number of output ports",	"","","","",		0,	1,		1,	1,1,0,0,BFMT(CF_LONG),	BFMT(CF_NULL),CA_READ,			GT_LIN,"",0,0,1,1,0,0,PRP_MPU_NUMOUTPUTPORTS,0 },
  { "CamIDInputPort",	"cam id on inport",			"","","","",		0,	1,		1,  1,1,0,0,BFMT(CF_NAME48),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPU_CAMIDINPUTPORT,0 },
  { "SrvIDOutputPort",  "srv id on outport",		"","","","",		0,	1,		1,  1,1,0,0,BFMT(CF_NAME48),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPU_SRVIDOUTPUTPORT,0 },
  { "CamIDOnOutputPort","cam id on outport",		"","","","",		0,	1,		1,  1,1,0,0,BFMT(CF_NAME48),	BFMT(CF_LONG),CA_READ|CA_WRITE,	GT_LIN,"",0,0,1,1,0,0,PRP_MPU_CAMIDONOUTPUTPORT,0 }
};


static CIniEx *m_pMPSIniFile;
static CIniEx *m_pSrvIniFile;
static CIniEx *m_pCamIniFile;
static time_t m_ServerStartTime;

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// socket and multiplexer access (telnet raw, verysimple)
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

int PingMultiplexer(int sock)
{
	//char cmdget1[]="GET 1\n";
	char cmdget1[]={'G','E','T',' ','1',13,0};
	timeval tmval;
	char returnbuf[10];
	char throwbuf[10];
	int numrecv;

	fd_set WRITE;
	fd_set READ;

	FD_ZERO( &WRITE );
	FD_SET( sock, &WRITE);

	tmval.tv_sec = 2;
	tmval.tv_usec = 0;

	int ret = select(1, NULL, &WRITE, NULL, &tmval);
	if (ret <= 0)
	{
		feclog("PingMultiplexer: Waiting for socket to write ERROR = %d\n", ret);

		return 0;
	}

	//SW! Apr 18, 2006 START

	int ret1 = send(sock, cmdget1, strlen(cmdget1), 0);
	if ((unsigned)ret1 != strlen(cmdget1))
	{
		feclog("PingMultiplexer: Error on send(\"GET 1\") to multiplexer = %d (WSAerr=%d)\n", ret1, WSAGetLastError());
		return 0;
	}

	//SW! Apr 18, 2006 END

	// receive until nothing comes in any more

	FD_ZERO( &READ );
	FD_SET( sock, &READ);
	tmval.tv_sec = 3; //SW! Apr 18, 2006
	tmval.tv_usec = 0;

	ret = select(1, &READ, NULL, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("PingMultiplexer: Waiting for socket to read back ERROR = %d\n", ret);
		return 0;
	}

	// receive until nothing comes in any more

	int left_read = 9;
	int finished = 0;
	char *startbuf = returnbuf;
	numrecv = 0;

	while (!finished)
	{
		ret1 = recv(sock, startbuf, left_read, 0);
		if (ret1 > 0)
		{
			left_read -= ret1;
			if (startbuf != throwbuf) numrecv += ret1;
			if (left_read < 0) 
			{
				//left_read = 0;
				startbuf = throwbuf;
				left_read = 9;
			}
		}
		else if (ret1 == 0)
		{
			feclog("PingMultiplexer: Telnet connection to multiplexer was closed unexpectedly. Err = %d", WSAGetLastError() );
			return 0;
		}
		else if (ret1 == SOCKET_ERROR)
		{
			if (WSAGetLastError() == WSAEWOULDBLOCK) finished = 1;
			else 
			{
				feclog("PingMultiplexer: Lost telnet connection to multiplexer. Err = %d", WSAGetLastError() );
				return 0;
			}
		}
		else
		{
			feclog("PingMultiplexer: Unknown error (recv telnet from multiplexer). Err = %d / %d", ret1, WSAGetLastError() );
			return 0;
		}

	}
		
	numrecv = ret1;
		//printf("Returned %d bytes\n", numrecv);
	returnbuf[numrecv] = 0;

	if (_stricmp(returnbuf, "ERROR") == 0) 
	{
		feclog("PingMultiplexer: Multiplexer returned ERROR(!)\n");
		return 0;
	}
	//printf("---Multiplexer PING successful\n");

	return 1;
}

// 1 == success
// 0 == error
int getMultiplexerCameraOnOutputPort( int sock, int mpout, int &mpin )
{
	char cmd[20];
	timeval tmval;
	char returnbuf[10];
	int ret, ret1;
	fd_set WRITE, READ;
	int numrecv;

	cmd[0] = 0;
	sprintf(cmd, "GET %d%c", mpout, 13);

	FD_ZERO( &WRITE );
	FD_SET( sock, &WRITE);

	tmval.tv_sec = 2;
	tmval.tv_usec = 0;

	ret = select(1, NULL, &WRITE, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("getMultiplexerCameraOnOutputPort: Waiting for socket to write ERROR = %d", ret);
		return 0;
	}

	ret1 = send(sock, cmd, strlen(cmd), 0);
	
	if (( ((unsigned)ret1) != strlen(cmd)) || (ret1 == SOCKET_ERROR))
	{
		feclog("getMultiplexerCameraOnOutputPort: error: send() ret1=%d WSAErr=%d", ret1, WSAGetLastError() );
		return 0;
	}


	FD_ZERO( &READ );
	FD_SET( sock, &READ);
	
	tmval.tv_sec = 3; //SW! 
	tmval.tv_usec = 0;

	ret = select(1, &READ, NULL, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("getMultiplexerCameraOnOutputPort: Waiting for socket to read back ERROR = %d", ret);
		return 0;
	}

	ret1 = recv(sock, returnbuf, 9, 0);
	
	if (ret1 == 0 || ret1 == SOCKET_ERROR)
	{
		feclog("getMultiplexerCameraOnOutputPort: error: recv() ret=%d\n", ret1);
		return 0;
	}

	numrecv = ret1;
	//feclog("Returned %d bytes\n", numrecv);
	returnbuf[numrecv] = 0;

	int num = atoi(returnbuf);

	mpin = num;

	return 1; // success

}

int connect_socket( SOCKET lsocket, char *lpszHostAddress, unsigned short nHostPort, int try_time )
{
	struct sockaddr_in C;

	// find out whether the lpszHostAddress is a dotted IP address
	C.sin_family = AF_INET;
	C.sin_port = htons(nHostPort);
	C.sin_addr.s_addr= inet_addr( lpszHostAddress );

	if (C.sin_addr.s_addr == INADDR_NONE) // aha, no dotted IP address, 
	{                                     // so look up hostname
		// pcHost isn't a dotted IP address, so send it to the resolver
		hostent* he = gethostbyname(lpszHostAddress);
		if (he == 0) 
		{
			// No one knows who lpszHostAddress is so fail connect
			return 0;
		}

		// host found, convert the address into the appropriate structure
		C.sin_addr.s_addr = *((unsigned long*)(he->h_addr));
	}

	time_t startTime;
	int i=0, ret=2667;

	startTime = time(NULL);

	while( time(NULL) < startTime+try_time )
	{
		i++;
  		feclog("connecttry #%d to ip=%s",i, inet_ntoa(C.sin_addr));
		ret = connect(lsocket, (const struct sockaddr *) &C, sizeof(C));

		if (ret != 0) 
		{
			feclog("NOT connected. Error was: ret=%d, WSAGetLastError=%d", ret, WSAGetLastError() );
			Sleep(500);
		}
		else break; // successful, connect happened
	}

	if (ret!=0) 
	{
		feclog("Connect: Unable to connect to multiplexer via telnet! Err = %d / %d", ret, WSAGetLastError() );
		return 0;
	}

	u_long arg = 1;
	ret = ioctlsocket(lsocket, FIONBIO, &arg);
	if (ret == SOCKET_ERROR)
	{
		feclog("Connect: Unable to set socket to nonblocking mode! Err = %d", WSAGetLastError() );
		return 0;
	}
	if (ret != 0) 
	{
		feclog("Connect: Unable to set socket to nonblocking mode! Unknown Err = %d / %d", ret, WSAGetLastError() );
		return 0;
	}

	return( 1 ); 
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// MISC support functions
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void putLeadingZeroes(char *outstr, const char *basicstr, int posnum)
{
	char format[20];
	char result[60];

	result[0] = 0;
	format[0] = 0;
	
	int pos = find(basicstr, "%");
	if (pos != -1)
	{
		Mid(format, basicstr, pos);
		if (pos > 0) Mid(result, basicstr, 0, pos);
	}

	if (strlen(format) == 0) strcpy(format, "%d");

	if (posnum < 100) addstr(result, "0");
	if (posnum < 10) addstr(result, "0");

	addstr(result, format);
	sprintf(outstr, result, posnum);
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// TINE support functions START
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void putCharString(DTYPE *dout, const char *str)
{
	if (dout == NULL) return;

	unsigned int length = strlen(str) + 1;

	//if (dout->dArrayLength < length) length = dout->dArrayLength;
	if (length > 48) length = 48;

	dout->dFormat = CF_NAME48;
	dout->dTimeStamp = makeDataTimeStamp();

	memcpy(dout->data.cptr, str, length);
}

int getCharString( DTYPE *din, char *str, unsigned int maxstrlength)
{
	if (din->dFormat != CF_TEXT) return illegal_format;
	if (din->dArrayLength > maxstrlength) return dimension_error;

	memcpy(str, din->data.cptr, din->dArrayLength);
	str[din->dArrayLength] = 0;

	return 0;
}


void putLong(DTYPE *dout, long int val)
{
	if (dout == NULL) return;

	dout->dArrayLength = 1;
	dout->dFormat = CF_LONG;
	dout->dTimeStamp = makeDataTimeStamp();

	memcpy(dout->data.lptr, &val, sizeof(long int));
}

void putDouble(DTYPE *dout, double val)
{
	if (dout == NULL) return;

	dout->dArrayLength = 1;
	dout->dFormat = CF_DOUBLE;
	dout->dTimeStamp = makeDataTimeStamp();

	memcpy(dout->data.lptr, &val, sizeof(double));
}

int getLong(DTYPE *din, long int &val)
{
	return( getValuesAsLong(din, (UINT32 *) &val, 1) );
}


int PutLongToServer(char *context, char *servername, char *property, long val)
{
  static char devstr[100];
  int rc;
  DTYPE din;
  static long localval;

  devstr[0] = 0;
  sprintf( devstr, "/%s/%s/GRABSRV", context, servername);

  localval = val;
  din.dArrayLength = 1;
  din.dFormat = CF_LONG;
  din.dTimeStamp = makeDataTimeStamp();
  din.data.lptr = (unsigned long *) &localval;
 
  rc = ExecLinkEx(devstr, property,NULL,&din,CA_WRITE,1000);

  if (rc != 0) 
  {
	  feclog("PutLongToServer errcode = %d", rc);
	  return 0; 
  }
  else return 1;
}

int GetLongFromServer(char *context, char *servername, char *property, long &val)
{
  static char devstr[100];
  int rc;
  DTYPE dout;
  static long localval;

  devstr[0] = 0;
  sprintf( devstr, "/%s/%s/GRABSRV", context, servername);

  localval = val;
  dout.dArrayLength = 1;
  dout.dFormat = CF_LONG;
  dout.dTimeStamp = makeDataTimeStamp();
  dout.data.lptr = (unsigned long *) &localval;
 
  rc = ExecLinkEx(devstr, property,&dout,NULL,CA_READ,1000);

  if (rc != 0) return 0; else { val = localval; return 1; }
}


int PutDoubleToServer(char *context, char *servername, char *property, double val)
{
  static char devstr[100];
  int rc;
  DTYPE din;
  static double localval;

  devstr[0] = 0;
  sprintf( devstr, "/%s/%s/GRABSRV", context, servername);

  localval = val;
  din.dArrayLength = 1;
  din.dFormat = CF_DOUBLE;
  din.dTimeStamp = makeDataTimeStamp();
  din.data.lptr = (unsigned long *) &localval;
 
  rc = ExecLinkEx(devstr, property,NULL,&din,CA_WRITE,1000);

  if (rc != 0) return 0; else return 1;
}

int PutStringToServer(char *context, char *servername, char *property, char *strval)
{
  static char devstr[100];
  int rc;
  DTYPE din;
  static char localval[100];

  devstr[0] = 0;
  localval[0] = 0;
  sprintf( devstr, "/%s/%s/GRABSRV", context, servername);

  copywholestring(localval, strval);

  din.dArrayLength = strlen(localval);
  din.dFormat = CF_TEXT;
  din.dTimeStamp = makeDataTimeStamp();
  din.data.lptr = (unsigned long *) &(localval[0]);
 
  rc = ExecLinkEx(devstr, property,NULL,&din,CA_WRITE,1000);

  if (rc != 0) return 0; else return 1;
}



int GetOwnPropertyId(char *devName, char *devProperty)
{
	XPropertyQueryStruct *xpqs;
	int size;

    xpqs = (XPropertyQueryStruct *) &MultiplexerServerXQueryStruct;
	size = sizeof(MultiplexerServerXQueryStruct)/sizeof(XPropertyQueryStruct);

	if (_strnicmp(devName, devmpu, strlen(devmpu)) == 0)
	{
		//return MultiplexerXQueryStruct
		xpqs = (XPropertyQueryStruct *) &MultiplexerUnitXQueryStruct;
		size = sizeof(MultiplexerUnitXQueryStruct)/sizeof(XPropertyQueryStruct);
	}

	if (_strnicmp(devName, devsrv, strlen(devsrv)) == 0)
	{
		//return MultiplexerXQueryStruct
		xpqs = (XPropertyQueryStruct *) &ServerXQueryStruct;
		size = sizeof(ServerXQueryStruct)/sizeof(XPropertyQueryStruct);
	}

	if (_strnicmp(devName, devcam, strlen(devcam)) == 0)
	{
		//return MultiplexerXQueryStruct;
		xpqs = (XPropertyQueryStruct *) &CameraXQueryStruct;
		size = sizeof(CameraXQueryStruct)/sizeof(XPropertyQueryStruct);
	}
	
	for (int i=0;i<size;i++)
	{
		if (strcmp(xpqs->prpName, devProperty) == 0) return xpqs->reserved[1];
		xpqs++;
	}

	return 0;
}


///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// TINE support functions END
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////


// 513: multiplexer.cfg malformed!
// 514: servers.cfg malformed
// 515: cameras.cfg malformed
// 516: server not found!
// 517: camera not found!
// 0: okay
int CheckNewSettings(char *newcamid, char *newserverid, long int newlock)
{
	int cc=0;
	char dummy[]="---empty---";
	char serversection[100]="";
	char camerasection[100]="";
	int serverfound = 0;
	int camerafound = 0;
	int cam;
	int srv;

	int numcameras = (int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
	int numservers = (int) m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);

	if ((numcameras == -1) || (numservers == -1)) ccerr(513);

	for (srv=0;srv<numservers;srv++)
	{
		char servertag[11]= "";

		putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
		char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

		if (strcmp(srvsect, dummy) == 0) ccerr( 514 );

		copywholestring(serversection, srvsect);

		char *serverid = m_pSrvIniFile->GetValue(serversection, "ServerID", dummy);

		if (strcmp(serverid, newserverid) == 0) { serverfound = 1; break; }
	}

	for (cam=0;cam<numcameras;cam++)
	{
		char cameratag[11]= "";

		putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
		char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

		if (strcmp(camsect, dummy) == 0) ccerr( 515 );

		copywholestring(camerasection, camsect);

		char *cameraid = m_pCamIniFile->GetValue(camerasection, "CamID", dummy);

		if (strcmp(cameraid, newcamid) == 0) { camerafound = 1; break; }
	}

	if (camerafound == 0) ccerr( 516 );
	if (serverfound == 0) ccerr( 517 );
	
	return 0; // success

err:
	if (cc == 513) SetRPCCompletion("multiplexer.cfg malformed!");
    else if (cc == 514) SetRPCCompletion("servers.cfg malformed!");
    else if (cc == 515) SetRPCCompletion("cameras.cfg malformed!");
    else if (cc == 517) SetRPCCompletion("server not found!");
    else if (cc == 516) SetRPCCompletion("camera not found!");
	else SetRPCCompletion("unknown internal error.");

// 513: multiplexer.cfg malformed!
// 514: servers.cfg malformed
// 515: cameras.cfg malformed
// 516: server not found!
// 517: camera not found!

	return cc;
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// REQUEST handlers START
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

int HandleServerRequest( int servernum, char *servdev, int propertyid,DTYPE *dout, DTYPE *din,short access)
{
	static char serversection[200];
	static char tempstring[200];
	char *rets;
	long retl;
	int retcode=0;

	serversection[0] = 0;
	tempstring[0] = 0;

	if (access & CA_WRITE) return illegal_read_write;

    rets = m_pSrvIniFile->GetValue("Default", servdev, "");
	if (strcmp(rets, "") == 0) return non_existent;
	copywholestring(serversection, rets);

	switch( propertyid )
	{
		case PRP_SRV_NAME:
				rets = m_pSrvIniFile->GetValue(serversection, "Name", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_SERVERID:
				rets = m_pSrvIniFile->GetValue(serversection, "ServerID", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;
		
		case PRP_SRV_TCPPORT:
				retl = m_pSrvIniFile->GetValueInt(serversection, "TCPPort", 31777);
				putLong(dout, retl);
				break;

		case PRP_SRV_TCPPORTLOSSY:
				retl = m_pSrvIniFile->GetValueInt(serversection, "TCPPortLossy", 31778);
				putLong(dout, retl);
				break;

		case PRP_SRV_TINEFRAMESNAME:
				rets = m_pSrvIniFile->GetValue(serversection, "TineFramesName", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_TINECONTROLSNAME:
				rets = m_pSrvIniFile->GetValue(serversection, "TineControlName", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_TINECONTEXT:
				rets = m_pSrvIniFile->GetValue(serversection, "TineContext", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_STS:
				retl = m_pSrvIniFile->GetValueInt(serversection, "STS", 0);
				putLong(dout, retl);
				break;

		case PRP_SRV_CURCAMID:
				rets = m_pSrvIniFile->GetValue(serversection, "CurCamID", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_CURCAMNAME:
				rets = m_pSrvIniFile->GetValue(serversection, "CurCamName", "");
				if (strcmp(rets, "") == 0) return non_existent;
				putCharString(dout, rets);
				break;

		case PRP_SRV_LOCK:
				rets = m_pSrvIniFile->GetValue(serversection, "Lock", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;
					
				putLong(dout, retl);
				break;

		case PRP_SRV_ISONMULTIPLEXER:
				rets = m_pSrvIniFile->GetValue(serversection, "IsOnMultiplexer", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;

				putLong(dout, retl);
				break;

		case PRP_SRV_MPOUTPUTPORT:
				retl = m_pSrvIniFile->GetValueInt(serversection, "MPOutputPort", -1);

				putLong(dout, retl);
				break;

		default: 
				retcode = illegal_property;
				break;

	}


	return retcode;
}

int HandleCameraRequest( int cameranum, char *camdev, int propertyid,DTYPE *dout, DTYPE *din,short access)
{
	static char camerasection[200];
	static char tempstring[200];
	char *rets;
	long retl;
	double retd;
	int retcode=0;

	camerasection[0] = 0;
	tempstring[0] = 0;
	
	if (access & CA_WRITE) return illegal_read_write;

    rets = m_pCamIniFile->GetValue("Default", camdev, "");
	if (strcmp(rets, "") == 0) return non_existent;
	copywholestring(camerasection, rets);

	switch( propertyid )
	{
		case PRP_CAM_NAME:
				rets = m_pCamIniFile->GetValue(camerasection, "Name", "");
				if (strcmp(rets, "") == 0) return non_existent;

				putCharString(dout, rets);
				break;

		case PRP_CAM_CAMID:
				rets = m_pCamIniFile->GetValue(camerasection, "CamID", "");
				if (strcmp(rets, "") == 0) return non_existent;

				putCharString(dout, rets);
				break;

		case PRP_CAM_STS:
				retl = m_pCamIniFile->GetValueInt(camerasection, "STS", 0);

				putLong(dout, retl);
				break;

		case PRP_CAM_ENABLED:
				rets = m_pCamIniFile->GetValue(camerasection, "Enabled", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;

				putLong(dout, retl);
				break;

		case PRP_CAM_EXTTRIG:
				rets = m_pCamIniFile->GetValue(camerasection, "ExtTrig", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;

				putLong(dout, retl);

				break;

		case PRP_CAM_TRIGMOD5:
				rets = m_pCamIniFile->GetValue(camerasection, "TrigMod5", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;

				putLong(dout, retl);

				break;

		case PRP_CAM_WIDTH:
				retl = m_pCamIniFile->GetValueInt(camerasection, "Width", 768);
				putLong(dout, retl);
				break;

		case PRP_CAM_HEIGHT:
				retl = m_pCamIniFile->GetValueInt(camerasection, "Height", 574);
				putLong(dout, retl);

				break;

		case PRP_CAM_BPP:
				retl = m_pCamIniFile->GetValueInt(camerasection, "BPP", 8);
				putLong(dout, retl);
				break;

		case PRP_CAM_ORIENTATION:
				rets = m_pCamIniFile->GetValue(camerasection, "Orientation", "NORM");
				putCharString(dout, rets);
				break;

		case PRP_CAM_SCALE:
				retd = m_pCamIniFile->GetValueDouble(camerasection, "Scale", 1.0000);
				putDouble(dout, retd);
				break;

		case PRP_CAM_NREF:
				retl = m_pCamIniFile->GetValueInt(camerasection, "NRef", 0);
				putLong(dout, retl);
				break;

		case PRP_CAM_PREF:
				retl = m_pCamIniFile->GetValueInt(camerasection, "PRef", 1000);
				putLong(dout, retl);
				break;

		case PRP_CAM_ISONMULTIPLEXER:
				rets = m_pCamIniFile->GetValue(camerasection, "IsOnMultiplexer", "FALSE");
				if (strcmp(rets, "") == 0) retl = 0;
				else if (_stricmp(rets, "true") == 0) retl = 1;
				else retl = 0;

				putLong(dout, retl);
				break;

		case PRP_CAM_LOCALPORT:
				retl = m_pCamIniFile->GetValueInt(camerasection, "LocalPort", 0);
				putLong(dout, retl);
				break;

		case PRP_CAM_DEDICATEDSERVERID:
				rets = m_pCamIniFile->GetValue(camerasection, "DedicatedServerID", "NONE");
				putCharString(dout, rets);
		
				break;

		case PRP_CAM_MPINPUTPORT:
				retl = m_pCamIniFile->GetValueInt(camerasection, "MPInputPort", -1);
				putLong(dout, retl);
				break;

		default: 
				retcode = illegal_property;
				break;

	}

	return retcode;
}

int HandleMultiplexerServerRequest( int propertyid,DTYPE *dout, DTYPE *din,short access )
{
    int retcode = 0;
	static char tempstr[500];
	static char tempstr1[500];
	struct tm *tmval;
	long templ;

	switch( propertyid )
	{
		case PRP_MPS_ALIVESINCE:	

				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				tmval = localtime( &m_ServerStartTime );
				strftime( tempstr, 499, "%d.%m.%Y %H:%M:%S", tmval );
				sprintf(tempstr1, "Server alive since: %s", tempstr);

				putCharString(dout, tempstr1);

				retcode = 0;
				break;


		case PRP_MPS_VERSION:
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}
	
				putCharString(dout, versionStr);

				retcode = 0;
				break;

		case PRP_MPS_STATUS:

				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				putLong(dout, 0);

				retcode = 0;
				break;

		case PRP_MPS_NEWCAMID:
				retcode = 0;

				if (access & CA_WRITE)
				{
					retcode = getCharString(din, newcamid, 10);
				}
				if (access & CA_READ)
				{
					putCharString(dout, newcamid);
				}

				break;
		case PRP_MPS_NEWSERVERID:
				retcode = 0;

				if (access & CA_WRITE)
				{
					retcode = getCharString(din, newserverid, 10);
				}
				if (access & CA_READ)
				{
					putCharString(dout, newserverid);
				}

				break;

		case PRP_MPS_SOMETHINGCHANGED:
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				putLong(dout, m_iSomethingChanged);

				retcode = 0;
				break;


		case PRP_MPS_NEWLOCK:
				retcode = 0;
				
				if (access & CA_WRITE)
				{
					retcode = getLong(din, newlock);
				}
				if (access & CA_READ)
				{
					putLong(dout, newlock);
				}

				break;
		case PRP_MPS_SELECT:
				retcode = 0;
				if (access & CA_WRITE)
				{
					retcode = CheckNewSettings(newcamid, newserverid, newlock); 
					if (retcode == 0)
					{
						// schedule switching on next background function

						if (selectOngoing == 1) { retcode = 524; break; }
						
						newcamid_forbkg[0] = 0;
						newserverid_forbkg[0] = 0;
						newlock_forbkg = newlock;

						copywholestring(newcamid_forbkg, newcamid);
						copywholestring(newserverid_forbkg, newserverid);

						selectOngoing = 1;
						m_lSelectFinishedSuccessfully = 0;

						if (access & CA_READ) putLong(dout, 1);
					}
					else
					{
					    if (access & CA_READ) putLong(dout, 0);
						retcode = 522;
					}
				}
				else if (access & CA_READ)
				{
					retcode = illegal_read_write; 
					break; 
				}

				break;

		case PRP_MPS_SELECTREADBACK:
				if (access & CA_WRITE)
				{
					retcode = illegal_read_write; 
					break; 
				}

				if (access & CA_READ)
				{
					putLong(dout, m_lSelectFinishedSuccessfully);
				}

				retcode = 0;
				break;

		case PRP_MPS_PROBESELECT:
				if (access & CA_WRITE) 
				{ 
					retcode = CheckNewSettings(newcamid, newserverid, newlock); 
					if (retcode == 0) putLong(dout, 1); else { putLong(dout, 0); retcode = 523; }
				}
				else if (access & CA_READ) 
				{ 
					retcode = illegal_read_write; 
				}
				
				break;

		case PRP_MPS_LASTERROR:
				retcode = 0;
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				sprintf(tempstr1, "There is no error. (Errorcode = 0)\n");
				putCharString(dout, tempstr1);

				break;
		
		case PRP_MPS_NUMCAMERAS:
				retcode = 0;
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				templ = (long int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
				putLong(dout, templ);
	
				break;
		
		case PRP_MPS_NUMSERVERS:
				retcode = 0;
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				templ = (long int) m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);
				putLong(dout, templ);
				
				break;

		default: 
				retcode = illegal_property;
				break;
	}

	if (retcode == 522) SetRPCCompletion("Settings failed verification. Select cancelled.");
	else if (retcode == 523) SetRPCCompletion("Probeselect: check failed !");
	else if (retcode == 524) SetRPCCompletion("Another select is still pending. Cancelled !");

	return retcode;
}

int HandleMultiplexerUnitRequest( int propertyid,DTYPE *dout, DTYPE *din,short access )
{
    int retcode = 0;
	static char tempstr[500];
	static char tempstr1[500];
	//struct tm *tmval;
	long templ;
	char dummy[]="---empty---";

	switch( propertyid )
	{
		case PRP_MPU_STATUS:
				retcode = 0;
				
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				templ = 1;
				putLong(dout, templ);
					
				break;
		
		case PRP_MPU_CONNSTATUS:
				retcode = 0;
				
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				if (tcp_sock != INVALID_SOCKET) templ = 1; else templ = 0;
				putLong(dout, templ);

				break;
		
		case PRP_MPU_CAMIDINPUTPORT:
				retcode = 0;
				
				if ((access & CA_WRITE) && (access & CA_READ)) 
				{ 
					retcode = getLong(din, templ);

					// seek over all cameras

					int numcameras = (int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);

					if (numcameras == -1) retcode = 515; // err in cfg: no cam found!
					int found = 0;

					for (int cam=0;cam<numcameras;cam++)
					{
						char cameratag[11];
						char camerasection[30];

						cameratag[0] = 0;
						camerasection[0] = 0;

						putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
						char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

						if (strcmp(camsect, dummy) == 0) continue;

						copywholestring(camerasection, camsect);

						char *rets1 = m_pCamIniFile->GetValue(camerasection, "IsOnMultiplexer", "FALSE");
						if (_stricmp(rets1, "true") == 0)
						{
							long retl2 = m_pCamIniFile->GetValueInt(camerasection, "MPInputPort", -1);

							if (retl2 == templ) 
							{
								char *tcameraid = m_pCamIniFile->GetValue(camerasection, "CamID", dummy);
								putCharString( dout, tcameraid );
								found = 1;
							}
						}
					}
					if (found == 0) retcode = 518; // no cam connected!
				}
				else
					retcode = illegal_read_write; 

				break;
		
		case PRP_MPU_SRVIDOUTPUTPORT:
				retcode = 0;
				if ((access & CA_WRITE) && (access & CA_READ)) 
				{ 
					retcode = getLong(din, templ);


					int numservers = (int) m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);

					if (numservers == -1) retcode = 514; // err in cfg: no srv found!
					int found = 0;

					for (int srv=0;srv<numservers;srv++)
					{
						char servertag[11];
						char serversection[30];

						servertag[0] = 0;
						serversection[0] = 0;

						putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
						char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

						if (strcmp(srvsect, dummy) == 0) continue;

						copywholestring(serversection, srvsect);

						char *rets1 = m_pCamIniFile->GetValue(serversection, "IsOnMultiplexer", "FALSE");
						if (_stricmp(rets1, "true") == 0)
						{
							long retl2 = m_pSrvIniFile->GetValueInt(serversection, "MPOutputPort", -1);

							if (retl2 == templ) 
							{
								char *tserverid = m_pSrvIniFile->GetValue(serversection, "ServerID", dummy);
								putCharString( dout, tserverid );
								found = 1;
							}
						}
					}
					if (found == 0) retcode = 519; // no cam connected!
				}
				else
					retcode = illegal_read_write; 

				break;

		case PRP_MPU_CAMIDONOUTPUTPORT:
				retcode = 0;
				if ((access & CA_WRITE) && (access & CA_READ)) 
				{ 
					retcode = getLong(din, templ);

					long temp2 = (long int) m_pMPSIniFile->GetValueInt("Multiplexer", "NumOutputPorts", -1);

					if (templ < 0 || templ > temp2)
					{
						retcode = 520; // bounds error
					}

					int mpin;

					int retv = getMultiplexerCameraOnOutputPort( tcp_sock, templ, mpin );
					if (retv == 1)
					{
						if (mpin == 0) { retcode = 525; break; } // no camera bound!

						int numcameras = (int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);

						if (numcameras == -1) retcode = 515; // err in cfg: no cam found!
						int found = 0;
	
						for (int cam=0;cam<numcameras;cam++)
						{
							char cameratag[11];
							char camerasection[30];

							cameratag[0] = 0;
							camerasection[0] = 0;

							putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
							char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

							if (strcmp(camsect, dummy) == 0) continue;

							copywholestring(camerasection, camsect);

							char *rets1 = m_pCamIniFile->GetValue(camerasection, "IsOnMultiplexer", "FALSE");
							if (_stricmp(rets1, "true") == 0)
							{
								long retl2 = m_pCamIniFile->GetValueInt(camerasection, "MPInputPort", -1);

								if (retl2 == mpin) 
								{
									char *tcameraid = m_pCamIniFile->GetValue(camerasection, "CamID", dummy);
									putCharString( dout, tcameraid );
									found = 1;
								}
							}
						}
						if (found == 0) retcode = 516; // cam was not found in cfg db

					}
					else retcode = 521; // MPU comm error!
				}
				else retcode = illegal_read_write;

				break;

		case PRP_MPU_NUMINPUTPORTS:
				retcode = 0;
				
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				templ = (long int) m_pMPSIniFile->GetValueInt("Multiplexer", "NumInputPorts", -1);
				putLong(dout, templ);
	
				break;
		
		case PRP_MPU_NUMOUTPUTPORTS:
				retcode = 0;
				if (access & CA_WRITE) 
				{ 
					retcode = illegal_read_write; 
					break; 
				}

				templ = (long int) m_pMPSIniFile->GetValueInt("Multiplexer", "NumOutputPorts", -1);
				putLong(dout, templ);
				
				break;

		default: 
				retcode = illegal_property;
				break;
	}

	if (retcode == 514) SetRPCCompletion("Servers.cfg malformed");
	else if (retcode == 515) SetRPCCompletion("Cameras.cfg malformed");
	else if (retcode == 518) SetRPCCompletion("there is no camera connected");
	else if (retcode == 519) SetRPCCompletion("there is no server connected");
	else if (retcode == 520) SetRPCCompletion("bounds exceeded");
	else if (retcode == 521) SetRPCCompletion("multiplexer unit communication error");
	else if (retcode == 525) SetRPCCompletion("No camera bound on this output port.");

	return retcode;
}


///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// REQUEST handlers END
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// Property query functions
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////


//SW! !!! Sept 08, 2005 - different devices with different properties
int XMultiplexerPropQueryFcn(char *devName,XPropertyQueryStruct **xpqs)
{
	long size;

	//printf("prop query fcn called: %s\n",devName);

    *xpqs = MultiplexerServerXQueryStruct;
	size = sizeof(MultiplexerServerXQueryStruct)/sizeof(XPropertyQueryStruct);

	if (_strnicmp(devName, devmpu, strlen(devmpu)) == 0)
	{
		//return MultiplexerXQueryStruct
		*xpqs = MultiplexerUnitXQueryStruct;
		size = sizeof(MultiplexerUnitXQueryStruct)/sizeof(XPropertyQueryStruct);
	}

	if (_strnicmp(devName, devsrv, strlen(devsrv)) == 0)
	{
		//return MultiplexerXQueryStruct
		*xpqs = ServerXQueryStruct;
		size = sizeof(ServerXQueryStruct)/sizeof(XPropertyQueryStruct);
	}

	if (_strnicmp(devName, devcam, strlen(devcam)) == 0)
	{
		//return MultiplexerXQueryStruct;
		*xpqs = CameraXQueryStruct;
		size = sizeof(CameraXQueryStruct)/sizeof(XPropertyQueryStruct);
	}

	return size;
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// Register properties
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////


int RegProp(char *propname, int size, short type, short access, char *desc, int id)
{
	int cc=0;

	if ((cc=RegisterPropertyEx(EQPMODNAME,propname,size,type,access,desc,id)) < 0)
	{
		feclog("Can't register property: <%s> (err=%d)\n",propname, -cc);
		return( 0 );
	}

	return( cc );
}


int RegPropFcn()
{
	int successful = 1;
	int i, a, b, c, d;
	XPropertyQueryStruct *tmp;

	a = sizeof(MultiplexerServerXQueryStruct)/sizeof(XPropertyQueryStruct);
	d = sizeof(MultiplexerUnitXQueryStruct)/sizeof(XPropertyQueryStruct);
	b = sizeof(ServerXQueryStruct)/sizeof(XPropertyQueryStruct);
	c = sizeof(CameraXQueryStruct)/sizeof(XPropertyQueryStruct);

	tmp = (XPropertyQueryStruct *) MultiplexerServerXQueryStruct;

	for (i=0;i<a;i++)
	{
		if ((tmp->reserved[2] = (unsigned short) RegProp(tmp->prpName, tmp->prpSize, tmp->prpFormat, tmp->prpAccess, tmp->prpDescription, tmp->reserved[1])) ==0) { successful = 0; return successful; }
		tmp++;
	}

	if (m_lMultiplexerDeviceNeeded == 1)
	{
		tmp = (XPropertyQueryStruct *) MultiplexerUnitXQueryStruct;
		for (i=0;i<d;i++)
		{
			if ((tmp->reserved[2] = (unsigned short) RegProp(tmp->prpName, tmp->prpSize, tmp->prpFormat, tmp->prpAccess, tmp->prpDescription, tmp->reserved[1])) ==0) { successful = 0; return successful; }
			tmp++;
		}
	}

	tmp = (XPropertyQueryStruct *) ServerXQueryStruct;
	for (i=0;i<b;i++)
	{
		if ((tmp->reserved[2] = (unsigned short) RegProp(tmp->prpName, tmp->prpSize, tmp->prpFormat, tmp->prpAccess, tmp->prpDescription, tmp->reserved[1])) ==0) { successful = 0; return successful; }
		tmp++;
	}

	tmp = (XPropertyQueryStruct *) CameraXQueryStruct;
	for (i=0;i<c;i++)
	{
		if ((tmp->reserved[2] = (unsigned short) RegProp(tmp->prpName, tmp->prpSize, tmp->prpFormat, tmp->prpAccess, tmp->prpDescription, tmp->reserved[1])) ==0) { successful = 0; return successful; }
		tmp++;
	}

	if (successful != 0) feclog("Registered all properties successful.");

	return successful;
}


int ValidityCheck()
{
	//does each section of server and cameras exist
	//are the settings reasonable (i.e. camera that has a dedicated server means that the 
	//  server is not connected to the multiplexer

	const int numorientations = 7;
	char orientations[numorientations][17]={"NORM","ROT90CW","ROT180CW","ROT270CW","MIRRORHORIZ","MIRRORVERT","ROT90CWMIRRHORIZ"};

	int maxlocalport = 3;
	int minlocalport = 0;
		
	// check multiplexerserver.cfg
	int cc =0;
	int tmp;
	char *tmps;
	char dummy[]="---dummy---";
	int numcameras;
	int numservers;
	int cam, srv;
	int numinputports;
	int numoutputports;

	m_lMultiplexerDeviceNeeded = 0;	

	tmp = m_pMPSIniFile->GetValueInt("Multiplexer", "NumInputPorts", -1);
	if (tmp == -1) ccerr(-1);
	numinputports = tmp;
	
	tmp = m_pMPSIniFile->GetValueInt("Multiplexer", "NumOutputPorts", -1);
	if (tmp == -1) ccerr(-2);
	numoutputports = tmp;

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "Context", "");
	if (strlen(tmps) == 0) ccerr(-3);

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "Server", "");
	if (strlen(tmps) == 0) ccerr(-4);

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "MultiplexerDevice", "");
	if (strlen(tmps) == 0) ccerr(-5);

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "");
	if (strlen(tmps) == 0) ccerr(-6);

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "");
	if (strlen(tmps) == 0) ccerr(-7);

    tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "MultiplexerTelnet", "");
	if (strlen(tmps) == 0) ccerr(-8);
	if ((_stricmp(tmps, "true") != 0) && (_stricmp(tmps, "false") != 0)) ccerr(-8);

	if (_stricmp(tmps, "true") == 0)
	{
		tmp = m_pMPSIniFile->GetValueInt("MultiplexerServer", "MultiplexerTelnetPort", -1);
		if (tmp == -1) ccerr(-9);

		tmps = m_pMPSIniFile->GetValue("MultiplexerServer", "MultiplexerTelnetServername", "");
		if (strlen(tmps) == 0) ccerr(-10);
	}

	// check servers.cfg
	// make sure whether all server sections exist

	numservers = (int) m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);
	if (numservers == -1) ccerr(-11);

	for (srv=0;srv<numservers;srv++)
	{
		char servertag[11]= "";
		char serversection[100];

		serversection[0] = 0;

		putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
		char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

		if (strcmp(srvsect, dummy) == 0) ccerr( -12 );
		
		copywholestring(serversection, srvsect);

		char *temp = m_pSrvIniFile->GetValue(serversection, "ServerID", "");
		if (strlen(temp) == 0) ccerr(-13);


		// reasonability checking
		// bounds checking
		// for each server check whether if IsOnMultiplexer==TRUE then MPOutputPort must have a reasonable value 

		temp = m_pSrvIniFile->GetValue(serversection, "IsOnMultiplexer", dummy);

		if (strcmp(temp,dummy) == 0) ccerr( -17 );

		if (_stricmp(temp, "true") == 0)
		{
			m_lMultiplexerDeviceNeeded = 1;
			int mpoutport = m_pSrvIniFile->GetValueInt(serversection, "MPOutputPort", -1);

			if ((mpoutport < 1) || (mpoutport > numoutputports))
			{
				ccerr(-18);
			}
		}
		else
		{
			if (_stricmp(temp, "false") != 0)
			{
				ccerr( -19 );
			}
		}
	}

	// check cameras.cfg
	// make sure whether all cameras exist
	numcameras = (int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
	if (numcameras == -1) ccerr(-14);

	for (cam=0;cam<numcameras;cam++)
	{
		char cameratag[11]= "";
		char camerasection[100];

		camerasection[0] = 0;

		putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
		char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

		if (strcmp(camsect, dummy) == 0) ccerr( -15 );

		copywholestring(camerasection, camsect);

		char *temp = m_pCamIniFile->GetValue(camerasection, "CamID", "");
		if (strlen(temp) == 0) ccerr(-16);


		// reasonability checking
		// bounds checking
		// for each camera check whether if IsOnMultiplexer==TRUE then MPInputPort must have a reasonable value 

		
		temp = m_pCamIniFile->GetValue(camerasection, "IsOnMultiplexer", dummy);

		if (strcmp(temp,dummy) == 0) ccerr( -20 );

		if (_stricmp(temp, "true") == 0)
		{
			m_lMultiplexerDeviceNeeded = 1;
			int mpinport = m_pCamIniFile->GetValueInt(camerasection, "MPInputPort", -1);

			if ((mpinport < 1) || (mpinport > numinputports))
			{
				ccerr(-21);
			}
		}
		else
		{
			if (_stricmp(temp, "false") != 0)
			{
				ccerr( -22 );
			}
			else
			{
				char dedservidbuf[40];
				char *dedservid = m_pCamIniFile->GetValue(camerasection, "DedicatedServerID", dummy);
				
				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ attention!!!

				if (strcmp(dedservid, dummy) == 0) ccerr( -26 );

				copywholestring(dedservidbuf, dedservid);

				int servfound = 0;

				for (srv=0;srv<numservers;srv++)
				{
					char servertag[11]= "";
					char serversection[100];

					serversection[0] = 0;

					putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
					char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

					if (strcmp(srvsect, dummy) == 0) ccerr( -27 );
					
					copywholestring(serversection, srvsect);

					char *temp = m_pSrvIniFile->GetValue(serversection, "ServerID", "");
					if (strcmp(temp, dedservidbuf) == 0) 
						servfound++;
				}
				
				if (servfound > 1) ccerr( -28 );
				if (servfound == 0) ccerr( -29 );

				//check "dedicated server id" whether it is reasonable (found inside servers.cfg, and server is not on multiplexer)
			}
		}


		temp = m_pCamIniFile->GetValue(camerasection, "Orientation", dummy);
		if (strcmp(temp,dummy) == 0) ccerr( -23 );
		
		int orientvalid = 0;
		for (int i=0;i<numorientations;i++)
		{
			if (strcmp(temp,orientations[i]) == 0) orientvalid = 1;
		}

		if (orientvalid == 0) ccerr(-24);

		int localport = m_pCamIniFile->GetValueInt(camerasection, "LocalPort", -1);

		if ((localport < minlocalport) || (localport > maxlocalport))
		{
			ccerr(-25);
		}
	}

	int  cnt;
	// check whether all ServerID are unique (inside Server.cfg)
	for (srv=0;srv<numservers;srv++)
	{
		char servertag[11]= "";
		char servertag2[11]= "";
		char serverid1[100];
		char TMPserversection[100];
				
		//TMPserversection[0] = 0;
		//serverid1[0] = 0;

		cnt = 0;
	
		putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
		char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

		if (strcmp(srvsect, dummy) != 0) 
		{
			copywholestring(TMPserversection, srvsect);

			char *t = m_pSrvIniFile->GetValue(TMPserversection, "ServerID", "");

			if (strcmp(t, "") != 0)
			{
				copywholestring(serverid1, t);
			
				for (int srv2=0;srv2<numservers;srv2++)
				{
					putLeadingZeroes(servertag2, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv2+1);

					char *ssect2tmp = m_pSrvIniFile->GetValue("Default", servertag2, dummy);
				
					if (strcmp(ssect2tmp, dummy) != 0)
					{
						copywholestring(TMPserversection, ssect2tmp);
						
						char *serverid2 = m_pSrvIniFile->GetValue(TMPserversection, "ServerID", "");

						if (strcmp(serverid1, serverid2) == 0) cnt++;
					}

				}
			}
		}

		if (cnt == 0) ccerr( -31 );
		if (cnt > 1) ccerr( -30 );
	}

	// check whether all camera ids are unique (pk value)!
	for (cam=0;cam<numcameras;cam++)
	{
		char cameratag[11]= "";
		char cameratag2[11]= "";
		char cameraid1[100];
		char TMPcamerasection[100];

		cnt = 0;
	
		putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
		char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

		if (strcmp(camsect, dummy) != 0) 
		{
			copywholestring(TMPcamerasection, camsect);

			char *t = m_pCamIniFile->GetValue(TMPcamerasection, "CamID", "");

			if (strcmp(t, "") != 0)
			{
				copywholestring(cameraid1, t);
			
				for (int cam2=0;cam2<numcameras;cam2++)
				{
					putLeadingZeroes(cameratag2, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam2+1);

					char *csect2tmp = m_pCamIniFile->GetValue("Default", cameratag2, dummy);
				
					if (strcmp(csect2tmp, dummy) != 0)
					{
						copywholestring(TMPcamerasection, csect2tmp);
						
						char *cameraid2 = m_pCamIniFile->GetValue(TMPcamerasection, "CamID", "");

						if (strcmp(cameraid1, cameraid2) == 0) cnt++;
					}

				}
			}
		}

		if (cnt == 0) ccerr( -33 );
		if (cnt > 1) ccerr( -32 );
	}

	feclog("--- SUCCESS: Validity check of configuration files succeeded.");

	if (m_lMultiplexerDeviceNeeded == 0)
	{
		feclog("----");
		feclog("By looking at the config files it was found out that");
		feclog("no camera or server is connected to a multiplexer device.");
		feclog("Due to that, all multiplexer device functionality was disabled.");
		feclog("----");
	}
	else
	{
		feclog("----");
		feclog("All multiplexer device functionality is enabled due to the");
		feclog("fact that there are cameras or servers connected to the");
		feclog("multiplexer device.");
		feclog("----");
	}

	return 1;

err:
	char errfirst[]="Validitycheck:";
	char erradd[]="not found in MultiplexerServer.cfg";
	char erradd1[]="not found or wrong setting in MultiplexerServer.cfg";
	
	char erraddsrv[]="not found or wrong in Servers.cfg";
	char erraddcam[]="not found or wrong in Cameras.cfg";

	char errinssrv[]="in >= 1 section inside Servers.cfg.";
	char errinscam[]="in >= 1 section inside Cameras.cfg.";
	
	switch(cc)
	{
		case -1: feclog("%s NumInputPorts %s", errfirst, erradd);
				 break;

		case -2: feclog("%s NumOutputPorts %s", errfirst, erradd);
				 break;

		case -3: feclog("%s Context %s", errfirst, erradd);
				 break;

		case -4: feclog("%s Server %s", errfirst, erradd);
				 break;

		case -5: feclog("%s MultiplexerDevice %s", errfirst, erradd);
				 break;

		case -6: feclog("%s ServerDevice %s", errfirst, erradd);
				 break;

		case -7: feclog("%s CameraDevice %s", errfirst, erradd);
				 break;

		case -8: feclog("%s MultiplexerTelnet %s", errfirst, erradd1);
				 break;

		case -9: feclog("%s MultiplexerTelnetPort %s", errfirst, erradd);
				 break;

		case -10:feclog("%s MultiplexerTelnetServerName %s", errfirst, erradd);
				 break;

		case -11:feclog("%s NumServers %s", errfirst, erraddsrv);
				 break;

		case -12:feclog("%s One of the servers was %s", errfirst, erraddsrv);
				 break;

		case -13:feclog("%s One of the server sections was %s", errfirst, erraddsrv);
				 break;
				 
		case -14:feclog("%s NumCameras %s", errfirst, erraddcam);
				 break;
		
		case -15:feclog("%s One of the cameras was %s", errfirst, erraddcam);
				 break;

		case -16:feclog("%s One of the camera sections was %s", errfirst, erraddcam);
				 break;

 		case -17:feclog("%s Mandatory entry IsOnMultiplexer is missin %s", errfirst, errinssrv);
				 break;

		case -18:feclog("%s MPOutputPort value is missing or out of bounds %s", errfirst, errinssrv);
				 break;

		case -19:feclog("%s String IsOnMultiplexer neither true nor false %s", errfirst, errinssrv);
				 break;

 		case -20:feclog("%s Mandatory string IsOnMultiplexer is missing %s", errfirst, errinscam);
				 break;

		case -21:feclog("%s MPInputPort is missing or out of bounds %s", errfirst, errinscam);
				 break;

		case -22:feclog("%s IsOnMultiplexer neither true nor false %s", errfirst, errinscam);
				 break;

		case -23:feclog("%s Mandatory string Orientation is missing %s", errfirst, errinscam);
				 break;

		case -24:feclog("%s Invalid Orientation string %s", errfirst, errinscam);
				 break;

		case -25:feclog("%s Mandatory Localport value is missing or out of bounds", errfirst);
				 feclog(" %s", errinscam);
				 break;

		case -26:feclog("%s Mandatory DedicatedServerID value (if IsOnMultiplexer=FALSE)", errfirst);
				 feclog(" is missing %s", errinscam);
				 break;

		case -27:feclog("%s One of the server sections (in [Default] of Servers.cfg) is", errfirst);
				 feclog(" %s", erraddsrv);
				 break;

		case -28:feclog("%s There seems to be more than one Server with a unique ServerID value.", errfirst);
				 break;

		case -29:feclog("%s At least one server that was referred by a", errfirst);
				 feclog("DedicatedServerID inside Cameras.cfg was not found.");
				 break;

		case -30:feclog("%s At least one ServerID value inside Servers.cfg is NOT unique !", errfirst);
				 break;

		case -31:feclog("%s Error inside program (-31). Please contact author.", errfirst);
				 break;

		case -32:feclog("%s At least one CamID value inside Cameras.cfg is NOT unique !", errfirst);
				 break;

		case -33:feclog("%s Error inside program (-33). Please contact author.", errfirst);
				 break;

		default: feclog("Validitycheck: Unknown error.");
				 break;
	}
	
	if (cc != 0) 
	{
		printf("=!=\nFATAL: Validity checking of cfg files failed (cc=%d)! Take a look at feclog error.\n=!=\n",cc);
		feclog("FATAL: Validity checking of cfg files failed! See error above.");
		exit(-1);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// Reprogram GrabSrv and Multiplexer functions
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

int AdjustServerWithoutMultiplexer( char *serversection, char *camerasection )
{
	static char servcontext[20];
	static char servname[20];
	//char dummy[]="---empty---";
	char *temp;
	int ret;
	long templ;
	double tempd;
	int cc=0;
	time_t start;
	time_t akt;

	servcontext[0] = 0;
	servname[0] = 0;

	temp = m_pSrvIniFile->GetValue(serversection, "TineContext", "");
	if (strcmp(temp, "") == 0) ccerr(-1);
	copywholestring(servcontext, temp);

	temp = m_pSrvIniFile->GetValue(serversection, "TineControlName", "");
	if (strcmp(temp, "") == 0) ccerr(-2);
	copywholestring(servname, temp);

	templ = m_pCamIniFile->GetValueInt(camerasection, "BPP", -1);
	if (templ == -1) ccerr(-3);
	ret = PutLongToServer(servcontext, servname, "C_BPP", templ);
    if (ret != 1) ccerr(-23);

	templ = m_pCamIniFile->GetValueInt(camerasection, "Height", -1);
	if (templ == -1) ccerr(-4);
	ret = PutLongToServer(servcontext, servname, "C_HEIGHT", templ);
	if (ret != 1) ccerr(-24);

	templ = m_pCamIniFile->GetValueInt(camerasection, "Width", -1);
	if (templ == -1) ccerr(-5);
	ret = PutLongToServer(servcontext, servname, "C_WIDTH", templ);
	if (ret != 1) ccerr(-25);
   
	templ = m_pCamIniFile->GetValueInt(camerasection, "NRef", -1);
	if (templ == -1) ccerr(-6);
	ret = PutLongToServer(servcontext, servname, "C_NREF", templ);
	if (ret != 1) ccerr(-26);

	templ = m_pCamIniFile->GetValueInt(camerasection, "PRef", -1);
	if (templ == -1) ccerr(-7);
	ret = PutLongToServer(servcontext, servname, "C_PREF", templ);
	if (ret != 1) ccerr(-27);

	templ = m_pCamIniFile->GetValueInt(camerasection, "LocalPort", -1);
	if (templ == -1) ccerr(-8);
	ret = PutLongToServer(servcontext, servname, "C_LOCALCAMPORT", templ);
	if (ret != 1) ccerr(-28);
	
	tempd = m_pCamIniFile->GetValueDouble(camerasection, "Scale", 10000.0);
	if ((tempd < 10000.1) && (tempd > 9999.9)) ccerr(-9);
	ret = PutDoubleToServer(servcontext, servname, "C_SCALE", tempd);
	if (ret != 1) ccerr(-29);

	temp = m_pCamIniFile->GetValue(camerasection, "Name", "");
	if (strcmp( temp, "") == 0) ccerr(-10);
	ret = PutStringToServer(servcontext, servname, "C_CAMNAME", temp);
	if (ret != 1) ccerr(-30);

	temp = m_pCamIniFile->GetValue(camerasection, "CamID", "");
	if (strcmp( temp, "") == 0) ccerr(-11);
	ret = PutStringToServer(servcontext, servname, "C_CAMID", temp);
	if (ret != 1) ccerr(-31);

	temp = m_pCamIniFile->GetValue(camerasection, "Orientation", "");
	if (strcmp( temp, "") == 0) ccerr(-12);
	ret = PutStringToServer(servcontext, servname, "C_ORIENTATION", temp);
	if (ret != 1) ccerr(-32);

	temp = m_pCamIniFile->GetValue(camerasection, "ExtTrig", "");
	if (_stricmp( temp, "true") == 0) templ = 1; else templ = 0;
	ret = PutLongToServer(servcontext, servname, "C_EXTTRIG", templ);
	if (ret != 1) ccerr(-33);

	temp = m_pCamIniFile->GetValue(camerasection, "TrigMod5", "");
	if (_stricmp( temp, "true") == 0) templ = 1; else templ = 0;
	ret = PutLongToServer(servcontext, servname, "C_TRIGMOD5", templ);
	if (ret != 1) ccerr(-34);

	ret = PutLongToServer(servcontext, servname, "S_UPDATE", 1);
	if (ret != 1) ccerr(-35);

	start = time(NULL);
	akt = start;

	cc = -38;

	while( akt < (start+15) )
	{
		ret = GetLongFromServer(servcontext, servname, "S_isUpdated", templ);
		//printf("----------------------------------Ret=%d   templ = %d\n", ret, templ);
		if ((ret == 1) && (templ == 1)) { cc = 0; break; }

		
		akt = time(NULL);
	}

	if (cc == 0)
	{
		m_pSrvIniFile->SetValue(serversection, "CurCamID", m_pCamIniFile->GetValue(camerasection, "CamID", "") );
		m_pSrvIniFile->SetValue(serversection, "CurCamName", m_pCamIniFile->GetValue(camerasection, "Name", "") );
		m_pSrvIniFile->WriteFile();

		m_iSomethingChanged++;
		SystemScheduleProperty(EQPMODNAME,CHANGEDSCHEDPROP);
	}
	else
	{
		feclog("AdjustServerWithoutMultiplexer: Timeout (15s) getting isUpdated == 1 from server.");
		feclog("Server could be in illegal state now!");

		m_iSomethingChanged++;
		SystemScheduleProperty(EQPMODNAME,CHANGEDSCHEDPROP);
	}


err:
	return( cc );
}



int AdjustMultiplexer( int sock, int mpin, int mpout)
{
	char cmd[20];
	timeval tmval;
	char returnbuf[10];
	char throwbuf[10];
	int ret, ret1;
	fd_set WRITE;
	fd_set READ;
	int numrecv;

	cmd[0] = 0;
	sprintf(cmd, "CONNECT %d %d%c", mpin, mpout, 13);

	FD_ZERO( &WRITE );
	FD_SET( sock, &WRITE);
	tmval.tv_sec = 2;
	tmval.tv_usec = 0;
	ret = select(1, NULL, &WRITE, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("AdjustMultiplexer: Waiting for socket to write ERROR = %d", ret);
		return 0;
	}

	ret1 = send(sock, cmd, strlen(cmd), 0);
	if (( ((unsigned)ret1) != strlen(cmd)) || (ret1 == SOCKET_ERROR))
	{
		feclog("AdjustMultiplexer: error: send() ret1=%d WSAErr=%d", ret1, WSAGetLastError() );
		return 0;
	}

		
	FD_ZERO( &READ );
	FD_SET( sock, &READ);
	tmval.tv_sec = 3;    // SW! Apr 18, 2006
	tmval.tv_usec = 0;

	ret = select(1, &READ, NULL, NULL, &tmval);
	if (ret <= 0)
	{
		feclog("AdjustMultiplexer: Waiting for socket to read back ERROR = %d\n", ret);
		return 0;
	}

	// receive until nothing comes in any more

	int left_read = 9;
	int finished = 0;
	char *startbuf = returnbuf;
	numrecv = 0;

	while (!finished)
	{
		ret1 = recv(sock, startbuf, left_read, 0);
		if (ret1 > 0)
		{
			left_read -= ret1;
			if (startbuf != throwbuf) numrecv += ret1;
			if (left_read < 0) 
			{
				//left_read = 0;
				startbuf = throwbuf;
				left_read = 9;
			}
		}
		else if (ret1 == 0)
		{
			feclog("AdjustMultiplexer: Telnet connection to multiplexer was closed unexpectedly. Err = %d", WSAGetLastError() );
			return 0;
		}
		else if (ret1 == SOCKET_ERROR)
		{
			if (WSAGetLastError() == WSAEWOULDBLOCK) finished = 1;
			else 
			{
				feclog("AdjustMultiplexer: Lost telnet connection to multiplexer. Err = %d", WSAGetLastError() );
				return 0;
			}
		}
		else
		{
			feclog("AdjustMultiplexer: Unknown error (recv telnet from multiplexer). Err = %d / %d", ret1, WSAGetLastError() );
			return 0;
		}

	}

	//printf("Returned %d bytes\n", numrecv);
	returnbuf[numrecv] = 0;

	if (_strnicmp(returnbuf, "ERROR", 5) == 0) 
	{
		feclog("AdjustMultiplexer: Multiplexer returned ERROR(!)\n");
		return 0;
	}

	if (_strnicmp(returnbuf, "OK", 2) != 0)
	{
		feclog("AdjustMultiplexer: Multiplexer did not return OK !!! (%s)\n", returnbuf);
		return 0;
	}


	cmd[0] = 0;
	sprintf(cmd, "UPDATE%c", 13);

	FD_ZERO( &WRITE );
	FD_SET( sock, &WRITE);
	tmval.tv_sec = 2;
	tmval.tv_usec = 0;

	ret = select(1, NULL, &WRITE, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("AdjustMultiplexer: Waiting for socket to write 2 ERROR = %d", ret);
		return 0;
	}

	ret1 = send(sock, cmd, strlen(cmd), 0);




	FD_ZERO( &READ );
	FD_SET( sock, &READ);
	tmval.tv_sec = 3; //SW! Apr 18, 2006
	tmval.tv_usec = 0;

	ret = select(1, &READ, NULL, NULL, &tmval);

	if (ret <= 0)
	{
		feclog("AdjustMultiplexer: Waiting for socket to read back ERROR = %d\n", ret);
		return 0;
	}

	// receive until nothing comes in any more

	left_read = 9;
	finished = 0;
	startbuf = returnbuf;
	numrecv = 0;

	while (!finished)
	{
		ret1 = recv(sock, startbuf, left_read, 0);
		if (ret1 > 0)
		{
			left_read -= ret1;
			if (startbuf != throwbuf) numrecv += ret1;
			if (left_read < 0)
			{
				//left_read = 0;
				startbuf = throwbuf;
				left_read = 9;
			}
		}
		else if (ret1 == 0)
		{
			feclog("AdjustMultiplexer: Telnet connection to multiplexer was closed unexpectedly. Err = %d", WSAGetLastError() );
			return 0;
		}
		else if (ret1 == SOCKET_ERROR)
		{
			if (WSAGetLastError() == WSAEWOULDBLOCK) finished = 1;
			else 
			{
				feclog("AdjustMultiplexer: Lost telnet connection to multiplexer. Err = %d", WSAGetLastError() );
				return 0;
			}
		}
		else
		{
			feclog("AdjustMultiplexer: Unknown error (recv telnet from multiplexer). Err = %d / %d", ret1, WSAGetLastError() );
			return 0;
		}

	}

	//printf("Returned %d bytes\n", numrecv);
	returnbuf[numrecv] = 0;

	if (_strnicmp(returnbuf, "ERROR", 5) == 0) 
	{
		feclog("AdjustMultiplexer: Multiplexer returned ERROR(!, (update))\n");
		return 0;
	}

	if (_strnicmp(returnbuf, "OK", 2) != 0)
	{
		feclog("AdjustMultiplexer: Multiplexer did not return OK (update)!!! (%s)\n", returnbuf);
		return 0;
	}

	//printf("---Multiplexer change OK!!!\n");
	feclog("-+- Multiplexer changed (inport %d routed to outport %d)", mpin, mpout);
	return 1; // success
}


int AdjustServerAndMultiplexer( char *serversection, char *camerasection, int mpinput, int mpoutput)
{
	static char servcontext[20];
	static char servname[20];
	char *temp;
	int ret;
	long templ;
	double tempd;
	int cc=0;
	time_t start;
	time_t akt;

	servcontext[0] = 0;
	servname[0] = 0;

	temp = m_pSrvIniFile->GetValue(serversection, "TineContext", "");
	if (strcmp(temp, "") == 0) ccerr(-1);
	copywholestring(servcontext, temp);

	temp = m_pSrvIniFile->GetValue(serversection, "TineControlName", "");
	if (strcmp(temp, "") == 0) ccerr(-2);
	copywholestring(servname, temp);

	templ = m_pCamIniFile->GetValueInt(camerasection, "BPP", -1);
	if (templ == -1) ccerr(-3);
	ret = PutLongToServer(servcontext, servname, "C_BPP", templ);
    if (ret != 1) ccerr(-23);

	templ = m_pCamIniFile->GetValueInt(camerasection, "Height", -1);
	if (templ == -1) ccerr(-4);
	ret = PutLongToServer(servcontext, servname, "C_HEIGHT", templ);
	if (ret != 1) ccerr(-24);

	templ = m_pCamIniFile->GetValueInt(camerasection, "Width", -1);
	if (templ == -1) ccerr(-5);
	ret = PutLongToServer(servcontext, servname, "C_WIDTH", templ);
	if (ret != 1) ccerr(-25);
   
	templ = m_pCamIniFile->GetValueInt(camerasection, "NRef", -1);
	if (templ == -1) ccerr(-6);
	ret = PutLongToServer(servcontext, servname, "C_NREF", templ);
	if (ret != 1) ccerr(-26);

	templ = m_pCamIniFile->GetValueInt(camerasection, "PRef", -1);
	if (templ == -1) ccerr(-7);
	ret = PutLongToServer(servcontext, servname, "C_PREF", templ);
	if (ret != 1) ccerr(-27);

	templ = m_pCamIniFile->GetValueInt(camerasection, "LocalPort", -1);
	if (templ == -1) ccerr(-8);
	ret = PutLongToServer(servcontext, servname, "C_LOCALCAMPORT", templ);
	if (ret != 1) ccerr(-28);
	
	tempd = m_pCamIniFile->GetValueDouble(camerasection, "Scale", 10000.0);
	if ((tempd < 10000.1) && (tempd > 9999.9)) ccerr(-9);
	ret = PutDoubleToServer(servcontext, servname, "C_SCALE", tempd);
	if (ret != 1) ccerr(-29);

	temp = m_pCamIniFile->GetValue(camerasection, "Name", "");
	if (strcmp( temp, "") == 0) ccerr(-10);
	ret = PutStringToServer(servcontext, servname, "C_CAMNAME", temp);
	if (ret != 1) ccerr(-30);

	temp = m_pCamIniFile->GetValue(camerasection, "CamID", "");
	if (strcmp( temp, "") == 0) ccerr(-11);
	ret = PutStringToServer(servcontext, servname, "C_CAMID", temp);
	if (ret != 1) ccerr(-31);

	temp = m_pCamIniFile->GetValue(camerasection, "Orientation", "");
	if (strcmp( temp, "") == 0) ccerr(-12);
	ret = PutStringToServer(servcontext, servname, "C_ORIENTATION", temp);
	if (ret != 1) ccerr(-32);

	temp = m_pCamIniFile->GetValue(camerasection, "ExtTrig", "");
	if (_stricmp( temp, "true") == 0) templ = 1; else templ = 0;
	ret = PutLongToServer(servcontext, servname, "C_EXTTRIG", templ);
	if (ret != 1) ccerr(-33);

	temp = m_pCamIniFile->GetValue(camerasection, "TrigMod5", "");
	if (_stricmp( temp, "true") == 0) templ = 1; else templ = 0;
	ret = PutLongToServer(servcontext, servname, "C_TRIGMOD5", templ);
	if (ret != 1) ccerr(-34);

	// stop grabserver
	ret = PutLongToServer(servcontext, servname, "S_STOP", 1);
	if (ret != 1) ccerr(-35);
	
	
	ret = AdjustMultiplexer( tcp_sock, mpinput, mpoutput);
	if (ret != 1) 
	{
		// switching of multiplexer failed, at least start grabbing without updating
		ret = PutLongToServer(servcontext, servname, "S_START", 1); // SW! no error checking here
		
		ccerr(-36);
	}
	
	//Sleep(1000); did not know why I had it in here, removed. SW! Apr 18, 2006

	// try it three times.
	ret = PutLongToServer(servcontext, servname, "S_UPDATE", 1);
	if (ret != 1) 
	{
		ret = PutLongToServer(servcontext, servname, "S_UPDATE", 1);
		if (ret != 1) 
		{
			ret = PutLongToServer(servcontext, servname, "S_UPDATE", 1);
			if (ret != 1) ccerr(-37);
		}
		
	}

	start = time(NULL);
	akt = time(NULL);

	cc = -38;

	while( akt < (start+15) )
	{
		ret = GetLongFromServer(servcontext, servname, "S_isUpdated", templ);

		if ((ret == 1) && (templ == 1)) { cc = 0; break; }
		
		akt = time(NULL);
	}

	if (cc == 0)
	{
		m_pSrvIniFile->SetValue(serversection, "CurCamID", m_pCamIniFile->GetValue(camerasection, "CamID", "") );
		m_pSrvIniFile->SetValue(serversection, "CurCamName", m_pCamIniFile->GetValue(camerasection, "Name", "") );
		m_pSrvIniFile->WriteFile();
		m_iSomethingChanged++;
		SystemScheduleProperty(EQPMODNAME,CHANGEDSCHEDPROP);
	}
	else
	{
		feclog("AdjustServerAndMultiplexer: Timeout (15s) getting isUpdated == 1 from server.");
		feclog("Server could be in illegal state now!");

		m_iSomethingChanged++;
		SystemScheduleProperty(EQPMODNAME,CHANGEDSCHEDPROP);
	}


err:
	feclog("AdjustServerAndMultiplexer: cc = %d", cc);

	return( cc );
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// TINE mps init, bkg, eqm START
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
void mpsinit(void)
{
	int i,n;
	char tempstr[100];

	time_t t = time(NULL);
	char ts[32];

	int err;

	sprintf(ts,ctime(&t));
  	feclog("-+- Multiplexer Server startup: the time is now %s (%d)\n",ts,t);

  	err = WSAStartup(0x0101,&wsaData);
	if ( err != 0 ) 
	{
		feclog("--- FATAL: Could not initialize Windows sockets 1.1.");
		feclog("--- FATAL: Maybe the dll ws2_32.dll is missing or could not be found.");

		printf("--- FATAL: Could not initialize Windows sockets 1.1.\n");
		printf("--- FATAL: Maybe the dll ws2_32.dll is missing or could not be found.\n");
   		
		exit(-5);
	}

	if (m_lMultiplexerDeviceNeeded == 1)
	{
		tcp_sock = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);

		int port = m_pMPSIniFile->GetValueInt("MultiplexerServer", "MultiplexerTelnetPort", -1);
		if (port == -1) 
		{
			feclog("--- FATAL: Could not get Multiplexer telnet port from cfg file!");
			printf("--- FATAL: Could not get Multiplexer telnet port from cfg file!\n");
			exit(-1);
		}

		char *servername = m_pMPSIniFile->GetValue("MultiplexerServer", "MultiplexerTelnetServername", "");

		if (strlen(servername) == 0)
		{
			feclog("--- FATAL: Could not get Multiplexer servername from cfg file !");
			printf("--- FATAL: Could not get Multiplexer servername from cfg file !\n");
			exit(-1);
		}

		feclog("Connecting to Multiplexer via Telnet: %s:%d...", servername, port);

		int ret = connect_socket(tcp_sock, servername, (unsigned short) port, 60); // try 60 seconds !

		if (ret != 1) 
		{
			feclog("--- FATAL: Could not connect to multiplexer via telnet. This is severe!!!");
			printf("--- FATAL: Could not connect to multiplexer via telnet. This is severe!!!\n");
		
			exit(-10);
		}

		feclog("Connected.");
	}

	//useMonthlyHistoryFiles = TRUE;
	//SystemSupressHeartbeatNotification = TRUE;

	n = sizeof(CameraXQueryStruct)/sizeof(XPropertyQueryStruct);

	if ((i=RegisterXPropertyQueryFunction(EQPMODNAME,XMultiplexerPropQueryFcn,n)) != 0)
	{
		feclog("--- FATAL: RegisterPropertyQueryFunction : %s",erlst[i]);
		printf("--- FATAL: RegisterPropertyQueryFunction : %s\n",erlst[i]);
		exit(-3);
	}

	int numcams = m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
	if (numcams == -1) 
	{
		feclog("--- FATAL: Camera configuration file does not contain proper \"NumCameras\" entry!");
		printf("--- FATAL: Camera configuration file does not contain proper \"NumCameras\" entry!\n");
		exit(-1);
	}

	int numservs = m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);
	if (numservs == -1) 
	{
		feclog("--- FATAL: Server configuration file does not contain proper \"NumServers\" entry!");
		printf("--- FATAL: Server configuration file does not contain proper \"NumServers\" entry!\n");
		exit(-1);
	}

	int modulenumber = 0;

	// register device names (more to come) 
	RegisterModuleName(EQPMODNAME,(char *) devmps,modulenumber++);
	if (m_lMultiplexerDeviceNeeded == 1) RegisterModuleName(EQPMODNAME,(char *) devmpu,modulenumber++);
	
	for (i=0;i<numcams;i++)
	{
		putLeadingZeroes(tempstr, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), i+1);
		
		RegisterModuleName(EQPMODNAME,tempstr,modulenumber++);
	}
	
	for (i=0;i<numservs;i++)
	{
		putLeadingZeroes(tempstr, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), i+1);

		RegisterModuleName(EQPMODNAME,tempstr,modulenumber++);
	}

    if (RegPropFcn() == 0) 
	{
		feclog("--- FATAL: Error on registering properties.");
		printf("--- FATAL: Error on registering properties.\n");

		exit(-3);
	}
}


void mpsexit(void)
{
	m_pMPSIniFile->WriteFile();
	m_pSrvIniFile->WriteFile();
	m_pCamIniFile->WriteFile();

	WSACleanup();
}

void mpsbkg(void)
{
	int cc = 0;
	char dummy[]="---empty---";
	static char serversection[100]="";
	static char camerasection[100]="";

	static char serverid[12];
	static char cameraid[12];

	int serverfound = 0;
	int camerafound = 0;
	int config;
	
	int cam;
	int srv;
	int srvmplex;
	int cammplex;
	int ret;
	char *temp;

	if (selectOngoing == 0) 
	{
		if (m_lMultiplexerDeviceNeeded == 1)
		{

			if (PingMultiplexer(tcp_sock) == 0)  
			{
				feclog("mpsbkg: Lost connection to multiplexer!\n");
				feclog(" Trying to restore connection now.\n");

				int port = m_pMPSIniFile->GetValueInt("MultiplexerServer", "MultiplexerTelnetPort", -1);
				char *servername = m_pMPSIniFile->GetValue("MultiplexerServer", "MultiplexerTelnetServername", "");

				if ((port == -1) || strlen(servername) == 0)
				{
					feclog(" Could not get valid TelnetServername and TelnetPort from configuration file. Abort.");
					return;
				}

				if (tcp_sock != INVALID_SOCKET)
				{
					ret = closesocket(tcp_sock);
					if (ret != 0) feclog(" Error on closesocket: ret=%d WSAerr=%d\n", ret, WSAGetLastError() );
				}

				tcp_sock = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
				if (tcp_sock != INVALID_SOCKET)
				{
					ret = connect_socket(tcp_sock, servername, (unsigned short) port, 1);

					if (ret != 1) 
					{
						feclog(" Connection to Multiplexer failed. Abort.\n");
						closesocket(tcp_sock); 
						tcp_sock=INVALID_SOCKET;
						return;
					}

					feclog(" (Re-)Connected.");
				}
				else
				{ // error on socket
					feclog(" Error on socket(): tcp_sock=%d WSAerr=%d\n", tcp_sock, WSAGetLastError() );
					tcp_sock = INVALID_SOCKET;
				}
			}
			
		}
		return;
	}

	serversection[0] = 0;
	camerasection[0] = 0;

	serverid[0] = 0;
	cameraid[0] = 0;

	// put camera on server
	// adjust multiplexer if necessary

	int numcameras = (int) m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
	int numservers = (int) m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);

	if ((numcameras == -1) || (numservers == -1)) ccerr(-1);

	for (srv=0;srv<numservers;srv++)
	{
		char servertag[11]= "";

		putLeadingZeroes(servertag, m_pMPSIniFile->GetValue("MultiplexerServer", "ServerDevice", "Server%d"), srv+1);
		char *srvsect = m_pSrvIniFile->GetValue("Default", servertag, dummy);

		if (strcmp(srvsect, dummy) == 0) ccerr( -2 );

		copywholestring(serversection, srvsect);

		char *tserverid = m_pSrvIniFile->GetValue(serversection, "ServerID", dummy);

		if (strcmp(tserverid, newserverid) == 0) { copywholestring(serverid, tserverid); serverfound = 1; break; }
	}

	for (cam=0;cam<numcameras;cam++)
	{
		char cameratag[11]= "";

		putLeadingZeroes(cameratag, m_pMPSIniFile->GetValue("MultiplexerServer", "CameraDevice", "Camera%d"), cam+1);
		char *camsect = m_pCamIniFile->GetValue("Default", cameratag, dummy);

		if (strcmp(camsect, dummy) == 0) ccerr( -3 );

		copywholestring(camerasection, camsect);

		char *tcameraid = m_pCamIniFile->GetValue(camerasection, "CamID", dummy);

		if (strcmp(tcameraid, newcamid) == 0) { copywholestring( cameraid, tcameraid); camerafound = 1; break; }
	}

	if (serverfound == 0 || camerafound == 0) ccerr( -4 );

	// verify whether camera is either directly connected to server or
	// both, server and camera are connected to the multiplexer

	temp = m_pSrvIniFile->GetValue(serversection, "IsOnMultiplexer", "");
	if (strcmp(temp, "") == 0) ccerr(-7);
	if (_stricmp(temp, "true") ==0) srvmplex = 1; else srvmplex = 0;

	temp = m_pCamIniFile->GetValue(camerasection, "IsOnMultiplexer", "");
	if (strcmp(temp, "") == 0) ccerr(-8);
	if (_stricmp(temp, "true") == 0) cammplex = 1; else cammplex = 0;

	temp = m_pCamIniFile->GetValue(camerasection, "DedicatedServerID", "");
	if (strcmp(temp, "") == 0) ccerr(-9);

	config = 0;

	if (_stricmp(temp, serverid) == 0) config = 1;
	else if (srvmplex == 1 && cammplex == 1) config = 2;

	if (config == 0) ccerr(-10);
	if ((config == 2) && (m_lMultiplexerDeviceNeeded == 0)) ccerr(-566);

	ret = 0;

	if (config == 1)
	{
		ret = AdjustServerWithoutMultiplexer( serversection, camerasection );
		if (ret != 0) ccerr(ret);
	}
	else if (config == 2)
	{
		int mpinputport = m_pCamIniFile->GetValueInt(camerasection, "MPInputPort", -1);
		if (mpinputport == -1) ccerr(-11);

		int mpoutputport = m_pSrvIniFile->GetValueInt(serversection, "MPOutputPort", -1);
		if (mpoutputport == -1) ccerr(-11);

	    ret = AdjustServerAndMultiplexer( serversection, camerasection, mpinputport, mpoutputport);
		if (ret != 0) ccerr(ret);
	}

	selectOngoing = 0;
	m_lSelectFinishedSuccessfully = 1;
	return;

err:
	m_lSelectFinishedSuccessfully = cc;
	selectOngoing = 0;

	return;
}

int mpseqm(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access)
{
	int prpid;
	//int cc;

	int ecode = 0;

	//printf("Equipment function called: devName=%s, devProperty=%s, access=%d\n", devName, devProperty, access);

	prpid = GetOwnPropertyId(devName,devProperty);

	//printf("Propertyid = %d\n", prpid);

	if (_strnicmp(devName, devsrv, strlen(devsrv)) == 0)
	{
		char server[5];
		memset(server, 0, 5);

		Right(server, devName, 3);
		int servnum = atoi(server);

		if (servnum > 0 && servnum < 1000) 
			ecode = HandleServerRequest( servnum, devName, prpid, dout, din, access  );

	}

	if (_strnicmp(devName, devcam, strlen(devcam)) == 0)
	{
		char camera[5];
		memset(camera, 0, 5);

		Right(camera, devName, 3);
		int camnum = atoi(camera);

		if (camnum > 0 && camnum < 1000) 
			ecode = HandleCameraRequest( camnum, devName, prpid, dout, din, access  );
	}

	if (_strnicmp(devName, devmps, strlen(devmps)) == 0)
	{
		ecode = HandleMultiplexerServerRequest( prpid, dout, din, access );
	}

	if (_strnicmp(devName, devmpu, strlen(devmpu)) == 0)
	{
		ecode = HandleMultiplexerUnitRequest( prpid, dout, din, access );
	}

	return ecode;
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// TINE mps init, bkg, eqm, exit END
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
//
// TINE Pre, PostSystemInit
//
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void PreSystemInit(void)
{
	// all feclog output here does not go into the proper dir+file (Env: FEC_LOG) but into
	// the program directory instead

	m_iSomethingChanged = 0;

	m_pMPSIniFile = new CIniEx();
	m_pSrvIniFile = new CIniEx();
	m_pCamIniFile = new CIniEx();

	if ((m_pMPSIniFile == NULL) || (m_pSrvIniFile == NULL) || ( m_pCamIniFile == NULL))
	{
		feclog("FATAL: At least one of the three configuration file classes could not be instantiated !");
		printf("FATAL: At least one of the three configuration file classes could not be instantiated !\n");
		exit(-1);
	}

	// !!! no error handling !!!
	if (m_pMPSIniFile->OpenAtExeDirectory( "MultiplexerServer.cfg" ) != TRUE) // open at the executable's directory 
	{
		feclog("FATAL: MultiplexerServer.cfg could not be opened or properly read in !");
		printf("FATAL: MultiplexerServer.cfg could not be opened or properly read in !\n");
		exit(-1);
	}

	if (m_pSrvIniFile->OpenAtExeDirectory( "Servers.cfg" ) != TRUE)  // open at the executable's directory 
	{
		feclog("FATAL: Servers.cfg could not be opened or properly read in !");
		printf("FATAL: Servers.cfg could not be opened or properly read in !\n");
		exit(-1);
	}

	if (m_pCamIniFile->OpenAtExeDirectory( "Cameras.cfg" ) != TRUE)  // open at the executable's directory 
	{
		feclog("FATAL: Cameras.cfg could not be opened or properly read in !");
		printf("FATAL: Cameras.cfg could not be opened or properly read in !\n");
		exit(-1);
	}

	ValidityCheck(); // adjusts also m_lMultiplexerDeviceNeeded to 1 if it is needed

	m_lSelectFinishedSuccessfully = 1;

	char strbuf[32];
	copywholestring(strbuf, m_pMPSIniFile->GetValue("MultiplexerServer", "Server", "VIDMUX"));
	addstr(strbuf, "FEC");

	RegisterFecNameEx(strbuf, "pitz video (multiplexer) server", "", "", "", "sweisse", 5, m_pMPSIniFile->GetValue("MultiplexerServer", "Context", "PITZ"));
}

/*
   PostSystemInit() is called in rpcserv.c, unless you have a good reason to do
   otherwise, you should leave rpcserv.c alone and make sure you have a
   function doinit of the following prototype.  This contains your own
   user specific initialization, such as the registration of equipment
   functions.
*/
void PostSystemInit(void)
{
	putCommandsInFeclog = TRUE;

	// SW! May 09, 2006: no error checking here, because ValidityCheck already showed that the parameters are alright
	int numcams = m_pCamIniFile->GetValueInt("Default", "NumCameras", -1);
	int numservs = m_pSrvIniFile->GetValueInt("Default", "NumServers", -1);

	int numdev = numcams+numservs+2;
	if (m_lMultiplexerDeviceNeeded == 0) numdev--;

	RegisterEquipmentModule(m_pMPSIniFile->GetValue("MultiplexerServer", "Server", "VIDMUX"),EQPMODNAME,numdev,mpseqm,mpsinit,mpsbkg,1000,mpsexit);

	MinPollingRate = 100;

#if defined(MSDOS) || defined(UNIX) || defined(WINNT)
  //RegisterUserCommand("stresstest",stest,NULL,NULL,CA_READ);
  //RegisterUserCommand("sinemode",NULL,(int *)&dataMode,NULL,CA_READ);
  //RegisterUserCommand("schedule",NULL,(int *)&scheduleIt,NULL,CA_READ|CA_WRITE);
  //RegisterUserFunction("testit",testit);

# ifdef CLIENT
  //RegisterUserCommand("hpenergy",NULL,NULL,&hpenergy,CA_READ);
# endif

	time(&m_ServerStartTime);
	
	selectOngoing = 0;
	newcamid[0] = 0;
	newserverid[0] = 0;
	newlock = 0;

#endif
}

