#include "platform.h"

// platform-specific include (required for ioctl function)
#include <sys/ioctl.h>

// constants required by source code
const	INT32	SOCKET_ERROR	= -1;
const	INT32	INVALID_SOCKET	= -1;

// sleep (pause thread execution for 'msec' milliseconds)
// less milliseconds than timeslice of system is not meaningful (e.g. Windows ~10ms)
VOID sleep_milliseconds( UINT32 msec )
{
	usleep( msec*1000 );
}

// remap winsock error code to a more general (useful on any platform) error code
INT32 getlasterror()
{
	// 4 linux:
	// pretty straigtforward implementation of getlasterror, as the error codes
	// on the socket interface are mapped to linux error codes
	
	return( errno );
}
// create a socket for use with TCP or UDP wrapper
socktype create_inet_socket( INT32 type )
{
	return( socket( AF_INET, type, 0 ) );
}

// connect a previously created socket to the remote endpoint defined via Saddr 
INT32 connect_socket(socktype s, const struct sockaddr * Saddr, size_t size_sockaddr)
{	
	return( connect( s, Saddr, size_sockaddr) );
}

// close or cleanup a socket (connected, but must work on unconnected sockets, too)
INT32	close_socket( socktype s )
{
	shutdown(s, SHUT_WR);

	return( close( s ) );
}

// wrapper for receive data on a connected socket
INT32 receive_socket_function( socktype s, UCHAR *buf, UINT32 buflen)
{
	return( recv(s, (CHAR *) buf, (INT32) buflen, 0) );
}

//
// wrapper in order to put the socket to nonblocking mode 
// (so that the inner loop of grabframes() works properly)
//
// 0 == success
// or SOCKET_ERROR on error
//
INT32 set_socket_nonblocking ( socktype s )
{
	unsigned int nonblock = 1;
	
	return( ioctl( s, FIONBIO, &nonblock) );
}
