// getsequence: acquire a frame sequence from PITZ Video System


#include "platform.h"
#include "videofileload.h"
#include "getframes.h"
#include "significancemap.h"


int main(int argc, char* argv[])
{
	/*struct VIDINFHDR vinf;
	INT32 ret;
	WSADATA wsad;

	WSAStartup( 0x0202, &wsad);

	initialize_vidinfhdr( &vinf );

	ret = grabframes( "znppitzfg2", 31777, 10, &vinf);

	if (ret == 0)
	{
		printf("Image data successfully grabbed.\n");
		ret = convert_format(&vinf, TYPE_IMC, FCN_NONE);
		if (ret == 0)
		{
			ret = save_file( TYPE_CHOOSE, "grabbed", &vinf); 
			if (ret == 0) 
				printf("Image data saved.\n");
			else
				printf("Error on save image data. ecode=%d\n", ret);
		}
		else printf("Error on converting image data to BKG file. ecode = %d\n", ret);
	}
	else
		printf("Error while grabbing image data. ecode=%d\n",ret);

	WSACleanup();*/
	

	struct VIDINFHDR signal, background, out;
	INT32 ret;

	if (argc != 4) 
	{
		printf("\nCould not understand commandline arguments.\n");
		printf("Usage: getsequence signal.imc background.bkc sigmask.imm\n");
		printf("\nIf there will be errors you will be notified.\n");

		exit(1);
	}

	
	initialize_vidinfhdr( &signal );
	initialize_vidinfhdr( &background );
	initialize_vidinfhdr( &out );

	ret = load_imc_file(argv[1], &signal);

	if (ret != 0) 
	{
		printf("Error loading signal file \"%s\". Errorcode was %d.\n", argv[1], ret);
		exit(2);
	}

	ret = load_bkc_file(argv[2], &background);
	if (ret != 0)
	{
		printf("Error loading background file \"%s\". Errorcode was %d.\n", argv[2], ret);
		free_vidinfhdr(&signal);

		exit(2);
	}

	ret = calculate_significance_map( &signal, &background, &out, 5.0);
	if (ret != 0)
	{
		printf("Error calculating significance map. Errorcode was %d.\n", ret);
		free_vidinfhdr(&background);
		free_vidinfhdr(&signal);
		
		exit(2);
	}

	out.type = TYPE_BKG;

	{
		UCHAR *data = out.framebits[0];
		UINT32 size_pixels = out.width*out.height;
		UINT32 k;

		for (k=0;k<size_pixels;k++)
		{
			if (data[k] > 0) data[k] = 255;
		}
	}

	ret = save_bkg_file(argv[3], &out);
	if (ret != 0)
	{
		printf("Error saving significance map to \"%s\". Errorcode was %d.\n", argv[3], ret);
		free_vidinfhdr(&out);
		free_vidinfhdr(&background);
		free_vidinfhdr(&signal);
		
		exit(2);
	}
	
	free_vidinfhdr( &out );
	free_vidinfhdr( &background);
	free_vidinfhdr( &signal);

	printf("Success.\n");
	
	return 0;
}

