// BogusTineDlg.cpp : implementation file
//

#include "stdafx.h"
#include "videoclnt.h"
#include "Queue.h"
class CVideoclntDlg;
#include "Compression.h"
#include "BogusTineDlg.h"
#include "headers.h"
#include "MemoryManagement.h"
#include "resource.h"
#include "Server.h"
#include "SSyncSocket.h"
#include "IniEx.h"
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "ImageAcquire.h"
#include "ImageSequence.h"
#include "AdditionalInformation.h"
#include "StatFile.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "PrinterSettings.h"
#include "MarkedComboBox.h"
#include "AboutDialog.h"
#include "InformationDialog.h"
#include "videoclntDlg.h"



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBogusTineDlg dialog
//
// This class encapsulates the polling and grabbing of video frames using
// TINE multicast mode.
//

/////////////////////////////////////////////////////////////////////////////
//
// TINE function calls inside eqp32.dll, needed for proper initialization of
// multicast mode.
//

extern "C" {

// receive buffer space on multicast socket
__declspec(dllexport) short PASCAL RPCAssignBufferSpace(int bufferSpace);

// switch synchronous notification on and off
// syncNotify must be TRUE for proper multicast operation
__declspec(dllexport) short PASCAL RPCSynchronousNotification(int syncNotify);

}

/////////////////////////////////////////////////////////////////////////////
//
// Construction of CBogusTineDlg
//
// Initialize the TINE dll and set boolean values so that grab and 
// poll mode are switched off
//
CBogusTineDlg::CBogusTineDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBogusTineDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBogusTineDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

    // initialize tine
    RPCAssignBufferSpace(0x300000);
    RPCSynchronousNotification(TRUE);
	
	// no grab transfer is running
	m_bTineGrabTransferRunning = FALSE;

	// no poll transfer is running
	m_bTinePollTransferRunning = FALSE;
}


void CBogusTineDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBogusTineDlg)
	DDX_Control(pDX, IDC_ACOPCTRL1_FRAMEDATA, m_CACOPFrameData);
	DDX_Control(pDX, IDC_ACOPCTRL2_CONTROL, m_CACOPControl);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBogusTineDlg, CDialog)
	//{{AFX_MSG_MAP(CBogusTineDlg)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBogusTineDlg message handlers

BEGIN_EVENTSINK_MAP(CBogusTineDlg, CDialog)
    //{{AFX_EVENTSINK_MAP(CBogusTineDlg)
	ON_EVENT(CBogusTineDlg, IDC_ACOPCTRL1_FRAMEDATA, 1 /* Receive */, OnReceiveAcopctrl1Framedata, VTS_I4 VTS_I4)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

/////////////////////////////////////////////////////////////////////////////
//
// OnReceiveAcopctrl1Framedata - equipment function (called via TINE ActiveX 
// control)
//
void CBogusTineDlg::OnReceiveAcopctrl1Framedata(long LinkIndex, long StatusCode) 
{
  // needed to count bad (error) status codes in poll mode
  static int count_bad_status=0;
  
  if (m_lReceiveTineLink == LinkIndex) // poll link response ? 
  {
    if (StatusCode == 0 || StatusCode == 607)
	{
	 count_bad_status = 0; // valid status code, so set bad status to zero
	 if (StatusCode == 0) // frame was send back ? 
	 {
      // acquire frame from tine memory block location, decompress it 
	  // if necessary and put it into the receive queue

      // read out video header 
	  VIDHDR vidH;
	  memman.memcpysw( (char *) &vidH, m_pTineData, sizeof(VIDHDR) );

      if (vidH.kennung == 0x11223344) // okay, it is a valid frame
      {
	    // check whether the poll queue has grown too big
		int size = m_pPollQueue->GetSize();
	    if (size > 10) // too big ? 
	    {
		    // clear 10 frames and free up the occupied memory
			for (int l=0;l<10;l++)
		    {
		  	    char *tmpptr = (char *) m_pPollQueue->RemoveHead();
			    if (tmpptr != NULL) memman.sw_free( (void *) tmpptr );
            }
	    }

        // allocate memory block for decompressed video frame
        char *memblock = (char *) memman.sw_malloc( (vidH.heightframe*vidH.widthframe*vidH.bpp/8)+sizeof(VIDHDR) );
		char *destptr = memblock;
		
        destptr += sizeof(VIDHDR);

        if (memblock != NULL) // memory block could be allocated ? 
		{
 	     char *ptr_fmthdr=NULL;
	     char *ptr_frame=NULL;

  	     if (vidH.compressed == 1) // frame compressed ? 
	     {
           // build up valid pointers
	       ptr_fmthdr = m_pTineData + sizeof(VIDHDR);
           ptr_frame = m_pTineData + sizeof(VIDHDR) + vidH.len_fmthdr;
 
   	       // decompress the frame
           m_pCompression->DecompressFrame( (LPBITMAPINFOHEADER) ptr_fmthdr, ptr_frame, m_pOutHdr, destptr);
	     } 
         else // frame is uncompressed, so just copy the frame data
	     {
           ptr_frame = m_pTineData + sizeof(VIDHDR);
           memman.memcpysw(destptr, ptr_frame, vidH.heightframe*vidH.widthframe*vidH.bpp/8 );
	     }
 	     
		 vidH.compressed = 0; // frame is not compressed any more
		 
		 if (vidH.bpp == 8) vidH.bpp_effective = 8; // SW! Sept 20, 2006 workaround for old servers!
		 
		 // write back video header
		 memman.memcpysw( (char *) memblock, (char *) &vidH, sizeof(VIDHDR) );

		 // put last acquired frame into queue
  	     m_pPollQueue->AddTail( (void *) memblock );
	     
		 // post notify message
	     AfxGetMainWnd()->PostMessage(ID_MSG_FRAMEARRIVED, 0, 0);
	    }// if (memblock != NULL)
      } // kennung == 0x11223344
	 } // == 0
    } // == 0 || == 607
	else
	{ 
	  // bad status code

	  // increment bad_status integer
      count_bad_status++;

	  //CString msg7;
	  //msg7.Format("Bad statuscode = %d", StatusCode );
	  //AfxMessageBox(msg7);

	  // 3 subsequent bad status codes, so post an error message 
	  if (count_bad_status == 3) AfxGetMainWnd()->PostMessage(ID_MSG_SOERR, (int) StatusCode, 1); // 1= TINE Error
	}
  } // link
  else if (m_lGrabTineLink == LinkIndex) // grab link response ?
  {
   // "good" status code and grabbing not yet finished ? 
   if ((StatusCode == 0 || StatusCode == 607) && (m_bGrabFinished != TRUE) )
   {
	 // last two request resulted in no frame sent back ?
	 // -> increment no frame integer or decrement, if one of the status codes was 
	 // valid
	 if (m_lLastStatusCode == 607 && StatusCode == 607) m_iNoFrame++; else m_iNoFrame = 0;
	 m_lLastStatusCode = StatusCode;

	 if (m_iNoFrame<40) // not yet 40 subsequent no_new_frame (607) messages
	 {
	  if (m_iFramesGrabbed<m_iGrabNumFrames) // grabbed_frames less than number of wanted frames?
	  {
       if (StatusCode == 0) // statuscode == success ? 
	   {
         // extract video header out of frame memory block
	     VIDHDR vidH;
	     memman.memcpysw( (char *) &vidH, m_pTineGrabData, sizeof(VIDHDR) );

         if (vidH.kennung == 0x11223344) // okay, it is a valid frame
         {
           // allocate memory needed to store frame data
		   char *memblock = (char *) memman.sw_malloc( (vidH.heightframe*vidH.widthframe*vidH.bpp/8)+sizeof(VIDHDR) );
		   char *destptr = memblock;
		
           destptr += sizeof(VIDHDR);

           if (memblock != NULL) // memory could be allocated ? 
		   {
 	        char *ptr_fmthdr=NULL;
	        char *ptr_frame=NULL;

  	        if (vidH.compressed == 1) // frame compressed ? 
	        {
              // build up valid pointers
	          ptr_fmthdr = m_pTineGrabData + sizeof(VIDHDR);
              ptr_frame = m_pTineGrabData + sizeof(VIDHDR) + vidH.len_fmthdr;
 
   	          // decompress it
              m_pCompression->DecompressFrame( (LPBITMAPINFOHEADER) ptr_fmthdr, ptr_frame, m_pOutHdr, destptr);
	        } 
            else // frame is uncompressed
	        {
              // just copy the frame data
			  ptr_frame = m_pTineGrabData + sizeof(VIDHDR);
              memman.memcpysw(destptr, ptr_frame, vidH.heightframe*vidH.widthframe*vidH.bpp/8 );
	        }
 	     
		    // frame is not compressed any more
		    vidH.compressed = 0;

			if (vidH.bpp == 8) vidH.bpp_effective = 8; // SW! Sept 20, 2006 workaround for old servers!
		    
			// write back video header
			memman.memcpysw( (char *) memblock, (char *) &vidH, sizeof(VIDHDR) );
	  
	        // put last acquired frame into queue
  	        m_pGrabQueue->AddTail( (void *) memblock );
		    
			// increment number of grabbed frames
			m_iFramesGrabbed++;

            // post notify message
			AfxGetMainWnd()->PostMessage(ID_MSG_FRAMEGRABBED, m_iFramesGrabbed, m_iGrabNumFrames );
           }//memblock != NULL
         }//vidH.kennung
       }// StatusCode == 0
	  } // framesgrabbed<grabnumframes
	  else
	  {
	   // all frames have been grabbed
       AfxGetMainWnd()->PostMessage(ID_MSG_GRABFINISHED, 0, 0); // grabbing has finished
	   // indicate finished grabbing
	   m_bGrabFinished = TRUE;
	  }
	 }
	 else
	 { // waited more than 2 seconds and no frame came in
       AfxGetMainWnd()->PostMessage(ID_MSG_SOERRGRAB, (int) StatusCode, 2); // 2= TIMEOUT
	   // indicate finished grabbing
	   m_bGrabFinished = TRUE;
	 }
   }
    else
   {
       // bad tine status code or grabbing has already finished, so post 
	   // error message
	   AfxGetMainWnd()->PostMessage(ID_MSG_SOERRGRAB, (int) StatusCode, 1); // 1= TINE ERROR 
	   // indicate finished grabbing
	   m_bGrabFinished = TRUE;
   }
  }
   else
  { // wrong link code
 
   AfxGetMainWnd()->PostMessage(ID_MSG_SOERRGRAB, (int) StatusCode, 1); // 1= TINE Error
   // indicate finished grabbing
   m_bGrabFinished = TRUE;
  }
}

/////////////////////////////////////////////////////////////////////////////
//
// StartTinePollTransfer - starts up polling of video frames using TINE multicast
//
// Starts up a TINE poll transfer of the video frames. Frame are put into the 
// pollqueue and for each frame a message to the main application message queue 
// is posted.
//
// Parameters:
//   context/server/device/property - Address of the TINE frame server
//   servernormal - normal server name of the grabserver (server - 'F')
//   pollqueue - queue to hold the received frames
//
// Return value:
//   TRUE  : polling is started up
//   FALSE : error while starting up poll mode
//
BOOL CBogusTineDlg::StartTinePollTransfer(CString context, CString server, CString device, CString property, CString servernormal, CQueue *pollqueue, int maxbuffersize, CString pollmode)
{
 // return value of the function
 BOOL retval = TRUE;
 
 // put the pollqueue reference into the proper variable
 m_pPollQueue = pollqueue;

 //BOOL ret = FALSE;
 
 // set up the context/server/device/property adress of the 
 // TINE ActiveX control
 m_CACOPFrameData.SetDeviceContext((LPCTSTR) context);
 m_CACOPFrameData.SetDeviceGroup((LPCTSTR) server);
 m_CACOPFrameData.SetDeviceName((LPCTSTR) device);
 m_CACOPFrameData.SetDeviceProperty((LPCTSTR) property);
 m_CACOPFrameData.SetReceiveQueueDepth(0);

 // variables needed to set up tine data transfer
 SAFEARRAY *psa;
 SAFEARRAYBOUND rgsabound[1];
 VARIANT null1, size, datatype, size0, datatype0;

 // initialize empty variant
 VariantInit( &null1 );
 V_VT( &null1 ) = VT_I1;
 null1.cVal = 1;

 // initialize maximum size of video frame data block 
 VariantInit( &size );
 V_VT( &size ) = VT_I4;
 size.lVal = maxbuffersize;

 // initialize empty size variant
 VariantInit( &size0 );
 V_VT( &size0 ) = VT_I4;
 size0.lVal = 1;

 // initialize data type identification
 CString str="BYTE";
 VariantInit( &datatype );
 V_VT( &datatype ) = VT_BSTR;
 V_BSTR( &datatype ) = str.AllocSysString();

 CString str1="BYTE";
 VariantInit( &datatype0 );
 V_VT( &datatype0 ) = VT_BSTR;
 V_BSTR( &datatype0 ) = str1.AllocSysString();

 // initialize a safe array width the maximum size of a 
 // video frame
 rgsabound[0].lLbound = 0;
 rgsabound[0].cElements = maxbuffersize;

 if ((psa=SafeArrayCreate(VT_I1,1,rgsabound)) == NULL)
 {
    //could not create safe array
	// so clean up and return false
    VariantClear( &null1 );
    VariantClear( &size );
    VariantClear( &datatype );
    VariantClear( &datatype0 );
    VariantClear( &size0 );
	retval = FALSE;
	return retval;
 }

 // initialize tine data block variant
 VariantInit( &m_vTineData );
 V_VT( &m_vTineData ) = VT_I1 | VT_ARRAY;
 V_ARRAY( &m_vTineData ) = psa;
 m_pTineData = (char *) psa->pvData;

 // set access mode to multicast 
 m_CACOPFrameData.SetAccessMode( (LPCTSTR) pollmode ); 
 // one request every 50 milliseconds (20 times per second)
 m_CACOPFrameData.SetAccessRate( "900" ); //SW! !!! Sept 08, 2005, changed for ScheduleProperty on Server side!
 
 // create tine link and start up transfer
 if ((m_lReceiveTineLink = m_CACOPFrameData.AttachLink(&m_vTineData, size, datatype, &null1, size0, datatype0)) <= 0) 
 {
      // error: present error message, return error and clean up variant
      //CString msg = CString("Could not connect Error AttachLink: ") + ;
      //AfxMessageBox( (LPCTSTR) msg, MB_OK | MB_ICONEXCLAMATION );

	  ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Could not connect Error AttachLink: %s", m_CACOPFrameData.GetStatus() );

	  retval = FALSE;
	  VariantClear( &m_vTineData );
 }
 else
 {
	 retval = TRUE; // success
	 // set boolean variable 
	 m_bTinePollTransferRunning = TRUE;
 }
   
 // clean up local variants
 VariantClear( &null1 );
 VariantClear( &size );
 VariantClear( &datatype );
 VariantClear( &datatype0 );
 VariantClear( &size0 );

 return(retval);
}


/////////////////////////////////////////////////////////////////////////////
//
// StopTinePollTransfer - stops a running poll transfer using TINE multicast
//
// Return values:
//  TRUE: poll transfer could be stopped
//  FALSE: poll transfer could not be stopped
//
BOOL CBogusTineDlg::StopTinePollTransfer()
{
 // default return value
 BOOL retval = TRUE;
 
 if (m_bTinePollTransferRunning == TRUE) // check whether a poll transfer is running
 {
   // close tine link
   // build up the variant in order to disable the link
   VARIANT tmp;
   VariantInit( &tmp );
   tmp.vt = VT_I4;
   tmp.lVal = m_lReceiveTineLink;

   // close the link
   long ret = m_CACOPFrameData.CloseLink(tmp);

   if (ret < 0) // error ? 
   {   
    retval = FALSE;
	((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0,"CloseLink: error: %s", (LPCTSTR) m_CACOPFrameData.GetStatus() );
   }
   
   // clean up the variants
   VariantClear( &tmp );
   VariantClear( &m_vTineData );
  
   // set default values
   m_CACOPFrameData.SetAccessMode("READ");
   m_CACOPFrameData.SetAccessRate("1000");

   // disable the receive tine link index
   m_lReceiveTineLink = -1;
 
   // set boolean variable
   m_bTinePollTransferRunning = FALSE;
 }

 return retval;
}

/////////////////////////////////////////////////////////////////////////////
//
// StartTineGrabTransfer - starts up grabbing of video frames using TINE multicast
//
// Starts up a TINE grab transfer of a certain number of video frames. 'numframes' frames are 
// put into the grabqueue and for each frame a message to the main application message queue 
// is posted.
//
// Parameters:
//   context/server/device/property - Address of the TINE frame server
//   servernormal - normal server name of the grabserver (server - 'F')
//   grabqueue - queue to hold the received frames
//   numframes - number of frames to be grabbed
//
// Return value:
//   TRUE  : grab mode is started up
//   FALSE : error while starting up grab mode
//
BOOL CBogusTineDlg::StartTineGrabTransfer(CString context, CString server, CString device, CString property, CString servernormal, CQueue *grabqueue, int numframes, int maxbuffersize, CString pollmode)
{
 // default return value
 BOOL retval = TRUE;

 if (m_bTineGrabTransferRunning == FALSE) // are we not running a poll mode ?
 {
  // set grab queue
  m_pGrabQueue = grabqueue;
  
  // set number of frames to grab
  m_iGrabNumFrames = numframes;
  
  // initialize grab variables
  m_iFramesGrabbed = 0;
  m_iNoFrame = 0;
  m_lLastStatusCode = 0;

  // set tine address (context/server/device/property)
  m_CACOPFrameData.SetDeviceContext((LPCTSTR) context);
  m_CACOPFrameData.SetDeviceGroup((LPCTSTR) server);
  m_CACOPFrameData.SetDeviceName((LPCTSTR) device);
  m_CACOPFrameData.SetDeviceProperty((LPCTSTR) property);

  // variables needed for initializing tine grab mode
  SAFEARRAY *psa;
  SAFEARRAYBOUND rgsabound[1];
  VARIANT null1, size, datatype, size0, datatype0;

  // initialize variants
  VariantInit( &null1 );
  V_VT( &null1 ) = VT_I1;
  null1.cVal = 1;

  VariantInit( &size );
  V_VT( &size ) = VT_I4;
  size.lVal = maxbuffersize;

  VariantInit( &size0 );
  V_VT( &size0 ) = VT_I4;
  size0.lVal = 1;

  CString str="BYTE";
  VariantInit( &datatype );
  V_VT( &datatype ) = VT_BSTR;
  V_BSTR( &datatype ) = str.AllocSysString();

  CString str1="BYTE";
  VariantInit( &datatype0 );
  V_VT( &datatype0 ) = VT_BSTR;
  V_BSTR( &datatype0 ) = str1.AllocSysString();

  // create safe array big enough to hold a single video frame
  rgsabound[0].lLbound = 0;
  rgsabound[0].cElements = maxbuffersize;
  if ((psa=SafeArrayCreate(VT_I1,1,rgsabound)) == NULL)
  {
    // could not create safe array
	// so clean up and return FALSE
    VariantClear( &null1 );
    VariantClear( &size );
    VariantClear( &datatype );
    VariantClear( &datatype0 );
    VariantClear( &size0 );
	retval = FALSE;
	return retval;
  }

  // initialize tine grab variant memory block
  VariantInit( &m_vTineGrabData );
  V_VT( &m_vTineGrabData ) = VT_I1 | VT_ARRAY;
  V_ARRAY( &m_vTineGrabData ) = psa;
  m_pTineGrabData = (char *) psa->pvData;

  // set mode to multicast and repetition rate to 
  // 20 Hz (one request each 50 msec)
  m_CACOPFrameData.SetAccessMode( (LPCTSTR) pollmode );
  m_CACOPFrameData.SetAccessRate( "900" ); //SW! !!! Apr 27, 2006, changed for ScheduleProperty on Server side!
  
  // initialize grav mode and start up grab link
  if ((m_lGrabTineLink = m_CACOPFrameData.AttachLink(&m_vTineGrabData, size, datatype, &null1, size0, datatype0)) <= 0) 
  {
      // error: display error message and clean up memory block variant
      ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox( MB_ICONEXCLAMATION, "Could not connect Error AttachLink: %s", m_CACOPFrameData.GetStatus() );
	  retval = FALSE;
	  VariantClear( &m_vTineGrabData );
  }
  else
  { 
	 // success
	 retval = TRUE;
	 // set boolean variable that grab mode is running
	 m_bTineGrabTransferRunning = TRUE; 
	 
	 // grabbing has not yet finished
	 m_bGrabFinished = FALSE;
  }
   
  // clean up variants
  VariantClear( &null1 );
  VariantClear( &size );
  VariantClear( &datatype );
  VariantClear( &datatype0 );
  VariantClear( &size0 );
 }

 return(retval);

}

/////////////////////////////////////////////////////////////////////////////
//
// StopTineGrabTransferCleanup - stops a running grab transfer or cleans up 
// after a grab transfer has been stopped
//
// Return values:
//  TRUE: grab transfer could be stopped and data structures are properly 
//        cleaned up
//  FALSE: poll transfer could not be stopped or cleanup has failed
//
BOOL CBogusTineDlg::StopTineGrabTransferCleanup()
{
 // default return value
 BOOL retval = TRUE;
 
 if (m_bTineGrabTransferRunning == TRUE) // are we running in grab mode ?
 {
   // close tine link
   // build up the variant in order to disable the link
   VARIANT tmp;
   VariantInit( &tmp );
   tmp.vt = VT_I4;
   tmp.lVal = m_lGrabTineLink;

   // close the link
   long ret = m_CACOPFrameData.CloseLink(tmp);

   if (ret < 0)
   {   
    // error: present error message and return FALSE
	retval = FALSE;
	((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "CloseLink: error: %s", (LPCTSTR) m_CACOPFrameData.GetStatus());
   }
   
   // clean up the variants
   VariantClear( &tmp );
   VariantClear( &m_vTineGrabData );
  
   // set default values
   m_CACOPFrameData.SetAccessMode("READ");
   m_CACOPFrameData.SetAccessRate("1000");

   // reset the grab tine link
   m_lGrabTineLink = -1;
 
   // set boolean variable
   m_bTineGrabTransferRunning = FALSE;
 }
 
 return retval;
}

/////////////////////////////////////////////////////////////////////////////
//
// SetCompressionClass - sets the compression class information 
//
// You have to pass the compression class and a pointer to a valid output format
// before starting up grab or poll mode.
//
// Parameters:
//   compr - pointer to the compression class
//   outfmt - pointer to BITMAPINFOHEADER structure that contains a valid 
//            output format (output = compressed frame)
//
void CBogusTineDlg::SetCompressionClass(CCompression *compr, LPBITMAPINFOHEADER outfmt)
{
 m_pCompression = compr;
 m_pOutHdr = outfmt;
}
