// ConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Compression.h"
#include "acop.h"
#include "headers.h"
#include "MemoryManagement.h"
#include "resource.h"
class CVideoclntDlg;
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "Queue.h"
#include "BogusTineDlg.h"
#include "Server.h"
#include "SSyncSocket.h"
#include "videoclnt.h"
#include "IniEx.h"
#include "ImageAcquire.h"
#include "ImageSequence.h"
#include "AdditionalInformation.h"
#include "StatFile.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "PrinterSettings.h"
#include "MarkedComboBox.h"
#include "AboutDialog.h"
#include "CATineDlg.h"
#include "InformationDialog.h"
#include "ConfigDlg.h"
#include "videoclntDlg.h"
#include "rpctypes.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConfigDlg dialog
//
// This dialog shows the configuration options of the videoclnt application.
// The user can set up the snapshot directory, force a reinitalization of server 
// settings and may choose between sockets and TINE multicast mode.
//
// The data exchange between the main program and the dialog is done via memory mapped
// initialisation files, where the values are also permanently remembered. If you press OK, 
// all values from the dialog are stored in the ini file. If you press Cancel, all values 
// in the ini file remain unchanged.
//

CConfigDlg::CConfigDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CConfigDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConfigDlg)
	//}}AFX_DATA_INIT

	m_pIniFileExeLocal = NULL; // initialize variable needed for accessing the ini file at executable location
	m_pIniFileExeGlobal = NULL; // initialize variable needed for accessing the ini file at executable location
	m_pIniFileProf = NULL; // initialize the ini file at profile directory
}


void CConfigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigDlg)
	DDX_Control(pDX, IDC_CHECK1, m_CCheckFilePreviewDisabled);
	DDX_Control(pDX, IDC_EDIT2_DAQDIR, m_CEditDAQDir);
	DDX_Control(pDX, IDC_EDIT1_DAQFILE, m_CEditDAQFile);
	DDX_Control(pDX, IDC_RADIO2_SOCKETS, m_CRadioSockets);
	DDX_Control(pDX, IDC_RADIO1_TINE, m_CRadioTine);
	DDX_Control(pDX, IDC_BUTTON2_ENABLEREINIT, m_CButtonEnableReInit);
	DDX_Control(pDX, IDC_COMBO2, m_CComboFramerate);
	DDX_Control(pDX, IDC_EDIT1_SNAPSHOTDIR, m_CEditSnapshot);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigDlg)
	ON_BN_CLICKED(IDC_BUTTON1, OnChooseSnapshot)
	ON_BN_CLICKED(IDC_BUTTON2_ENABLEREINIT, OnButton2Enablereinit)
	ON_BN_CLICKED(IDC_RADIO1_TINE, OnRadio1Tine)
	ON_BN_CLICKED(IDC_RADIO2_SOCKETS, OnRadio2Sockets)
	ON_BN_CLICKED(IDC_BUTTON2_DAQFILE, OnButton2Daqfile)
	ON_BN_CLICKED(IDC_BUTTON3_DAQDIR, OnButton3Daqdir)
	ON_BN_CLICKED(IDC_CHECK1, OnCheckDisabledFilePreviewDialog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigDlg message handlers


/////////////////////////////////////////////////////////////////////////////
// This is the callback function when the user presses the OK button.
//
//

void CConfigDlg::OnOK() 
{
    // get the snapshot directory from control
	CString Snapshotdir;
	m_CEditSnapshot.GetWindowText(Snapshotdir);

	// get last character of snapshot directory
	TCHAR t1 = Snapshotdir.GetAt(Snapshotdir.GetLength()-1);
	// if the directory's last char is not a '\', then append it
    if (t1 != '\\') Snapshotdir += "\\";

    // check whether the directory exists and we can write to it

	if (CheckDirectory(Snapshotdir)<1)
	{
       // display error message and do not close dialog
       ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "The snapshot path does not exist or is not writable.\nPlease choose a path that does exist.");
	   return;
	}

	CString daqdir;
	m_CEditDAQDir.GetWindowText(daqdir);

	// get last character of snapshot directory
	t1 = daqdir.GetAt(daqdir.GetLength()-1);
	// if the directory's last char is not a '\', then append it
    if (t1 != '\\') daqdir += "\\";

    // check whether the directory exists and we can write to it

	if (CheckDirectory(daqdir)<1)
	{
       // display error message and do not close dialog
       ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "The daq directory does not exist or is not writable.\nPlease choose a path that does exist.");
	   return;
	}

    // if it does then save it into the profile ini file
    m_pIniFileProf->SetValue("MAIN", "SnapshotDir", Snapshotdir);

	CString daqfile;
	m_CEditDAQFile.GetWindowText( daqfile );
	m_pIniFileProf->SetValue("MAIN", "DaqConfigFile", daqfile );

	if (m_CCheckFilePreviewDisabled.GetCheck() == 1) 
		m_pIniFileProf->SetValue("MAIN", "LoadPreviewDialogDisabled", "TRUE");
	else
		m_pIniFileProf->SetValue("MAIN", "LoadPreviewDialogDisabled", "FALSE");

	m_pIniFileProf->SetValue("MAIN", "DaqDirectory", daqdir );

    // flush ini file
    m_pIniFileProf->WriteFile();

    // put framerate id into ini file 
    int pos = m_CComboFramerate.GetCurSel();
    
	m_pIniFileExeLocal->SetValueInt("MAIN", "Framerate", pos );
    m_pIniFileExeLocal->WriteFile();

    CDialog::OnOK();

}

/////////////////////////////////////////////////////////////////////////////
// This is the callback function for the 'Cancel' button. No values are stored 
// in the registry.
//
void CConfigDlg::OnCancel() 
{
	// close dialog
	CDialog::OnCancel();
}

/////////////////////////////////////////////////////////////////////////////
// This is the callback function for the WM_INITDIALOG message. It is called
// whenever this dialog is about to be created. It just reads the variables 
// out of the registry and into the appropriate controls.
//
//
BOOL CConfigDlg::OnInitDialog() 
{
	// call Base-class
	CDialog::OnInitDialog();

    // server reinitialization: FALSE
    m_bReInit = FALSE;

    // update framerate control
    CString Framerate = m_pIniFileExeLocal->GetValue("MAIN", "Framerate", "0");
	int frate = atoi( (const char *) Framerate );
	if (frate < 0 || frate > 10) frate = 0;
	m_CComboFramerate.SetCurSel( frate );

    // fill in the proper values for the snapshot directory
	
	// get the snapshotdir
	CString Snapshotdir = m_pIniFileProf->GetValue("MAIN", "SnapshotDir", "h:\\");

	CString sdisabled = m_pIniFileProf->GetValue("MAIN", "LoadPreviewDialogDisabled", "FALSE");

	if (sdisabled == "TRUE") m_CCheckFilePreviewDisabled.SetCheck(1); else m_CCheckFilePreviewDisabled.SetCheck(0);

	// put snapshot directory into the appropriate control
	m_CEditSnapshot.SetWindowText( (LPCTSTR) Snapshotdir );

	CString daqdeffile = m_pIniFileExeLocal->GetValue("MAIN", "DaqDefaultConfigFile", "daqproperties_default.cfg");
	CString daqfile = m_pIniFileProf->GetValue("MAIN", "DaqConfigFile", daqdeffile);

	m_CEditDAQFile.SetWindowText( (LPCTSTR) daqfile );
	
	CString daqdir = m_pIniFileProf->GetValue("MAIN", "DaqDirectory", "H:\\");
	m_CEditDAQDir.SetWindowText( (LPCTSTR) daqdir);

	if (m_bCanTineBeEnabled == FALSE) // tine can not be enabled ?
	{
     // uncheck tine, check socket radio button
	 m_CRadioTine.SetCheck(0);
     m_CRadioSockets.SetCheck(1);
	 // disable the radio buttons 
	 m_CRadioTine.EnableWindow(FALSE);
	 m_CRadioSockets.EnableWindow(FALSE);
	 
	 // tine can not be enabled, so set transfer mode to sockets
	 m_bTransferModeTine = FALSE;
	}
	 else
	{ // tine can be enabled
     if (m_bIsTineEnabled == TRUE) // tine is enabled ? 
	 {
      // check tine radio button, uncheck sockets radio button
	  m_CRadioTine.SetCheck(1);
      m_CRadioSockets.SetCheck(0);
	  m_bTransferModeTine = TRUE; // set transfer mode to TINE
     }
	 else
	 { // tine is not enabled
      
      // uncheck tine radio button, check sockets radio button
	  m_CRadioTine.SetCheck(0);
      m_CRadioSockets.SetCheck(1);
	  m_bTransferModeTine = FALSE; // set transfer mode to sockets
	 }
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// This is the callback function for the 'Choose' button. It opens a dialog 
// box where the user can choose an appropriate directory for storing the 
// snapshot images.
//
//
void CConfigDlg::OnChooseSnapshot() 
{
    BROWSEINFO  Bi;
    LPITEMIDLIST  lpItemLst;
    char lpName[255];
    HRESULT  rc;
    LPMALLOC pMalloc=NULL;

    Bi.hwndOwner = m_hWnd;
    Bi.pidlRoot = NULL;
    Bi.pszDisplayName = lpName;
    Bi.lpszTitle = "Snapshot directory";
    Bi.ulFlags = NULL;
    Bi.lpfn = NULL;
    Bi.lParam = NULL;
    Bi.iImage = 0;

    lpItemLst = SHBrowseForFolder (&Bi);
	if (lpItemLst == NULL) return;
    SHGetPathFromIDList (lpItemLst, lpName);
    if ( SUCCEEDED(SHGetMalloc (&pMalloc)) ) 
	{
		pMalloc->Free ((void *)lpItemLst);
		rc = pMalloc->Release ();
	}

	// get selected directory path
	CString snapshotdir(lpName);

	// obtain last character of it
	TCHAR t1 = snapshotdir.GetAt(snapshotdir.GetLength()-1);

	// it the string does not end with an '\', append one
    if (t1 != '\\') snapshotdir += "\\";

	// check, whether the directory exist
	if (GetFileAttributes((LPCTSTR) snapshotdir) != 0xFFFFFFFF)
	{
	   // okay, directory exists
	   m_CEditSnapshot.SetWindowText( (LPCTSTR) snapshotdir );
	}
     else
	{
       // does not exist, display error message
	   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "The path does not exist. Please choose a path that exists.");
	}

}

/////////////////////////////////////////////////////////////////////////////
// This function sets a reference to the two ini files of the main application.
//
// One ini file resides at the executable's location, the other ini file resides 
// at the profile directory of the currently logged on user
void CConfigDlg::SetIniFile(CIniEx *the_ini_exe_global, CIniEx *the_ini_exe_local, CIniEx *the_ini_prof)
{
  m_pIniFileExeGlobal = the_ini_exe_global;
  m_pIniFileExeLocal = the_ini_exe_local;
  m_pIniFileProf = the_ini_prof;
}

/////////////////////////////////////////////////////////////////////////////
// ReturnServerReInitialization - return whether the user has choosed to 
// reinitialize the server data structures
//
// TRUE  : server structures should be reinitialized
// FALSE : server structures should not be reinitialized
BOOL CConfigDlg::ReturnServerReInititialisation()
{
 return( m_bReInit );
}


/////////////////////////////////////////////////////////////////////////////
// OnButton2Enablereinit - called when the user clicks on the 
// 'Enable reinitialization' button
//
void CConfigDlg::OnButton2Enablereinit() 
{
	// change text on button
	m_CButtonEnableReInit.SetWindowText("Enabled!");

    // reinitialization choosed
    m_bReInit = TRUE;
	// disable button
	m_CButtonEnableReInit.EnableWindow(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// SetTineProperties - sets the TINE boolean variables
//
// This function has be called before the dialog is shown on screen !
//
// Parameters:
//  IsTineEnabled: TRUE: check tine radio button, FALSE: check sockets radio button
//  CanTineBeEnabled: FALSE: disable (gray out) the radio buttons tine and sockets
//                    
void CConfigDlg::SetTineProperties(BOOL IsTineEnabled, BOOL CanTineBeEnabled)
{
 m_bIsTineEnabled = IsTineEnabled;
 m_bCanTineBeEnabled = CanTineBeEnabled;
}

/////////////////////////////////////////////////////////////////////////////////////
// OnRadio1Tine - called when the user clicks on the radio button 'use tine protocol'
//
//
void CConfigDlg::OnRadio1Tine() 
{
	// check TINE radio button
	m_CRadioTine.SetCheck(1);
	// uncheck Sockets radio button
	m_CRadioSockets.SetCheck(0);
	
	// set transfer mode to TINE
	m_bTransferModeTine = TRUE;
}

//////////////////////////////////////////////////////////////////////////////////
// OnRadio2Sockets - called when the user clicks on the radio button 'use sockets'
//
//
void CConfigDlg::OnRadio2Sockets() 
{
	//uncheck TINE radio button
	m_CRadioTine.SetCheck(0);
	// check sockets radio button
	m_CRadioSockets.SetCheck(1);
	
	// set transfer mode to sockets
	m_bTransferModeTine = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// ReturnTransferMode - returns the actually set transfer mode
//
// Return value:
//  TRUE  :  TINE
//  FALSE : Sockets
BOOL CConfigDlg::ReturnTransferMode()
{
 return( m_bTransferModeTine );
}

void CConfigDlg::OnButton2Daqfile() 
{
 // create file filter string
 CString filefilter = "Configuration Files (*.cfg)|*.cfg|All Files (*.*)|*.*||";

 // instantiate file open dialog
 CFileDialog fileDlg(TRUE,NULL,NULL, OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST, filefilter);
 
 // display dialog and check whether user pressed cancel
 if(fileDlg.DoModal()==IDCANCEL)
	return; // user pressed cancel -> get out of here
	
 // make extensions upper case

 m_CEditDAQFile.SetWindowText( (LPCTSTR) fileDlg.GetPathName() );
}


void CConfigDlg::OnButton3Daqdir() 
{
    BROWSEINFO  Bi;
    LPITEMIDLIST  lpItemLst;
    char lpName[255];
    HRESULT  rc;
    LPMALLOC pMalloc = NULL;

    Bi.hwndOwner = m_hWnd;
    Bi.pidlRoot = NULL;
    Bi.pszDisplayName = lpName;
    Bi.lpszTitle = "Online DAQ storage directory";
    Bi.ulFlags = NULL;
    Bi.lpfn = NULL;
    Bi.lParam = NULL;
    Bi.iImage = 0;

    lpItemLst = SHBrowseForFolder (&Bi);
	if (lpItemLst == NULL) return;
    SHGetPathFromIDList (lpItemLst, lpName);
    if ( SUCCEEDED(SHGetMalloc (&pMalloc)) )
	{
		pMalloc->Free ((void *)lpItemLst);
		rc = pMalloc->Release ();
	}

	// get selected directory path
	CString daqdir(lpName);

	// obtain last character of it
	TCHAR t1 = daqdir.GetAt(daqdir.GetLength()-1);

	// it the string does not end with an '\', append one
    if (t1 != '\\') daqdir += "\\";


	if (CheckDirectory(daqdir)<1)
	{
       // does not exist, display error message
	   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "The path does not exist or is not writable. Please choose a different path.");
	}
	else
	{
	   // okay, directory exists
	   m_CEditDAQDir.SetWindowText( (LPCTSTR) daqdir );
	}
}


//  1 exists and is writable
// -1 does not exists
// -2 exists but is not writable
int CConfigDlg::CheckDirectory(CString dir)
{
    BOOL dir_exists=FALSE;
	BOOL dir_writable=FALSE;

	// check, whether the directory exists
	if (GetFileAttributes((LPCTSTR) dir) != 0xFFFFFFFF)
	{
      dir_exists = TRUE;

      CString filename = dir+"___123456789.tmp";
	 
  	  FILE *fp = fopen((LPCTSTR) filename, "wb");
	  if (fp != NULL)
	  {
	   char buffer[10];
	   int ret = fwrite( buffer, 1, 10, fp);
	   if (ret == 10) dir_writable = TRUE;
	   fclose(fp);
	   remove( filename );
      }
	}

	if (dir_exists == FALSE) return( -1 );
	if (dir_writable == FALSE) return( -2 );

    return( 1 );
}

void CConfigDlg::OnCheckDisabledFilePreviewDialog() 
{
	// TODO: Add your control notification handler code here
}
