// Fit.cpp: implementation of the CFit class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MemoryManagement.h"
#include "headers.h"
#include "FitParam.h"
#include "Stat.h"
#include "FourierStat.h"
#include "Fit.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
/*
CFit::CFit()
{	
	offset_X=0;
	offset_Y=0;
	len_X=ImageWidth;
	len_Y=ImageHeight;

//	TEMP_XDIST=new double[ImageWidth];
//	TEMP_YDIST=new double[ImageHeight];
//	FitDist_X=new double[ImageWidth];
//	FitDist_Y=new double[ImageHeight];
//	tolerances = new double[4];

	TEMP_XDIST=(double *) memman.sw_malloc( sizeof(double)*ImageWidth);
	TEMP_YDIST=(double *) memman.sw_malloc( sizeof(double)*ImageHeight);
	FitDist_X=(double *) memman.sw_malloc( sizeof(double)*ImageWidth);
	FitDist_Y= (double *) memman.sw_malloc( sizeof(double)*ImageHeight);
	tolerances = (double *) memman.sw_malloc( 4*sizeof(double) );

	memman.memcpysw(TEMP_XDIST,XDIST,ImageWidth*sizeof(double));
	memman.memcpysw(TEMP_YDIST,YDIST,ImageHeight*sizeof(double));
	ZeroMemory(FitDist_X,ImageWidth*sizeof(double));
	ZeroMemory(FitDist_Y,ImageHeight*sizeof(double));
	
	sigma_X= XRMS;
	sigma_Y= YRMS;


}
*/
CFit::~CFit()
{
	if (TEMP_XDIST != NULL) memman.sw_free( (void *) TEMP_XDIST );
	if (TEMP_YDIST != NULL) memman.sw_free( (void *) TEMP_YDIST );
	if (FitDist_X  != NULL) memman.sw_free( (void *) FitDist_X );
	if (FitDist_Y  != NULL) memman.sw_free( (void *) FitDist_Y );
	if (tolerances != NULL) memman.sw_free( (void *) tolerances );
}

CFit::CFit(unsigned char* data,unsigned char* Back, unsigned char *out, CRect AOI,BOOL XFilt, BOOL circular, BOOL normalize,
		unsigned short deg,unsigned short recdep,unsigned int ImWi,unsigned int ImHi, unsigned int rep_rate, unsigned int bpp, unsigned int bpp_eff):
			CFourierStat(data,Back,out, AOI,XFilt,circular, normalize, deg,recdep, ImWi,ImHi, bpp, bpp_eff)
{
    CStat::START();
	CFourierStat::START();

    m_iRepRate = rep_rate;

    /* calculate min and max xydist */
    double *xdist = X_DIST();
	double *ydist = Y_DIST();

	if (xdist == NULL || ydist == NULL) return;

	MinXDIST = 1e10;
	MaxXDIST = -1e10;

	MinYDIST = 1e10;
	MaxYDIST = -1e10;

	for (int i=left; i<right+1; i++)
	{
     if (xdist[i] < MinXDIST) MinXDIST = xdist[i];
	 if (xdist[i] > MaxXDIST) MaxXDIST = xdist[i];
	}

	for (i=up; i<down+1; i++)
	{
     if (ydist[i] < MinYDIST) MinYDIST = ydist[i];
	 if (ydist[i] > MaxYDIST) MaxYDIST = ydist[i];
	}

	offset_X=MinXDIST;
	offset_Y=MinYDIST;
	len_X=right-left+1;
	len_Y=down-up+1;

	TEMP_XDIST=(double *) memman.sw_malloc( sizeof(double)*ImageWidth);
	TEMP_YDIST=(double *) memman.sw_malloc( sizeof(double)*ImageHeight);
	FitDist_X=(double *) memman.sw_malloc( sizeof(double)*ImageWidth);
	FitDist_Y= (double *) memman.sw_malloc( sizeof(double)*ImageHeight);
	tolerances = (double *) memman.sw_malloc( 4*sizeof(double) );

	if (TEMP_XDIST == NULL || TEMP_YDIST == NULL || FitDist_X == NULL || FitDist_Y == NULL || tolerances == NULL)
	{
     return;
	}

	memman.memcpysw( (char *) TEMP_XDIST, (char *) CStat::X_DIST(),ImageWidth*sizeof(double));
	memman.memcpysw( (char *) TEMP_YDIST, (char *) CStat::Y_DIST(),ImageHeight*sizeof(double));
	ZeroMemory(FitDist_X,ImageWidth*sizeof(double));
	ZeroMemory(FitDist_Y,ImageHeight*sizeof(double));
	
	double *xP= TEMP_XDIST;
	double *yP= TEMP_YDIST;
	double sum=1e-15;
	
	for ( i = left; i<=right; i++) 
	{
	 xP[i] -= offset_X;
	 sum += xP[i];
	}
	ASSERT( sum > 0.0 );
	for ( i = left; i<=right; i++) 
	{
	 xP[i] /= sum;
	}
	 
    sum = 1e-15;

	for (i = up; i<=down;i++) 
	{
	 yP[i] -= offset_Y;
	 sum += yP[i];
	}
	ASSERT( sum > 0.0 );
	 
	for (i = up; i<=down;i++)
	{
	 yP[i] /= sum;
	}
	
	double xm = CFourierStat::CalcXMEAN(xP, 1, left, right);
	double ym = CFourierStat::CalcYMEAN(yP, 1, up, down);
// 0-sigma, 1-amplitude, 2-position, 3-offset
	sigma_X=CStat::CalcXRMS(xP, xm ); 
	sigma_Y=CStat::CalcYRMS(yP, ym ); 

//	sigma_X= PreFit(xdist, left, right, xm, 0.0*sigma_X, 2.5*sigma_X, TRUE);
//	sigma_Y= PreFit(ydist, up  , down , ym, 0.0*sigma_Y, 2.5*sigma_Y, FALSE);

	tolerances[0]=3;
	tolerances[1]=MaxXDIST*0.01;
	tolerances[2]=3;
	tolerances[3]=.05*(MaxXDIST-MinXDIST);
	FitParam p1(left, up, right, down, CFourierStat::XM(), CFourierStat::YM(), MinXDIST, MaxXDIST, MinYDIST, MaxYDIST, sigma_X, sigma_Y, offset_X, offset_Y, TRUE);
	FitParam p2(left, up, right, down, CFourierStat::XM(), CFourierStat::YM(), MinXDIST, MaxXDIST, MinYDIST, MaxYDIST, sigma_X, sigma_Y, offset_X, offset_Y, TRUE);

    FitPar_X = FitMultiGauss(&p1, &p2,0, &(xdist[left]), len_X,TRUE);
	
	
	tolerances[0]=3;
	tolerances[1]=MaxYDIST*0.01;
	tolerances[2]=3;
	tolerances[3]=.05*(MaxYDIST-MinYDIST);
	FitParam p3(left, up, right, down, CFourierStat::XM(), CFourierStat::YM(), MinXDIST, MaxXDIST, MinYDIST, MaxYDIST, sigma_X, sigma_Y, offset_X, offset_Y, FALSE);
	FitParam p4(left, up, right, down, CFourierStat::XM(), CFourierStat::YM(), MinXDIST, MaxXDIST, MinYDIST, MaxYDIST, sigma_X, sigma_Y, offset_X, offset_Y, FALSE);
      
	
	FitPar_Y = FitMultiGauss(&p3, &p4,	0, &(ydist[up]), len_Y,FALSE);


	FillFitDist(FitDist_X, FitPar_X ,0, ImageWidth-1);
	FillFitDist(FitDist_Y, FitPar_Y ,0, ImageHeight-1);
    pos_X=FitPar_X->FitPar[2].CurrentValue;
    pos_Y=FitPar_Y->FitPar[2].CurrentValue;
}

double CFit::Residual(double sigma, double xmean, double *Source, int leftborder, int rightborder, BOOL XY, double maxval)
{
    if (Source == NULL) return 0.0;

	double retval=0;
	double delta,gauss,offset = XY?offset_X:offset_Y,
		nominator=maxval*sigma*2.506628274631000502415765284811;
	for(int i=leftborder;i<=rightborder;i++)
	{									
		gauss=CalcGauss(sigma,i,xmean)*nominator;
		delta=pow(gauss-Source[i],2);		
		retval+=delta;
	}
	return retval;
}

double CFit::CalcGauss(double sigma, double x, double xmean)
{
	double retval=1/(sigma*2.506628274631000502415765284811);
	double power=-pow(x-xmean,2);
	power=power/(2*pow(sigma,2));
	retval=retval*exp(power);
	return retval;
}

double CFit::PreFit(double *Source, int leftborder, int rightborder, double xmean, double sigmamin, double sigmamax, BOOL XY)
{
	if (Source == NULL) return 0.0;

	double cval,dsigma=(sigmamax-sigmamin)/2,lmin=sigmamin,lmax=dsigma;
	double ampl=XY?MaxXDIST:MaxYDIST;
	cval=Residual((sigmamin+dsigma)*1.001,xmean,Source,leftborder,rightborder,XY,ampl)-
			Residual((sigmamin+dsigma),xmean,Source,leftborder,rightborder,XY,ampl);
	cval/=((sigmamin+dsigma)*.001);
	lmax+=(lmin+=((double)(cval<0)+.5*((double)(cval==0)))*dsigma);
	if(dsigma<1)return (lmax+lmin)/2;
	else return PreFit(Source,leftborder,rightborder,xmean,lmin,lmax,XY);
}

double CFit::Derivative(FitParam *inParam, double *Source, int length, int currnum,BOOL XY)
{
	if (inParam == NULL || Source == NULL) return 0.0;
	double retval;
	double fwd=1.005,bckwd=.995, denom=fwd-bckwd;
	FitParam localFV2,localFV1;
	localFV2= *inParam;
	localFV1= *inParam;
	localFV2.FitPar[currnum].CurrentValue*=fwd;
	localFV1.FitPar[currnum].CurrentValue*=bckwd;
	retval=MutipleRes(&localFV2,Source,length,XY, m_iRepRate)-MutipleRes(&localFV1,Source,length,XY, m_iRepRate);
	retval/=inParam->FitPar[currnum].CurrentValue*denom;
	return retval;
}


// rep_rate = 1..10 repetition rate in hertz
// rep_Rate = 0 offline calculation, full data is analysed
double CFit::MutipleRes(FitParam* inParam,double* Source,int length,BOOL XY, int rep_rate)
{
    if (inParam == NULL || Source == NULL) return 0.0;
	
	double retval=0,nominator,sigma,xm,delta,
		off=inParam->FitPar[3].CurrentValue;
	int lb=XY?left:up;
	
	int step;

	if (rep_rate != 0) // fixed repetition rate
	{
	 double divider = (1.0 / ((double) rep_rate)) * 70;
	 step = (int) (1.0 + ((length*1.0) / divider));
    }
    else step = 1;

	for(int j=0;j<length;j+=step)
	{
		sigma=inParam->FitPar[0].CurrentValue;
		xm=inParam->FitPar[2].CurrentValue;
		nominator=inParam->FitPar[1].CurrentValue*
			sigma*2.506628274631000502415765284811;
		delta=CalcGauss(sigma,j+lb,xm)*nominator;
		retval+=pow(Source[j]-delta-off,2);
	}
	return retval;
}

FitParam* CFit::FitMultiGauss(FitParam* inParam,FitParam* OutVal,int currnum,double* Source, int len,BOOL XY)
{ 
  if (inParam == NULL || OutVal == NULL || Source == NULL) return NULL;

  double cval,sigmamin,sigmamax,li=1, sd=0,su=1;
  sigmamin=inParam->FitPar[currnum].LowerLimit;
  sigmamax=inParam->FitPar[currnum].UpperLimit;
  double dsigma=fabs((sigmamax-sigmamin)/2);//0-sigma 1-amplitude 2-position
  inParam->FitPar[currnum].CurrentValue=sigmamin+(li)*dsigma;

  for (int i=currnum+1;i<inParam->TotalNumber;i++)
  {
		inParam->FitPar[i].LowerLimit=OutVal->FitPar[i].LowerLimit;
		inParam->FitPar[i].UpperLimit=OutVal->FitPar[i].UpperLimit;
  }

  if( currnum < inParam->TotalNumber-1 ) FitMultiGauss(inParam,OutVal,currnum+1,Source, len,XY);
 
  cval=Derivative(inParam,Source,len,currnum,XY);
  
  su+=(sd+=((double)(cval<0))+.5*((double)(cval==0)));
 
  inParam->FitPar[currnum].LowerLimit=sigmamin+(sd)*dsigma;
  inParam->FitPar[currnum].UpperLimit=sigmamin+(su)*dsigma;

  if (dsigma<=tolerances[currnum])
  {
   return inParam;
  }
  else 
  {
    return FitMultiGauss(inParam,OutVal,currnum,Source, len,XY );
  }

}

void CFit::FillFitDist(double *ziel, FitParam *FP, int l, int r)
{
	if ( ziel == NULL || FP == NULL) return;
	
	double pos= FP->FitPar[2].CurrentValue, sigma=FP->FitPar[0].CurrentValue, 
		   off= FP->FitPar[3].CurrentValue, ampl= FP->FitPar[1].CurrentValue,
		nominator=ampl*sigma*2.506628274631000502415765284811;
	for (int i=l;i<=r;i++)
	{
		ziel[i]=off+nominator*CalcGauss(sigma,(double)i,pos);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the X distribution array for further processing in the 
// main program. ttention: After destructing the class this pointer is no longer valid.
//
double*  CFit::X_DIST_FIT()
{
	return FitDist_X;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the Y distribution array for further processing in the 
// main program. ttention: After destructing the class this pointer is no longer valid.
//
double*  CFit::Y_DIST_FIT()
{
	return FitDist_Y;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the x coordinate of the spot size for further processing 
// in the main program. 
//
double  CFit::X_RMS()
{
	return sigma_X;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the y coordinate of the spot size for further processing 
// in the main program. 
//
double  CFit::Y_RMS()
{
	return sigma_Y;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the x coordinate of the spot center for further processing 
// in the main program. 
//
double  CFit::XM()
{
	return pos_X;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the y coordinate of the spot center for further processing 
// in the main program. 
//
double  CFit::YM()
{
	return pos_Y;
}
