// FitParam.cpp: implementation of the FitParam class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MemoryManagement.h"
#include "headers.h"
#include "Stat.h"
#include "FourierStat.h"
#include "FitParam.h"
#include "Fit.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FitParam::FitParam()
{TotalNumber=4;FitPar=new Component[TotalNumber];}

FitParam::~FitParam()
{if (FitPar != NULL) delete [] FitPar;}

FitParam::FitParam(int left, int up, int right, int down, double FTMeanX, double FTMeanY, 
                   double MinXDIST, double MaxXDIST, double MinYDIST, double MaxYDIST, 
				   double sigma_X, double sigma_Y, double offset_X, double offset_Y, BOOL XY)
{
	TotalNumber=4;// 0-sigma, 1-amplitude, 2-position, 3-offset
	FitPar=new Component[TotalNumber];
	if (FitPar == NULL) return;

	int LBorders=XY?left:up;
	int RBorders=XY?right:down;
	int MaxPosition=XY?(int)FTMeanX:(int)FTMeanY;
	double MaxD=XY?MaxXDIST:MaxYDIST;
	double MinD=XY?MinXDIST:MinYDIST;
	double sigma=XY?sigma_X:sigma_Y;
	double delta_offset = (MaxD-MinD)/2;

	FitPar[0].LowerLimit=sigma*0.2;//sigma
	FitPar[0].UpperLimit=sigma*1.5;
	FitPar[0].CurrentValue=sigma;

	FitPar[1].LowerLimit=(MaxD-MinD)*.5;//amplitude
	FitPar[1].UpperLimit=(MaxD-MinD)*1.5;
	FitPar[1].CurrentValue=(MaxD-MinD);

	FitPar[2].LowerLimit=MaxPosition-0.2*(MaxPosition-LBorders);//position
	FitPar[2].UpperLimit=MaxPosition+0.2*(RBorders-MaxPosition);
	FitPar[2].CurrentValue=MaxPosition;

	FitPar[3].CurrentValue=MinD;//offset
	FitPar[3].LowerLimit=MinD;
	FitPar[3].UpperLimit=(MaxD-MinD)/2;
}

FitParam& FitParam::operator =(FitParam &EQV)
{
	if (FitPar == NULL) return *this;
	TotalNumber=EQV.TotalNumber;
	delete [] FitPar;
	FitPar=new Component[TotalNumber];
	if (FitPar == NULL) return *this;
	for(int i = 0; i < TotalNumber; i++ )
	{
		FitPar[i].LowerLimit=EQV.FitPar[i].LowerLimit;
		FitPar[i].UpperLimit=EQV.FitPar[i].UpperLimit;
		FitPar[i].CurrentValue=EQV.FitPar[i].CurrentValue;
	}
	return *this;
}


void FitParam::VM (FitParam* fV)
{ 

	if (FitPar == NULL) return;
	double format[6]={0.5 , 1.5 , 0.5 , 1.5 , 0.2 , 0.2 };
//amplitude
	FitPar[1].LowerLimit=FitPar[1].CurrentValue*format[0];
	FitPar[1].UpperLimit=FitPar[1].CurrentValue*format[1];
//sigma
	FitPar[0].LowerLimit=FitPar[0].CurrentValue*format[2];
	FitPar[0].UpperLimit=FitPar[0].CurrentValue*format[3];
//position
	FitPar[2].LowerLimit=FitPar[2].CurrentValue-
			format[4]*(FitPar[2].CurrentValue-fV->FitPar[2].LowerLimit );

	FitPar[2].UpperLimit=FitPar[2].CurrentValue+
			format[5]*(fV->FitPar[2].UpperLimit -FitPar[2].CurrentValue);

	FitPar[3].LowerLimit=0;
	FitPar[3].UpperLimit=fV->FitPar[3].UpperLimit;
}