// IMCFileHelper.cpp: implementation of the CIMCFileHelper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "headers.h"
#include "videoclnt.h"
#include <vfw.h>
#include "Queue.h"
#include "BMPFileHelper.h"
#include "compression.h"
#include "EnvelopeAverage.h"
class CVideoclntDlg;
#include "IniEx.h"
#include "Server.h"
#include "ServerList.h"
#include "SSyncSocket.h"
#include "configdlg.h"
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "ImageSequence.h"
#include "Stat.h"
#include "BogusTineDlg.h"
#include "ImageAcquire.h"
#include "FourierStat.h"
#include "StatFile.h"
#include "StatEmpty.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "acop.h"
#include "oleauto.h"
#include "AdditionalInformation.h"
#include "PrinterSettings.h"
#include "MarkedComboBox.h"
#include "AboutDialog.h"
#include "CATineDlg.h"
#include "InformationDialog.h"
#include "videoclntDlg.h"
#include "IMCFileHelper.h"
#include "zconf.h"
#include "zlib.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction - just empty
//////////////////////////////////////////////////////////////////////

CIMCFileHelper::CIMCFileHelper()
{
}

CIMCFileHelper::~CIMCFileHelper()
{
}

//////////////////////////////////////////////////////////////////////
// SaveToFile(CString filename, CImageSequence *seq)
//
// This function just stores the ImageSequence seq into the IMC 
// (IMage Compressed) file 'filename'.
//
// Returnvalue
//  True  : file was successfully saved  
//  False : file was not successfully saved  
//////////////////////////////////////////////////////////////////////
BOOL CIMCFileHelper::SaveToFile(CString filename, CImageSequence *seq)
{
 // no frames inside the sequence -> error
 if (seq->Count() == 0) return FALSE;

 // open file in write binary mode
 FILE *fp = fopen(filename, "wb");
 if (fp == NULL) // could not create file: error
 {
  return FALSE;
 }

 // loop over all frames
 for (int i=0;i<seq->Count();i++)
 {
  VIDHDR vidH;
  FHDR fHdr;
  IMHDR imHdr;
  char *frameptr;

  // get frame # i
  seq->GetFrame(i, (const char **) &frameptr );

  // get video header out of frameptr
  memman.memcpysw( (char *) &vidH, (char *) frameptr, sizeof(VIDHDR));

  // create file header
  fHdr.width = vidH.widthframe;
  fHdr.height = vidH.heightframe;
  fHdr.num_frames = seq->Count();
  
  if (vidH.bpp_effective == 8) vidH.bpp_effective = 0;

  // encode effective bpp properly SW! Sept 21, 2006
  fHdr.bpp = ((vidH.bpp_effective<<16)&0xffff0000) + (vidH.bpp&0xffff);

  if (i==0) // only save file header if it is the first frame
  {
   // write out file header  
   int ret2 = fwrite( &fHdr, 1, sizeof(FHDR), fp);
   if (ret2 != sizeof(FHDR) ) // error while writing ? 
   {
    fclose(fp); // close file, return error
	return FALSE;
   }
  }

  // create image header
  imHdr.scale = vidH.scale;
  imHdr.len_uncompressed = (fHdr.width*fHdr.height*(fHdr.bpp&0xffff)) / 8;

  // allocate memory for temorarily storing the compressed frame bits
  char *compressed = (char *) memman.sw_malloc( (imHdr.len_uncompressed*3) / 2 );
  if (compressed == NULL) // error allocating ? 
  {
   fclose(fp);
   return FALSE; // -> return error
  }
  
  int  comprlen= (imHdr.len_uncompressed*3) / 2;
  
  // compress the frame using zlib
  int retc = compress ((Bytef *)compressed, (uLongf *) &comprlen, (const Bytef *) (frameptr+sizeof(VIDHDR)), imHdr.len_uncompressed); 
  if (retc != Z_OK) // error ? 
  {
   memman.sw_free( (void *) compressed );
   fclose(fp);
   return FALSE; // return error
  }

  // fill out last remaining field of image header
  imHdr.len_compressed = comprlen;

  // write out image header
  int ret3 = fwrite( &imHdr, 1, sizeof(IMHDR), fp);
  if (ret3 != sizeof(IMHDR) ) // error ? 
  {
   memman.sw_free( (void *) compressed );
   fclose(fp);
   return FALSE; // return error
  }

  // write out compressed frame bits
  ret3 = fwrite( compressed, 1, comprlen, fp); 
  if (ret3 != comprlen) // error ?  
  {
   memman.sw_free( (void *) compressed );
   fclose(fp);
   return FALSE; // return error
  }

  // free compressed ptr
  memman.sw_free( (void *) compressed );
 }

 // close file
 fclose(fp);
 
 // success
 return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
// LoadFromFile(CString filename, CImageSequence *seq, CCATineDlg *m_pTineDlg)
//
// This function just reads in the IMC file 'filename and stores the frames 
// into the ImageSequence seq 
//
// Returnvalue
//  True  : file was successfully read  
//  False : file was not successfully read  
///////////////////////////////////////////////////////////////////////////////
BOOL CIMCFileHelper::LoadFromFile(CString filename, CImageSequence *seq, CCATineDlg *m_pTineDlg, BOOL tinealinit, CTineALDlg *tinealdlg, CVideoclntDlg *maindlg)
{
 FHDR fHdr;
 IMHDR imHdr;
 CImageSequence *new_seq;
 
 // create new temporarily sequence
 new_seq = new CImageSequence(m_pTineDlg, tinealinit, tinealdlg, maindlg);
 if (new_seq == NULL)
 {
  return FALSE;
 }

 // open file in read binary mode
 FILE *fp = fopen( (const char *) (LPCTSTR) filename, "rb");
 if (fp == NULL)
 {
  delete new_seq; 
  return( FALSE );
 }

 // read in file header
 int ret = fread( &fHdr, 1, sizeof(FHDR), fp);

 if (ret != sizeof(FHDR))
 {
  delete new_seq;
  fclose(fp);
  return( FALSE );
 }

 // loop over number of frames
 for (unsigned int i=0;i<fHdr.num_frames;i++)
 {
  // read in image header of image # i 
  ret = fread( &imHdr, 1, sizeof(IMHDR), fp);

  if (ret != sizeof(IMHDR)) // error while reading ? -> clean up and close
  {
   delete new_seq; 
   fclose(fp);
   return FALSE;
  }

  unsigned char *compressed = (unsigned char *) memman.sw_malloc( imHdr.len_compressed );
  if (compressed == NULL) // error while allocating ? -> clean up and close
  {
   delete new_seq; 
   fclose(fp);
   return FALSE;
  }

  ret = fread( (void *) compressed, 1, imHdr.len_compressed, fp);
  if (ret != ((int)imHdr.len_compressed)) // error while reading ? -> clean up and close
  {
   delete new_seq; 
   memman.sw_free( (void *) compressed );
   fclose(fp);
   return( FALSE );
  }

  unsigned char *uncompressed = (unsigned char *) memman.sw_malloc( imHdr.len_uncompressed + sizeof(VIDHDR) );
  if (uncompressed == NULL) // error while allocating ? -> clean up and close
  {
   delete new_seq; 
   memman.sw_free( (void *) compressed);
   fclose(fp);
   return( FALSE );
  }

  // build up proper video header for image
  VIDHDR vidH;

  // upper 16 bits of file header 'bpp' filled -> this is the effective bpp
  if (fHdr.bpp&0xffff0000)
  {
	  vidH.bpp_effective = (fHdr.bpp>>16)&0xffff;
	  vidH.bpp = fHdr.bpp & 0xffff;
  }
  else
  {
	  vidH.bpp = fHdr.bpp;
	  vidH.bpp_effective = vidH.bpp;
  }
  
  vidH.heightframe = fHdr.height;
  vidH.widthframe = fHdr.width;
  vidH.cameraport = -3;
  vidH.scale = imHdr.scale;
  vidH.len_fmthdr = 0;
  vidH.len_total = sizeof(VIDHDR) + imHdr.len_uncompressed;
  vidH.len_data = imHdr.len_uncompressed;
  vidH.compressed = 0;
  vidH.compression = 0;
  vidH.frameno = 0;
  vidH.kennung = 0x11223344;
  
  // attach video header to front of memory location
  memman.memcpysw( (char *) uncompressed, (char *) &vidH, sizeof(VIDHDR) );

  unsigned char *uncompr_data = uncompressed + sizeof(VIDHDR);

  // uncompress video frame bits
  int retc = uncompress ((Bytef *) uncompr_data, (unsigned long *) &imHdr.len_uncompressed, (const Bytef *) compressed, imHdr.len_compressed);
  if (retc != Z_OK)
  {
   delete new_seq; 
   memman.sw_free( (void *) uncompressed );
   memman.sw_free( (void *) compressed );
   fclose(fp);
   return( FALSE );
  }

  // add uncompressed frame to temp sequence
  new_seq->Add( uncompressed );
  // set scale value accordingly
  new_seq->SetScale( imHdr.scale );

  // free compressed memory location
  memman.sw_free( (void *) compressed );
 }

 // now all frames are properly read in, so copy new sequence to parameter passed sequence
 // and delete new sequence

 seq->EmptySequence();
 
 int cnt = new_seq->Count();

 // copy all frames
 for (int k=0;k<cnt;k++)
 {
  char *frame;
  new_seq->GetFrame(k, (const char **) &frame);
  seq->Add( (unsigned char *) frame );
 }
 
 // copy scale from new sequence to sequence
 seq->SetScale( new_seq->GetScale() );

 // remove all frames from new sequence
 for (int m=0;m<cnt;m++)
 {
  new_seq->RemoveFrame(0);
 }
 
 // free new sequence
 delete new_seq;

 // close file
 fclose( fp );

 // success
 return( TRUE );  
}
