// ImageAcquire.cpp: implementation of the CImageAcquire class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <winsock.h>
#include <vfw.h>
#include <math.h>
#include "Compression.h"
#include "headers.h"
#include "MemoryManagement.h"
#include "resource.h"
class CVideoclntDlg;
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "Queue.h"
#include "BogusTineDlg.h"
#include "Server.h"
#include "SSyncSocket.h"
#include "videoclnt.h"
#include "IniEx.h"
#include "ImageAcquire.h"
#include "ImageSequence.h"
#include "AdditionalInformation.h"
#include "StatFile.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "PrinterSettings.h"
#include "MarkedComboBox.h"
#include "AboutDialog.h"
#include "CATineDlg.h"
#include "InformationDialog.h"
#include "videoclntDlg.h"
#include "rpctypes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CImageAcquire::CImageAcquire( CIniEx *global, CCATineDlg *tinedlg )
{
 // initialize reference to bogus tine dialog
 m_pTineDlg = NULL;
 // not yet initialized 
 m_bSuccessfulInitialized = FALSE;
 
 // default transfer mode: sockets
 m_bUseTine = FALSE;

 m_pCompression = NULL;
 m_pOutFmt = NULL;

 m_pTineRateDlg = tinedlg;

 m_pIniFileGlobal = global;
}


// plain destruction
CImageAcquire::~CImageAcquire()
{

}


///////////////////////////////////////////////////////////////////////
// CheckIP - checks whether one of the IPs of the computer is on the 
// 141.34.30.xx subnet
//
// Return value:
//  TRUE  : one of the IPs is on the 141.34.30.xx subnet
//  FALSE : PC is not connected on the 141.34.30.xx subnet
//
/*
BOOL CImageAcquire::CheckIP()
{
	// get hostname
	char ac[80];
    if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) {
        return FALSE;
    }

    // get hostentry structure for hostname   
    struct hostent *phe = gethostbyname(ac);
    if (phe == 0) {
        return FALSE;
    }

    // seek through hostentry structure and check IPs 
    for (int i = 0; phe->h_addr_list[i] != 0; i++) 
	{
        struct in_addr addr;
        memcpy(&addr, phe->h_addr_list[i], sizeof(struct in_addr));

        if (addr.S_un.S_un_b.s_b1 == 141 && addr.S_un.S_un_b.s_b2 == 34 && addr.S_un.S_un_b.s_b3 == 30)
		{ 
		 // got it !
		 return( TRUE );
		}

    }

	return( FALSE );
}
*/
///////////////////////////////////////////////////////////////////////
// checkFile() - checks whether the given file is newer than the 
// specified date/time
//
// Return value:
//  TRUE  : file is newer than the specified date
//  FALSE : file is older than the specified date
//
/*
BOOL CImageAcquire::checkFile( CString file, int year, int month, int day, int hour, int minute)
{
 CFileStatus status;
 if( CFile::GetStatus( (LPCTSTR) file, status ) )
 {
       CTime t(status.m_mtime);
       if (t.GetYear() < year) return( FALSE );
	   if (t.GetYear() > year) return( TRUE );
	   if (t.GetYear() == year)
	   {
	    if (t.GetMonth() < month) return( FALSE );
		if (t.GetMonth() > month) return( TRUE );
		if (t.GetMonth() == month)
		{
		 if (t.GetDay() < day) return( FALSE );
		 if (t.GetDay() > day) return( TRUE );
		 if (t.GetDay() == day)
		 {
		  if (t.GetHour() < hour) return( FALSE );
		  if (t.GetHour() > hour) return( TRUE );
		  if (t.GetHour() == hour) 
		  {
		   if (t.GetMinute() < minute) return( FALSE );
		   if (t.GetMinute() > minute) return( TRUE );
		   if (t.GetMinute() == minute) return( TRUE );
          }
         }
		}
	   }
	   
 }
 return FALSE;
}
*/

///////////////////////////////////////////////////////////////////////
// checkDLLs() - checks whether the required TINE dlls are new enough 
// to support TINE multicast
//
// Return value:
//  TRUE  : all DLLs are new enough
//  FALSE : at least one of the DLLs failed the date/time test or is 
//          too old
//
/*
BOOL CImageAcquire::CheckDLLs()
{
    char buf[MAX_PATH+1];

    UINT ret = GetSystemDirectory( buf, MAX_PATH );
    if (ret == 0) return( FALSE );

    CString path(buf);

    if (path.GetAt( path.GetLength()-1 ) != '\\')
    {
     path = path + "\\";
    }

    if (checkFile( path + "Acop.dll", 2002, 11, 29, 14, 40) == FALSE) return( FALSE );
    if (checkFile( path + "Acop.ocx", 2002, 12, 2, 13, 7) == FALSE) return( FALSE );
    if (checkFile( path + "Eqp32.dll", 2002, 12, 9, 12, 46) == FALSE) return( FALSE );

    return TRUE;

}
*/

///////////////////////////////////////////////////////////////////////
// AutoDetectMulticast() - checks whether the IP is on the required subnet
// and whether all DLLs are up to date
//
// Return value:
//  TRUE  : TINE multicast can be enabled, because every check succeeded
//  FALSE : TINE multicast can not be enabled - the PC is either on the 
//          wrong subnet or the TINE DLLs are not new enough
//
/*
BOOL CImageAcquire::AutoDetectMulticast()
{
  if (CheckIP() == TRUE && CheckDLLs() == TRUE) return( TRUE );

  return FALSE;
}
*/

///////////////////////////////////////////////////////////////////////
// Init() - initializes image acquire subsystem
//
// Parameter:
//  useTINE : overrides autodetection of TINE, if FALSE the use socket
//            transfer no matter what TINE autodetection may decide
//
// Return value:
//  TRUE  : subsystem is initialized properly
//  FALSE : error while initializing subsystem (this is FATAL)
//
BOOL CImageAcquire::Init( BOOL useTine, CString TinePollMode )
{
 // autodetect TINE multicast mode
 //m_bUseTine = AutoDetectMulticast();
 
 m_sTinePollMode = TinePollMode;

 m_bUseTine = useTine;

 // override if useTINE is FALSE, then use socket transfer mode
 //if (useTine == FALSE) m_bUseTine = FALSE;
 //m_bUseTine = FALSE;

 // construct bogus tine dialog class
 m_pTineDlg = new CBogusTineDlg();
 if (m_pTineDlg == NULL)
 {
  return FALSE;
 }
 
 // create bogus tine dialog
 BOOL ret = m_pTineDlg->Create(IDD_DIALOG2_BOGUSTINE, AfxGetMainWnd()->GetDesktopWindow() );
 if (ret == FALSE)
 {
  delete m_pTineDlg;
  return FALSE;
 }
 
 // everything fine, so we are successfully initialized
 m_bSuccessfulInitialized = TRUE;

 // initialize socket reference to no reference yet 
 m_pSock = NULL;

 // no poll and no grab mode running at the moment
 m_bPollMode = FALSE;
 m_bGrabMode = FALSE;

 // success
 return( TRUE );
}

///////////////////////////////////////////////////////////////////////
// Done() - deinitializes image acquire subsystem
//
//
// Return value:
//  TRUE  : subsystem is deinitialized properly
//  FALSE : error while deinitializing subsystem (not returned at the moment)
//
BOOL CImageAcquire::Done()
{
 // not yet initialized: so done was successfull
 if (m_bSuccessfulInitialized == FALSE) return( TRUE );
 
 // destroy bogus tine dialog 
 if (m_pTineDlg != NULL)
 {
  m_pTineDlg->DestroyWindow();
  delete m_pTineDlg;
 }

 // deinitialize compression
 DeinitializeCompression();

 // not initialized any more
 m_bSuccessfulInitialized = FALSE;

 // success
 return( TRUE );
}

///////////////////////////////////////////////////////////////////////
// StartPollMode() - starts up poll mode on 'the_server' and put frames
//                   into pollqueue
//
// Return value:
//  TRUE  : poll mode is started up
//  FALSE : poll mode could not be started
//
BOOL CImageAcquire::StartPollMode( CServer the_server, CQueue *pollqueue )
{
 BOOL ret;
 int w, h, bpp;

 // class not yet initialized -> return FALSE
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 if (GetWidthHeightBppFromServer(w, h, bpp, the_server) == FALSE) return FALSE;

 // initialize compression 
 ret = InitializeCompression( w, h, bpp );
 if (ret == FALSE)
 {
  return FALSE;
 }

 // put compression parameters into bogus tine dialog class
 m_pTineDlg->SetCompressionClass( m_pCompression, m_pOutFmt);

 if (m_bUseTine == TRUE) // TINE multicast ? 
 {
  // start up TINE multicast transfer
  int maxtinesize = (w*h*bpp/8)+sizeof(VIDHDR);
  ret = m_pTineDlg->StartTinePollTransfer(the_server.getTineContext(), the_server.getTineFrameEqpName(), "device_0", "FRAME.GET", the_server.getServername(), pollqueue, maxtinesize, m_sTinePollMode );
 }
 else
 {
  // start up socket transfer
  ret = StartSocketPollTransfer( the_server.getServername(), the_server.getListenport(), pollqueue);
 }

 if (ret == TRUE) //success ? 
 {
  m_bPollMode = TRUE; // poll mode is running
 }

 return( ret );
}


///////////////////////////////////////////////////////////////////////
// SwitchServer() - switches the server while poll mode is running, switch
//                  to the 'new_server' and put frames into 'new_queue'.
//
// Return value:
//  TRUE  : server was switched
//  FALSE : server was not switched
//
BOOL CImageAcquire::SwitchServer( CServer new_server, CQueue *new_queue )
{
 BOOL retval=TRUE;

 // grab mode is running, so don't switch
 if (m_bGrabMode == TRUE) return( FALSE );
 
 // poll mode running ? 
 if (m_bPollMode == TRUE)
 {
  // then stop it
  BOOL ret1 = StopPollMode();
  // error? -> return FALSE
  if (ret1 == FALSE) retval = FALSE;
  // start up poll mode using 'new_server' and 'new_queue'
  ret1 = StartPollMode( new_server, new_queue);
  // error ? -> return FALSE
  if (ret1 == FALSE) retval = FALSE;
 }
 
 return retval;
}

///////////////////////////////////////////////////////////////////////
// StopPollMode() - stops previously started poll mode
//
// Return value:
//  TRUE  : poll mode was shut down properly
//  FALSE : poll mode could not be stopped
//
BOOL CImageAcquire::StopPollMode( )
{
 BOOL retval;
	// not yet initialized: return FALSE
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 if (m_bUseTine == TRUE) // tine multicast transfer ?
 {
  // stop tine polling
  retval = m_pTineDlg->StopTinePollTransfer();
 }
  else
 {
  // stop socket polling
  retval = StopSocketPollTransfer(); 
 }

 DeinitializeCompression();

 return( retval );
}

///////////////////////////////////////////////////////////////////////
// GrabFrames() - starts up grab mode on 'the_server' and put 'numframes' 
//                frames into 'grab_queue'; timeout is calculated using
//                framerate (Hz)
//
// Return value:
//  TRUE  : grab mode is started up
//  FALSE : grab mode could not be started
//
BOOL CImageAcquire::GrabFrames( CServer the_server, CQueue *grab_queue, int numframes, int framerate )
{
 BOOL ret;
 int w, h, bpp;

 // not yet initialized: return FALSE
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );

 if (GetWidthHeightBppFromServer(w, h, bpp, the_server) == FALSE) return FALSE;

 // initialize compression 
 ret = InitializeCompression( w, h, bpp );
 if (ret == FALSE)
 {
  return FALSE;
 }

 // put compression parameters into bogus tine dialog class
 m_pTineDlg->SetCompressionClass( m_pCompression, m_pOutFmt);
 
 if (m_bUseTine == TRUE) // TINE multicast mode ?
 {
  // start grab mode using TINE multicast

  int maxtinesize = (w*h*bpp/8)+sizeof(VIDHDR);

  ret = m_pTineDlg->StartTineGrabTransfer(the_server.getTineContext(), the_server.getTineFrameEqpName(), "device_0", "FRAME.GET", the_server.getServername(), grab_queue, numframes, maxtinesize, m_sTinePollMode); 
 }
  else
 {
  // start up socket grab mode
  ret = StartSocketGrabMode( the_server.getServername(), the_server.getListenport(), grab_queue, numframes, AutoDetectFramerate(framerate));
 }

 if (ret == TRUE) // success ? 
 {
  m_bGrabMode = TRUE; // grab mode is running
 }
 
 return ret;
}


///////////////////////////////////////////////////////////////////////////////
// GrabFramesCleanup()
//
// This function stops a grab mode (if falure happened) and cleans up after
// a successful grab mode. This function has to be called after finishing a 
// grab mode.
// 
// Return value:
//   TRUE - cleanup was successful
//  FALSE - cleanup was not successful
//
BOOL CImageAcquire::GrabFramesCleanup()
{
 BOOL ret;

 // not yet initialized? -> return FALSE
 if (m_bSuccessfulInitialized == FALSE) return( FALSE );
 
 DeinitializeCompression();

 if (m_bUseTine == TRUE) // TINE multicast mode ? 
 {
  // clean up after TINE grab mode
  ret = m_pTineDlg->StopTineGrabTransferCleanup();
 }
 else
 {
  // clean up after socket grab mode
  ret = StopSocketGrabModeCleanup();
 }

 // grab mode is not running any more
 m_bGrabMode = FALSE;

 return ret;
}

///////////////////////////////////////////////////////////////////////////////
// StopSocketGrabModeCleanup()
//
// This function stops a grab mode (if falure happened) and cleans up after
// a successful grab mode. This function has to be called after finishing a 
// socket grab mode.
// 
// Return value:
//   TRUE - returns always true, but cleans up only if necessary
//
BOOL CImageAcquire::StopSocketGrabModeCleanup()
{
  // close the socket
  m_pSock->Close();
  
  // wait for grab thread to finish (if timeout occured)
  while( m_bGrabThreadRunning == TRUE)
  {
	  ::Sleep(50);
  }
  
  // clean up socket
  delete m_pSock;
  m_pSock = NULL;

  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the static function used to initialize the thread function 
// thread_receive(). This function just does a dynamic cast to call 
// the proper thread_receive function.
//
UINT CImageAcquire::thread_receive_static(LPVOID *p)
{
	return reinterpret_cast<CImageAcquire *>(p)->thread_receive();
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the actual receive thread which reads in the frames that arrive on the 
// socket when in poll mode. After succesfully reading a frame it is transferred 
// into the video queue. In addition, a message is posted to notify the main thread
// that there is a new frame to be processed.
//
// muss noch dekomprimieren
UINT CImageAcquire::thread_receive()
{
 VIDHDR vidH;
 char *memblock;
 int ret=0;
 //char stringbuf[200];
 int firstfrm = -1;

 m_bReceiveThreadRunning = TRUE;

 for (;;) // thread loop
 {
  /* receive video header */
  vidH.kennung = 0;
  ret = 0;
  ret = m_pSock->Receive((void *)&vidH, sizeof(VIDHDR));
  if (ret == SOCKET_ERROR) 
  {
    int tmp1 = WSAGetLastError();

	if (tmp1 != WSAEINTR) 
	{
		break;
	}
	 else
	{
	}
  }
  else if (ret == 0)
  {
   ret = SOCKET_ERROR; 
   break;
  }

  if (firstfrm == -1) firstfrm = vidH.frameno;

  // create memory location for the frame to be acquired
  memblock = (char *) memman.sw_malloc( vidH.len_total );
  if (memblock == NULL) 
  {
	  ret = SOCKET_ERROR;
	  break;
  }
  memman.memcpysw((char *) memblock, (char *) &vidH, sizeof(VIDHDR) ); 

  // read in the frame data
  char *moveptr = memblock + sizeof(VIDHDR);
  ret = m_pSock->Receive((void *) moveptr, vidH.len_total - sizeof(VIDHDR));
  if (ret == SOCKET_ERROR) 
  {
    int tmp = WSAGetLastError();

	if (tmp != WSAEINTR) 
	{
		break;
	}
	 else
	 {
	 }
  }
  else if (ret == 0)
  {
	  ret = SOCKET_ERROR;
      break;
  }
  
  if (vidH.kennung == 0x11223344) // okay, it is a valid frame
  {
	    // if there are more than 10 frames waiting to be processed it is most likely that
	    // the main application/operating system is busy doing something else
	    // we better drop the frames so that the queue does not grow too much
	    // (!) here we are dropping frames (!) 
	    int size = m_pSocketPollQueue->GetSize();
	    if (size > 10)
	    {
		    for (int l=0;l<10;l++)
		    {
		  	    char *tmpptr = (char *) m_pSocketPollQueue->RemoveHead();
			    if (tmpptr != NULL) memman.sw_free( (void *) tmpptr );
            }
	    }

        char *decompblock = (char *) memman.sw_malloc( (vidH.heightframe*vidH.widthframe*vidH.bpp/8)+sizeof(VIDHDR)+20 );
		char *destptr = decompblock;
		
        destptr += sizeof(VIDHDR);

        if (decompblock != NULL)
		{
 	     char *ptr_fmthdr=NULL;
	     char *ptr_frame=NULL;

  	     if (vidH.compressed == 1) // frame compressed ? 
	     {
		   // build up valid pointers
	       ptr_fmthdr = memblock + sizeof(VIDHDR);
           ptr_frame = memblock + sizeof(VIDHDR) + vidH.len_fmthdr;
 
   	       // decompress it
           ret = m_pCompression->DecompressFrame( (LPBITMAPINFOHEADER) ptr_fmthdr, ptr_frame, m_pOutFmt, destptr);
	     } 
         else // frame is uncompressed
	     {
           ptr_frame = memblock + sizeof(VIDHDR);
           memman.memcpysw(destptr, ptr_frame, vidH.heightframe*vidH.widthframe*vidH.bpp/8 );
	     }
 	     
		 vidH.compressed = 0;
		 
		 if (vidH.bpp == 8) vidH.bpp_effective = 8; // SW! Sept 20, 2006 workaround for old servers!
		 
		 memman.memcpysw( (char *) decompblock, (char *) &vidH, sizeof(VIDHDR) );
         
		 
		 // put last acquired frame into queue
  	     m_pSocketPollQueue->AddTail( (void *) decompblock );
		 //memman.sw_free( (void *) decompblock );
	     // post notify message
	     AfxGetMainWnd()->PostMessage(ID_MSG_FRAMEARRIVED, 0, 0);
	    }// if (memblock != NULL)

  }
  else
  {
  }

  memman.sw_free( (void *) memblock );
  memblock = NULL;
 }

 // if there is a socket error and we are not forcing the thread to stop (and thus 
 // created a socket error by ourselves (!), post the error notify message
 if (memblock != NULL) memman.sw_free( (void *) memblock );

 if (ret == SOCKET_ERROR && m_bForceReceiveThreadToStop == FALSE)
 {
  AfxGetMainWnd()->PostMessage(ID_MSG_SOERR, WSAGetLastError(), NULL);
 }

 // thread is not running any more
 m_bReceiveThreadRunning = FALSE;
 return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the static function used to initialize the thread function 
// thread_grab(). This function just does a dynamic cast to call 
// the proper thread_grab() function.
//
UINT CImageAcquire::thread_grab_static(LPVOID *p)
{
	return reinterpret_cast<CImageAcquire *>(p)->thread_grab();
}

/////////////////////////////////////////////////////////////////////////////
//
// This is the actual grab thread which reads in the frames that arrive on the 
// socket when in grab mode. It also grabs background images. It reads up to 
// m_iGrabThreadCount frames. After succesfully reading the frame(s) the thread 
// shuts down itself. You can shut down the thread hard by closing the socket and 
// waiting for m_bGrabThreadRunning to be set to false.
//
UINT CImageAcquire::thread_grab()
{
 char *destptr;
 VIDHDR vidH;
 char *memblock;
 int ret=0;

 // thread is running
 m_bGrabThreadRunning = TRUE;
 // 0 pictures grabbed up to now
 m_iGrabThreadCaptured = 0;

 // for-loop to grab up to m_iGrabThreadCount frames
 for (int i=0;i<m_iGrabThreadCount;i++)
 {
  vidH.kennung = 0;
  ret = 0;
  // read in video header
  ret = m_pSock->Receive((void *)&vidH, sizeof(VIDHDR));
  if (ret == SOCKET_ERROR) 
  {
    int tmp1 = WSAGetLastError();

	if (tmp1 != WSAEINTR) 
	{
		break;
	}
	 else
	{
	}
  }
  else if (ret == 0)
  {
	ret = SOCKET_ERROR; break;
  }

  // allocate memory required for whole frame
  memblock = (char *) memman.sw_malloc( vidH.len_total );
  if (memblock == NULL)
  {
	  ret = SOCKET_ERROR;
	  break;
  }
  // copy video header into storage location
  memman.memcpysw((char *) memblock, (char *) &vidH, sizeof(VIDHDR) ); 
  char *moveptr = memblock + sizeof(VIDHDR);

  // obtain the rest of the frame
  ret = m_pSock->Receive((void *) moveptr, vidH.len_total - sizeof(VIDHDR));
  if (ret == SOCKET_ERROR) 
  {
    int tmp = WSAGetLastError();

	if (tmp != WSAEINTR) // any error except WSAEINTR will shut down grab thread
	{
		break;
	}
  }
  else if (ret == 0) // connection shut down
  {
	  ret = SOCKET_ERROR; break; // return socket error and leave for-loop
  }
  
  if (vidH.kennung == 0x11223344) // okay, it is a frame
  {
    destptr = (char *) memman.sw_malloc( (vidH.heightframe*vidH.widthframe*vidH.bpp/8)+sizeof(VIDHDR)+20 );
	if (destptr != NULL)
	{
	 	// copy video header into memory location in which the frames will be stored
	    
		if (vidH.bpp == 8) vidH.bpp_effective = 8; // SW! Sept 20, 2006 workaround for old servers!
		
		memman.memcpysw( destptr, (char *) &vidH, sizeof(VIDHDR) );
	    
	    char *ptr_fmthdr=NULL;
	    char *ptr_frame=NULL;

  	    if (vidH.compressed == 1) // frame compressed ? 
	    {
          // build up valid pointers
	      ptr_fmthdr = memblock + sizeof(VIDHDR);
          ptr_frame = memblock + sizeof(VIDHDR) + vidH.len_fmthdr;
	      
		  // decompress it
          ret = m_pCompression->DecompressFrame( (LPBITMAPINFOHEADER) ptr_fmthdr, ptr_frame, m_pOutFmt, destptr+sizeof(VIDHDR));
	    } 
        else // frame is uncompressed
	    { 
          ptr_frame = memblock + sizeof(VIDHDR);
          memman.memcpysw(destptr+sizeof(VIDHDR), ptr_frame, (vidH.widthframe*vidH.heightframe*vidH.bpp/8) );
	    }
        // put frame into grab queue

        m_pSocketGrabQueue->AddTail( (void *) destptr );
     
	    // increment grabbed frames counter
	    m_iGrabThreadCaptured++;

		AfxGetMainWnd()->PostMessage(ID_MSG_FRAMEGRABBED, m_iGrabThreadCaptured, m_iGrabThreadCount );
	}
  }

  // free temporary frame storage
  if (memblock != NULL) { memman.sw_free( (void *) memblock ); memblock = NULL; }
  
 }//for
 
 if (memblock != NULL) { memman.sw_free( (void *) memblock ); memblock = NULL; }

 m_bGrabThreadRunning = FALSE;

 return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
// StartSocketGrabMode() - starts up a socket-based grab transfer
//
// Parameters:
//  hostname       - host name of the server computer
//  listenport     - listen port of the server computer 
//  the_queue      - reference to the queue where to put the grabbed frames
//  numframes      - number of frames to grab
//  framerate      - framerate in Hz of the server (needed for calculating timeouts)
//
// Return value:
//  TRUE  : socket grab transfer is started up
//  FALSE : error while starting up grab transfer
//
BOOL CImageAcquire::StartSocketGrabMode(CString hostname, int listenport, CQueue *the_queue, int numframes, int framerate)
{
  CString msg;
  
  CString servername = hostname;
  int iPortnumber = listenport;
  m_pSocketGrabQueue = the_queue;

  // if there is no socket instantiated: instantiate it
  if (m_pSock == NULL) 
  {
   m_pSock = new SSyncSocket();
  }

  if (m_pSock == NULL) return FALSE;

  // create the socket 
  BOOL ret = m_pSock->Create();
  if (ret == FALSE) 
  {
    ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Unable to create Socket.");
    return FALSE;
  }
  
  // connect to the server
  BOOL sockRet = m_pSock->Connect(servername, iPortnumber);

  if (sockRet == FALSE) // there was an error ? 
  {
    // build base error message
    CString baseerr = "Could not connect to server. ";
    CString msg;

	// obtain detailed error code to build specific error message
    int errorcode = m_pSock->GetLastError();

    switch( errorcode )
	{
	      case 10061: // refused
		       msg = "Connection refused.";
			   break; 
 
		  case 10051: // net unreachable
			   msg = "Network is unreachable.";
			   break;
 
	      case 10060: // timeout
		       msg = "Connection attempt timed out.";
			   break;

		  case 10049: //WSAEADDRNOTAVAIL
			   msg = "Address is not valid.";
			   break;

		  case 11001: // WSAHOST_NOT_FOUND
			   msg = "Host not found.";
               break;

	      default:
		       msg.Format("Unknown error. Errorcode = %d (%x h)", errorcode, errorcode );
			   break;
	}

	// print out error message
	((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, (LPCTSTR) (baseerr+msg));

	// clean up the socket
	m_pSock->Close();
	delete m_pSock;
	m_pSock = NULL;

	return FALSE; // error
  }

  // initialize proper grab thread variables
  m_iGrabThreadCount = numframes; // we want exactly 1 frame
  m_iGrabThreadCaptured = 0; // up to now no pictures were grabbed
  m_bGrabThreadRunning = TRUE; // we want the grab thread running

  int timeoutperframe = 1000 / framerate;

  // fire up grab thread
  CWinThread *m_pThread = AfxBeginThread((AFX_THREADPROC) thread_grab_static, this);

  // get system tick count (time)
  DWORD start = ::GetTickCount();
  DWORD akt = start;

  // wait for either timeout or end of grab thread (m_bGrabThreadRunning becomes FALSE)
  while( m_bGrabThreadRunning == TRUE && akt <= (start+(timeoutperframe*numframes)+1200))
  {
	  ::Sleep(100);
	  akt = ::GetTickCount();
  }

  if (akt > (start+(timeoutperframe*numframes)+1200))
  {
   AfxGetMainWnd()->PostMessage(ID_MSG_SOERRGRAB, 0, 2); // post error message
  }
  else if (m_iGrabThreadCaptured != numframes ) // error while grabbing? 
  {
   AfxGetMainWnd()->PostMessage(ID_MSG_SOERRGRAB, 0, 0); // post error message
  }
  else
  { // everything fine ? 
   AfxGetMainWnd()->PostMessage(ID_MSG_GRABFINISHED, 0, 0); // success message
  }

  return TRUE; // success
}

/////////////////////////////////////////////////////////////////////////////
// StartSocketPollTransfer() - starts up a socket-based poll transfer
//
// Parameters:
//  hostname       - host name of the server computer
//  listenport     - listen port of the server computer 
//  the_poll_queue - reference to the queue where to put polled frames
//
// Return value:
//  TRUE  : socket poll transfer is started up
//  FALSE : error while starting up polling 
//
BOOL CImageAcquire::StartSocketPollTransfer(CString hostname, int listenport, CQueue *the_poll_queue)
{

	// copy queue reference
	  m_pSocketPollQueue = the_poll_queue;

	  // initialize receive thread variables, so that the receive thread is 
	  // started up properly
	  m_bForceReceiveThreadToStop = FALSE;
	  m_bReceiveThreadRunning = FALSE;
	  
	  // if the socket is not instantiated, then instantiate it 
	  if (m_pSock == NULL) 
	  {
	   m_pSock = new SSyncSocket();
	  }

	  if (m_pSock == NULL) return FALSE;
	  
	  // get server name and port number from ini file
	  CString msg;

	  CString servername = hostname;
	  int iPortnumber = listenport;

	  // prints out message "Connecting to servername"
      //msg.Format("Connecting to %s:%d ...", servername, iPortnumber );
      //PrintServerMessage(msg);

	  // create the socket
      BOOL ret = m_pSock->Create();
      if (ret == FALSE) 
	  {
       ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Unable to create Socket.");
       //PrintServerMessage("Unable to create socket.");
       return FALSE;
	  }
    
	  // connect to server
	  BOOL sockRet = m_pSock->Connect(servername, iPortnumber);

      if (sockRet == FALSE) // error while connecting ? 
	  {
       // build error message consisting of base error message and a detailed one 
   	   CString baseerr = "Could not connect to server. ";
       CString msg;

	   // get last error from socket class to build proper detailed error message
       int errorcode = m_pSock->GetLastError();

       switch( errorcode )
	   {
	      case 10061: // refused
		       msg = "Connection refused.";
			   break; 
 
		  case 10051: // net unreachable
			   msg = "Network is unreachable.";
			   break;
 
	      case 10060: // timeout
		       msg = "Connection attempt timed out.";
			   break;

		  case 10049: //WSAEADDRNOTAVAIL
			   msg = "Address is not valid.";
			   break;

		  case 11001: // WSAHOST_NOT_FOUND
			   msg = "Host not found.";
			   break;

	      default:
		       msg.Format("Unknown error. Errorcode = %d (%x h)", errorcode, errorcode );
			   break;
	   }
       // prints error message
       //PrintServerMessage(baseerr+msg);
	   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION , (LPCTSTR) (baseerr+msg));
	  }
	  else
	  { // okay, everything went fine (we are connected)
   	   
		  // start up receive thread 
		  m_pReceiveThread = AfxBeginThread((AFX_THREADPROC) thread_receive_static, this); //, THREAD_PRIORITY_TIME_CRITICAL

	      return( TRUE );
	  }

	  // there was an error: return FALSE
	  return FALSE;

}

/////////////////////////////////////////////////////////////////////////////
// StopSocketPollTransfer() - stops a socket-based poll transfer
//
// Return value:
//  TRUE: socket poll transfer is stopped
//
// Remark:
//  This function returns always true.
BOOL CImageAcquire::StopSocketPollTransfer()
{
   // tell the receive thread that we want an immediate stop, so that the socket 
   // error handling routine ( OnSocketError() ) does not get called.
   m_bForceReceiveThreadToStop = TRUE;

   // for safety: if socket is properly initialized 
   if (m_pSock != NULL)
   {
     m_pSock->Close(); // close socket, so that the receive thread stops
     
	 // wait for the receive thread to shut down
     while( m_bReceiveThreadRunning == TRUE)
	 {
      ::Sleep(100);
	 }

	 // free up memory and deinitialize socket
     delete m_pSock;
     m_pSock = NULL;
   }

   return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//
// This function initializes the (de-)compression class. Using this class, 
// the compressed images which come from the socket can be decompressed.
//
BOOL CImageAcquire::InitializeCompression( int width, int height, int bpp)
{
  // allocate space needed for output format, which is needed to tell certain function
  // which format the decompressed bytes have

  m_pOutFmt = (LPBITMAPINFOHEADER) memman.sw_malloc( sizeof(BITMAPINFOHEADER) );
  if (m_pOutFmt == NULL) return( FALSE ); 
  memset( m_pOutFmt, 0, sizeof(BITMAPINFOHEADER));

  int biwidth = width;
  int biheight = height;

  // inistialize structure members 
  m_pOutFmt->biSize          = sizeof( BITMAPINFOHEADER);
  m_pOutFmt->biWidth		 = biwidth; // because of 24bit (3 bytes per pixel for the decompression routines)
  m_pOutFmt->biHeight	     = biheight; // we set the width to 768/3 = 256 
  m_pOutFmt->biPlanes        = 1;
  m_pOutFmt->biBitCount      = 24;
  m_pOutFmt->biCompression   = BI_RGB;
  m_pOutFmt->biSizeImage     = width*height*bpp/8;
  m_pOutFmt->biClrUsed       = 0;
  m_pOutFmt->biXPelsPerMeter = 0;
  m_pOutFmt->biYPelsPerMeter = 0;
  m_pOutFmt->biClrImportant  = 0;

  // initialize decompression class
  m_pCompression = new CCompression();
  if (m_pCompression == NULL) return( FALSE );
  if (m_pCompression->Init(MAKEFOURCC('H','F','Y','U')) == FALSE) return FALSE;

  // everything went fine
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//
// This function deinitializes the part of the code that is used for decompression 
// of the compressed frames. Mainly the memory is freed.
//
BOOL CImageAcquire::DeinitializeCompression()
{
 // CCompression class valid ? 
 if (m_pCompression != NULL)
 {
	m_pCompression->Done(); // deinitialize decompression
    delete m_pCompression; // destruct and free memory
 }
 m_pCompression = NULL;

 if (m_pOutFmt != NULL) memman.sw_free( (void *) m_pOutFmt ); // free memory
 m_pOutFmt = NULL;

 // return always true
 return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
// CanTineBeEnabled() - returns whether Tine multicast mode can be enabled
// 
// Return value:
//  TRUE - Tine is enabled
// FALSE - Tine is not enabled, but can be enabled
//
BOOL CImageAcquire::IsTineEnabled()
{
 return( m_bUseTine );
}

//////////////////////////////////////////////////////////////////////////////
// CanTineBeEnabled() - returns whether Tine multicast mode can be enabled
// 
// Return value:
//  TRUE - Tine can be enabled
// FALSE - Tine can not be enabled, because autodetection showed that the PC
//         is not able to do so (either files are out of date or the PC is 
//         not in the proper subnet
BOOL CImageAcquire::CanTineBeEnabled()
{
 return( m_bUseTine );
}

//////////////////////////////////////////////////////////////////////////////
// AutoDetectFramerate() - auto detects repetition rate (=framerate) of 
//                         experiment
// 
// Parameter:
//  (int) framerate =  0: use autodetection
//                  <> 0: use the framerate specified
// Return value:
//  (int) - Framerate in Hz
//
int CImageAcquire::AutoDetectFramerate(int framerate)
{
 // != 0: I will not use autodetection but the forced framerate in 'framerate'
 if (framerate != 0) return( framerate ); 

 //////////////////////////////////////////////////////////////////////////////////////////////////////////
 //

 // no reference to TINE dialog: error !
 if (m_pTineRateDlg == NULL) return( 1 );

 m_pTineRateDlg->m_CACOPCA.SetDeviceContext( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Context", "PITZ.DIAG") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceGroup( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Server", "REPRATE") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceName( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Device", "REP_RATE") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceProperty( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Property", "M")  );
 m_pTineRateDlg->m_CACOPCA.SetAccessMode("READ");
 m_pTineRateDlg->m_CACOPCA.SetAccessRate("1000");

 CString dt = m_pIniFileGlobal->GetValue("RepetitionRateRead", "Datatype", "INT");
 dt.MakeUpper();

 int resultint=1;
 int paramint=0;
 float paramfloat=0.0;
 CComVariant
	olvint(paramint),
	olvdx,
	dataxint(paramint),
	olvfloat(paramfloat),
	dataxfloat(paramfloat);

 // set proper access parameters

 long ret1;

 if (dt == "INT")
 {
	// call the server
	ret1 = m_pTineRateDlg->m_CACOPCA.Execute( &olvint, olvdx, olvdx, &dataxint, olvdx, olvdx);
    if (ret1 == 0) // server returned successful call
	{
      resultint = (int) olvint.lVal;
	}

 }
 else
 {
	ret1 = m_pTineRateDlg->m_CACOPCA.Execute( &olvfloat, olvdx, olvdx, &dataxfloat, olvdx, olvdx);    
    if (ret1 == 0) // server returned successful call
	{
      resultint = (int) olvfloat.fltVal;
	}
 }

 if (resultint == 0) resultint = 1;

 // clean up variants
 olvint.Clear();
 olvdx.Clear();
 dataxint.Clear();
 olvfloat.Clear();
 dataxfloat.Clear();

 // return framerate
 return( resultint );
}

//////////////////////////////////////////////////////////////////////////////
// AutoDetectFramerateFIT() - auto detects repetition rate (=framerate) of 
//                         experiment to be used for fitting
// 
// Parameter:
//  (int) framerate =  0: use autodetection
//                  <> 0: use the framerate specified
// Return value:
//  (int) - Framerate in Hz
//
// if the autodetection fails, the function returns 10 (10 Hz), so that the machine does
// not get overloaded by too much fitting
int CImageAcquire::AutoDetectFramerateFIT(int framerate)
{
 // != 0: I will not use autodetection but the forced framerate in 'framerate'
 if (framerate != 0) return( framerate ); 

 //////////////////////////////////////////////////////////////////////////////////////////////////////////
 //

 // no reference to TINE dialog: error !
 if (m_pTineRateDlg == NULL) return( 1 );

 m_pTineRateDlg->m_CACOPCA.SetDeviceContext( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Context", "PITZ.DIAG") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceGroup( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Server", "REPRATE") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceName( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Device", "REP_RATE") );
 m_pTineRateDlg->m_CACOPCA.SetDeviceProperty( m_pIniFileGlobal->GetValue("RepetitionRateRead", "Property", "M")  );
 m_pTineRateDlg->m_CACOPCA.SetAccessMode("READ");
 m_pTineRateDlg->m_CACOPCA.SetAccessRate("1000");

 CString dt = m_pIniFileGlobal->GetValue("RepetitionRateRead", "Datatype", "INT");
 dt.MakeUpper();

 int resultint=10;
 int paramint=0;
 float paramfloat=0.0;
 CComVariant
	olvint(paramint),
	olvdx,
	dataxint(paramint),
	olvfloat(paramfloat),
	dataxfloat(paramfloat);

 // set proper access parameters

 long ret1;

 if (dt == "INT")
 {
	// call the server
	ret1 = m_pTineRateDlg->m_CACOPCA.Execute( &olvint, olvdx, olvdx, &dataxint, olvdx, olvdx);
    if (ret1 == 0) // server returned successful call
	{
      resultint = (int) olvint.lVal;
	}

 }
 else
 {
	ret1 = m_pTineRateDlg->m_CACOPCA.Execute( &olvfloat, olvdx, olvdx, &dataxfloat, olvdx, olvdx);    
    if (ret1 == 0) // server returned successful call
	{
      resultint = (int) olvfloat.fltVal;
	}
 }

 if (resultint == 0) resultint = 10;

 // clean up variants
 olvint.Clear();
 olvdx.Clear();
 dataxint.Clear();
 olvfloat.Clear();
 dataxfloat.Clear();

 // return framerate
 return( resultint );
}


BOOL CImageAcquire::GetWidthHeightBppFromServer( int &width, int &height, int &bpp, CServer &srv )
{
	BOOL retval = FALSE;

	m_pTineRateDlg->m_CACOPCA.SetDeviceContext( (LPCTSTR) srv.getTineContext() );
	m_pTineRateDlg->m_CACOPCA.SetDeviceGroup( (LPCTSTR) srv.getTineControlName());
	m_pTineRateDlg->m_CACOPCA.SetDeviceName("GRABSRV");

    m_pTineRateDlg->m_CACOPCA.SetAccessMode(ACCMODE);
    m_pTineRateDlg->m_CACOPCA.SetAccessRate(ACCRATEWHB);

    // set up variant variables needed for the tine 
	// call 
   
    VARIANT widthdata, heightdata, bppdata, camsize, camtype, null0, size0, datatype0;
	long retw, reth, retbpp;

    VariantInit( &widthdata );
    V_VT( &widthdata ) = VT_I4;
    widthdata.lVal = 1;

	VariantInit( &heightdata );
    V_VT( &heightdata ) = VT_I4;
    heightdata.lVal = 1;

	VariantInit( &bppdata );
    V_VT( &bppdata ) = VT_I4;
    bppdata.lVal = 1;

    VariantInit( &camsize );
    V_VT( &camsize ) = VT_I4;
    camsize.lVal = 1;

	VariantInit( &camtype );
	V_VT( &camtype ) = VT_I4;
    camtype.lVal = CF_LONG;

	VariantInit( &null0 );
	V_VT( &null0 ) = VT_I1;
	null0.cVal = 1;

    VariantInit( &size0 );
	V_VT( &size0 ) = VT_I4;
	size0.lVal = 1;

	CString str1="BYTE";
	VariantInit( &datatype0 );
	V_VT( &datatype0 ) = VT_BSTR;
	V_BSTR( &datatype0 ) = str1.AllocSysString();

	// call the server
	m_pTineRateDlg->m_CACOPCA.SetDeviceProperty( (LPCTSTR) "C_WIDTH" );
	retw = m_pTineRateDlg->m_CACOPCA.Execute( &widthdata, camsize, camtype, &null0, size0, datatype0);

	m_pTineRateDlg->m_CACOPCA.SetDeviceProperty( (LPCTSTR) "C_HEIGHT" );
	reth = m_pTineRateDlg->m_CACOPCA.Execute( &heightdata, camsize, camtype, &null0, size0, datatype0);

	m_pTineRateDlg->m_CACOPCA.SetDeviceProperty( (LPCTSTR) "C_BPP" );
	retbpp = m_pTineRateDlg->m_CACOPCA.Execute( &bppdata, camsize, camtype, &null0, size0, datatype0);

	if ((retw == 0) && (reth == 0) && (retbpp == 0)) // server returned successful call
	{
		width = (int) widthdata.lVal;
		height = (int) heightdata.lVal;
		bpp = (int) bppdata.lVal;
		retval = TRUE;
	}

	VariantClear( &widthdata );
	VariantClear( &heightdata );
	VariantClear( &bppdata );

	VariantClear( &camsize );
	VariantClear( &camtype );
	VariantClear( &null0 );
	VariantClear( &size0 );
	VariantClear( &datatype0 );

	return( retval );
}

