// ImageAcquire.h: interface for the CImageAcquire class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IMAGEACQUIRE_H__A4654545_2D33_11D7_AFE1_0002B34B0F4F__INCLUDED_)
#define AFX_IMAGEACQUIRE_H__A4654545_2D33_11D7_AFE1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CImageAcquire  
{
public:
    // plain construction destruction
	CImageAcquire( CIniEx *global, CCATineDlg *tinedlg);
	virtual ~CImageAcquire();

    // TRUE: TINE can be enabled, FALSE: TINE can not be enabled
	BOOL CanTineBeEnabled();

    // autodetects framerate, if possible 
	int AutoDetectFramerate( int framerate );

    // autodetects framerate for fitting, if possible 
	int AutoDetectFramerateFIT( int framerate );
	
	// TRUE: TINE is enabled, FALSE: TINE is not enabled
	BOOL IsTineEnabled();

    // initializes the image acquire subsystem
	BOOL Init( BOOL useTine, CString TinePollMode );
	
	// deinitializes image acquire subsystem
	BOOL Done();

    // start poll mode on 'the_server' and put video frames into pollqueue
    BOOL StartPollMode( CServer the_server, CQueue *pollqueue);
	
	// switch to a different server, replace old queue by new_queue
	BOOL SwitchServer( CServer new_server, CQueue *new_queue );
	
	// stop poll mode
	BOOL StopPollMode( );

    // grab frames from server 'the_server', put frames into 'grab_queue', grab 'numframes' frames and
	// calculate timeout using 'framerate' framerate (Hz)
    BOOL GrabFrames( CServer the_server, CQueue *grab_queue, int numframes, int framerate );
	
	// clean up after grabbing has finished or after grabbing has produced an error
	BOOL GrabFramesCleanup();



private:
	BOOL GetWidthHeightBppFromServer( int &width, int &height, int &bpp, /*const!!! SW! Apr 26, 2006 */ CServer &srv );

	
	CIniEx * m_pIniFileGlobal;
	CCATineDlg * m_pTineRateDlg;
	CString m_sTinePollMode;
    // TRUE: multicast mode can be used, FALSE: multicast can not be used 
    //BOOL AutoDetectMulticast();
    
	// check whether IP number is capable for multicasting
	//BOOL CheckIP();
    
    // check whether TINE dll's are up to date (and thus, multicast can be used)
	//BOOL CheckDLLs();
    
	// called by CheckDLLs, checks whether a single file is newer than
	// year/month/day, hour:minute
	BOOL checkFile( CString file, int year, int month, int day, int hour, int minute);

	// compression variables
	// output format for decompressed frames
	LPBITMAPINFOHEADER m_pOutFmt;
	// reference to compression class
	CCompression * m_pCompression;
	
	// TRUE: grab mode is in progress
	BOOL m_bGrabMode;
	// TRUE: poll mode is in progress
	BOOL m_bPollMode;

    // queues for grabbed and polled frames (in socket mode) 
	CQueue * m_pSocketGrabQueue;
	CQueue * m_pSocketPollQueue;

    // TRUE: image acquire subsystem was initialized properly
	BOOL m_bSuccessfulInitialized;
	
	// reference to bogus TINE dialog (grabbing and polling using multicast)	
	CBogusTineDlg * m_pTineDlg;
	
	// TRUE: TINE multicast will be used for transfers
	BOOL m_bUseTine;

	// synchronous socket for getting the frames from the server
	// in socket mode 
	SSyncSocket *m_pSock;

	/* thread to continously receive images in socket mode */
	static UINT thread_receive_static(LPVOID *p);
	UINT thread_receive();
	CWinThread *m_pReceiveThread; // thread object
	BOOL m_bForceReceiveThreadToStop; // TRUE: receive thread should shut down itself
	BOOL m_bReceiveThreadRunning; // TRUE: receive thread is running

	/* thread to grab a certain number of images in socket mode */
	static UINT thread_grab_static(LPVOID *p);
	UINT thread_grab();
	CWinThread *m_pGrabThread;
	int m_iGrabThreadCaptured; // number of frames actually grabbed
	BOOL m_bGrabThreadRunning; // TRUE: grab thread is running
	int m_iGrabThreadCount;    // how many pictures should be grabbed

    // initialize (de-)compression, TRUE: successful 
    BOOL InitializeCompression( int width, int height, int bpp);
    // deinitialize (de-)compression, TRUE: successful 
	BOOL DeinitializeCompression();

    
    // starts up socket poll transfer using hostname:listenport and puts frames into 'the_poll_queue'
	// TRUE: successful
	BOOL StartSocketPollTransfer( CString hostname, int listenport, CQueue *the_poll_queue);
	
	// stops socket poll transfer
	BOOL StopSocketPollTransfer();

    // starts up socket grab transfer using hostname:listenport and puts 'numframes' frames into 'the_queue'
	// framerate is calculated using 'framerate'
	// TRUE: successful
    BOOL StartSocketGrabMode(CString hostname, int listenport, CQueue *the_queue, int numframes, int framerate);
	
	// must be called after grabbing has finished or produced an error
	BOOL StopSocketGrabModeCleanup();

};

#endif // !defined(AFX_IMAGEACQUIRE_H__A4654545_2D33_11D7_AFE1_0002B34B0F4F__INCLUDED_)
