// ImageSequence.cpp: implementation of the CImageSequence class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "videoclnt.h"
#include "headers.h"
#include "MemoryManagement.h"
#include <vfw.h>
#include <atlbase.h>
#include "rpcdefs.h"
#include "Queue.h"
#include "compression.h"
#include "BMPFileHelper.h"
class CVideoclntDlg;
class CImageSequence;
#include <windows.h>
#include <winsock.h>
#undef fd_set
#include "acop.h"
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "IniEx.h"
#include "Server.h"
#include "ServerList.h"
#include "SSyncSocket.h"
#include "BogusTineDlg.h"
#include "ImageAcquire.h"
#include "configdlg.h"
#include "Stat.h"
#include "FourierStat.h"
#include "StatFile.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "PrinterSettings.h"
#include "oleauto.h"
#include "AdditionalInformation.h"
#include "MarkedComboBox.h"
#include "InformationDialog.h"
#include "videoclntDlg.h"
#include "ImageSequence.h"
#include "IMCFileHelper.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CImageSequence::CImageSequence( CCATineDlg *catinedlg, BOOL tinealinit, CTineALDlg *tinealdlg, CVideoclntDlg *maindlg)
{
 // properties not yet acquired
 m_bDoocsPropertiesAcquired = FALSE;

 m_bTineALInitialized = tinealinit;
 m_pTineALDlg = tinealdlg;
 m_pMainDlg = maindlg;
 m_bPropReadoutSucceeded = FALSE;
 
 // set reference to TINE dialog for magnets
 m_pTineDlg = catinedlg;

 // set up default scale
 m_dScale = DEFAULT_SCALE_FACTOR;

 m_iBpp = 8;
 m_iBppEffective = 8;
}

CImageSequence::~CImageSequence()
{
 // throw away all frames
 EmptySequence();
}

////////////////////////////////////////////////////////////////////////
// Count() - returns the number of frames that are stored inside the 
//           image sequence
// 
int  CImageSequence::Count()
{
 return( m_Queue.GetSize() );
}

////////////////////////////////////////////////////////////////////////
// ReadDoocsFloat() - reads a single doocs float property
//
// Parameters: 
//  facility/device/location/property - address of the doocs property
//  resultdbl                         - the read out float is put there
//
// Return value:
//   TRUE: property was read out
//  FALSE: error during readout
//DEL BOOL CImageSequence::ReadDoocsFloat(CString facility, CString device, CString location, CString property, double &resultdbl)
//DEL {
//DEL  // no reference to TINE dialog: error !
//DEL  if (m_pTineDlg == NULL) return( FALSE );
//DEL 
//DEL  float param=0;
//DEL  CComVariant
//DEL 	olv(param),
//DEL 	olvdx,
//DEL 	datax(param);
//DEL 
//DEL  // set proper access parameters
//DEL  m_pTineDlg->m_CACOPCA.SetDeviceContext(facility);
//DEL  m_pTineDlg->m_CACOPCA.SetDeviceGroup(device);
//DEL  m_pTineDlg->m_CACOPCA.SetDeviceName(location);
//DEL  m_pTineDlg->m_CACOPCA.SetDeviceProperty(property);
//DEL  m_pTineDlg->m_CACOPCA.SetAccessMode("READ");
//DEL  m_pTineDlg->m_CACOPCA.SetAccessRate("250");
//DEL 
//DEL  // call the server
//DEL  long ret1 = m_pTineDlg->m_CACOPCA.Execute( &olv, olvdx, olvdx, &datax, olvdx, olvdx);
//DEL  
//DEL  if (ret1 == 0) // server returned successful call
//DEL  {
//DEL     resultdbl = (double) olv.fltVal;
//DEL  }
//DEL 
//DEL  // clean up variants
//DEL  olv.Clear();
//DEL  olvdx.Clear();
//DEL  datax.Clear();
//DEL 
//DEL  if (ret1 != 0) return FALSE;
//DEL 
//DEL  // success
//DEL  return TRUE;
//DEL }

////////////////////////////////////////////////////////////////////////
// AcquireDoocsProperties() - reads out all doocs properties and the 
//                            currents of the magnets, too
//
// Return value:
//   TRUE: properties were read out (always true)
//
// Remarks:
//   If properties could not read out then the variable of the
//   property that could not be read out becomes '-7123.0'.
//
BOOL CImageSequence::AcquireDoocsProperties()
{
 if (m_pTineALDlg == NULL) return FALSE;

 BOOL ret = FALSE;
 if (m_bTineALInitialized == TRUE) 
 {
	 ret = m_pTineALDlg->returnWholePropertyStringForDAQ( m_sPropStr );
 }

 ret = m_pTineALDlg->snapshotAdditionalPropertiesToAdditionalPropertyStructure();

 if (ret == TRUE) 
 {
	 m_bDoocsPropertiesAcquired = TRUE;
 }
 else
 {
    m_pMainDlg->OnDaqPublic();
	((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Acquiring properties for DAQ function failed.\nDAQ was stopped. Data was saved.\nTry again the DAQ function. If this does not work, please restart the program.");
	m_pTineALDlg->DeinitializeTine();
	m_pTineALDlg->InitializeTine( m_pMainDlg, m_pMainDlg->ReturnAdditionalProperties() );
 }

 m_bPropReadoutSucceeded = ret;

 return ret;

}

BOOL CImageSequence::returnDoocsProperties( CString &propStr )
{
 if (m_bPropReadoutSucceeded == TRUE) propStr = m_sPropStr;

 return m_bPropReadoutSucceeded;
}


BOOL CImageSequence::returnPropertyDescriptionString( CString &propdescstr )
{
 if (m_pTineALDlg == NULL) return FALSE;
	
 BOOL ret = m_pTineALDlg->returnWholePropertyDescriptionStringForDAQ( propdescstr ); // returns always TRUE at the moment

 return ret;
}

////////////////////////////////////////////////////////////////////////
// Add() - adds a single video frame to the end of the image sequence 
//
// Parameter:
//   frame - pointer to the memory location where the Videoheader plus 
//           the frame is stored
// 
// Return value:
//   TRUE: frame was added to the end of the sequence (always true)
//
BOOL CImageSequence::Add( unsigned char *frame )
{
 // add frame to tail of queue
 m_Queue.AddTail( (void *) frame ); // insert pointer to end of list
 
 // get video header
 VIDHDR vidH;
 memman.memcpysw( (char *) &vidH, (char *) frame, sizeof(VIDHDR) );

 if (vidH.bpp_effective == 0) vidH.bpp_effective = vidH.bpp;

 // update width and height according to video header
 m_iImageWidth = (int) vidH.widthframe;
 m_iImageHeight = (int) vidH.heightframe;
 m_iBpp = (int) vidH.bpp;
 m_iBppEffective = (int) vidH.bpp_effective;
 

 m_dScale = vidH.scale;

 // success
 return TRUE;
}

////////////////////////////////////////////////////////////////////////
// AddSequence() - adds a sequence of video frames to the end of the 
//                 image sequence 
//
// Parameters:
//   frames              - pointer to the memory location where the 
//                         Videoheader(s) plus the frame(s) are stored
//   numframes           - number of frames
//   size_oneframe_bytes - size of one frame in bytes (video header + frame data)
// 
// Return value:
//   TRUE: frames were added to the end of the sequence (always true)
//
/* SW! Apr 26, 2006 rausgenommen

  BOOL CImageSequence::AddSequence( unsigned char *frames, int numframes, int size_oneframe_bytes )
{
 if (numframes > 0) // numframes > 0 ? 
 {
  VIDHDR vidH;

  memman.memcpysw( (char *) &vidH, (char *) frames, sizeof(VIDHDR) );

  // update width and height according to video header
  m_iImageWidth = (int) vidH.widthframe;
  m_iImageHeight = (int) vidH.heightframe;
  m_iBpp = (int) vidH.bpp;
	 
  for (int i=0;i<numframes;i++) // loop over all frames
  {
   // extract pointer to actual frame
   unsigned char *tmpptr = frames+(i*size_oneframe_bytes);
   // add frame to tail of queue
   m_Queue.AddTail( (void *) tmpptr );
  }
 }

 // success
 return TRUE;
}
*/

////////////////////////////////////////////////////////////////////////
// EmptySequence() - empties the sequence of frames
//                    
// Return value:
//   TRUE: frames were deleted (always true)
//
BOOL CImageSequence::EmptySequence()
{
 int max = m_Queue.GetSize(); // get number of elements in list
 
 if (max > 0)
 {
  for (int i=0;i<max;i++) // iterate all data blocks
  {
   void *tmpptr;

   if (m_Queue.IsEmpty() == TRUE) // is the list empty ? 
   {
    tmpptr = NULL; // then return NULL
   }
    else
   {
    tmpptr = m_Queue.RemoveHead(); // return first item of linked 
   }                             //list and remove it from list
 
   if (tmpptr != NULL) memman.sw_free( tmpptr ); // if pointer is valid: delete the memory attached to it
  }
 }

 m_dScale = DEFAULT_SCALE_FACTOR;

 // doocs properties not yet acquired
 m_bDoocsPropertiesAcquired = FALSE;
 
 // success
 return TRUE;
}

////////////////////////////////////////////////////////////////////////
// GetFrame() - returns a reference of a specified frame of the sequence
//
// Parameters:
//   framenum - frame number 
//   here     - frame reference is put there
// 
// Return value:
//   TRUE: frame was found and is returned
//  FALSE: frame was not found and is not returned
//
BOOL CImageSequence::GetFrame( int framenum, const char **here)
{
 BOOL retval = FALSE;
 void *tmpptr;
 if (m_Queue.IsEmpty() == TRUE) // is the list empty ? 
 {
  tmpptr = NULL; // then return NULL
 }
  else
 {
	  // return child at specific position, which belongs to the 
	  // 'index'
  	  tmpptr = m_Queue.GetElem( framenum );
	  if (tmpptr!=NULL) retval = TRUE;
 }
 *here = (const char *) tmpptr;
 return( retval );
}

////////////////////////////////////////////////////////////////////////
// SaveToFile() - saves the images inside the sequence to file 
//
// Parameters:
//   bmp          - TRUE: write BMP, FALSE: write IMM
//   with_text    - TRUE: write descriptive text file together with the 
//                        image data file(s), FALSE: write no text file
//   basefilename - base filename of the image file 
//                  (path+name without . and without extension)
//   filenametext - file name for txt file
//   compression  - NOT used at the moment
//   hpal         - pointer to the palette that is used for saving (when writing to BMP)
//   info_return  - pointer to CString to return the additional information string (NULL: no info will be returned)
//   error        - pointer to additional error int (pass NULL to not store additional error information)
// 
// Return value:
//   TRUE: frames were saved
//  FALSE: frames were not saved
//   
//  error: valid if return value is FALSE
//  error == 1 -> error writing image file
//  error == 2 -> error writing additional text file
//
BOOL CImageSequence::SaveToFile( int bmp, BOOL with_text, BOOL auto_text, CString basefilename, CString filenametext, int compression, PALETTEENTRYSW *hpal, CString *info_return, CString *passed_info, int *error )
{
 BOOL retval = TRUE;
 int txtsaved = 1;

 // no pictures to save: return FALSE
 if (m_Queue.GetSize() == 0) return FALSE;

 if (with_text == TRUE) // save additional text file ? 
 {
  // 0 = not saved 
  // 1 = saved
  // call dialog in order to precise the information (by the user) that will be saved
  // if auto_text == TRUE -> store automatically, don't ask user for additional information

  if (passed_info != NULL)
  {
    txtsaved = PutAdditionalInformation( filenametext, *passed_info );
  }
   else
  {
   txtsaved = ChooseAndPutAdditionalInformation( filenametext, auto_text, info_return );
  }
 }
 
 if (bmp == FILE_IMM) // we want to save an IMM file
 {
  FILE *fp;

  CString filename = basefilename + ".imm";

  // open file for writing
  fp = fopen((LPCTSTR) filename, "wb");
   
  if (fp != NULL) // file could be opened
  {
   
   // loop over grab buffer (number of images that are into it)
   for (int i=0;i<m_Queue.GetSize();i++)
   {
    char *frame;
	VIDHDR vidH;

    GetFrame(i, (const char **) &frame);
	memman.memcpysw((char *) &vidH, frame, sizeof(VIDHDR) );

	// encode bpp in upper word of width integer 

	unsigned int w_write = ((vidH.bpp&0xFF)<<16) + (vidH.widthframe&0xFFFF);
	if (vidH.bpp == 8) w_write  = (vidH.widthframe&0xFFFF);

	unsigned int h_write = ((vidH.bpp_effective&0xFF)<<16) + (vidH.heightframe&0xFFFF);
	if (vidH.bpp == 8) h_write  = (vidH.heightframe&0xFFFF);
    int ret;
	
	// write width and height of frame
	ret = fwrite(&w_write, 1, 4, fp);
    ret += fwrite(&h_write, 1, 4, fp);

    // calculate pointer to specific image in grab buffer
	char *datapointer = frame;
    datapointer += sizeof(VIDHDR);

    // write out image bits
    ret += fwrite( datapointer, 1, (vidH.widthframe*vidH.heightframe*vidH.bpp/8), fp);
    
	// get and write out scale value of the frame
	
	double scale=m_dScale; 
	ret += fwrite( &scale, 1, sizeof(double), fp);

    if ( ((unsigned int)ret) != (vidH.widthframe*vidH.heightframe*vidH.bpp/8)+8+8) // error while writing ? 
    {
  	 // print error message
	 if (error != NULL) *error = 1; // error writing file
	 /*
	 CString msg;
	 msg.Format("Error while writing imm file \"%s\".", filename);
     AfxMessageBox((LPCTSTR) msg);
	 */
	 retval = FALSE;
     break; // leave for-loop
    }

   }
   // close file
   fclose(fp);
  }
 }
 else if (bmp == FILE_BMP) // save BMP file 
 {
  // save pictures from grab-buffer as bmp images 
  // if there is more than one picture inside the grab buffer then
  // the images are written out as a sequence.
  // The actual background is saved accordingly.
	 
  BOOL ret;

  // loop over all pictures that are in the grab buffer
  for (int i=0;i<m_Queue.GetSize();i++) 
  {
   // format string for filename
   CString writename;
   writename.Format("%s%d%s", basefilename, i, ".bmp" );

   VIDHDR vidH;
   char *frame;

   GetFrame(i, (const char **) &frame);
   memman.memcpysw((char *) &vidH, frame, sizeof(VIDHDR) );

   char *datapointer = frame;
   datapointer += sizeof(VIDHDR);
   char *datapointer8 = NULL;

   if (vidH.bpp != 8) // 16 or 32 bit data ? -> image must be downscaled in order to be saved as BMP
   {
	   datapointer8 = (char *) memman.sw_malloc( vidH.widthframe*vidH.heightframe );
	   if (datapointer8 == NULL)
	   {
		   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Error allocating memory while saving BMP file to disk.");
		   if (error != NULL) *error = 1;
           return FALSE;
	   }

	   DownScaleTo8Bit( (unsigned char *) datapointer, (unsigned char *) datapointer8, vidH.widthframe, vidH.heightframe, vidH.bpp);
	   datapointer = datapointer8;
   }

   CBMPFileHelper help;

   // write out the bmp image using helper function
   ret = help.WriteBMP( (char *) (LPCTSTR) writename, datapointer, hpal, vidH.widthframe, vidH.heightframe, vidH.bpp, vidH.bpp_effective, FALSE );

   if (datapointer8 != NULL)
   {
	   memman.sw_free( (void *) datapointer8 );
   }

   if (ret == FALSE) // error while writing: print error message
   {
  	 if (error != NULL) *error = 1; // error writing file
	 /*
	 CString msg;
	 msg.Format("Error while writing bmp file #%d out of %d.", i, m_Queue.GetCount());
     AfxMessageBox((LPCTSTR) msg);
	 */
	 retval = FALSE;
     break;
   }
  }
 }
 else if( bmp == FILE_IMC) // IMage Compressed file ? 
 {
  CString filename = basefilename + ".imc"; // create filename
  // instantiate helper class for IMC loading and saving
  CIMCFileHelper help;
  
  // save 'this' image sequence to IMC file
  BOOL ret = help.SaveToFile( filename, this ); 
  retval = ret;
 }
 else if( bmp == FILE_BKC) // IMage Compressed file ? 
 {
  CString filename = basefilename + ".bkc"; // create filename
  // instantiate helper class for IMC loading and saving
  CIMCFileHelper help;
  
  // save 'this' image sequence to IMC file
  BOOL ret = help.SaveToFile( filename, this ); 
  retval = ret;
 }

 if (txtsaved != 1) // error while saving txt file ? 
 {
  if (error != NULL) *error = 2; // could not save text file
  //AfxMessageBox("Could not save the additional text file.", MB_OK | MB_ICONEXCLAMATION );
  retval = FALSE;
 }

 return( retval );
}


////////////////////////////////////////////////////////////////////////
// LoadFromFile() - load an image sequence from file
//
// Parameters:
//   bmp          - TRUE: read BMP, FALSE: read IMM
//   filename     - filename of the image file 
// 
// Return value:
//   TRUE: frames were loaded
//  FALSE: frames were not loaded properly
//
BOOL CImageSequence::LoadFromFile( int bmp, CString filename)
{
 BOOL retval = TRUE;

 if (bmp == FILE_IMM) // we want to load an IMM file
 {
  FILE *fp;
  VIDHDR vidH;
  int images_loaded = 0;

  // set camera port to 'Loaded' value
  vidH.cameraport = -3;

  // open the imm file
  fp = fopen((LPCTSTR) filename, "rb");
  
  if (fp != NULL)
  {
   // calculate number of frames stored in file based on the file size and the size of a single frame
   // ( 768*574+16 bytes )
   unsigned int w, h, bpp, bpp_eff;
   unsigned int ret = fread( &w, 1, 4, fp );
   ret += fread( &h, 1, 4, fp );
   
   // mask out bpp, store as bits per pixel
   bpp = (w&0x00FF0000) >> 16;
   bpp_eff = (h&0x00FF0000) >> 16;
   // convert back to real width
   w = w&0xffff;
   h = h&0xffff;
   // old file: bpp == 0 so make it 8
   if (bpp == 0) bpp = 8;
   if (bpp_eff == 0) bpp_eff = 8;

   if (ret == 8)
   {
    long pos = ftell( fp);
    fseek( fp, 0, SEEK_END);
    long size = ftell(fp);
    long num_images = size / (w*h*bpp/8+8+8 );
    fseek( fp, 0, SEEK_SET);

    if (num_images > 0) // there are images in the file (check is for safety)
    {
     // create buffer that is large enough to hold the data
	 
	 char *buffer = (char *) memman.sw_malloc( num_images*((w*h*bpp/8)+sizeof(VIDHDR)) ); // (!!!!) bpp not yet
	 images_loaded = 0;// variable to hold the number of frames already loaded in
	 if (buffer != NULL)
	 {
	  char *temp = buffer;
	   
	  for (int i=0;i<num_images;i++) // loop over all frames
	  {
       unsigned int lw, lh, lbpp, lbpp_eff;
     
       // read in width and height	 
	   fread(&lw, 1, 4, fp);
       fread(&lh, 1, 4, fp);

	   // mask out bpp, store as bits per pixel
       lbpp = (lw&0x00FF0000) >> 16;
       // convert back to real width
       lw = lw&0xffff;
	   if (lbpp == 0) lbpp = 8;

	   lbpp_eff = (lh&0x00FF0000) >> 16;
	   if (lbpp_eff == 0) lbpp_eff = 8;
	   lh = lh&0xffff;

	   if ((lw != w) || (lh != h) || (lbpp != bpp) || (lbpp_eff != bpp_eff)) // check for right width and height
       {
	    ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "The IMM File you are trying to load is corrupt !"); 
	    break; // get out of the for-loop
       }
       else
       {
        // copy video header into grab buffer (just a dummy header with the 
	    // proper camera port value '-2'== 'Loaded Image')
	    vidH.heightframe = lh;
	    vidH.widthframe = lw;
		vidH.bpp = lbpp;
		vidH.bpp_effective = lbpp_eff; // SW! Sept 21, 2006

		// SW! Sept 21, 2006 quick hack!
		//if (vidH.bpp == 8) vidH.bpp_effective = 8; else vidH.bpp_effective = vidH.bpp;

	    vidH.kennung = 0x11223344;
		vidH.cameraport = -3;
	    vidH.compressed = 0;
	    vidH.compression = 0;
	    vidH.frameno = 0;
	    vidH.len_data = (vidH.heightframe * vidH.widthframe * vidH.bpp) / 8;
	    vidH.len_fmthdr = 0;
	    vidH.len_total = vidH.len_data + sizeof(VIDHDR);
	    
        temp += sizeof(VIDHDR);

        // read in frame data and scale value
	    unsigned int ret = fread( temp, 1, lw*lh*lbpp/8, fp);
	    double scale;
	    unsigned int ret1 = fread( &scale, 1, sizeof(double), fp);

	    if (ret1 == 8) // scale read successfully ? 
	    {
	     m_dScale = scale;
        }
      
		// update width and height according to video header
        m_iImageWidth = (int) vidH.widthframe;
        m_iImageHeight = (int) vidH.heightframe;
        m_iBpp = (int) vidH.bpp;
		m_iBppEffective = (int) vidH.bpp_effective;

	    vidH.scale = scale;
        memman.memcpysw( temp-sizeof(VIDHDR), (char *) &vidH, sizeof(VIDHDR));

	    // check whether all bitmap bits have been read and print error otherwise 
	    if (ret != (lw*lh*lbpp/8))
	    {
	      ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "Error while reading from file.");
	 	  retval = FALSE;
 		  break; // get out of the for-loop
 	    }
	    else
	    {
          temp += ret; // adjust moving pointer
		  images_loaded++; // increment loaded images
	    }  
	   }
	  } // for
     }
	  else
	 {
	  ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Out of memory while loading file.");
	 }
	
	 if (images_loaded == 0) // no images loaded
	 {
	   retval = FALSE; // return ERROR
	 }
      else
	 { // frames were loaded so put them into the sequence
         
		 // clear the sequence
		 EmptySequence();

		 char *temp1 = buffer;
		 VIDHDR vidH1;

		 memman.memcpysw( (char *) &vidH1, temp1, sizeof(VIDHDR));

         // loop over all images
		 for (int k=0;k<images_loaded;k++)
		 {
          // !!!! vidH.bpp
		  char *temp1 = buffer + k*((vidH1.widthframe*vidH1.heightframe*vidH1.bpp/8)+sizeof(VIDHDR));
		  char *newbuf = (char *) memman.sw_malloc( (vidH1.widthframe*vidH1.heightframe*vidH1.bpp/8)+sizeof(VIDHDR) );
		  if (newbuf != NULL)
		  {
			  // copy image
  		      memman.memcpysw( newbuf, temp1, (vidH1.widthframe*vidH1.heightframe*vidH1.bpp/8)+sizeof(VIDHDR) );

              // add image to sequence 
              Add( (unsigned char *) newbuf );
  		  }
		  else
		  {
           retval = FALSE;
		  }

         }
 	 }

     // free buffer 
     if (buffer != NULL) memman.sw_free( (void *) buffer );

    }//if num_images>0
	else
	{
	 retval = FALSE;
	}
   }// if ret == 8
   else
   {
    retval = FALSE;
   }
  } // if fp != NULL
   else
  {// fp == NULL
   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, "Could not open imm file.");
   retval = FALSE;
  }
  // close file
  fclose(fp);

 }
 else if (bmp == FILE_BMP)// BMP file loading
 {
  // here: load bmp file
  CBitmap bmp;

  CBMPFileHelper help;

  // load BMP file using own helper function
  HBITMAP hBmp = help.LoadDIBSectionFromFile((LPCTSTR) filename, NULL, NULL, 0);
  
  if (hBmp == NULL) // could not load bitmap, print error message and exit function
  {
	  //AfxMessageBox("Error loading bmp file.");
	  retval = FALSE;
	  return( retval );
  }

  // attach bitmap handle to CBitmap class
  BOOL ret = bmp.Attach( hBmp );

  // get width and height of bitmap
  BITMAP Bitmap;
  GetObject(hBmp, sizeof(BITMAP), &Bitmap);
  LONG cx = Bitmap.bmWidth;
  LONG cy = Bitmap.bmHeight;

  if (Bitmap.bmBitsPixel != 8) // check for wrong color depth
  {
   CString msg8;
   msg8 = ("Image has wrong color depth. It must be of 8 bit color depth.\n");
   msg8 += ("Please load the picture into your favourite picture editor and modify it to\n");
   msg8 += ("meet the requirements (8 bit color depth (256 colors)).");

   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(0, (LPCTSTR) msg8);

   // detach bitmap
   HBITMAP del = (HBITMAP) bmp.Detach();
   // delete bitmap object (free memory and resources)
   if (del != NULL) ::DeleteObject( (HGDIOBJ) del );
   
   retval = FALSE; // exit function
   return( retval );
  }

  // we have successfully loaded the frame, now we have to 
  // initialize the grab buffer by first freeing the old grab buffer,
  // allocating sufficent memory location and initialize grab buffer
  // variables
  
  EmptySequence();

  char *buffer = (char *) memman.sw_malloc( sizeof(VIDHDR)+cx*cy );
  if (buffer != NULL)
  {
    // create dummy video header (camera port set to 'Loaded Image')
    // and attach it to the frame (copy to grab buffer)
    VIDHDR vidH;
    vidH.cameraport = -3;
    vidH.widthframe = (int) cx;
    vidH.heightframe = (int) cy;
	vidH.bpp = 8;
	vidH.bpp_effective = 8; // for BMP this is okay SW! Sept 21, 2006
	vidH.kennung = 0x11223344;
	vidH.compressed = 0;
	vidH.compression = 0;
	vidH.frameno = 0;
	vidH.len_data = (vidH.heightframe * vidH.widthframe * vidH.bpp) / 8;
	vidH.len_fmthdr = 0;
	vidH.len_total = vidH.len_data + sizeof(VIDHDR);
	vidH.scale = DEFAULT_SCALE_FACTOR;
    memman.memcpysw((char *) buffer, (char *) &vidH, sizeof(VIDHDR));
  
    // get bitmap bits from bitmap attached to CBitmap class and store
    // it into the grab buffer
    bmp.GetBitmapBits( cx*cy, (LPVOID) (buffer+sizeof(VIDHDR)) );

	// update width and height according to video header
    m_iImageWidth = (int) vidH.widthframe;
    m_iImageHeight = (int) vidH.heightframe;
    m_iBpp = (int) vidH.bpp;
	m_iBppEffective = (int) vidH.bpp_effective;


    // we no longer need the bitmap handle, so detach and free it
    HBITMAP del = (HBITMAP) bmp.Detach();
    if (del != NULL) ::DeleteObject( (HGDIOBJ) del );

    // add image to sequence 
	Add( (unsigned char *) buffer );
  }
  else
  {
   retval = FALSE;
   ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, "Out of memory while loading image.");
  }

 }
 else if (bmp == FILE_IMC) // IMage Compressed ? 
 {
  // instantiate helper class for IMC loading and saving
  CIMCFileHelper help;
  // load IMC file into 'this' image sequence 
  BOOL ret = help.LoadFromFile( filename, this, this->m_pTineDlg, m_bTineALInitialized, m_pTineALDlg, m_pMainDlg); 
  ret = retval;
 }
 else if (bmp == FILE_BKC) // BacKground Compressed ? 
 {
  // instantiate helper class for IMC loading and saving
  CIMCFileHelper help;
  // load IMC file into 'this' image sequence 
  BOOL ret = help.LoadFromFile( filename, this, this->m_pTineDlg, m_bTineALInitialized, m_pTineALDlg, m_pMainDlg); 
  m_dScale = DEFAULT_SCALE_FACTOR; // no scale since it is a background file !
  ret = retval;
 }


 return( retval );
}

////////////////////////////////////////////////////////////////////////////////
// ChooseAndPutAdditionalInformation() 
//
// This function displays a dialog where the user can precise the additional 
// information that is saved together with the image file(s). After pressing
// OK on the dialog the information is saved to text file 'filename'.
//
// Parameter:
//  filename - file name and path to the txt file
// auto_text - TRUE: auto save, only save automatically acquired information, do not ask for
//             additional information
//             FALSE: ask for additional information using dialog box
//
// Return value:
//  1 = file was successfully saved
//  0 = error while saving
//
int CImageSequence::ChooseAndPutAdditionalInformation(CString filename, BOOL auto_text, CString *info_return )
{
 BOOL retval = 0;

 // present additional information dialog
 CAdditionalInformation add( m_pMainDlg->ReturnNumAdditionalProperties(), m_pMainDlg->ReturnAdditionalProperties());

 if (auto_text == FALSE) add.DoModal(); else add.OnOKAuto();

 // get the information string
 CString info;
 add.ReturnInformation( info );

 if (info_return != NULL) *info_return = info; 

 // open file for writing text
 FILE *fp = fopen( (const char *) filename, "wt");
 if (fp != NULL)
 {
  // put data 
  int ret1 = fputs( (const char *) info, fp);
  if (ret1 != EOF) retval = 1;
  fclose(fp);
 }
 
 // return 
 return retval;
}

////////////////////////////////////////////////////////////////////////////////
// PutAdditionalInformation() 
//
// This function puts additional information 'infotext' into the text file 'filename'.
//
// Parameter:
//  filename - file name and path to the txt file
//  infotext - text to be saved into the file
//
// Return value:
//  1 = file was successfully saved
//  0 = error while saving
//
int CImageSequence::PutAdditionalInformation(CString filename, CString infotext)
{
 BOOL retval = 0;

 // open file for writing text
 FILE *fp = fopen( (const char *) filename, "wt");
 if (fp != NULL)
 {
  // put data 
  int ret1 = fputs( (const char *) infotext, fp);
  if (ret1 != EOF) retval = 1;
  fclose(fp);
 }
 
 // return 
 return retval;
}


////////////////////////////////////////////////////////////////////////////////
// SetScale() - sets the scale associated with the images 
//
void CImageSequence::SetScale(double scale)
{
 m_dScale = scale;
}

////////////////////////////////////////////////////////////////////////////////
// GetScale() - returns the scale of the images inside the sequence
//
double CImageSequence::GetScale()
{
 return( m_dScale );
}

////////////////////////////////////////////////////////////////////////////////
// GetImageWidth() - returns the width of the images inside the sequence
//
int CImageSequence::GetImageWidth()
{
 return( m_iImageWidth );
}

////////////////////////////////////////////////////////////////////////////////
// GetImageHeight() - returns the height of the images inside the sequence
//
int CImageSequence::GetImageHeight()
{
 return( m_iImageHeight );
}


////////////////////////////////////////////////////////////////////////
// RemoveFrame() - removes a single frame out of the sequence
//                 the frame bits memory location is not deleted
//
// Return value:
//   TRUE: frame was removed
//  FALSE: frame was not removed, either because it was not there or
//         the list is empty
//
BOOL CImageSequence::RemoveFrame(int frameno)
{
 if ((m_Queue.IsEmpty() == TRUE) ||(frameno >= m_Queue.GetSize())) // is the list empty ? 
 {
  return FALSE; // then return NULL
 }
  else
 {
  //void *tmpptr = m_Queue.GetElem( frameno );
  
  // remove the element 'frameno'
  m_Queue.RemoveElem( frameno );

  //if (tmpptr != NULL) memman.sw_free( tmpptr );
  return TRUE;
 }

}

////////////////////////////////////////////////////////////////////////
// DownScaleTo8Bit() 
//
// This mathod scales image src (attributes: width, height and bpp) into 
// 8 bits memory location dst (attributes: width, height, bpp=8).
//
// One has to take care that memory location src is at least as big as 
//  width*height*bpp/8 
// and that dst is at least as big as
//  width*height
//
void CImageSequence::DownScaleTo8Bit(unsigned char *src, unsigned char *dst, unsigned int width, unsigned int height, unsigned int bpp)
{

 if (bpp == 8)  // 8 bits per pixel, then just copy the data
 {
  memman.memcpysw( (char *) dst, (char *) src, width*height);
 }
 else if (bpp == 16) // 16 bits per pixel 
 {
  unsigned short maxval=0;
  unsigned short *srci = (unsigned short *) src;

  // look into source for maximum value
  for (unsigned int j=0;j<width*height;j++)
  {
   if (srci[j] > maxval) maxval = srci[j];
  }
  
  // scale into 8 bits accordingly to the maximum value
  unsigned long tmp = 65536L / (((unsigned long)maxval)+1);
  unsigned short tmp1 = (unsigned short) tmp;
  for (j=0;j<width*height;j++)
  {
   dst[j] = (unsigned char) ((srci[j] * tmp1) >> 8);
  }
 }
 else if (bpp == 32) // 32 bits per pixel
 {
  unsigned long *srcl = (unsigned long *) src;
  unsigned long maxval=0;

  // look into source for maximum value
  for (unsigned int j=0;j<width*height;j++)
  {
   if (srcl[j] > maxval) maxval = srcl[j];
  }
  
  // scale into 8 bits accordingly to the maximum value
  unsigned long tmp = 0xFFFFFFFF / (maxval);
  for (j=0;j<width*height;j++)
  {
   dst[j] = (unsigned char) ((srcl[j] * tmp) >> 24);
  }
 
 }

}

////////////////////////////////////////////////////////////////////////////////
// GetScale() - returns the bpp (bits per pixel) of the images inside the sequence
//
int CImageSequence::GetBpp()
{
 return( m_iBpp );
}

////////////////////////////////////////////////////////////////////////////////
// SetBpp() - sets the bpp (bits per pixel) associated with the images 
//
void CImageSequence::SetBpp(int bpp)
{
 m_iBpp = bpp;
}

////////////////////////////////////////////////////////////////////////////////
// GetScale() - returns the bpp (bits per pixel) of the images inside the sequence
//
int CImageSequence::GetBppEffective()
{
 return( m_iBppEffective );
}

////////////////////////////////////////////////////////////////////////////////
// SetBpp() - sets the bpp (bits per pixel) associated with the images 
//
void CImageSequence::SetBppEffective(int bpp_eff)
{
 m_iBppEffective = bpp_eff;
}
