// ImageSequence.h: interface for the CImageSequence class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IMAGESEQUENCE_H__B3937E03_2C6A_11D7_AFE1_0002B34B0F4F__INCLUDED_)
#define AFX_IMAGESEQUENCE_H__B3937E03_2C6A_11D7_AFE1_0002B34B0F4F__INCLUDED_

#include "CATineDlg.h"	// Added by ClassView
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CImageSequence  
{
public:
	// construction/destruction
    CImageSequence( CCATineDlg *catinedlg, BOOL tinealinit, CTineALDlg *tinealdlg, CVideoclntDlg *maindlg);
	virtual ~CImageSequence();

    // get image width and height of stored images		
	int GetImageHeight();
	int GetImageWidth();
	
	// get and set scale associated with images
	double GetScale();
	void SetScale( double scale);

	// get and set bits per pixel associated with images
	void SetBpp( int bpp);
	int GetBpp();

	void SetBppEffective( int bpp_eff);
	int GetBppEffective();


    // returns the amount of video frames that are stored
    int  Count();
    
	// acquires the DOOCS properties (Gun FWP, Klystron FWP, SP Voltage, SP Phase etc.)
	BOOL AcquireDoocsProperties();
	// returns the acquired doocs properties
    BOOL returnDoocsProperties( CString &propStr );
	BOOL returnPropertyDescriptionString( CString &propdescstr );

	
	// adds a single frame to the end of the image sequence
	BOOL Add( unsigned char *frame );
	// adds a certain amount of frame to the end of the image sequence
	//BOOL AddSequence( unsigned char *frames, int numframes, int size_oneframe_bytes  );
	// throws away all images
	BOOL EmptySequence();
	// gets a single frame 
	BOOL GetFrame( int framenum, const char **here);

	// removes a single frame out of the sequence
	BOOL RemoveFrame( int frameno );

    // saves the image sequence to file bmp=FILE_IMM FILE_BMP FILE_IMC
    BOOL SaveToFile( int bmp, BOOL with_text, BOOL auto_text, CString basefilename, CString filenametext, int compression, PALETTEENTRYSW *hpal, CString *info_return, CString *passed_info, int *error );
	
	// loads in an image sequence from file bmp=FILE_IMM FILE_BMP FILE_IMC
	BOOL LoadFromFile( int bmp, CString filename );

private:
	CVideoclntDlg * m_pMainDlg;

    BOOL m_bTineALInitialized;
    CTineALDlg *m_pTineALDlg;
	// bits per pixel of the associated images
	int m_iBpp;
	int m_iBppEffective;

	// TINE dialog for acquiring magnets information
	CCATineDlg *m_pTineDlg;
	
	// acquires Imain and Ibuck (magnets current)

    // TRUE: doocs properties were acquired    
	BOOL m_bDoocsPropertiesAcquired;
    // reads a single doocs float value into 'resultdbl'

    // image width of the images in the sequence
    int m_iImageWidth;
	// image height of the images in the sequence
	int m_iImageHeight;
	
	// scale value of the images
	double m_dScale;

    // present dialog for additional information when saving sequence to file
    int ChooseAndPutAdditionalInformation(CString filename, BOOL auto_text, CString *info_return );
    int PutAdditionalInformation(CString filename, CString infotext);
    // queue to hold the images in the sequence
	CQueue m_Queue;

	// scale 16 and 32 bpp images down to 8 bits 
	void DownScaleTo8Bit(unsigned char *src, unsigned char *dst, unsigned int width, unsigned int height, unsigned int bpp);

	CString m_sPropStr;
	BOOL m_bPropReadoutSucceeded;


};


#endif // !defined(AFX_IMAGESEQUENCE_H__B3937E03_2C6A_11D7_AFE1_0002B34B0F4F__INCLUDED_)
