// MemoryManagement.cpp: Implementierung der Klasse CMemoryManagement.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MemoryManagement.h"
#include <malloc.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// construction
//
CMemoryManagement::CMemoryManagement()
{
    ::InitializeCriticalSection(&lock);
}

//////////////////////////////////////////////////////////////////////
// destruction
//
CMemoryManagement::~CMemoryManagement()
{
    ::DeleteCriticalSection(&lock);
}

//////////////////////////////////////////////////////////////////////
// sw_malloc - allocates aligned memory
//
// Parameter:
//   size    - size of the memory block in bytes
//
// Return-Value:
//   NULL        - could not allocate memory block
//   other_value - pointer to memory block
//
void *CMemoryManagement::sw_malloc( int size )
{
	void *ptr;
 
	if (size == 0) return( NULL );

	::EnterCriticalSection(&lock);
 
#ifdef __INTEL__
	ptr = _mm_malloc( size, 16 ); 
#else
	//ptr = _aligned_malloc( size, 16 );

	ptr = (void *) new char[size];

#endif

	::LeaveCriticalSection(&lock);

	return( ptr );
}


//////////////////////////////////////////////////////////////////////
// sw_free - free up memory block allocated by sw_malloc()
//
// Parameter:
//   ptr    - pointer that was returned by sw_malloc()
//
void  CMemoryManagement::sw_free( void *ptr )
{
	if (ptr == NULL) return;

	::EnterCriticalSection(&lock);

#ifdef __INTEL__
	_mm_free( ptr );
#else
	//_aligned_free( (char *) ptr );

	delete [] ptr;

#endif
	::LeaveCriticalSection(&lock);
}

//////////////////////////////////////////////////////////////////////
// memcpysw - encapsulation of copying memory blocks
//
// Parameters:
//     dst      - destination pointer
//     src      - source pointer
//  lengthsize  - number of bytes to copy
//
void CMemoryManagement::memcpysw( char *dst, char *src, int lengthsize)
{
 //memcpy( (void *) dst, (const void *) src, lengthsize );
 __asm
   {
	 emms
     pushfd
     mov esi,src
     mov edi,dst
     mov ecx, lengthsize
	 mov edx,ecx
	 shr ecx,3
	 and edx,7
	 cmp ecx,0
	 je overbigcopy
	 sub edi,8

    bigcopy:
	 add edi,8
	 movq mm1, [esi]
	 movq [edi],mm1
	 add esi,8
	 //add edi,8
	 dec ecx
	 jnz bigcopy

     add edi,8
    overbigcopy:
	 cmp edx,0
	 je  outthere

    smallcopy:
	 mov al,[esi]
	 mov [edi],al
	 inc esi
	 inc edi
	 dec edx
	 jnz smallcopy

    outthere:
	 popfd
     emms
   }
} 


// declare the memory management class in global namespace
CMemoryManagement memman;