#include "stdafx.h"
#include "memorymanagement.h"
#include "Queue.h"

CQueue::CQueue()
{
     ::InitializeCriticalSection(&lock);
} // Queue

CQueue::~CQueue()
{
      ::DeleteCriticalSection(&lock);
} // ~Queue


// add an element to the end of the list
BOOL CQueue::AddTail(LPVOID p)
{ 
	  BOOL result=TRUE;
      ::EnterCriticalSection(&lock);
	  queue.AddTail(p);
      ::LeaveCriticalSection(&lock);
	  return result;
} // AddTail


// removes the first element from the list and returns it, if queue is empty, return NULL
LPVOID CQueue::RemoveHead()
{ 
      LPVOID result;

              ::EnterCriticalSection(&lock);
			  if (queue.IsEmpty() == TRUE) result = NULL; else result = queue.RemoveHead();
              ::LeaveCriticalSection(&lock);
              return result;

} // RemoveHead


// return number of elements in queue
int CQueue::GetSize()
{
      int result1;

      ::EnterCriticalSection(&lock);
      if (queue.IsEmpty() == TRUE) result1 = 0; else result1 = queue.GetCount();
      ::LeaveCriticalSection(&lock);
      return result1;
}


// return element # 'index' from list, if list is empty return null
LPVOID CQueue::GetElem( int index )
{
      LPVOID result;

      ::EnterCriticalSection(&lock);
	  if (queue.IsEmpty() == TRUE) result = NULL; else result = queue.GetAt( queue.FindIndex(index));
      ::LeaveCriticalSection(&lock);
      return result;

}

// remove element at specified index, if queue is empty, do nothing
void CQueue::RemoveElem( int index )
{
      ::EnterCriticalSection(&lock);
      if (queue.IsEmpty() == FALSE)
	  {
   	   queue.RemoveAt( queue.FindIndex(index));
	  }
      ::LeaveCriticalSection(&lock);
}


// remove element 'ptr' from list, if list is empty or 'ptr' not found return null else return '1'
LPVOID CQueue::RemoveElem( void *ptr )
{
      LPVOID result;

      ::EnterCriticalSection(&lock);

	  if (queue.IsEmpty() == FALSE) 
	  {
 	    POSITION pos = queue.Find((void *) ptr);
	    if (pos == NULL) result = NULL; else { result = (void *)  1; queue.RemoveAt( pos ); }
	  }
	  else result = NULL;

      ::LeaveCriticalSection(&lock);
      return result;

}



// get head element from list and return it, if queue is empty return NULL
LPVOID CQueue::GetHead()
{ 
 LPVOID result;

 ::EnterCriticalSection(&lock);

 if (queue.IsEmpty())
 {
  result = NULL;
 }
 else
 {
  result = queue.GetHead();
 }
 
 ::LeaveCriticalSection(&lock);
 
 return result;
} // RemoveHead


// is queue empty ? True: yes, false: no
BOOL CQueue::IsEmpty()
{
  ::EnterCriticalSection(&lock);
  BOOL retval = queue.IsEmpty();
  ::LeaveCriticalSection(&lock);
  return( retval );
}


void CQueue::EmptyQueue()
{
 ::EnterCriticalSection(&lock);

 if (queue.IsEmpty() == FALSE)
 {
   int size = queue.GetCount();
 
   for (int a=0;a<size;a++)
   {
	   LPVOID result = queue.RemoveHead();

	   //AfxMessageBox("Empty.");

	   if (result != NULL) memman.sw_free( (void *) result );
   }
 }

 ::LeaveCriticalSection(&lock);

}
