// ShutterControl.cpp: implementation of the CShutterControl class.
//
// Helper class to open and close the laser shutter for semiautomatical 
// datataking.
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "videoclnt.h"
#include "CATineDlg.h"
#include "IniEx.h"
#include "ShutterControl.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction - just empty
//////////////////////////////////////////////////////////////////////

CShutterControl::CShutterControl( CIniEx *IniFileGlobal, CCATineDlg *tinedlg )
{
 m_pTineDlg = tinedlg;
 m_pIniFileGlobal = IniFileGlobal;
}

CShutterControl::~CShutterControl()
{
}


//////////////////////////////////////////////////////////////////////
// OpenShutter()
//
// This function opens the shutter and reads back until it is open 
// or a timeout (4 seconds) has occured.
//
// Returnvalue:
//   True : shutter was successfully opened
//  False : shutter could not be opened
//////////////////////////////////////////////////////////////////////
BOOL CShutterControl::OpenShutter()
{
 //return TRUE;

 if (m_pTineDlg == NULL) return FALSE;
 
 int err=1;
 int intparam=0;
 float floatparam=0.0;
 double d;

 //
 CString dt = m_pIniFileGlobal->GetValue("ShutterWrite", "Datatype", "INT");

 dt.MakeUpper();

 if (dt == "INT") intparam = m_pIniFileGlobal->GetValueInt("ShutterWrite", "OpenShutterWrite", 0);
 else if (dt == "FLOAT") 
 {
	 d = m_pIniFileGlobal->GetValueDouble("ShutterWrite", "OpenShutterWrite", 0.0);
	 floatparam = (float) d;
 }
 else return FALSE;

 CComVariant
	olvint(intparam),
	olvdx,
	dataxint(intparam),
	olvfloat(floatparam),
	dataxfloat(floatparam);

 // set proper access parameters
 m_pTineDlg->m_CACOPCA.SetDeviceContext( m_pIniFileGlobal->GetValue("ShutterWrite", "Context", "PITZ.I_LOCK") );
 m_pTineDlg->m_CACOPCA.SetDeviceGroup( m_pIniFileGlobal->GetValue("ShutterWrite", "Server", "BIS") );
 m_pTineDlg->m_CACOPCA.SetDeviceName( m_pIniFileGlobal->GetValue("ShutterWrite", "Device", "BIS2") );
 m_pTineDlg->m_CACOPCA.SetDeviceProperty( m_pIniFileGlobal->GetValue("ShutterWrite", "Property", "REG_14")  );
 m_pTineDlg->m_CACOPCA.SetAccessMode("WRITE");
 m_pTineDlg->m_CACOPCA.SetAccessRate("1000");

 long ret1;

 if (dt == "INT")
 {
   // call the server
   ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvint, olvdx, olvdx, &dataxint, olvdx, olvdx);
 }
 else
 {
   ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvfloat, olvdx, olvdx, &dataxfloat, olvdx, olvdx);
 }
 
 if (ret1 == 0) // server returned successful call
 {
   err = 0;
 }

 // clean up variants
 olvint.Clear();
 olvdx.Clear();
 dataxint.Clear();

 olvfloat.Clear();
 dataxfloat.Clear();

 if (err != 0) // error? 
 {
  return FALSE;
 }
  else
 {
  BOOL loop = TRUE;
  int count = 0;
  
  while (loop == TRUE)
  {
   BOOL error;
   BOOL ret = CShutterControl::IsShutterOpen( &error );

   if (error == TRUE) return FALSE;
   if (ret == TRUE) loop = FALSE;
   ::Sleep(100);
   count++;
   if (count >= 80) return FALSE; // after approx. 8 seconds: timeout
  }
 }

 // when it was successfully opened, wait 1.5 seconds in order for old frames to vanish from the server's
 // memory
 ::Sleep(1500);
 return TRUE;
}

//////////////////////////////////////////////////////////////////////
// CloseShutter()
//
// This function closes the shutter and reads back until it is closed 
// or a timeout (4 seconds) has occured.
//
// Returnvalue:
//   True : shutter was successfully closed
//  False : shutter could not be closed
//////////////////////////////////////////////////////////////////////
BOOL CShutterControl::CloseShutter() 
{
 if (m_pTineDlg == NULL) return FALSE;
 
 int err=1;
 int intparam=1;
 float floatparam=1.0;
 double d;

 //
 CString dt = m_pIniFileGlobal->GetValue("ShutterWrite", "Datatype", "INT");

 dt.MakeUpper();

 if (dt == "INT") intparam = m_pIniFileGlobal->GetValueInt("ShutterWrite", "CloseShutterWrite", 1);
 else if (dt == "FLOAT") 
 {
	 d = m_pIniFileGlobal->GetValueDouble("ShutterWrite", "CloseShutterWrite", 1.0);
	 floatparam = (float) d;
 }
 else return FALSE;

 CComVariant
	olvint(intparam),
	olvdx,
	dataxint(intparam),
	olvfloat(floatparam),
	dataxfloat(floatparam);

 // set proper access parameters
 m_pTineDlg->m_CACOPCA.SetDeviceContext( m_pIniFileGlobal->GetValue("ShutterWrite", "Context", "PITZ.I_LOCK") );
 m_pTineDlg->m_CACOPCA.SetDeviceGroup( m_pIniFileGlobal->GetValue("ShutterWrite", "Server", "BIS") );
 m_pTineDlg->m_CACOPCA.SetDeviceName( m_pIniFileGlobal->GetValue("ShutterWrite", "Device", "BIS2") );
 m_pTineDlg->m_CACOPCA.SetDeviceProperty( m_pIniFileGlobal->GetValue("ShutterWrite", "Property", "REG_14")  );
 m_pTineDlg->m_CACOPCA.SetAccessMode("WRITE");
 m_pTineDlg->m_CACOPCA.SetAccessRate("1000");

 long ret1;

 if (dt == "INT")
 {
   // call the server
   ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvint, olvdx, olvdx, &dataxint, olvdx, olvdx);
 }
 else
 {
   ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvfloat, olvdx, olvdx, &dataxfloat, olvdx, olvdx);
 }

 if (ret1 == 0) // server returned successful call
 {
   err = 0;
 }

 // clean up variants
 olvint.Clear();
 olvdx.Clear();
 dataxint.Clear();

 olvfloat.Clear();
 dataxfloat.Clear();
 
 if (err != 0) // error? 
 {
  return FALSE;
 }
  else
 {
  BOOL loop = TRUE;
  int count = 0;
  
  while (loop == TRUE) // loop until it is really closed or a timeout occurs
  {
   BOOL error;
   BOOL ret = CShutterControl::IsShutterOpen( &error );

   if (error == TRUE) return FALSE;
   if (ret == FALSE) loop = FALSE;
   ::Sleep(100);
   count++;
   if (count >= 80) return FALSE; // timeout after about 4 seconds
  }
 }

 // when it was successfully closed, wait 1.5 seconds in order for old frames to vanish from the server's
 // memory
 ::Sleep(1500); 
 return TRUE;
}

//////////////////////////////////////////////////////////////////////
// IsShutterOpen()
//
// This function check whether the shutter is opened or closed.
//
// Parameters:
//   error [out] : TRUE: error happened, it is undefined whether 
//                       shutter is open or closed
//                FALSE: return value shows state of shutter
// Returnvalue:
//   True : shutter is open (applies only if error==FALSE)
//  False : shutter is closed (applies only if error==FALSE)
//////////////////////////////////////////////////////////////////////
BOOL CShutterControl::IsShutterOpen( BOOL *error )
{
 if (m_pTineDlg == NULL)
 {
  if (error != NULL) *error = TRUE;
  return FALSE;
 }

 //
 CString dt = m_pIniFileGlobal->GetValue("ShutterRead", "Datatype", "INT");
 dt.MakeUpper();
 int intopen, intclosed;
 double doubleopen, doubleclosed;

 if (dt == "INT") 
 {
	 intopen = m_pIniFileGlobal->GetValueInt("ShutterRead", "ResultOpen", 0);
	 intclosed = m_pIniFileGlobal->GetValueInt("ShutterRead", "ResultClosed", 1);
 }
 else if (dt == "FLOAT") 
 {
	 doubleopen = m_pIniFileGlobal->GetValueDouble("ShutterWrite", "ResultOpen", 0.0);
	 doubleclosed = m_pIniFileGlobal->GetValueDouble("ShutterWrite", "ResultClosed", 1.0);
 }
 else return FALSE;

 int resultint;
 double resultdouble;
 int paramint=0;
 float paramfloat=0.0;
 CComVariant
	olvint(paramint),
	olvdx,
	dataxint(paramint),
	olvfloat(paramfloat),
	dataxfloat(paramfloat);

 // set proper access parameters

 m_pTineDlg->m_CACOPCA.SetDeviceContext( m_pIniFileGlobal->GetValue("ShutterRead", "Context", "PITZ.I_LOCK") );
 m_pTineDlg->m_CACOPCA.SetDeviceGroup( m_pIniFileGlobal->GetValue("ShutterRead", "Server", "BIS") );
 m_pTineDlg->m_CACOPCA.SetDeviceName( m_pIniFileGlobal->GetValue("ShutterRead", "Device", "BIS1") );
 m_pTineDlg->m_CACOPCA.SetDeviceProperty( m_pIniFileGlobal->GetValue("ShutterRead", "Property", "DI_4")  );

 m_pTineDlg->m_CACOPCA.SetAccessMode("READ");
 m_pTineDlg->m_CACOPCA.SetAccessRate("1000");

 long ret1;

 if (dt == "INT")
 {
	// call the server
	ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvint, olvdx, olvdx, &dataxint, olvdx, olvdx);
    if (ret1 == 0) // server returned successful call
	{
      resultint = (int) olvint.lVal;
	}

 }
 else
 {
	ret1 = m_pTineDlg->m_CACOPCA.Execute( &olvfloat, olvdx, olvdx, &dataxfloat, olvdx, olvdx);    
    if (ret1 == 0) // server returned successful call
	{
      resultdouble = (double) olvfloat.fltVal;
	}
 }

 // clean up variants
 olvint.Clear();
 olvdx.Clear();
 dataxint.Clear();

 olvfloat.Clear();
 dataxfloat.Clear();

 if (ret1 != 0)
 {
  if (error != NULL) *error = TRUE;
  return FALSE; 
 }

 if (error != NULL) *error = FALSE;

 if (dt == "INT")
 {
   if (resultint == intclosed) return FALSE; // shutter is closed
   if (resultint == intopen) return TRUE; //shutter is open;
 }
 else
 {
	 /// !!! SW!!!! unsafe, may not work !!!
   if (resultdouble == doubleclosed) return FALSE; // shutter is closed
   if (resultdouble == doubleopen) return TRUE; //shutter is open;
 }

 if (error != NULL) *error = TRUE;
 return FALSE;
}
