// Stat.h: interface for the CStat class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STATEMPTY_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
#define AFX_STATEMPTY_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////////////////
//
// This class is a barebone class based on straightforward statistical analysis class CStat. 
// In this class no calculations are made, just the background is subtracted from the 
// video data. Most function do simply nothing at all.
//
class CStatEmpty  
{
public:
	// construction of the class, for description see StatEmpty.cpp
	CStatEmpty(unsigned char* data,unsigned char* Back, unsigned char *out, CRect AOI,BOOL XFilt, BOOL circular, BOOL normalize, unsigned int ImWi,unsigned int lmHi, unsigned int bpp, unsigned int bpp_eff);
	
	// destruction of the class
	virtual ~CStatEmpty();

    // returns x mean value (spot center) in x coordinate
	// does nothing here in CStatEmpty
	double XM();
	
    // returns y mean value (spot center) in y coordinate
	// does nothing here in CStatEmpty
	double YM();

    // returns x rms value (spot size) in x coordinate
	// does nothing here in CStatEmpty
	double X_RMS();
	
    // returns y rms value (spot size) in y coordinate
	// does nothing here in CStatEmpty
	double Y_RMS();
	
    // returns distribution array in x coordinate
	// does nothing here in CStatEmpty, just returns
	// zero floats in the array
	double* X_DIST();
	
    // returns distribution array in y coordinate
	// does nothing here in CStatEmpty, just returns
	// zero floats in the array
	double* Y_DIST();
	
	// function to do calculation stuff
	// does nothing here in CStatEmpty
	void START();

    unsigned int SUM_PIXELS();

protected:
    // members to hold the (empty) distribution arrays
	double* XDIST;
	double* YDIST;

    // integer distribution array (needed for calculation in inner loop)
	double *YDISTd;
	double *XDISTd;

    // members to hold the (empty; 0.0) calculated variables
	double XMEAN;
	double YMEAN;
	double XRMS;
	double YRMS;

    // members needed for class
	unsigned char* array; // pointer to source frame
	unsigned char* BackGr; // pointer to background image
	
	// positions of the area of interest
	int left;
	int right;
	int up;
	int down;

	int center_x;
	int center_y;
	int length;

    // image width and height
	int ImageWidth;
	int ImageHeight;
	
	// calculation functions, no calculations take place
	// here in the CStatEmpty class 
	double CalcXMEAN(double* dist);
	double CalcYMEAN(double* dist);
	double CalcXRMS(double* dist,double xm);
	double CalcYRMS(double* dist,double ym);

    // calculation of distributions _after_ successful call to NormalizeImage()

	void NormalizeImage( void *inout, BOOL circ );

	// normalizes the image 
	void NormalizeImageRectangular( void *datablock);

	// normalizes the image using circular area of interest
	void NormalizeImageCircular( void *datablock);

    void GetLeftAndRight( int ypos, int &leftval, int &rightval);

	BOOL filter; // TRUE: apply x-ray filtering
	
	// does x-ray filtering of data
	void XFilter(void* data);
private:
	int m_iBpp;
	int m_iBppEffective;
	unsigned int m_uiSumPixels;


    void BackSubtraction(unsigned char *inout_buffer, unsigned char *backbuf);
    void CalcProjection(unsigned char *in, BOOL circular);

};



#endif // !defined(AFX_STAT_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
