// CStatFile.cpp: implementation of the CCStatFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "videoclnt.h"
#include <vfw.h>
#include "headers.h"
#include "Queue.h"
#include "compression.h"
class CVideoclntDlg;
#include <windows.h>
#include <winsock.h>
#undef fd_set
#include "IniEx.h"
#include "Server.h"
#include "ServerList.h"
#include "SSyncSocket.h"
#include "configdlg.h"
#include "CATineDlg.h"
#include "TineALDlg.h"
#include "Stat.h"
#include "StatStreak.h"
#include "BogusTineDlg.h"
#include "ImageAcquire.h"
#include "FourierStat.h"
#include "ImageSequence.h"
#include "StatFile.h"
#include "StatEmpty.h"
#include "FitParam.h"
#include "Fit.h"
#include "MessagesDlg.h"
#include "MyVideoFrame.h"
#include "acop.h"
#include "oleauto.h"
#include "AdditionalInformation.h"
#include "PrinterSettings.h"
#include "MarkedComboBox.h"
#include "InformationDialog.h"
#include "videoclntDlg.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////////////////
// 
// The constructor of CStatFile just does some basic initialization.
//
// Parameters:
//  Num      - number of images that reside inside data
//  data     - pointer to the to be analysed images (must contain at least (ImWi*ImHi+sizeof(VIDHDR))*Num bytes)
//  back     - pointer to the to be subtracted background image (must contain at least ImWi*ImHi bytes)
//  out      - pointer where the analyzed image should be stored (must contain at least ImWi*ImHi bytes)
//  AOI      - rectangular region of area of interest inside frame - ranges must be x = [0,ImWi] y = [0,ImHi]
//  XFilt    - x-ray filtering TRUE=on FALSE=off
//  useclass - which analysis should take place
//             1=fourier, 2=statistical 3=no calculations, just background subtr. and normalization 
//             4=fitting
//  ImWi     - width of the image: _must_ be set to 768 (because of tight optimization)
//  ImHi     - height of the image: _must_ be set to 574 (because of tight optimization)
//
CStatFile::CStatFile(CImageSequence *seq, unsigned char* Back, unsigned char *out, CRect AOI, BOOL XFilt, BOOL circular, BOOL normalize, 
				   int useclass,unsigned int ImWi,unsigned int ImHi, unsigned int Bpp, unsigned int bpp_eff, unsigned int rep_rate)
//CStatFile::CStatFile(unsigned char Num, char* data, unsigned char* Back, unsigned char *out, CRect AOI, BOOL XFilt,
//				   int useclass,unsigned int ImWi,unsigned int ImHi, unsigned int Bpp)
{
	// initialize member variables using parameters
	NumSamples=seq->Count();
	filter=XFilt;
	BackGr=Back;
	outdata = out;
	ImageWidth=ImWi;
	ImageHeight=ImHi;
	ImageBpp = Bpp;
	ImageBppEffective = bpp_eff;
	//array=(unsigned char *) data; //new unsigned char[NumSamples*(ImageWidth*ImageHeight+2*sizeof(int)/sizeof(unsigned char))];
	fourier=useclass; // 1= fourier (CFourierStat) 2=stat (CStat) 3=off (CStatEmpty)
	AOIlocal.CopyRect(&AOI);
		
	// initialize members needed for calculation to have a defined starting value
	XMEAN=0;
	YMEAN=0;
	XRMS=0;
	YRMS=0;
	m_dSumPixels = 0.0;
	m_pSeq = seq;
	m_iRepRate = rep_rate;
	m_bNormalization = normalize;
	m_bCircularAOI = circular;
	// allocate memory needed for calculation
	XDIST= (double *) memman.sw_malloc( sizeof(double)*ImageWidth );
	YDIST= (double *) memman.sw_malloc( sizeof(double)*ImageHeight);

    XDIST_FIT = (double *) memman.sw_malloc( sizeof(double)*ImageWidth );
	YDIST_FIT = (double *) memman.sw_malloc( sizeof(double)*ImageHeight);

	if (XDIST == NULL || YDIST == NULL || XDIST_FIT == NULL || YDIST_FIT == NULL) return;

    for (unsigned int i=0;i<ImageWidth;i++)
	{
     XDIST[i] = 0.0;
	 XDIST_FIT[i] = 0.0;
	}

    for (i=0;i<ImageHeight;i++)
	{
     YDIST[i] = 0.0;
	 YDIST_FIT[i] = 0.0;
	}



}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// In the destructor just the dynamically allocated variables are freed.
//
CStatFile::~CStatFile()
{
	if (XDIST != NULL) memman.sw_free( (void *) XDIST );
	if (YDIST != NULL) memman.sw_free( (void *) YDIST );

	if (XDIST_FIT != NULL) memman.sw_free( (void *) XDIST_FIT );
	if (YDIST_FIT != NULL) memman.sw_free( (void *) YDIST_FIT );
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// In this function the actual calculation takes place. By using the fourier parameter we 
// choose the proper analysis class and do the statistical analysis.
// If we have just one picture (member NumSamples), we subtract the background, normalize the 
// area of interest, do the calculations and put the analysed image into the memory location 
// 'out' which was set by the constructor.
//
// If there is more than one picture to be processed all calculations and normalization and 
// background subtraction is done on _each_ picture. After doing this, all pictures and the 
// results of the calculations are _averaged_.
//
// Please note: When using no statistics no part of the image is normalized. Just the background 
// is subtracted because of optimization reasons.
//
void  CStatFile::START()
{
 CString msg("Error allocating memory in CStatFile. Image(s) were not processed !");

	if (XDIST == NULL || XDIST_FIT == NULL || YDIST == NULL || YDIST_FIT == NULL) 
	{
		((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, msg);
		return;
	}

	{
		if (NumSamples > 1) // we have more than one image to process
		{
		  // allocate temporary buffer needed for temporary storage of 
		  // the output for each processed image
		  unsigned char *outtemp = (unsigned char *) memman.sw_malloc( ImageWidth*ImageHeight*ImageBpp/8 );
		  if (outtemp == NULL) 
		  {
			  ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, msg);
			  return;
		  }
		  
		  // allocate temporary buffer needed for averaging the outputs
		  // of the single image processing
		  unsigned int *averaged = (unsigned int *) memman.sw_malloc( sizeof(int)*ImageWidth*ImageHeight );
		  if (averaged == NULL) 
		  {
			  memman.sw_free( (void *) outtemp );
			  ((CVideoclntDlg *)AfxGetMainWnd())->OwnMessageBox(MB_ICONEXCLAMATION, msg);
			  return;
		  }
		  
		  // clear average buffer
		  memset( (void *) averaged, 0, ImageWidth*ImageHeight*sizeof(unsigned int) );

          // clear XDIST
		  for (register unsigned int j=0;j<ImageWidth;j++)
		  {
           XDIST[j] = 0.0;
		   XDIST_FIT[j] = 0.0;
		  }

          // clear YDIST  
		  for (j=0;j<ImageHeight;j++)
		  {
           YDIST[j] = 0.0;
		   YDIST_FIT[j] = 0.0;
		  }

          // walk over each frame
		  for(unsigned int i=0;i<NumSamples;i++)
		  {
			BOOL ret = m_pSeq->GetFrame( i, (const char **) &pDATA);
			pDATA += sizeof(VIDHDR);
			
			//if (ret == FALSE) continue;
			// calculate starting point in 'data' of i-th image 
			//pDATA= (unsigned char *) m_pSeq->  &array[(i*size)+sizeof(VIDHDR)];
			int w=ImageWidth,h=ImageHeight,bpp=ImageBpp, bpp_eff=ImageBppEffective;
			if(fourier==1) // fourier analysis
			{
				//instantiate fourier class and do analysis
				CFourierStat st(pDATA,BackGr,outtemp, AOIlocal,filter, m_bCircularAOI, m_bNormalization,5,1,w,h, bpp, bpp_eff);
				st.START();
				
				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;
				
				// add projections to XDIST and YDIST array
				// (will be averaged later)
				double *disttemp = st.X_DIST();
				for (register unsigned int t=0;t<ImageWidth;t++)
				{
					XDIST[t] += disttemp[t];
                }

				disttemp = st.Y_DIST();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST[t] += disttemp[t];
                }
			}
			else if(fourier == 2)
			{
				//instantiate straightforward class and do analysis
				CStat st(pDATA,BackGr,outtemp, AOIlocal,filter, m_bCircularAOI, m_bNormalization, w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// add projections to XDIST and YDIST array
				// (will be averaged later)
				double *disttemp = st.X_DIST();
				for (register unsigned int t=0;t<ImageWidth;t++)
				{
					XDIST[t] += disttemp[t];
                }

				disttemp = st.Y_DIST();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST[t] += disttemp[t];
                }
			}
			else if(fourier == 3){

				//instantiate 'empty' class and do analysis
				CStatEmpty st(pDATA,BackGr, outtemp, AOIlocal,filter, m_bCircularAOI,m_bNormalization, w,h, bpp, bpp_eff);
				st.START();
				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// add projections to XDIST and YDIST array
				// (will be averaged later)
				double *disttemp = st.X_DIST();
				for (register unsigned int t=0;t<ImageWidth;t++)
				{
					XDIST[t] += disttemp[t];
                }

				disttemp = st.Y_DIST();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST[t] += disttemp[t];
                }
			}
			else if(fourier == 4)
			{
                CFit  fit(pDATA, BackGr, outtemp, AOIlocal, filter, m_bCircularAOI, m_bNormalization, 5, 1, w, h, 0, bpp, bpp_eff); // rep_Rate=0->offline calc

				XMEAN+=fit.XM() / NumSamples;
				YMEAN+=fit.YM() / NumSamples;
				XRMS+=fit.X_RMS() / NumSamples;
				YRMS+=fit.Y_RMS() / NumSamples;
				m_dSumPixels += (1.0*fit.SUM_PIXELS())/NumSamples;

				// add projections to XDIST and YDIST array
				// (will be averaged later)
				double *disttemp = fit.X_DIST();
				for (register unsigned int t=0;t<ImageWidth;t++)
				{
					XDIST[t] += disttemp[t];
                }

				disttemp = fit.Y_DIST();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST[t] += disttemp[t];
                }

				disttemp = fit.X_DIST_FIT();
				for (t=0;t<ImageWidth;t++)
				{
					XDIST_FIT[t] += disttemp[t];
                }

				disttemp = fit.Y_DIST_FIT();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST_FIT[t] += disttemp[t];
                }
			}
			else if(fourier == 5) // Streak mode
			{
				//instantiate straightforward class and do analysis
				CStatStreak st(pDATA,BackGr,outtemp, AOIlocal,filter, m_bCircularAOI, m_bNormalization, w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// add projections to XDIST and YDIST array
				// (will be averaged later)
				double *disttemp = st.X_DIST();
				for (register unsigned int t=0;t<ImageWidth;t++)
				{
					XDIST[t] += disttemp[t];
                }

				disttemp = st.Y_DIST();
				for (t=0;t<ImageHeight;t++)
				{
					YDIST[t] += disttemp[t];
                }
			}


            // add output image (normalized and background subtracted and possibly x-ray filtered)
			// to averaged buffer
			if (ImageBpp == 8)
			{
				for (register int m=0;m<w*h;m++)
				{
					averaged[m] += (unsigned int) outtemp[m];
				}
			}
			else
			{
				unsigned short *outtemp16 = (unsigned short *) outtemp;
				for (register int m=0;m<w*h;m++)
				{
					averaged[m] += (unsigned int) outtemp16[m];
				}
			}

		  } // for

          
		  if (ImageBpp == 8)
		  {
			// average the averaged buffer by dividing each pixel by the 'NumSamples' count
			for (register unsigned int m=0;m<ImageWidth*ImageHeight;m++)
			{
		  		outdata[m] = (unsigned char) (averaged[m] / ((unsigned int) NumSamples));
			}
		  }
		  else if (ImageBpp == 16)
		  {
			  unsigned short *outdata16 = (unsigned short *) outdata;

			// average the averaged buffer by dividing each pixel by the 'NumSamples' count
			for (register unsigned int m=0;m<ImageWidth*ImageHeight;m++)
			{
		  		outdata16[m] = (unsigned short) (averaged[m] / ((unsigned int) NumSamples));
			}
		  }
		  
		  // average XDIST array
		  for (register unsigned int m=0;m<ImageWidth;m++)
		  {
		 	XDIST[m] /= (double) NumSamples;
			XDIST_FIT[m] /= (double) NumSamples;
		  }
			
          // average YDIST array
		  for (m=0;m<ImageHeight;m++)
		  {
			YDIST[m] /= (double) NumSamples;
			YDIST_FIT[m] /= (double) NumSamples;
		  }
         
          // delete temporary allocated memory
		  memman.sw_free( (void *) outtemp );
		  memman.sw_free( (void *) averaged );
		} 
		else // just one picture to process 
		{
		    unsigned int i = 0;
            // calculate starting position of frame inside video data buffer
		
			BOOL ret = m_pSeq->GetFrame( i, (const char **) &pDATA);
			pDATA += sizeof(VIDHDR);

			//pDATA=&array[(i*size)+sizeof(VIDHDR)];
			int w=ImageWidth,h=ImageHeight,bpp=ImageBpp, bpp_eff=ImageBppEffective;
			if(fourier==1) // fourier mode
			{
				//instantiate fourier class and do analysis
				CFourierStat st(pDATA,BackGr,outdata, AOIlocal,filter, m_bCircularAOI,m_bNormalization, 5,1,w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;
				
				// copy distributions from the class CFourierStat into the arrays of CStatFile
				memcpy(XDIST,st.X_DIST(),ImageWidth*sizeof(double));
				memcpy(YDIST,st.Y_DIST(),ImageHeight*sizeof(double));

			}
			else if(fourier == 2) // straightforward mode
			{
				//instantiate straightforward class and do analysis

				CStat st(pDATA,BackGr,outdata, AOIlocal,filter, m_bCircularAOI,m_bNormalization, w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// copy distributions from the class CFourierStat into the arrays of CStatFile
				memcpy(XDIST,st.X_DIST(),ImageWidth*sizeof(double));
				memcpy(YDIST,st.Y_DIST(),ImageHeight*sizeof(double));
			}
			else if(fourier == 3) // no calculations
			{
				//instantiate 'empty' class and do analysis
				CStatEmpty st(pDATA,BackGr, outdata, AOIlocal,filter, m_bCircularAOI,m_bNormalization, w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// copy distributions from the class CFourierStat into the arrays of CStatFile
				memcpy(XDIST,st.X_DIST(),ImageWidth*sizeof(double));
				memcpy(YDIST,st.Y_DIST(),ImageHeight*sizeof(double));
			}
			else if(fourier == 4)
			{
                CFit  fit(pDATA, BackGr, outdata, AOIlocal, filter, m_bCircularAOI, m_bNormalization, 5, 1, w, h, m_iRepRate, bpp, bpp_eff);

				XMEAN+=fit.XM() / NumSamples;
				YMEAN+=fit.YM() / NumSamples;
				XRMS+=fit.X_RMS() / NumSamples;
				YRMS+=fit.Y_RMS() / NumSamples;
				m_dSumPixels += (1.0*fit.SUM_PIXELS())/NumSamples;

				memcpy(XDIST,fit.X_DIST(),ImageWidth*sizeof(double));
				memcpy(YDIST,fit.Y_DIST(),ImageHeight*sizeof(double));
				memcpy(XDIST_FIT,fit.X_DIST_FIT(),ImageWidth*sizeof(double));
				memcpy(YDIST_FIT,fit.Y_DIST_FIT(),ImageHeight*sizeof(double));
			}
			else if(fourier == 5) // Streak mode
			{
				//instantiate straightforward class and do analysis

				CStatStreak st(pDATA,BackGr,outdata, AOIlocal,filter, m_bCircularAOI,m_bNormalization, w,h, bpp, bpp_eff);
				st.START();

				// add the calculated spot center and size to real variables (divided by count of pictures)
				XMEAN+=st.XM()/NumSamples;
				YMEAN+=st.YM()/NumSamples;
				XRMS+=st.X_RMS()/NumSamples;
				YRMS+=st.Y_RMS()/NumSamples;
				m_dSumPixels += (1.0*st.SUM_PIXELS())/NumSamples;

				// copy distributions from the class CFourierStat into the arrays of CStatFile
				memcpy(XDIST,st.X_DIST(),ImageWidth*sizeof(double));
				memcpy(YDIST,st.Y_DIST(),ImageHeight*sizeof(double));
			}


		}

	}

}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the X distribution array for further processing in the 
// main program. ttention: After destructing the class this pointer is no longer valid.
//
double*  CStatFile::X_DIST()
{
	return XDIST;
}

double*  CStatFile::X_DIST_FIT()
{
	return XDIST_FIT;
}


///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the Y distribution array for further processing in the 
// main program. ttention: After destructing the class this pointer is no longer valid.
//
double*  CStatFile::Y_DIST()
{
	return YDIST;
}

double*  CStatFile::Y_DIST_FIT()
{
	return YDIST_FIT;
}


///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the x coordinate of the spot size for further processing 
// in the main program. 
//
double  CStatFile::X_RMS()
{
	return XRMS;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the y coordinate of the spot size for further processing 
// in the main program. 
//
double  CStatFile::Y_RMS()
{
	return YRMS;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the x coordinate of the spot center for further processing 
// in the main program. 
//
double  CStatFile::XM()
{
	return XMEAN;
}

///////////////////////////////////////////////////////////////////////////////////////////
// 
// This public function just returns the y coordinate of the spot center for further processing 
// in the main program. 
//
double  CStatFile::YM()
{
	return YMEAN;
}

double CStatFile::SUM_PIXELS()
{
 return( m_dSumPixels );
}
