// StatFile.h: interface for the CStatFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STATFILE_H__B552A98A_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
#define AFX_STATFILE_H__B552A98A_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* 
   This class is the class that does the statistical analysis. It makes use of CStatEmpty, 
   CStat and CFourierStat classes to support no, straightforward and fourier statistics.

   It is kind of an extension to the other stat. classes to support analysis of multiple
   images in a batch.

   In addition to the analysis the image(s) are also background subtracted, normalized 
   (on the area of interest) and x-ray filtered.
*/

class CStatFile  
{
public:
    // construction and initialization of some basic variables
	//CStatFile(unsigned char Num, char* name, unsigned char* Back, unsigned char *out, CRect AOI,BOOL XFilt,
	//	int useclass,unsigned int ImWi,unsigned int ImHi, unsigned int Bpp);
    CStatFile::CStatFile(CImageSequence *seq, unsigned char* Back, unsigned char *out, CRect AOI, BOOL XFilt, BOOL circular, BOOL normalize, 
				   int useclass,unsigned int ImWi,unsigned int ImHi, unsigned int Bpp, unsigned int bpp_eff, unsigned int rep_rate);
    
	virtual ~CStatFile();

    // do real calculations
	void START();

    // returns the x coordinate of the spot center
	double XM();
	// returns the y coordinate of the spot center
	double YM();
    // returns the x coordinate of the spot sizer
	double X_RMS();
	// returns the y coordinate of the spot sizer
	double Y_RMS();
	// returns the x distribution array
	double* X_DIST();
	// returns the y distribution array
	double* Y_DIST();

    double*  X_DIST_FIT();
    double*  Y_DIST_FIT();

    // returns the sum of all pixels (for laser power measurement)
	double SUM_PIXELS();

protected:

    // width and height of the to be analysed image
	unsigned int ImageWidth;
	unsigned int ImageHeight;

    // spot center coordinates
	double XMEAN;
	double YMEAN;

    // spot size coordinates
	double XRMS;
	double YRMS;

    /* distribution arrays */
	double* XDIST;
	double* YDIST;

    /* distribution arrays */
	double* XDIST_FIT;
	double* YDIST_FIT;

    // 1=fourier 2=straightforward 3=no calculations 4=fitting
	int fourier;

    // area of interest rectangle inside the video frame
	CRect AOIlocal;
	
	// x-ray filtering: TRUE = on
	BOOL filter;

	// x-ray filtering procedure

    // temporary variables 
	unsigned char  MaxVal;
	unsigned char *pDATA; // temp data pointer
	//unsigned char *array; // actual data pointer
	unsigned char *BackGr;// pointer to background image
	unsigned char  NumSamples;
	unsigned char *outdata; // pointer to buffer for processed image
private:
	BOOL m_bCircularAOI;
	BOOL m_bNormalization;
	int m_iRepRate;
	CImageSequence * m_pSeq;
	int ImageBpp;
	int ImageBppEffective;
	double m_dSumPixels;
};

#endif // !defined(AFX_STATFILE_H__B552A98A_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)

