// StatStreak.h: interface for the CStat class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STATSTREAK_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
#define AFX_STATSTREAK_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////////////////
//
// This class does straightforward statistical analysis. Using weight of the masses 
// in x projection and y projetion the class is able to detect the spot center coor-
// dinates and the spot size. Straightforward statistics fail in some cases, e.g. if 
// there is more than one spot on the video frame. In addition to calculating spot
// center and size, this class also does subtraction of the background image and 
// normalization of the area of interest.
//

class CStatStreak 
{
public:
	unsigned int SUM_PIXELS();
	// construction of the class
	CStatStreak(unsigned char* data,unsigned char* Back, unsigned char *out, CRect AOI,BOOL XFilt, BOOL circular, BOOL normalize, unsigned int ImWi,unsigned int lmHi, unsigned int bpp, unsigned int bpp_eff);
	
	// destruction of the class
	virtual ~CStatStreak();

    // returns x mean value (spot center on x-axis)
	double XM();
	// returns y mean value (spot center on y-axis)
	double YM();
	
	// returns x rms value (spot size on x-axis)
	double X_RMS();
	// returns y rms value (spot size on y-axis)
	double Y_RMS();
	
    // returns the x-distribution (dist over all columns)
	double* X_DIST();
	// returns the y-distribution (dist over all rows)
	double* Y_DIST();
	
	// do the calculation of spot center and size
	void START();

protected:
    // floating point distribution arrays
	double* XDIST;
	double* YDIST;
 
    // integer distribution array (needed for calculation in inner loop)
	double *YDISTd;
	double *XDISTd;

    // member variables used to store the calculated spot center and size 
	double XMEAN;
	double YMEAN;
	double XRMS;
	double YRMS;

    // member variables to store important variables
	BOOL filter; // TRUE: apply x-ray filtering
	unsigned char* array; // input array
	unsigned char* BackGr; // background image
	int left; // leftmost pixel position of area of interest
	int right; // rightmost pixel position of area of interest
	int up; // topmost pixel position of area of interest
	int down; // bottom-most pixel position of area of interest
	int ImageWidth; // width of the to-be-analyzed image
	int ImageHeight; // height of the to-be-analyzed image
	int center_x;
	int center_y;
	int length;
	
	// calculate x distribution: deprecated, replaced by NormalizeImage() and CalcXYDistSW()
	// calculates y distribution: deprecated, replaced by NormalizeImage() and CalcXYDistSW()
	
    // calculation of distributions _after_ successful call to NormalizeImage()
	
	// calculates x mean value
	double CalcXMEAN(double* dist);
	// calculates y mean value
	double CalcYMEAN(double* dist);
	// calculates x rms value
	double CalcXRMS(double* dist,double xm);
	// calculates y rms value
	double CalcYRMS(double* dist,double ym);
	
	void NormalizeImage( void *inout, BOOL circ );

	// normalizes the image 
	void NormalizeImageRectangular(void *datablock);

	// normalizes the image using circular area of interest
	void NormalizeImageCircular(void *datablock);

    // does the x-ray filtering
	void XFilter(void* data);

    // not used at the moment

    // fast memory copy routine
private:
	void CalcProjection( unsigned char *in, BOOL circular);
	void BackSubtraction( unsigned char *inout_buffer, unsigned char *backbuf);
	int m_iBpp;
	int m_iBppEffective;
	void GetLeftAndRight( int ypos, int &leftval, int &rightval);
	unsigned int m_uiSumPixels;
};



#endif // !defined(AFX_STAT_H__B552A988_6D65_11D6_AFB1_0002B34B0F4F__INCLUDED_)
