#ifndef _ALMSRV_H_
#define _ALMSRV_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#include <time.h>
#ifndef BYTE
#  define BYTE unsigned char
#endif
#ifndef UINT16
#  define UINT16 unsigned short
#endif
#ifndef UINT32
#  define UINT32 unsigned long
#endif

#define MAXALARMS 100
#define DEFAULT_ALARM_WINDOW 3
#define DEFAULT_ALARM_HEARTBEAT 1200
#define DEFAULT_INSTANT_WINDOW 4
#define DEFAULT_TERMINATION_WINDOW 4

#define almNEWALARM     0x01
#define almHEARTBEAT    0x02
#define almOSCILLATION  0x04
#define almDATACHANGE   0x08
#define almINSTANT      0x10
#define almTERMINATE    0x40
#define almTEST         0x80

typedef struct AlarmMsgStruct
{
  char tag[16];
  char device[16];
  char alarmTag[16];
  UINT32 alarmCode;
  time_t timestamp;
  UINT32 alarmMask;
  BYTE alarmData[6];
  BYTE alarmDataFormat;
  BYTE alarmDataArraySize;
  BYTE severity;
  BYTE descriptor;
  UINT16 alarmSystem;
} AMS;                    /* AMS sent out upon request */
#define AMS_SIZE  72

typedef struct ADStag    /* Alarm Definition Structure */
{
   char alarmTag[16];
   UINT32 alarmCode;
   UINT32 alarmMask;
   short severity;
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   UINT32 timeRes;
   short hitsChannel;
   short hitsModule;
   char alarmText[40];
   char deviceText[38];
   UINT16 alarmSystem;  /* take 2 bytes from deviceText to hold the system ID */
   char dataText[40];
   char url[40];
   UINT32 ndefs;
} ADS;
#define ADS_SIZE  200

typedef struct AMRBtag
{
  UINT32 almStartTime;
  UINT32 almStopTime;
  UINT32 almMinSeverity;
  UINT32 reservered;
} AMRB;                   /* Alarm Message Request Block */
#define AMRB_SIZE  16

typedef struct ALARMstruct/* local alarm list          */
{
  time_t timestamp;       /* our old friend                                */
  long   alarmCode;       /* standardized return code (implies severity)   */
  BYTE   alarmData[6];    /* alarm specific data (database interpreted)    */
  BYTE   alarmStatus;     /* corresponds to the descriptor                 */
  BYTE   clrCount;        /* record of clear requests                      */
  struct ALARMstruct *next;
} ALARM;

extern ALARM *almList;
extern short EqpFormats[];

int SetDeviceAlarm(char *name,char *devname,long almCode,BYTE *almData,BYTE flags);
int SetAlarmEx(char *name,short devNr,long almCode,BYTE *almData,BYTE almflags);
int SetAlarm(char *tag,short devNr,long almCode,BYTE *almData);
int getAlarmsEx(char *eqpname,AMS *ams,short num,short minSeverity,time_t starttime,time_t stoptime,short startnum);
int getAlarms(char *tag,AMS *ams,short num,short minSeverity);
int numAlarmsEx(char *eqpname,time_t *almTimeStamp,short *almSeverity,short *NrAtAlmTimeStamp,short *NrAtAlmSeverity);
int numAlarms(char *tag,time_t *almTimeStamp,short *almSeverity);
void ClearAlarm(char *tag,short devNr);
void ClearDeviceAlarm(char *name,char *devname);
void RemoveDeviceAlarm(char *name,char *devname,long code);
void RemoveAlarm(char *name,short devNr,long code);
short findSeverity(char *name,long cc);
int RegisterSeverity(int almCode,int almSeverity);
int fillHRTstamp(BYTE *HRTstamp);
void getAlarmInfo(void);
int getAlarmDefs(char *eqm,ADS **ads);
int numAlarmDefs(char *eqm);
ADS *getAlarmTable(char *eqm,long code);
int getAlarmInfoTable(char *eqname);
int initAlarmServer(void);
int GetAlarm(char *eqpname,char *devname,AMS *ams);
int SetAlarmByDeviceNumber(char *name,short devNr,long almCode,BYTE *almData,BYTE flags,time_t t);
int SetHrtAlarm(char *name,short devNr,long almCode,BYTE *almData,BYTE flags,time_t t);

extern char *tagList[];
extern int numStoredAlarms;
extern int AlarmWindow;
extern int AlarmHeartbeat;
extern int AlarmInstantWindow;
extern int NTAGS;
extern int NSEVERITYCODE;
extern int MaxNumberAlarms;
extern int AlarmTestState;
extern int ALMdebug;

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* _ALMSRV_H_ */
